Package: PRA
Type: Package
Title: Project Risk Analysis
Version: 0.6.0
Authors@R: person("Paul", "Govan", email = "paul.govan2@gmail.com", role = c("aut", "cre", "cph"), comment = c(ORCID = "0000-0002-1821-8492"))
Description: Data analysis for Project Risk Management via the Second Moment Method, 
  Monte Carlo Simulation, Contingency Analysis, Sensitivity Analysis, Earned Value Management, 
  Learning Curves, Bayesian Methods, and more.
Depends: R (>= 4.1.0)
Imports: mc2d, minpack.lm, stats
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
URL: https://paulgovan.github.io/PRA/, https://github.com/paulgovan/PRA
BugReports: https://github.com/paulgovan/PRA/issues
Suggests: base64enc, corrplot, devtools, ellmer, ggplot2, igraph,
        jsonlite, knitr, networkD3, remotes, rmarkdown, scales,
        mcptools, testthat (>= 3.0.0), withr
Config/testthat/edition: 3
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-08-25 23:30:26 UTC; paulgovan
Author: Paul Govan [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-1821-8492>)
Maintainer: Paul Govan <paul.govan2@gmail.com>
Repository: CRAN
Date/Publication: 2026-08-28 14:40:17 UTC
Built: R 4.5.2; ; 2026-08-28 18:02:21 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(plot,dsm)
S3method(print,dsm)
S3method(print,mcs)
S3method(print,pra_sigmoidal_fit)
S3method(print,smm)
export(ac)
export(contingency)
export(cor_matrix)
export(cost_pdf)
export(cost_post_pdf)
export(cpi)
export(cv)
export(eac)
export(etc)
export(ev)
export(fit_sigmoidal)
export(grandparent_dsm)
export(mcs)
export(parent_dsm)
export(plot_sigmoidal)
export(pra_mcp_server)
export(pra_tools)
export(predict_sigmoidal)
export(prob_net)
export(prob_net_learn)
export(prob_net_sim)
export(prob_net_update)
export(pv)
export(risk_post_prob)
export(risk_prob)
export(sensitivity)
export(smm)
export(spi)
export(sv)
export(tcpi)
export(vac)
importFrom(grDevices,heat.colors)
importFrom(graphics,axis)
importFrom(graphics,image)
importFrom(graphics,lines)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(mc2d,qtriang)
importFrom(mc2d,rtriang)
importFrom(minpack.lm,nlsLM)
importFrom(stats,coef)
importFrom(stats,cor)
importFrom(stats,df.residual)
importFrom(stats,median)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,qnorm)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,qunif)
importFrom(stats,rbinom)
importFrom(stats,rlnorm)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,var)
importFrom(stats,vcov)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# PRA <a href="https://paulgovan.github.io/PRA/"><img src="man/figures/logo.png" align="right" height="137" alt="PRA website" /></a>

<!-- badges: start -->

[![Project Status: Active – The project has reached a stable, usable
state and is being actively
developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
[![CRAN
status](https://www.r-pkg.org/badges/version/PRA)](https://CRAN.R-project.org/package=PRA)
[![R-CMD-check](https://github.com/paulgovan/PRA/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/paulgovan/PRA/actions/workflows/R-CMD-check.yaml)
[![Codecov test
coverage](https://codecov.io/gh/paulgovan/PRA/graph/badge.svg)](https://app.codecov.io/gh/paulgovan/PRA)
[![](http://cranlogs.r-pkg.org/badges/grand-total/PRA)](https://cran.r-project.org/package=PRA)
[![](http://cranlogs.r-pkg.org/badges/last-month/PRA)](https://cran.r-project.org/package=PRA)
[![](https://img.shields.io/badge/doi-10.32614/CRAN.package.PRA-green.svg)](https://doi.org/10.32614/CRAN.package.PRA)
<!-- badges: end -->

## Introduction

Welcome to **PRA**! This project provides a set of tools for performing
Project Risk Analysis (PRA) using various quantitative methods. It is
designed to help project analysts assess and manage risks associated
with project schedules, costs, and performance.

PRA can be used as a traditional R package with direct function calls,
or as an **AI-powered risk analysis agent** with three input modes:

| Input | Route | When to use |
|----|----|----|
| `/command` | Deterministic tool execution | Reliable, instant results |
| Natural language with data | LLM selects and calls tools | Exploratory analysis |
| Conceptual question | RAG knowledge base | “What is CPI?”, “How does MCS work?” |

## Key Features

### Analytical Methods

- [Second Moment
  Analysis](https://paulgovan.github.io/PRA/articles/SMM.html)
- [Monte Carlo
  Simulation](https://paulgovan.github.io/PRA/articles/MCS.html)
- [Contingency
  Analysis](https://paulgovan.github.io/PRA/articles/MCS.html#contingency)
- [Sensitivity
  Analysis](https://paulgovan.github.io/PRA/articles/MCS.html#sensitivity)
- [Earned Value
  Management](https://paulgovan.github.io/PRA/articles/evm.html)
- [Learning
  Curves](https://paulgovan.github.io/PRA/articles/sigmoidal.html)
- [Bayesian
  Methods](https://paulgovan.github.io/PRA/articles/Bayes.html)

### AI Agent Framework

- **Slash commands** (`/mcs`, `/evm`, `/risk`, …) — deterministic tool
  calls that bypass the LLM for instant, reliable results
- **Natural language interface** — ask risk analysis questions in plain
  English; the agent calls the right tools when you provide numerical
  data
- **RAG-enhanced reasoning** — conceptual questions answered from domain
  knowledge with source citations
- **Fully local** — runs on your machine via
  [Ollama](https://ollama.com), no API keys or cloud services
- **Interactive Shiny app** — chat UI with `/commands`, data upload,
  inline plots, and model settings
- **Evaluation framework** — measure tool-calling accuracy with
  [vitals](https://vitals.tidyverse.org)

See the [Agentic Risk
Analysis](https://paulgovan.github.io/PRA/articles/agent.html) vignette
for details.

## Installation

To install the release version of PRA, use:

``` r
install.packages('PRA')
```

You can install the development version of PRA like so:

``` r
devtools::install_github('paulgovan/PRA')
```

## Usage

### Traditional R Interface

Here is a simple example of how to use the package for a common PRA
task.

First, load the package:

``` r
library(PRA)
```

Suppose you have a simple project with 3 tasks (A, B, and C), but the
duration of each task is uncertain. You can describe the uncertainty of
each task using probability distributions and then run a Monte Carlo
Simulation (MCS) to estimate the overall project duration.

To do so, set the number of simulations and describe probability
distributions for each work package. In this case, run 10,000
simulations with the following distributions:

``` r
num_simulations <- 10000
task_distributions <- list(
  list(type = "normal", mean = 10, sd = 2), # Task A: Normal distribution
  list(type = "triangular", a = 5, b = 10, c = 15), # Task B: Triangular distribution
  list(type = "uniform", min = 8, max = 12) # Task C: Uniform distribution
)
```

Then run the simulation using the `mcs` function and store the results:

``` r
results <- mcs(num_simulations, task_distributions)
```

To visualize the results, you can create a histogram of the total
project duration. You can also overlay a normal distribution curve based
on the mean and standard deviation of the results:

``` r
hist(results$total_distribution,
  freq = FALSE, breaks = 50, main = "Distribution of Total Project Duration",
  xlab = "Total Duration", col = "skyblue", border = "white"
)
curve(dnorm(x, mean = results$total_mean, sd = results$total_sd), add = TRUE, col = "darkblue")
```

<img src="man/figures/README-unnamed-chunk-5-1.png" width="100%" />

This will give you a visual representation of the uncertainty in the
total project duration based on the individual task distributions. On
average, the project is expected to take around 30 time units, but could
take as little as 20 or as much as 40 time units.

### AI Agent Interface

PRA provides three ways to run risk analyses through its agent
framework.

**Slash commands** (deterministic, no LLM required):

``` r
# Monte Carlo simulation
r <- PRA:::execute_command(
  '/mcs tasks=[{"type":"normal","mean":10,"sd":2},{"type":"triangular","a":5,"b":10,"c":15}]'
)
cat(r$result)

# Chain: contingency reserve from the MCS result above
r <- PRA:::execute_command("/contingency phigh=0.95 pbase=0.50")
cat(r$result)

# Full EVM analysis
r <- PRA:::execute_command(
  "/evm bac=500000 schedule=[0.2,0.4,0.6,0.8,1.0] period=3 complete=0.35 costs=[90000,195000,310000]"
)
cat(r$result)

# Bayesian risk probability
r <- PRA:::execute_command("/risk causes=[0.3,0.2] given=[0.8,0.6] not_given=[0.2,0.4]")
cat(r$result)

# List all commands
r <- PRA:::execute_command("/help")
cat(r$result)
```

**Chat interface** (natural language, requires Ollama):

``` r
chat <- pra_chat(model = "llama3.2")

# Numerical data → LLM calls the appropriate tool
chat$chat("Run a Monte Carlo simulation for a 3-task project with
  Task A ~ Normal(10, 2), Task B ~ Triangular(5, 10, 15),
  Task C ~ Uniform(8, 12). Use 10,000 simulations.")

# Conceptual question → answered from RAG knowledge base
chat$chat("What is the difference between SPI and CPI?")
```

**Interactive Shiny app:**

``` r
pra_app()
```

The app supports all three input modes — `/commands` for instant
results, natural language for LLM tool calls, and conceptual questions
for RAG-powered answers:

<img src="man/figures/pra-app-welcome.png" width="100%" />

### Custom Knowledge Base

Extend the agent’s domain knowledge with your own documents:

``` r
store <- build_knowledge_base()
add_documents(store, "path/to/project_docs/")
```

## More Resources

This project was inspired by the book [Data Analysis for Engineering and
Project Risk Managment](https://doi.org/10.1007/978-3-030-14251-3) by
Ivan Damnjanovic and Ken Reinschmidt and is highly recommended.

## Code of Conduct

Please note that the PRA project is released with a [Contributor Code of
Conduct](https://contributor-covenant.org/version/2/1/CODE_OF_CONDUCT.html).
By contributing to this project, you agree to abide by its terms.

# PRA Agent Skills

These are [Agent Skills](https://modelcontextprotocol.io) that document when and
how an AI agent should call PRA's analytical tools. They are designed to pair
with the PRA MCP server (`pra_mcp_server()`), which serves the same tools that
`pra_tools()` defines.

Each subdirectory contains a `SKILL.md` with YAML frontmatter (`name`,
`description`) and instructions:

| Skill | Covers | Primary MCP tools |
|---|---|---|
| `pra-overview` | Routing / entry point | (directs to the tools below) |
| `pra-monte-carlo` | Schedule/cost uncertainty | `mcs_tool`, `smm_tool` |
| `pra-sensitivity-contingency` | Reserves & risk drivers | `contingency_tool`, `sensitivity_tool` |
| `pra-earned-value` | Performance & forecasting | `evm_analysis_tool` |
| `pra-bayesian-risk` | Risk probability & cost | `risk_prob_tool`, `risk_post_prob_tool`, `cost_pdf_tool`, `cost_post_pdf_tool` |
| `pra-learning-curves` | S-curve fitting/forecasts | `fit_and_predict_sigmoidal_tool` |
| `pra-dsm` | Task dependency structure | `parent_dsm_tool`, `grandparent_dsm_tool` |

## Using the skills with the MCP server

Register the server with an MCP-compatible client, then make the skills
available to the agent. For Claude Code:

```sh
claude mcp add -s project pra -- Rscript -e "PRA::pra_mcp_server()"
```

The skill files can be copied to (or referenced from) your agent's skills
directory. Locate the installed copies with:

```r
system.file("skills", package = "PRA")
```

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
set.seed(42)

## ----setup--------------------------------------------------------------------
library(PRA)

## -----------------------------------------------------------------------------
num_simulations <- 10000
task_distributions <- list(
  list(type = "normal", mean = 10, sd = 2), # Task A
  list(type = "triangular", a = 5, b = 10, c = 15), # Task B
  list(type = "uniform", min = 8, max = 12) # Task C
)

## -----------------------------------------------------------------------------
correlation_matrix <- matrix(c(
  1.0, 0.5, 0.3,
  0.5, 1.0, 0.4,
  0.3, 0.4, 1.0
), nrow = 3, byrow = TRUE)

## -----------------------------------------------------------------------------
results <- mcs(num_simulations, task_distributions, correlation_matrix)

## ----results='asis'-----------------------------------------------------------
cat("Mean Total Duration:     ", round(results$total_mean, 2), "weeks\n")
cat("Variance of Duration:    ", round(results$total_variance, 2), "\n")
cat("Std Dev of Duration:     ", round(results$total_sd, 2), "weeks\n")

## -----------------------------------------------------------------------------
hist_data <- results$total_distribution

hist(hist_data,
  breaks = 50, freq = FALSE,
  main = "Monte Carlo Simulation - Total Project Duration",
  xlab = "Total Duration (weeks)", col = "steelblue", border = "white"
)
lines(density(hist_data), col = "tomato", lwd = 2)
abline(v = results$total_mean, col = "black", lty = 2, lwd = 1.5)
legend("topright",
  legend = c("Density", paste0("Mean = ", round(results$total_mean, 1), " wks")),
  col = c("tomato", "black"), lty = c(1, 2), lwd = 2, bty = "n"
)

## -----------------------------------------------------------------------------
knitr::kable(
  data.frame(
    Percentile = c("P5", "P50 (Median)", "P95"),
    Duration = round(results$percentiles, 1),
    Meaning = c(
      "5% chance of finishing this fast or faster",
      "Equal chance of finishing above or below this",
      "95% chance of finishing by this date"
    )
  ),
  caption = "Simulation Percentiles"
)

## ----results='asis'-----------------------------------------------------------
contingency_val <- contingency(results, phigh = 0.95, pbase = 0.50)
cat("Schedule contingency (P95 − P50):", round(contingency_val, 2), "weeks\n")
cat(
  "There is a 95% chance the project will finish within",
  round(results$percentiles["95%"], 1), "weeks.\n"
)

## -----------------------------------------------------------------------------
sensitivity_results <- sensitivity(task_distributions, correlation_matrix)

sens_data <- data.frame(
  Task        = c("Task A (Normal)", "Task B (Triangular)", "Task C (Uniform)"),
  Sensitivity = sensitivity_results
)

p <- ggplot2::ggplot(
  sens_data,
  ggplot2::aes(x = Sensitivity, y = reorder(Task, Sensitivity))
) +
  ggplot2::geom_col(fill = "steelblue") +
  ggplot2::geom_text(ggplot2::aes(label = round(Sensitivity, 3)),
    hjust = -0.1, size = 3.5
  ) +
  ggplot2::labs(
    title = "Tornado Chart - Task Sensitivity",
    x     = "Sensitivity Coefficient",
    y     = NULL
  ) +
  ggplot2::xlim(0, max(sensitivity_results) * 1.2) +
  ggplot2::theme_minimal()

print(p)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(PRA)

## -----------------------------------------------------------------------------
bac <- 500000
schedule <- c(0.10, 0.25, 0.50, 0.75, 1.00)
time_period <- 3

## ----results='asis'-----------------------------------------------------------
pv_val <- pv(bac, schedule, time_period)
cat("Planned Value (PV): $", format(pv_val, big.mark = ","), "\n")

## ----results='asis'-----------------------------------------------------------
actual_per_complete <- 0.40
ev_val <- ev(bac, actual_per_complete)
cat("Earned Value (EV): $", format(ev_val, big.mark = ","), "\n")

## ----results='asis'-----------------------------------------------------------
period_costs <- c(45000, 110000, 135000)
ac_val <- ac(period_costs, time_period, cumulative = FALSE)
cat("Actual Cost (AC): $", format(ac_val, big.mark = ","), "\n")

## ----results='asis'-----------------------------------------------------------
sv_val <- sv(ev_val, pv_val)
cv_val <- cv(ev_val, ac_val)
spi_val <- spi(ev_val, pv_val)
cpi_val <- cpi(ev_val, ac_val)

cat("Schedule Variance (SV):          $", format(sv_val, big.mark = ","), "\n")
cat("Cost Variance (CV):              $", format(cv_val, big.mark = ","), "\n")
cat("Schedule Performance Index (SPI):", round(spi_val, 3), "\n")
cat("Cost Performance Index (CPI):    ", round(cpi_val, 3), "\n")

## ----results='asis'-----------------------------------------------------------
eac_typical <- eac(bac, method = "typical", cpi = cpi_val)
eac_atypical <- eac(bac, method = "atypical", ac = ac_val, ev = ev_val)
eac_combined <- eac(bac,
  method = "combined", cpi = cpi_val, ac = ac_val,
  ev = ev_val, spi = spi_val
)

cat("EAC (typical):  $", format(round(eac_typical), big.mark = ","), "\n")
cat("EAC (atypical): $", format(round(eac_atypical), big.mark = ","), "\n")
cat("EAC (combined): $", format(round(eac_combined), big.mark = ","), "\n")

## -----------------------------------------------------------------------------
eac_table <- data.frame(
  Method = c("Typical", "Atypical", "Combined"),
  EAC = c(round(eac_typical), round(eac_atypical), round(eac_combined)),
  Overrun = c(
    round(eac_typical - bac),
    round(eac_atypical - bac),
    round(eac_combined - bac)
  ),
  Assumption = c(
    "Current CPI continues",
    "Future work at planned rate",
    "CPI and SPI both factor in"
  )
)
knitr::kable(eac_table,
  format.args = list(big.mark = ","),
  caption = "EAC Comparison by Method"
)

## ----results='asis'-----------------------------------------------------------
etc_val <- etc(bac, ev_val, cpi_val)
vac_val <- vac(bac, eac_typical)

# TCPI to meet original BAC
tcpi_bac <- tcpi(bac, ev_val, ac_val, target = "bac")

# TCPI to meet revised EAC (typical)
tcpi_eac <- tcpi(bac, ev_val, ac_val, target = "eac", eac = eac_typical)

cat("Estimate to Complete (ETC):         $", format(round(etc_val), big.mark = ","), "\n")
cat("Variance at Completion (VAC):       $", format(round(vac_val), big.mark = ","), "\n")
cat("TCPI (to meet BAC):                ", round(tcpi_bac, 3), "\n")
cat("TCPI (to meet EAC):                ", round(tcpi_eac, 3), "\n")

## ----warning=FALSE------------------------------------------------------------
time_periods <- c(1, 2, 3)
actual_pct <- c(0.08, 0.22, 0.40)
p_costs <- c(45000, 110000, 135000)

pv_vals <- sapply(time_periods, function(t) pv(bac, schedule, t))
ac_vals <- cumsum(p_costs)
ev_vals <- sapply(actual_pct, function(a) ev(bac, a))

trend_data <- data.frame(
  Period = time_periods,
  PV     = pv_vals,
  AC     = ac_vals,
  EV     = ev_vals
)

p <- ggplot2::ggplot(trend_data, ggplot2::aes(x = Period)) +
  ggplot2::geom_line(ggplot2::aes(y = PV, color = "Planned Value (PV)"), linewidth = 1.2) +
  ggplot2::geom_line(ggplot2::aes(y = AC, color = "Actual Cost (AC)"), linewidth = 1.2) +
  ggplot2::geom_line(ggplot2::aes(y = EV, color = "Earned Value (EV)"), linewidth = 1.2) +
  ggplot2::geom_point(ggplot2::aes(y = PV, color = "Planned Value (PV)"), size = 3) +
  ggplot2::geom_point(ggplot2::aes(y = AC, color = "Actual Cost (AC)"), size = 3) +
  ggplot2::geom_point(ggplot2::aes(y = EV, color = "Earned Value (EV)"), size = 3) +
  ggplot2::geom_hline(yintercept = bac, linetype = "dashed", color = "black", linewidth = 0.8) +
  ggplot2::geom_hline(yintercept = eac_typical, linetype = "dotted", color = "darkred", linewidth = 0.8) +
  ggplot2::annotate("text", x = 2.8, y = bac + 12000, label = "BAC = $500K", size = 3.5) +
  ggplot2::annotate("text", x = 2.8, y = eac_typical + 12000, label = "EAC = $725K", size = 3.5) +
  ggplot2::scale_color_manual(values = c(
    "Planned Value (PV)" = "steelblue",
    "Actual Cost (AC)"   = "tomato",
    "Earned Value (EV)"  = "forestgreen"
  )) +
  ggplot2::scale_y_continuous(labels = scales::label_dollar(scale = 1e-3, suffix = "K")) +
  ggplot2::labs(
    title  = "Earned Value Management - Performance Trend",
    x      = "Time Period",
    y      = "Value",
    color  = NULL
  ) +
  ggplot2::theme_minimal() +
  ggplot2::theme(legend.position = "bottom")

print(p)

library(hexSticker)
library(ggplot2)

hex <- sticker(
  "inst/gauge.svg",
  s_x       = 1,
  s_y       = 0.72,
  s_width   = 0.55,
  package   = "PRA",
  p_size    = 24,
  p_color   = "#fcbba1",
  p_y       = 1.42,
  h_fill    = "#67000d",
  h_color   = "#fb6a4a",
  h_size    = 2.0,
  url       = "paulgovan.github.io/PRA",
  u_size    = 5,
  u_color   = "#fc9272",
  filename  = "inst/PRA_hex_sticker.png"
)
plot(hex)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
set.seed(42)

## ----setup--------------------------------------------------------------------
library(PRA)

## -----------------------------------------------------------------------------
task_means <- c(10, 15, 20) # Expected duration for each task (weeks)
task_vars <- c(4, 9, 16) # Variance of each task duration
cor_mat <- matrix(c(
  1.0, 0.5, 0.3,
  0.5, 1.0, 0.4,
  0.3, 0.4, 1.0
), nrow = 3, byrow = TRUE)

## ----results='asis'-----------------------------------------------------------
result <- smm(task_means, task_vars, cor_mat)
cat("Total Mean Duration:    ", round(result$total_mean, 2), "weeks\n")
cat("Total Variance:         ", round(result$total_var, 2), "\n")
cat("Total Std Deviation:    ", round(result$total_std, 2), "weeks\n")

## ----results='asis'-----------------------------------------------------------
total_mean <- result$total_mean
total_sd <- result$total_std
ci_lower <- total_mean - 1.96 * total_sd
ci_upper <- total_mean + 1.96 * total_sd
cat("95% CI: [", round(ci_lower, 1), ",", round(ci_upper, 1), "] weeks\n")

## -----------------------------------------------------------------------------
x_range <- seq(total_mean - 4 * total_sd, total_mean + 4 * total_sd, length.out = 300)
y_range <- dnorm(x_range, mean = total_mean, sd = total_sd)

plot(x_range, y_range,
  type = "l", lwd = 2, col = "steelblue",
  main = "SMM - Implied Project Duration Distribution",
  xlab = "Total Duration (weeks)", ylab = "Density"
)

# Shade 95% CI region
x_ci <- x_range[x_range >= ci_lower & x_range <= ci_upper]
y_ci <- dnorm(x_ci, mean = total_mean, sd = total_sd)
polygon(c(ci_lower, x_ci, ci_upper), c(0, y_ci, 0),
  col = "lightblue", border = NA
)

abline(v = total_mean, col = "black", lty = 2, lwd = 1.5)
legend("topright",
  legend = c("Normal density", "95% CI", "Mean"),
  col = c("steelblue", "lightblue", "black"),
  lty = c(1, NA, 2), lwd = c(2, NA, 1.5),
  pch = c(NA, 15, NA), pt.cex = 1.5,
  bty = "n"
)

## -----------------------------------------------------------------------------
# Represent each task as a normal distribution for MCS comparison (independent case)
task_dists_for_mcs <- list(
  list(type = "normal", mean = task_means[1], sd = sqrt(task_vars[1])),
  list(type = "normal", mean = task_means[2], sd = sqrt(task_vars[2])),
  list(type = "normal", mean = task_means[3], sd = sqrt(task_vars[3]))
)

# Run MCS without correlation (identity = fully independent)
mcs_result <- mcs(10000, task_dists_for_mcs)

## -----------------------------------------------------------------------------
# SMM variance without correlation = sum of individual variances
smm_var_nocor <- sum(task_vars)

comparison <- data.frame(
  Method          = c("SMM (independent)", "Monte Carlo (10,000 runs)"),
  Total_Mean      = round(c(result$total_mean, mcs_result$total_mean), 2),
  Total_Variance  = round(c(smm_var_nocor, mcs_result$total_variance), 2),
  Total_StdDev    = round(c(sqrt(smm_var_nocor), mcs_result$total_sd), 2)
)
knitr::kable(comparison, caption = "SMM vs. Monte Carlo Comparison (independent tasks)")

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
set.seed(42)

## ----setup--------------------------------------------------------------------
library(PRA)

## -----------------------------------------------------------------------------
cause_probs <- c(0.3, 0.2)
risks_given_causes <- c(0.8, 0.6)
risks_given_not_causes <- c(0.2, 0.4)

## ----results='asis'-----------------------------------------------------------
prior_risk <- risk_prob(cause_probs, risks_given_causes, risks_given_not_causes)
cat("Prior probability of risk event R:", round(prior_risk, 3), "\n")

## -----------------------------------------------------------------------------
risk_probs <- c(0.3, 0.5, 0.2)
means_given_risks <- c(10000, 15000, 5000)
sds_given_risks <- c(2000, 1000, 1000)
base_cost <- 2000

## -----------------------------------------------------------------------------
prior_samples <- cost_pdf(
  num_sims          = 5000,
  risk_probs        = risk_probs,
  means_given_risks = means_given_risks,
  sds_given_risks   = sds_given_risks,
  base_cost         = base_cost
)

## -----------------------------------------------------------------------------
# C1 observed as present; C2 not yet assessed
observed_causes <- c(1, NA)

## ----results='asis'-----------------------------------------------------------
posterior_risk <- risk_post_prob(
  cause_probs, risks_given_causes,
  risks_given_not_causes, observed_causes
)
cat("Posterior probability of risk event R:", round(posterior_risk, 3), "\n")

## -----------------------------------------------------------------------------
prob_data <- data.frame(
  Stage       = c("Prior", "Posterior"),
  Probability = c(prior_risk, posterior_risk)
)

p <- ggplot2::ggplot(prob_data, ggplot2::aes(x = Stage, y = Probability, fill = Stage)) +
  ggplot2::geom_col(width = 0.5, show.legend = FALSE) +
  ggplot2::geom_text(ggplot2::aes(label = round(Probability, 3)),
    vjust = -0.4, size = 4.5
  ) +
  ggplot2::scale_fill_manual(values = c("Prior" = "steelblue", "Posterior" = "tomato")) +
  ggplot2::scale_y_continuous(limits = c(0, 1), labels = scales::percent) +
  ggplot2::labs(
    title = "Bayesian Update: Risk Probability",
    x     = NULL,
    y     = "P(Risk Event R)"
  ) +
  ggplot2::theme_minimal(base_size = 13)

print(p)

## -----------------------------------------------------------------------------
observed_risks <- c(1, NA, 1) # Risk 1 and 3 confirmed; Risk 2 not yet assessed

## -----------------------------------------------------------------------------
posterior_samples <- cost_post_pdf(
  num_sims          = 5000,
  observed_risks    = observed_risks,
  means_given_risks = means_given_risks,
  sds_given_risks   = sds_given_risks,
  base_cost         = base_cost
)

## -----------------------------------------------------------------------------
xlim_range <- range(c(prior_samples, posterior_samples))

# Prior cost histogram
hist(prior_samples,
  breaks = 40, freq = FALSE,
  col = rgb(0.27, 0.51, 0.71, 0.5), # steelblue, semi-transparent
  border = "white",
  xlim = xlim_range,
  main = "Prior vs. Posterior Cost Distribution",
  xlab = "Total Cost ($)",
  ylab = "Density"
)

# Posterior cost histogram (overlaid)
hist(posterior_samples,
  breaks = 40, freq = FALSE,
  col = rgb(0.84, 0.24, 0.31, 0.5), # tomato, semi-transparent
  border = "white",
  add = TRUE
)

abline(v = mean(prior_samples), col = "steelblue", lty = 2, lwd = 2)
abline(v = mean(posterior_samples), col = "tomato", lty = 2, lwd = 2)

legend("topright",
  legend = c(
    paste0("Prior  (mean = $", format(round(mean(prior_samples)), big.mark = ",")),
    paste0("Posterior (mean = $", format(round(mean(posterior_samples)), big.mark = ","))
  ),
  fill = c(rgb(0.27, 0.51, 0.71, 0.5), rgb(0.84, 0.24, 0.31, 0.5)),
  bty = "n"
)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(PRA)

## -----------------------------------------------------------------------------
data <- data.frame(
  time       = 1:9,
  completion = c(5, 15, 40, 60, 70, 75, 80, 85, 90)
)

## -----------------------------------------------------------------------------
fit <- fit_sigmoidal(data, "time", "completion", "logistic")

## -----------------------------------------------------------------------------
summary(fit)

## -----------------------------------------------------------------------------
plot_sigmoidal(
  fit, data, "time", "completion", "logistic",
  conf_level = 0.95,
  main = "Logistic Learning Curve - Completion Forecast",
  xlab = "Week",
  ylab = "Completion (%)"
)

## -----------------------------------------------------------------------------
future_times <- seq(1, 12, length.out = 100)
predictions <- predict_sigmoidal(fit, future_times, "logistic", conf_level = 0.95)

knitr::kable(
  tail(round(predictions, 1), 5),
  caption = "Predicted completion (final 5 points)",
  row.names = FALSE
)

## ----warning=FALSE------------------------------------------------------------
fit_logistic <- fit_sigmoidal(data, "time", "completion", "logistic")
fit_pearl <- fit_sigmoidal(data, "time", "completion", "pearl")
fit_gompertz <- fit_sigmoidal(data, "time", "completion", "gompertz")

## -----------------------------------------------------------------------------
# Residual standard errors for comparison
rse <- function(fit) summary(fit)$sigma

comparison <- data.frame(
  Model = c("Logistic", "Pearl", "Gompertz"),
  Residual_StdError = round(c(rse(fit_logistic), rse(fit_pearl), rse(fit_gompertz)), 3)
)
knitr::kable(comparison, caption = "Model Fit Comparison (lower RSE = better fit)")

## -----------------------------------------------------------------------------
x_seq <- seq(1, 12, length.out = 200)

pred_log <- predict_sigmoidal(fit_logistic, x_seq, "logistic")
pred_prl <- predict_sigmoidal(fit_pearl, x_seq, "pearl")
pred_gom <- predict_sigmoidal(fit_gompertz, x_seq, "gompertz")

# Base plot with observed data
plot(data$time, data$completion,
  pch = 16, xlim = c(1, 12), ylim = c(0, 105),
  main = "Learning Curve: Model Comparison",
  xlab = "Week", ylab = "Completion (%)"
)
lines(pred_log$x, pred_log$pred, col = "steelblue", lwd = 2)
lines(pred_prl$x, pred_prl$pred, col = "tomato", lwd = 2, lty = 2)
lines(pred_gom$x, pred_gom$pred, col = "darkgreen", lwd = 2, lty = 3)
legend("bottomright",
  legend = c("Logistic", "Pearl", "Gompertz"),
  col = c("steelblue", "tomato", "darkgreen"),
  lty = c(1, 2, 3), lwd = 2, bty = "n"
)

# PRA Agent Evaluation Framework using vitals
#
# This script defines evaluation tasks for systematically measuring and
# improving the PRA chat agent's tool selection accuracy and response quality.
#
# Usage:
#   source(system.file("eval/pra_eval.R", package = "PRA"))
#   results <- run_pra_eval(model = "llama3.2")
#
#   # Repeated trials with confidence intervals
#   results <- run_pra_eval(model = "llama3.2", n_trials = 3)
#
#   # Compare models
#   results <- run_pra_comparison(models = c("llama3.2", "qwen2.5"))
#
# Requirements: vitals (>= 0.2.0), ellmer, PRA (with Ollama running)

# ============================================================================
# Evaluation Dataset: 25 scenarios across 3 tiers
# ============================================================================

#' Build the PRA evaluation dataset
#'
#' @return A tibble with columns: input, target, tier, expected_tools,
#'   expected_keywords
pra_eval_dataset <- function() {
  tibble::tibble(
    input = c(
      # -- Tier 1: Single-tool (8 scenarios) --
      paste0(
        "Run a Monte Carlo simulation with 10000 iterations for a 3-task project: ",
        "Task A has a normal distribution with mean 10 and sd 2, ",
        "Task B has a triangular distribution with min 5, mode 10, max 15, ",
        "Task C has a uniform distribution between 8 and 12."
      ),
      "What is the Second Moment Method estimate for a project with task means [10, 15, 20, 8] and variances [4, 9, 16, 2]?",
      paste0(
        "Calculate EVM metrics for my project: BAC = $500,000, ",
        "cumulative planned schedule is [0.2, 0.4, 0.6, 0.8, 1.0], ",
        "we are at period 3, actual percent complete is 55%, ",
        "cumulative actual costs are [$95000, $200000, $310000]."
      ),
      paste0(
        "I have a risk event with 2 root causes. ",
        "Prior probabilities: P(C1) = 0.3, P(C2) = 0.2. ",
        "Conditional: P(R|C1) = 0.8, P(R|C2) = 0.6. ",
        "Background: P(R|not C1) = 0.2, P(R|not C2) = 0.4. ",
        "What is the overall risk probability?"
      ),
      paste0(
        "Calculate EVM metrics: BAC = $100,000, schedule [0.25, 0.5, 0.75, 1.0], ",
        "period 2, actual percent complete is 40%, ",
        "cumulative actual costs [$22000, $48000]. ",
        "What is the schedule variance?"
      ),
      paste0(
        "I have a dependency matrix for my project: ",
        "[[1,1,0,0],[0,1,1,0],[0,0,1,1],[0,0,0,1]]. ",
        "Show me the parent DSM."
      ),
      paste0(
        "Fit a sigmoidal learning curve (logistic model) to this data: ",
        "x = [1,2,3,4,5,6,7,8], y = [2,8,25,50,72,88,95,98]. ",
        "Predict the value at x = 10."
      ),
      paste0(
        "Run a Monte Carlo simulation for 2 tasks only: ",
        "Task 1 is uniform(5, 15), Task 2 is normal(20, 3). ",
        "Use 5000 simulations."
      ),

      # -- Tier 2: Multi-tool chain (9 scenarios) --
      paste0(
        "Simulate a 3-task project (A normal(10,2), B triangular(5,10,15), C uniform(8,12)) ",
        "with 10000 iterations and then calculate the contingency reserve at 95% confidence."
      ),
      paste0(
        "My project has BAC=$100,000, schedule [0.1,0.3,0.6,0.8,1.0], ",
        "period 3, 50% complete, costs [12000,28000,65000]. ",
        "Run a full EVM analysis and tell me what the forecast completion cost will be."
      ),
      paste0(
        "I have 2 root causes with P(C1)=0.3, P(C2)=0.2, ",
        "P(R|C1)=0.8, P(R|C2)=0.6, P(R|~C1)=0.2, P(R|~C2)=0.4. ",
        "After investigation, C1 was observed to have occurred but C2 is unknown. ",
        "What is the updated risk probability?"
      ),
      paste0(
        "Run a sensitivity analysis on a 3-task project: ",
        "Task A normal(10,2), Task B triangular(5,10,15), Task C uniform(8,12). ",
        "Which task drives the most variance?"
      ),
      paste0(
        "Fit a pearl learning curve to this data: ",
        "x = [1,2,3,4,5,6,7,8,9,10], y = [5,15,40,60,70,75,80,85,90,95]. ",
        "Then predict the value at x = 12."
      ),
      paste0(
        "Run a Monte Carlo simulation for tasks: A normal(15,3), B triangular(8,12,20), ",
        "C uniform(5,10). Then run a sensitivity analysis to find the biggest risk driver."
      ),
      paste0(
        "Simulate a 4-task project: A normal(10,2), B triangular(5,10,15), ",
        "C uniform(8,12), D normal(20,5). Then calculate the contingency at P80 ",
        "and P95 confidence levels."
      ),
      paste0(
        "I have a 3x3 dependency matrix [[1,1,0],[1,1,1],[0,1,1]]. ",
        "Show me the parent DSM, then compute the grandparent DSM."
      ),
      paste0(
        "Calculate the prior risk probability for 3 causes: P(C1)=0.4, P(C2)=0.1, P(C3)=0.3, ",
        "P(R|C1)=0.9, P(R|C2)=0.5, P(R|C3)=0.7, ",
        "P(R|~C1)=0.1, P(R|~C2)=0.5, P(R|~C3)=0.3. ",
        "Then update: C1 occurred, C2 did not occur, C3 is unknown."
      ),

      # -- Tier 3: Open-ended / conceptual (8 scenarios) --
      paste0(
        "My construction project has 5 work packages. ",
        "I think there's significant schedule risk. ",
        "How should I assess the schedule uncertainty?"
      ),
      paste0(
        "My project's CPI is 0.85 and SPI is 0.92. ",
        "We're at period 4 of 8. Should I be concerned? What should I do next?"
      ),
      "How do Bayesian risk methods differ from Monte Carlo simulation for project risk?",
      "What is the difference between the Second Moment Method and Monte Carlo simulation?",
      "What is contingency reserve and how is it calculated from simulation results?",
      "Explain what earned value management is and define CPI and SPI.",
      "What sigmoidal learning curve models does PRA support and when should I use each?",
      "How does sensitivity analysis help identify schedule risk drivers?"
    ),

    target = c(
      # Tier 1 targets
      "Calls mcs_tool with correct distributions and returns percentile results",
      "Calls smm_tool with means [10,15,20,8] and variances [4,9,16,2]",
      "Calls evm_analysis_tool with bac=500000, correct schedule, period=3, 0.55, costs",
      "Calls risk_prob_tool with correct cause probs and conditional probabilities",
      "Calls evm_analysis_tool and reports SV=-10000 (EV=40000, PV=50000)",
      "Calls parent_dsm_tool with the 4x4 matrix",
      "Calls fit_and_predict_sigmoidal_tool with logistic model, x data, y data, predict_x=10",
      "Calls mcs_tool with 2 tasks and 5000 simulations",

      # Tier 2 targets
      "Calls mcs_tool then contingency_tool with phigh=0.95",
      "Calls evm_analysis_tool and interprets EAC forecasts",
      "Calls risk_post_prob_tool with observed=[1, null]",
      "Calls sensitivity_tool with correct distributions",
      "Calls fit_and_predict_sigmoidal_tool with pearl model and predict_x=[12]",
      "Calls mcs_tool then sensitivity_tool",
      "Calls mcs_tool then contingency_tool (possibly twice for P80 and P95)",
      "Calls parent_dsm_tool then grandparent_dsm_tool",
      "Calls risk_prob_tool then risk_post_prob_tool with observed=[1,0,null]",

      # Tier 3 targets
      "Recommends MCS approach and asks for distribution estimates per work package",
      "Interprets CPI<1 and SPI<1 as behind schedule and over budget, suggests corrective action",
      "Explains Bayesian is for root-cause analysis, MCS for full distribution simulation",
      "Explains SMM is analytical approximation, MCS is full simulation with distributions",
      "Explains contingency as difference between percentile and base estimate from MCS",
      "Defines EVM, CPI = EV/AC measures cost efficiency, SPI = EV/PV measures schedule efficiency",
      "Lists logistic, gompertz, weibull, pearl models; explains S-curve shape and use cases",
      "Explains sensitivity analysis identifies which tasks contribute most to total variance"
    ),

    tier = c(
      rep("single_tool", 8),
      rep("multi_tool", 9),
      rep("open_ended", 8)
    ),

    expected_tools = list(
      # Tier 1
      "mcs_tool",
      "smm_tool",
      "evm_analysis_tool",
      "risk_prob_tool",
      "evm_analysis_tool",
      "parent_dsm_tool",
      "fit_and_predict_sigmoidal_tool",
      "mcs_tool",

      # Tier 2
      c("mcs_tool", "contingency_tool"),
      "evm_analysis_tool",
      "risk_post_prob_tool",
      "sensitivity_tool",
      "fit_and_predict_sigmoidal_tool",
      c("mcs_tool", "sensitivity_tool"),
      c("mcs_tool", "contingency_tool"),
      c("parent_dsm_tool", "grandparent_dsm_tool"),
      c("risk_prob_tool", "risk_post_prob_tool"),

      # Tier 3 — no specific tools expected; scored on content quality
      character(0),
      character(0),
      character(0),
      character(0),
      character(0),
      character(0),
      character(0),
      character(0)
    ),

    # Keywords that open-ended responses should contain for quality scoring
    expected_keywords = list(
      # Tier 1 — not used (tool accuracy matters)
      character(0), character(0), character(0), character(0),
      character(0), character(0), character(0), character(0),

      # Tier 2 — not used
      character(0), character(0), character(0), character(0),
      character(0), character(0), character(0), character(0), character(0),

      # Tier 3 — keywords for quality scoring
      c("monte carlo", "simulation", "distribution"),
      c("cpi", "spi", "behind", "over budget", "corrective"),
      c("bayesian", "root cause", "monte carlo", "simulation"),
      c("second moment", "analytical", "monte carlo", "simulation"),
      c("contingency", "percentile", "p50", "p80", "p95", "reserve"),
      c("earned value", "cpi", "spi", "cost", "schedule"),
      c("sigmoidal", "learning curve", "logistic", "gompertz"),
      c("sensitivity", "variance", "risk driver", "tornado")
    )
  )
}

# ============================================================================
# Package environment for tracking tool calls across solver invocations
# ============================================================================

.pra_eval_env <- new.env(parent = emptyenv())
.pra_eval_env$tool_calls <- list()

# ============================================================================
# Custom Solver
# ============================================================================

#' Create a PRA solver for vitals evaluation
#'
#' @param model Ollama model name
#' @param rag Whether to enable RAG
#' @return A solver function compatible with vitals (>= 0.2.0)
pra_solver <- function(model = "llama3.2", rag = TRUE) {
  force(model)
  force(rag)

  function(input) {
    n <- length(input)
    results <- character(n)
    solver_chats <- vector("list", n)

    # Reset tool tracking
    .pra_eval_env$tool_calls <- vector("list", n)

    for (i in seq_len(n)) {
      chat <- PRA::pra_chat(model = model, rag = rag)

      # Track which tools are called via chat turns
      tools_called <- character(0)

      response <- tryCatch({
        resp <- chat$chat(input[i])
        # Extract tool calls from chat turns
        turns <- tryCatch(chat$get_turns(), error = function(e) list())
        for (turn in turns) {
          tryCatch({
            if (!is.null(turn@role) && turn@role == "assistant") {
              for (content in turn@contents) {
                if (any(grepl("ToolRequest", class(content)))) {
                  tools_called <- c(tools_called, content@name)
                }
              }
            }
          }, error = function(e) NULL)
        }
        resp
      },
      error = function(e) paste0("ERROR: ", e$message)
      )

      results[i] <- response
      solver_chats[[i]] <- chat
      .pra_eval_env$tool_calls[[i]] <- tools_called
    }

    list(result = results, solver_chat = solver_chats)
  }
}

# ============================================================================
# Custom Scorer: Tool Selection + Content Quality (vitals 0.2.0 interface)
# ============================================================================

#' Score tool selection accuracy and open-ended response quality
#'
#' For tool-calling scenarios: checks if expected tools were called.
#' For open-ended scenarios: checks for domain-relevant keywords in the response.
#'
#' @param dataset The evaluation dataset (must have expected_tools and
#'   expected_keywords columns)
#' @return A scorer function
tool_selection_scorer <- function(dataset) {
  expected <- dataset$expected_tools
  keywords <- dataset$expected_keywords
  force(expected)
  force(keywords)

  function(samples, ...) {
    n <- nrow(samples)
    scores <- character(n)
    explanations <- character(n)

    for (i in seq_len(n)) {
      exp_tools <- expected[[i]]
      actual_tools <- .pra_eval_env$tool_calls[[i]]
      if (is.null(actual_tools)) actual_tools <- character(0)

      result_text <- samples$result[i]

      if (length(exp_tools) == 0) {
        # Open-ended: score on keyword coverage
        kws <- keywords[[i]]
        if (length(kws) == 0 || grepl("^ERROR:", result_text)) {
          if (nchar(result_text) > 50 && !grepl("^ERROR:", result_text)) {
            scores[i] <- "C"
            explanations[i] <- "Substantive response (no keywords defined)"
          } else {
            scores[i] <- "I"
            explanations[i] <- "Empty or error response"
          }
        } else {
          result_lower <- tolower(result_text)
          matched <- vapply(kws, function(kw) grepl(kw, result_lower, fixed = TRUE), logical(1))
          hit_rate <- mean(matched)

          if (hit_rate >= 0.5) {
            scores[i] <- "C"
            explanations[i] <- paste0("Keyword coverage: ", sum(matched), "/",
                                       length(kws), " (",
                                       paste(kws[matched], collapse = ", "), ")")
          } else if (hit_rate > 0) {
            scores[i] <- "P"
            explanations[i] <- paste0("Partial keyword coverage: ", sum(matched), "/",
                                       length(kws), " (",
                                       paste(kws[matched], collapse = ", "),
                                       "; missing: ", paste(kws[!matched], collapse = ", "), ")")
          } else {
            scores[i] <- "I"
            explanations[i] <- paste0("No keywords matched from: ",
                                       paste(kws, collapse = ", "))
          }
        }
      } else if (all(exp_tools %in% actual_tools)) {
        scores[i] <- "C"
        explanations[i] <- paste("All expected tools called:",
                                  paste(actual_tools, collapse = ", "))
      } else if (any(exp_tools %in% actual_tools)) {
        scores[i] <- "P"
        missing <- setdiff(exp_tools, actual_tools)
        explanations[i] <- paste("Partial: called",
                                  paste(intersect(exp_tools, actual_tools), collapse = ", "),
                                  "but missing", paste(missing, collapse = ", "))
      } else {
        scores[i] <- "I"
        explanations[i] <- paste("Expected:",
                                  paste(exp_tools, collapse = ", "),
                                  "| Got:",
                                  if (length(actual_tools) > 0) paste(actual_tools, collapse = ", ") else "(none)")
      }
    }

    tibble::tibble(
      score = factor(scores, levels = c("I", "P", "C"),
                     labels = c("Incorrect", "Partial", "Correct"),
                     ordered = TRUE),
      explanation = explanations
    )
  }
}

# ============================================================================
# Runner with repeated trials support
# ============================================================================

#' Run the PRA evaluation
#'
#' @param model Ollama model name
#' @param rag Whether to enable RAG
#' @param scorer_type "tool" for tool selection, "model" for model-graded QA
#' @param n_trials Number of repeated evaluation runs (default 1). Multiple
#'   trials enable confidence interval estimation.
#' @return A list with: task (vitals Task from last trial), summary (per-tier
#'   accuracy), trial_scores (accuracy per trial for CI calculation)
run_pra_eval <- function(model = "llama3.2", rag = TRUE,
                         scorer_type = "tool", n_trials = 1L) {
  if (!requireNamespace("vitals", quietly = TRUE)) {
    stop("Package 'vitals' is required. Install with: install.packages('vitals')")
  }

  dataset <- pra_eval_dataset()
  solver <- pra_solver(model = model, rag = rag)

  scorer <- if (scorer_type == "model") {
    vitals::model_graded_qa()
  } else {
    tool_selection_scorer(dataset)
  }

  trial_scores <- numeric(n_trials)
  tier_scores <- list()
  last_task <- NULL

  for (trial in seq_len(n_trials)) {
    if (n_trials > 1) message("=== Trial ", trial, " of ", n_trials, " ===")

    task <- vitals::Task$new(
      dataset = dataset,
      solver = solver,
      scorer = scorer,
      name = paste0("pra_eval_", model, if (rag) "_rag" else "_norag",
                     if (n_trials > 1) paste0("_t", trial) else "")
    )

    task$eval()
    last_task <- task

    # Extract scores from the task
    samples <- tryCatch(task$get_samples(), error = function(e) NULL)
    if (!is.null(samples)) {
      correct <- sum(samples$score == "Correct", na.rm = TRUE)
      total <- nrow(samples)
      trial_scores[trial] <- correct / total

      # Per-tier breakdown
      for (t in unique(dataset$tier)) {
        idx <- which(dataset$tier == t)
        tier_correct <- sum(samples$score[idx] == "Correct", na.rm = TRUE)
        tier_total <- length(idx)
        tier_key <- paste0(t, "_trial_", trial)
        tier_scores[[tier_key]] <- tier_correct / tier_total
      }
    }
  }

  # Build summary
  summary <- summarize_eval(trial_scores, tier_scores, dataset, n_trials, model, rag)

  list(
    task = last_task,
    summary = summary,
    trial_scores = trial_scores
  )
}

#' Summarize evaluation results with per-tier accuracy and optional CIs
#'
#' @param trial_scores Numeric vector of per-trial overall accuracy
#' @param tier_scores Named list of per-tier per-trial accuracies
#' @param dataset The evaluation dataset
#' @param n_trials Number of trials
#' @param model Model name
#' @param rag RAG setting
#' @return A list with formatted summary
summarize_eval <- function(trial_scores, tier_scores, dataset, n_trials, model, rag) {
  tiers <- unique(dataset$tier)

  cat("\n")
  cat("========================================\n")
  cat("  PRA Evaluation Summary\n")
  cat("========================================\n")
  cat("Model:  ", model, "\n")
  cat("RAG:    ", if (rag) "enabled" else "disabled", "\n")
  cat("Trials: ", n_trials, "\n")
  cat("Scenarios: ", nrow(dataset), "\n")
  cat("----------------------------------------\n")

  overall_mean <- mean(trial_scores)
  cat(sprintf("Overall accuracy: %.1f%%", overall_mean * 100))

  if (n_trials >= 3) {
    ci <- compute_ci(trial_scores)
    cat(sprintf(" (95%% CI: %.1f%% - %.1f%%)", ci[1] * 100, ci[2] * 100))
  }
  cat("\n\n")

  cat("Per-tier breakdown:\n")
  tier_summary <- list()
  for (t in tiers) {
    tier_vals <- unlist(tier_scores[grepl(paste0("^", t, "_trial_"), names(tier_scores))])
    tier_mean <- mean(tier_vals)
    tier_n <- sum(dataset$tier == t)
    cat(sprintf("  %-15s %5.1f%% (%d scenarios)", t, tier_mean * 100, tier_n))

    if (n_trials >= 3 && length(tier_vals) >= 3) {
      ci <- compute_ci(tier_vals)
      cat(sprintf("  [95%% CI: %.1f%% - %.1f%%]", ci[1] * 100, ci[2] * 100))
    }
    cat("\n")

    tier_summary[[t]] <- list(mean = tier_mean, n = tier_n, values = tier_vals)
  }
  cat("========================================\n\n")

  list(
    model = model,
    rag = rag,
    n_trials = n_trials,
    overall_mean = overall_mean,
    overall_scores = trial_scores,
    per_tier = tier_summary
  )
}

#' Compute 95% confidence interval using t-distribution
#' @param x Numeric vector of scores
#' @return Length-2 vector (lower, upper)
compute_ci <- function(x) {
  n <- length(x)
  if (n < 2) return(c(NA, NA))
  m <- mean(x)
  se <- stats::sd(x) / sqrt(n)
  t_crit <- stats::qt(0.975, df = n - 1)
  c(max(0, m - t_crit * se), min(1, m + t_crit * se))
}

# ============================================================================
# Comparative evaluation
# ============================================================================

#' Run comparative evaluation across models and RAG settings
#'
#' @param models Character vector of model names to compare
#' @param rag_options Logical vector of RAG settings to test (default both)
#' @param n_trials Number of trials per configuration (default 1)
#' @return List of evaluation result objects
run_pra_comparison <- function(models = c("llama3.2", "qwen2.5"),
                               rag_options = c(TRUE, FALSE),
                               n_trials = 1L) {
  if (!requireNamespace("vitals", quietly = TRUE)) {
    stop("Package 'vitals' is required. Install with: install.packages('vitals')")
  }

  results <- list()
  for (m in models) {
    for (r in rag_options) {
      label <- paste0(m, if (r) "_rag" else "_norag")
      message("\n>>> Evaluating: ", label, " <<<\n")
      tryCatch({
        results[[label]] <- run_pra_eval(model = m, rag = r, n_trials = n_trials)
      }, error = function(e) {
        message("  Failed: ", e$message)
      })
    }
  }

  # Print comparison table
  if (length(results) > 1) {
    cat("\n========================================\n")
    cat("  Comparison Summary\n")
    cat("========================================\n")
    cat(sprintf("%-25s %10s %12s %12s %12s\n",
                "Configuration", "Overall", "Single-tool", "Multi-tool", "Open-ended"))
    cat(strrep("-", 75), "\n")
    for (label in names(results)) {
      r <- results[[label]]$summary
      cat(sprintf("%-25s %9.1f%%",
                  label, r$overall_mean * 100))
      for (t in c("single_tool", "multi_tool", "open_ended")) {
        if (!is.null(r$per_tier[[t]])) {
          cat(sprintf(" %11.1f%%", r$per_tier[[t]]$mean * 100))
        } else {
          cat("          NA")
        }
      }
      cat("\n")
    }
    cat("========================================\n")
  }

  message("\nEvaluation complete. Use vitals::vitals_view() to inspect results.")
  results
}

#' Create PRA Tool Definitions for LLM Agent
#'
#' Creates a list of ellmer tool objects that wrap PRA's exported functions for
#' use with an LLM agent. Each tool includes a description that helps the LLM
#' select the appropriate analysis method and properly format parameters.
#'
#' @return A list of ellmer tool objects.
#'
#' @details
#' Tool wrappers handle serialization between the LLM (JSON strings) and R
#' (lists, matrices, data.frames). Complex inputs like task distribution lists
#' and correlation matrices are accepted as JSON strings and deserialized
#' internally.
#'
#' Large output vectors (e.g., Monte Carlo simulation samples) are summarized
#' to mean, sd, and key percentiles rather than returning the full vector to
#' the LLM. The full results are stored in the package environment for use by
#' downstream tools (e.g., contingency analysis needs the full distribution).
#'
#' When used with shinychat, tool results include rich HTML display with inline
#' plots via `ellmer::ContentToolResult`.
#'
#' @examples
#' \dontrun{
#' tools <- pra_tools()
#' chat <- ellmer::chat_ollama(model = "llama3.2")
#' for (tool in tools) chat$register_tool(tool)
#' }
#'
#' @export
pra_tools <- function() {
  if (!requireNamespace("ellmer", quietly = TRUE)) {
    stop("Package 'ellmer' is required for pra_tools(). Install with: install.packages('ellmer')")
  }

  list(
    # ---- Monte Carlo Simulation ----
    ellmer::tool(
      mcs_tool,
      "Run a Monte Carlo simulation to estimate total project cost or duration. Use when the user provides task-level uncertainty distributions (normal, triangular, uniform) and wants the range of possible project outcomes. Returns mean, standard deviation, and key percentiles (P5, P50, P95). Follow up with contingency_tool or sensitivity_tool.",
      arguments = list(
        num_sims = ellmer::type_integer("Number of simulations to run, typically 10000."),
        task_dists_json = ellmer::type_string(
          "JSON array of task distributions. Each element is an object with 'type' and parameters. Example: [{\"type\":\"normal\",\"mean\":10,\"sd\":2},{\"type\":\"triangular\",\"a\":5,\"b\":10,\"c\":15},{\"type\":\"uniform\",\"min\":8,\"max\":12}]"
        ),
        cor_mat_json = ellmer::type_string(
          "Optional JSON 2D array for the correlation matrix. Use 'null' or omit if tasks are independent. Example for 3 tasks: [[1,0.5,0.3],[0.5,1,0.4],[0.3,0.4,1]]"
        )
      ),
      annotations = ellmer::tool_annotations(title = "Monte Carlo Simulation")
    ),

    # ---- Second Moment Method ----
    ellmer::tool(
      smm_tool,
      "Quick analytical estimate of total project mean and variance using the Second Moment Method. Use when you need a fast approximation without running a full simulation, or when only means and variances are known (not full distributions).",
      arguments = list(
        mean_json = ellmer::type_string("JSON array of task mean values. Example: [10, 12, 8]"),
        var_json = ellmer::type_string("JSON array of task variance values. Example: [4, 9, 2]"),
        cor_mat_json = ellmer::type_string(
          "Optional JSON 2D array for the correlation matrix. Use 'null' or omit if tasks are independent."
        )
      ),
      annotations = ellmer::tool_annotations(title = "Second Moment Method")
    ),

    # ---- Contingency ----
    ellmer::tool(
      contingency_tool,
      "Calculate the contingency reserve from Monte Carlo simulation results. MUST be called after mcs_tool. Returns the dollar/time buffer needed between a base estimate (P50) and a high-confidence estimate (P95).",
      arguments = list(
        phigh = ellmer::type_number("High percentile for contingency (default 0.95)."),
        pbase = ellmer::type_number("Base percentile (default 0.50).")
      ),
      annotations = ellmer::tool_annotations(title = "Contingency Analysis")
    ),

    # ---- Sensitivity ----
    ellmer::tool(
      sensitivity_tool,
      "Identify which tasks contribute most to total project uncertainty. Returns sensitivity values for each task. Higher values mean the task is a bigger driver of total variance.",
      arguments = list(
        task_dists_json = ellmer::type_string("JSON array of task distributions (same format as mcs_tool)."),
        cor_mat_json = ellmer::type_string("Optional JSON 2D array for the correlation matrix. Use 'null' or omit if independent.")
      ),
      annotations = ellmer::tool_annotations(title = "Sensitivity Analysis")
    ),

    # ---- EVM: Full Analysis (composite) ----
    ellmer::tool(
      evm_analysis_tool,
      "Run a complete Earned Value Management analysis in one call. Computes PV, EV, AC, SV, CV, SPI, CPI, EAC (all 3 methods), ETC, VAC, and TCPI. Use this instead of individual EVM tools. Returns a full dashboard table.",
      arguments = list(
        bac = ellmer::type_number("Budget at Completion (total planned budget)."),
        schedule_json = ellmer::type_string("JSON array of cumulative planned completion fractions. Example: [0.1, 0.2, 0.4, 0.7, 1.0]"),
        time_period = ellmer::type_integer("Current time period (1-based index into schedule)."),
        actual_per_complete = ellmer::type_number("Actual percent complete as a decimal (e.g., 0.35 for 35%)."),
        actual_costs_json = ellmer::type_string("JSON array of cumulative actual costs per period. Example: [10000, 22000, 37000]"),
        cumulative = ellmer::type_enum("Whether costs are already cumulative ('true') or per-period ('false'). Default 'true'.", values = c("true", "false"))
      ),
      annotations = ellmer::tool_annotations(title = "Earned Value Management")
    ),

    # ---- Bayesian: Prior Risk Probability ----
    ellmer::tool(
      risk_prob_tool,
      "Calculate the prior probability of a risk event from root causes. Use when you have identified root causes with estimated probabilities and conditional risk probabilities.",
      arguments = list(
        cause_probs_json = ellmer::type_string("JSON array of root cause probabilities. Example: [0.3, 0.2]"),
        risks_given_causes_json = ellmer::type_string("JSON array of P(Risk|Cause). Example: [0.8, 0.6]"),
        risks_given_not_causes_json = ellmer::type_string("JSON array of P(Risk|Not Cause). Example: [0.2, 0.4]")
      ),
      annotations = ellmer::tool_annotations(title = "Bayesian Risk (Prior)")
    ),

    # ---- Bayesian: Posterior Risk Probability ----
    ellmer::tool(
      risk_post_prob_tool,
      "Update risk probability after observing whether causes occurred. Use after risk_prob_tool when new information is available about root causes.",
      arguments = list(
        cause_probs_json = ellmer::type_string("JSON array of root cause probabilities."),
        risks_given_causes_json = ellmer::type_string("JSON array of P(Risk|Cause)."),
        risks_given_not_causes_json = ellmer::type_string("JSON array of P(Risk|Not Cause)."),
        observed_causes_json = ellmer::type_string("JSON array of observed cause states: 1 (occurred), 0 (did not occur), null (unknown). Example: [1, null]")
      ),
      annotations = ellmer::tool_annotations(title = "Bayesian Risk (Posterior)")
    ),

    # ---- Bayesian: Prior Cost Distribution ----
    ellmer::tool(
      cost_pdf_tool,
      "Simulate prior cost distribution from risk probabilities. Returns summary statistics (mean, sd, percentiles) of simulated project costs.",
      arguments = list(
        num_sims = ellmer::type_integer("Number of simulations (typically 10000)."),
        risk_probs_json = ellmer::type_string("JSON array of risk probabilities (one per risk)."),
        means_given_risks_json = ellmer::type_string("JSON array of mean additional cost if risk occurs."),
        sds_given_risks_json = ellmer::type_string("JSON array of SD of additional cost if risk occurs."),
        base_cost = ellmer::type_number("Baseline project cost before risk impacts (default 0).")
      ),
      annotations = ellmer::tool_annotations(title = "Prior Cost Distribution")
    ),

    # ---- Bayesian: Posterior Cost Distribution ----
    ellmer::tool(
      cost_post_pdf_tool,
      "Simulate posterior cost distribution after observing risk outcomes. Use after observing which risks actually occurred.",
      arguments = list(
        num_sims = ellmer::type_integer("Number of simulations."),
        observed_risks_json = ellmer::type_string("JSON array of observed risk states: true (occurred), false (did not), null (unknown)."),
        means_given_risks_json = ellmer::type_string("JSON array of mean additional cost if risk occurs."),
        sds_given_risks_json = ellmer::type_string("JSON array of SD of additional cost if risk occurs."),
        base_cost = ellmer::type_number("Baseline project cost (default 0).")
      ),
      annotations = ellmer::tool_annotations(title = "Posterior Cost Distribution")
    ),

    # ---- Learning Curves: Fit and Predict (composite) ----
    ellmer::tool(
      fit_and_predict_sigmoidal_tool,
      "Fit a sigmoidal learning curve model and generate predictions. Use when the user has historical time/completion data and wants to model the learning curve. Models: pearl, gompertz, logistic.",
      arguments = list(
        x_json = ellmer::type_string("JSON array of x values (time or unit number). Example: [1,2,3,4,5,6,7,8,9,10]"),
        y_json = ellmer::type_string("JSON array of y values (cost, completion, etc.). Example: [5,15,40,60,70,75,80,85,90,95]"),
        model_type = ellmer::type_enum("Sigmoidal model type.", values = c("pearl", "gompertz", "logistic")),
        predict_x_json = ellmer::type_string("JSON array of x values to predict at. If omitted, predicts at the original x values."),
        conf_level = ellmer::type_number("Confidence level for prediction intervals (e.g., 0.95). Omit for no intervals.")
      ),
      annotations = ellmer::tool_annotations(title = "Learning Curve")
    ),

    # ---- DSM: Parent ----
    ellmer::tool(
      parent_dsm_tool,
      "Compute the resource-task dependency structure matrix (DSM). Shows which tasks share resources and are therefore coupled. Input is a resources x tasks matrix.",
      arguments = list(
        matrix_json = ellmer::type_string("JSON 2D array for a resources x tasks matrix. Example: [[1,0,1,0],[0,1,0,1],[1,0,1,1]]")
      ),
      annotations = ellmer::tool_annotations(title = "Parent DSM")
    ),

    # ---- DSM: Grandparent ----
    ellmer::tool(
      grandparent_dsm_tool,
      "Compute the risk-based task dependency structure matrix. Shows how tasks are coupled through shared risk-resource pathways. Requires both a resource-task matrix (S) and a risk-resource matrix (R).",
      arguments = list(
        s_matrix_json = ellmer::type_string("JSON 2D array for the resource-task matrix S (resources x tasks)."),
        r_matrix_json = ellmer::type_string("JSON 2D array for the risk-resource matrix R (risks x resources).")
      ),
      annotations = ellmer::tool_annotations(title = "Grandparent DSM")
    )
  )
}

# Suppress R CMD check note for 'x' used in graphics::curve() expressions
utils::globalVariables("x")

# ============================================================================
# /command registry — direct tool invocation without LLM
# ============================================================================

#' Get the slash-command registry
#'
#' Returns a named list of command definitions. Each entry has:
#' \describe{
#'   \item{title}{Display name}
#'   \item{description}{One-line summary}
#'   \item{args}{Named list of argument specs (name, type, required, description, example)}
#'   \item{fn}{Function to call with parsed arguments}
#' }
#' @return Named list of command definitions.
#' @keywords internal
pra_command_registry <- function() {
  list(
    mcs = list(
      title = "Monte Carlo Simulation",
      description = "Run a Monte Carlo simulation for task-level uncertainty.",
      args = list(
        list(
          name = "tasks", type = "json", required = TRUE,
          description = "Task distributions as JSON array",
          example = '[{"type":"normal","mean":10,"sd":2},{"type":"triangular","a":5,"b":10,"c":15},{"type":"uniform","min":8,"max":12}]'
        ),
        list(
          name = "n", type = "integer", required = FALSE,
          description = "Number of simulations (default: 10000)",
          example = "10000"
        )
      ),
      fn = function(args) {
        n <- if (!is.null(args$n)) as.integer(args$n) else 10000L
        mcs_tool(n, args$tasks)
      }
    ),
    smm = list(
      title = "Second Moment Method",
      description = "Quick analytical estimate of total project mean and variance.",
      args = list(
        list(
          name = "means", type = "json", required = TRUE,
          description = "JSON array of task mean values",
          example = "[10, 15, 20, 8]"
        ),
        list(
          name = "variances", type = "json", required = TRUE,
          description = "JSON array of task variance values",
          example = "[4, 9, 16, 2]"
        )
      ),
      fn = function(args) {
        smm_tool(args$means, args$variances)
      }
    ),
    contingency = list(
      title = "Contingency Analysis",
      description = "Calculate contingency reserve from the last MCS run.",
      args = list(
        list(
          name = "phigh", type = "number", required = FALSE,
          description = "High percentile (default: 0.95)",
          example = "0.95"
        ),
        list(
          name = "pbase", type = "number", required = FALSE,
          description = "Base percentile (default: 0.50)",
          example = "0.50"
        )
      ),
      fn = function(args) {
        ph <- if (!is.null(args$phigh)) as.numeric(args$phigh) else 0.95
        pb <- if (!is.null(args$pbase)) as.numeric(args$pbase) else 0.50
        contingency_tool(ph, pb)
      }
    ),
    sensitivity = list(
      title = "Sensitivity Analysis",
      description = "Identify which tasks drive the most variance.",
      args = list(
        list(
          name = "tasks", type = "json", required = TRUE,
          description = "Task distributions as JSON array (same format as /mcs)",
          example = '[{"type":"normal","mean":10,"sd":2},{"type":"uniform","min":8,"max":12}]'
        )
      ),
      fn = function(args) {
        sensitivity_tool(args$tasks)
      }
    ),
    evm = list(
      title = "Earned Value Management",
      description = "Run a complete EVM analysis (PV, EV, AC, SV, CV, SPI, CPI, EAC, TCPI).",
      args = list(
        list(
          name = "bac", type = "number", required = TRUE,
          description = "Budget at Completion",
          example = "500000"
        ),
        list(
          name = "schedule", type = "json", required = TRUE,
          description = "Cumulative planned completion fractions",
          example = "[0.2, 0.4, 0.6, 0.8, 1.0]"
        ),
        list(
          name = "period", type = "integer", required = TRUE,
          description = "Current time period (1-based)",
          example = "3"
        ),
        list(
          name = "complete", type = "number", required = TRUE,
          description = "Actual percent complete (0-1 or 0-100)",
          example = "0.35"
        ),
        list(
          name = "costs", type = "json", required = TRUE,
          description = "Cumulative actual costs per period",
          example = "[90000, 195000, 310000]"
        )
      ),
      fn = function(args) {
        evm_analysis_tool(
          bac = args$bac,
          schedule_json = args$schedule,
          time_period = args$period,
          actual_per_complete = args$complete,
          actual_costs_json = args$costs,
          cumulative = "true"
        )
      }
    ),
    risk = list(
      title = "Bayesian Risk (Prior)",
      description = "Calculate prior risk probability from root causes.",
      args = list(
        list(
          name = "causes", type = "json", required = TRUE,
          description = "Root cause probabilities",
          example = "[0.3, 0.2]"
        ),
        list(
          name = "risk_given", type = "json", required = TRUE,
          description = "P(Risk | Cause) for each cause",
          example = "[0.8, 0.6]"
        ),
        list(
          name = "risk_not_given", type = "json", required = TRUE,
          description = "P(Risk | Not Cause) for each cause",
          example = "[0.2, 0.4]"
        )
      ),
      fn = function(args) {
        risk_prob_tool(args$causes, args$risk_given, args$risk_not_given)
      }
    ),
    risk_post = list(
      title = "Bayesian Risk (Posterior)",
      description = "Update risk probability after observing causes.",
      args = list(
        list(
          name = "causes", type = "json", required = TRUE,
          description = "Root cause probabilities",
          example = "[0.3, 0.2]"
        ),
        list(
          name = "risk_given", type = "json", required = TRUE,
          description = "P(Risk | Cause) for each cause",
          example = "[0.8, 0.6]"
        ),
        list(
          name = "risk_not_given", type = "json", required = TRUE,
          description = "P(Risk | Not Cause) for each cause",
          example = "[0.2, 0.4]"
        ),
        list(
          name = "observed", type = "json", required = TRUE,
          description = "Observed causes: 1 (occurred), 0 (not), null (unknown)",
          example = "[1, null]"
        )
      ),
      fn = function(args) {
        risk_post_prob_tool(args$causes, args$risk_given, args$risk_not_given, args$observed)
      }
    ),
    learning = list(
      title = "Learning Curve",
      description = "Fit a sigmoidal learning curve (pearl/gompertz/logistic) and predict.",
      args = list(
        list(
          name = "x", type = "json", required = TRUE,
          description = "X values (time or unit number)",
          example = "[1,2,3,4,5,6,7,8,9,10]"
        ),
        list(
          name = "y", type = "json", required = TRUE,
          description = "Y values (cost, completion, etc.)",
          example = "[5,15,40,60,70,75,80,85,90,95]"
        ),
        list(
          name = "model", type = "string", required = TRUE,
          description = "Model type: pearl, gompertz, or logistic",
          example = "logistic"
        ),
        list(
          name = "predict", type = "json", required = FALSE,
          description = "X values to predict at (default: original x values)",
          example = "[12, 15]"
        )
      ),
      fn = function(args) {
        fit_and_predict_sigmoidal_tool(args$x, args$y, args$model, args$predict)
      }
    ),
    dsm = list(
      title = "Parent DSM",
      description = "Compute resource-task dependency structure matrix (S * S').",
      args = list(
        list(
          name = "matrix", type = "json", required = TRUE,
          description = "Resource-task matrix as JSON 2D array",
          example = "[[1,0,1],[0,1,0],[1,0,1]]"
        )
      ),
      fn = function(args) {
        parent_dsm_tool(args$matrix)
      }
    )
  )
}

#' Format help text for a single command
#' @keywords internal
format_command_help <- function(cmd_name, cmd) {
  arg_lines <- vapply(cmd$args, function(a) {
    req <- if (a$required) " (required)" else " (optional)"
    paste0("  **", a$name, "**", req, " - ", a$description, "\n    Example: `", a$example, "`")
  }, character(1))

  paste0(
    "### /", cmd_name, " - ", cmd$title, "\n\n",
    cmd$description, "\n\n",
    "**Arguments:**\n\n",
    paste(arg_lines, collapse = "\n\n"), "\n\n",
    "**Usage:**\n```\n/", cmd_name,
    paste0(" ", vapply(cmd$args, function(a) {
      if (a$required) {
        paste0(a$name, "=", a$example)
      } else {
        paste0("[", a$name, "=", a$example, "]")
      }
    }, character(1)), collapse = ""),
    "\n```"
  )
}

#' Format the /help overview
#' @keywords internal
format_help_overview <- function() {
  registry <- pra_command_registry()
  lines <- vapply(names(registry), function(name) {
    cmd <- registry[[name]]
    paste0("  **/", name, "** - ", cmd$description)
  }, character(1))

  paste0(
    "### Available Commands\n\n",
    "Type a command to run a tool directly (no LLM needed). ",
    "Type `/command` with no arguments to see detailed help.\n\n",
    paste(lines, collapse = "\n\n"),
    "\n\n  **/help** - Show this list\n\n",
    "You can also type natural language questions to use the AI agent."
  )
}

#' Parse and execute a slash command
#'
#' @param input The raw user input string starting with "/".
#' @return A list with `ok` (logical) and `result` (character or ContentToolResult).
#' @keywords internal
execute_command <- function(input) {
  input <- trimws(input)
  registry <- pra_command_registry()

  # Parse: /command arg1=val1 arg2=val2
  # Or:    /command (no args -> show help)
  # Or:    /help
  parts <- strsplit(input, "\\s+", perl = TRUE)[[1]]
  cmd_name <- sub("^/", "", tolower(parts[1]))

  # /help

  if (cmd_name == "help") {
    return(list(ok = TRUE, result = format_help_overview()))
  }

  # Unknown command
  if (!cmd_name %in% names(registry)) {
    suggestions <- names(registry)
    return(list(ok = FALSE, result = paste0(
      "Unknown command: **/", cmd_name, "**\n\n",
      "Available commands: ", paste0("`/", suggestions, "`", collapse = ", "),
      "\n\nType `/help` for details."
    )))
  }

  cmd <- registry[[cmd_name]]

  # No args -> show detailed help for this command
  if (length(parts) == 1) {
    return(list(ok = TRUE, result = format_command_help(cmd_name, cmd)))
  }

  # Parse key=value arguments
  arg_strings <- parts[-1]
  parsed_args <- list()
  parse_errors <- character(0)

  for (s in arg_strings) {
    if (!grepl("=", s)) {
      parse_errors <- c(parse_errors, paste0("Invalid argument `", s, "` - use `name=value` format."))
      next
    }
    eq_pos <- regexpr("=", s)
    key <- substr(s, 1, eq_pos - 1)
    val <- substr(s, eq_pos + 1, nchar(s))
    parsed_args[[key]] <- val
  }

  if (length(parse_errors) > 0) {
    return(list(ok = FALSE, result = paste0(
      "**Parse errors:**\n\n", paste("- ", parse_errors, collapse = "\n"), "\n\n",
      format_command_help(cmd_name, cmd)
    )))
  }

  # Check required arguments
  required <- vapply(cmd$args, function(a) if (a$required) a$name else NA_character_, character(1))
  required <- required[!is.na(required)]
  missing <- setdiff(required, names(parsed_args))

  if (length(missing) > 0) {
    missing_help <- vapply(missing, function(m) {
      a <- cmd$args[[which(vapply(cmd$args, function(x) x$name, character(1)) == m)]]
      paste0("  **", m, "** - ", a$description, "\n    Example: `", m, "=", a$example, "`")
    }, character(1))

    return(list(ok = FALSE, result = paste0(
      "**Missing required argument", if (length(missing) > 1) "s" else "", ":**\n\n",
      paste(missing_help, collapse = "\n\n"),
      "\n\n**Full usage:**\n\n",
      format_command_help(cmd_name, cmd)
    )))
  }

  # Execute
  tryCatch(
    {
      result <- cmd$fn(parsed_args)
      list(ok = TRUE, result = result)
    },
    error = function(e) {
      list(ok = FALSE, result = paste0(
        "**Error running /", cmd_name, ":** ", e$message, "\n\n",
        "Check your arguments and try again. Type `/", cmd_name,
        "` for usage details."
      ))
    }
  )
}

# ============================================================================
# Internal tool wrapper functions
# ============================================================================

# Package-level environment to store full simulation results for chaining
.pra_agent_env <- new.env(parent = emptyenv())

#' @keywords internal
check_package <- function(pkg, purpose = NULL) {
  if (!requireNamespace(pkg, quietly = TRUE)) {
    msg <- sprintf("Package '%s' is required", pkg)
    if (!is.null(purpose)) msg <- paste0(msg, " for ", purpose)
    stop(paste0(msg, ". Install with: install.packages('", pkg, "')"), call. = FALSE)
  }
}

#' @keywords internal
parse_json_param <- function(json_str, param_name, tool_name) {
  if (is.null(json_str) || json_str == "null" || json_str == "") {
    return(NULL)
  }
  check_package("jsonlite")
  tryCatch(
    jsonlite::fromJSON(json_str, simplifyVector = TRUE),
    error = function(e) {
      stop(sprintf(
        "Invalid JSON for '%s' in %s: %s\nReceived: %s",
        param_name, tool_name, e$message, json_str
      ), call. = FALSE)
    }
  )
}

#' @keywords internal
parse_json <- function(json_str) {
  check_package("jsonlite")
  if (is.null(json_str) || json_str == "null" || json_str == "") {
    return(NULL)
  }

  # Clean common LLM formatting mistakes before parsing
  cleaned <- json_str

  # Remove smart quotes that some models produce
  cleaned <- gsub("\u201c|\u201d", "\"", cleaned)
  cleaned <- gsub("\u2018|\u2019", "'", cleaned)

  # Strip leading/trailing whitespace

  cleaned <- trimws(cleaned)

  # If it looks like a bare comma-separated list (no brackets), wrap in []
  if (!grepl("^\\s*[\\[{]", cleaned) && grepl("^\\s*-?[0-9]", cleaned)) {
    cleaned <- paste0("[", cleaned, "]")
  }

  # Try strict parse first, then fallback with cleanup
  tryCatch(
    jsonlite::fromJSON(cleaned, simplifyVector = TRUE),
    error = function(e) {
      # Try removing trailing commas: [1, 2,] -> [1, 2]
      cleaned2 <- gsub(",\\s*([\\]\\}])", "\\1", cleaned)
      # Try replacing single quotes with double quotes
      cleaned2 <- gsub("'", "\"", cleaned2)
      tryCatch(
        jsonlite::fromJSON(cleaned2, simplifyVector = TRUE),
        error = function(e2) {
          # Last resort: try to extract numbers from the string
          nums <- regmatches(cleaned, gregexpr("-?[0-9]*\\.?[0-9]+", cleaned))[[1]]
          if (length(nums) > 0) {
            as.numeric(nums)
          } else {
            stop("Cannot parse JSON: ", json_str, call. = FALSE)
          }
        }
      )
    }
  )
}

#' Coerce an input to an R object
#'
#' Accepts either a JSON string (from LLM tool calls) or a native R object
#' (from direct R usage). If the input is a single character string, it is
#' parsed as JSON. Otherwise it is returned as-is.
#' @keywords internal
as_r_input <- function(x) {
  if (is.null(x)) {
    return(NULL)
  }
  if (is.character(x) && length(x) == 1) {
    parse_json(x)
  } else {
    x
  }
}

#' Coerce input to a numeric vector
#'
#' Parses JSON input and ensures the result is a numeric vector. Handles
#' cases where LLMs send string representations of numbers.
#' @keywords internal
as_numeric_input <- function(x) {
  val <- as_r_input(x)
  if (is.null(val)) {
    return(NULL)
  }
  as.numeric(val)
}

#' Validate task distributions parsed from JSON
#' @keywords internal
validate_task_dists <- function(task_dists, tool_name = "mcs_tool") {
  if (!is.list(task_dists) || length(task_dists) == 0) {
    stop(tool_name, ": task_dists_json must be a non-empty JSON array of distribution objects.", call. = FALSE)
  }
  valid_types <- c("normal", "triangular", "uniform")
  required_params <- list(
    normal = c("mean", "sd"),
    triangular = c("a", "b", "c"),
    uniform = c("min", "max")
  )
  for (i in seq_along(task_dists)) {
    d <- task_dists[[i]]
    dtype <- d$type
    if (is.null(dtype) || !dtype %in% valid_types) {
      stop(
        sprintf(
          "%s: Task %d has invalid type '%s'. Must be one of: %s",
          tool_name, i, as.character(dtype), paste(valid_types, collapse = ", ")
        ),
        call. = FALSE
      )
    }
    missing <- setdiff(required_params[[dtype]], names(d))
    if (length(missing) > 0) {
      stop(
        sprintf(
          "%s: Task %d (%s) is missing required parameters: %s",
          tool_name, i, dtype, paste(missing, collapse = ", ")
        ),
        call. = FALSE
      )
    }
  }
  invisible(TRUE)
}

#' @noRd
format_result_table <- function(...) {
  pairs <- list(...)
  nms <- names(pairs)
  vals <- vapply(pairs, function(v) {
    if (is.numeric(v)) format(round(v, 4), big.mark = ",", scientific = FALSE) else as.character(v)
  }, character(1))
  max_n <- max(nchar(nms))
  lines <- paste0("  ", formatC(nms, width = -max_n), "  ", vals)
  paste(lines, collapse = "\n")
}

#' @keywords internal
summarize_distribution <- function(x) {
  pcts <- stats::quantile(x, c(0.05, 0.10, 0.25, 0.50, 0.75, 0.90, 0.95))
  list(
    mean = mean(x),
    sd = stats::sd(x),
    min = min(x),
    max = max(x),
    percentiles = list(
      P5  = unname(pcts[1]),
      P10 = unname(pcts[2]),
      P25 = unname(pcts[3]),
      P50 = unname(pcts[4]),
      P75 = unname(pcts[5]),
      P90 = unname(pcts[6]),
      P95 = unname(pcts[7])
    )
  )
}

#' @noRd
format_distribution <- function(x) {
  s <- summarize_distribution(x)
  paste0(
    "Summary Statistics:\n",
    format_result_table(
      Mean = s$mean, SD = s$sd, Min = s$min, Max = s$max
    ),
    "\n\nPercentiles:\n",
    format_result_table(
      P5 = s$percentiles$P5, P10 = s$percentiles$P10,
      P25 = s$percentiles$P25, P50 = s$percentiles$P50,
      P75 = s$percentiles$P75, P90 = s$percentiles$P90,
      P95 = s$percentiles$P95
    )
  )
}

#' @keywords internal
parse_task_dists <- function(x) {
  if (is.list(x) && !is.data.frame(x) && length(x) > 0 && is.list(x[[1]])) {
    return(x) # Already a list-of-lists from direct R call
  }
  raw <- as_r_input(x)
  if (is.data.frame(raw)) {
    lapply(seq_len(nrow(raw)), function(i) as.list(raw[i, , drop = FALSE]))
  } else {
    raw
  }
}

#' Save a plot to a temp file and return path
#' @keywords internal
save_pra_plot <- function(plot_fn, name, width = 700, height = 450) {
  plot_path <- file.path(tempdir(), paste0(
    "pra_plot_", name, "_",
    format(Sys.time(), "%H%M%S"), ".png"
  ))
  grDevices::png(plot_path, width = width, height = height, res = 100)
  tryCatch(plot_fn(), finally = grDevices::dev.off())
  plot_path
}

#' Generate a base64-encoded HTML img tag from a plot function
#' @keywords internal
plot_to_html <- function(plot_fn, width = 700, height = 450) {
  check_package("base64enc")
  tmp <- tempfile(fileext = ".png")
  on.exit(unlink(tmp), add = TRUE)
  grDevices::png(tmp, width = width, height = height, res = 100)
  tryCatch(plot_fn(), finally = grDevices::dev.off())
  raw_bytes <- readBin(tmp, "raw", file.info(tmp)$size)
  b64 <- base64enc::dataURI(raw_bytes, mime = "image/png")
  paste0('<img src="', b64, '" style="max-width:100%;border-radius:8px;margin-top:8px;">')
}

#' Build an HTML table from name-value pairs
#' @keywords internal
html_result_table <- function(...) {
  pairs <- list(...)
  nms <- names(pairs)
  vals <- vapply(pairs, function(v) {
    if (is.numeric(v)) format(round(v, 4), big.mark = ",", scientific = FALSE) else as.character(v)
  }, character(1))
  rows <- paste0(
    "<tr><td style='padding:4px 12px;font-weight:600;'>", nms,
    "</td><td style='padding:4px 12px;'>", vals, "</td></tr>"
  )
  paste0(
    "<table style='border-collapse:collapse;margin:8px 0;font-size:0.9em;'>",
    paste(rows, collapse = ""), "</table>"
  )
}

#' Build a ContentToolResult or fall back to plain text
#' @keywords internal
tool_result <- function(text, title = NULL, html = NULL) {
  # Try to return rich ContentToolResult for shinychat display
  if (!is.null(html)) {
    tryCatch(
      {
        display <- list(html = html)
        if (!is.null(title)) display$title <- title
        ellmer::ContentToolResult(value = text, extra = list(display = display))
      },
      error = function(e) {
        # Fall back to plain text if ContentToolResult not available
        text
      }
    )
  } else {
    text
  }
}

# ---- MCS ----
#' @keywords internal
mcs_tool <- function(num_sims, task_dists_json, cor_mat_json = "null") {
  num_sims <- as.integer(unlist(num_sims))[1]
  if (is.na(num_sims) || num_sims < 1) {
    stop("mcs_tool: num_sims must be a positive integer.", call. = FALSE)
  }
  task_dists <- parse_task_dists(task_dists_json)
  validate_task_dists(task_dists, "mcs_tool")
  cor_mat <- as_r_input(cor_mat_json)
  if (!is.null(cor_mat)) cor_mat <- as.matrix(as.data.frame(cor_mat))
  result <- mcs(num_sims, task_dists, cor_mat)
  .pra_agent_env$last_mcs <- result

  s <- summarize_distribution(result$total_distribution)

  # Plain text for LLM
  text <- paste0(
    "Monte Carlo Simulation Results (n = ", format(num_sims, big.mark = ","), "):\n\n",
    format_distribution(result$total_distribution)
  )

  # Rich HTML display
  html <- tryCatch(
    {
      stats_html <- html_result_table(
        Mean = s$mean, `Std Dev` = s$sd, Min = s$min, Max = s$max,
        P5 = s$percentiles$P5, P25 = s$percentiles$P25,
        P50 = s$percentiles$P50, P75 = s$percentiles$P75,
        P90 = s$percentiles$P90, P95 = s$percentiles$P95
      )
      plot_img <- plot_to_html(function() {
        graphics::hist(result$total_distribution,
          freq = FALSE, breaks = 50,
          main = "Monte Carlo Simulation Results",
          xlab = "Total Project Duration/Cost",
          col = "#18bc9c80", border = "white"
        )
        graphics::curve(stats::dnorm(x, mean = result$total_mean, sd = result$total_sd),
          add = TRUE, col = "#2c3e50", lwd = 2
        )
        graphics::abline(
          v = stats::quantile(result$total_distribution, c(0.50, 0.95)),
          col = c("#3498db", "#e74c3c"), lty = 2, lwd = 1.5
        )
        graphics::legend("topright",
          legend = c("Normal fit", "P50", "P95"),
          col = c("#2c3e50", "#3498db", "#e74c3c"),
          lty = c(1, 2, 2), lwd = c(2, 1.5, 1.5),
          cex = 0.8, bg = "white"
        )
      })
      paste0(stats_html, plot_img)
    },
    error = function(e) NULL
  )

  tool_result(text, title = "Monte Carlo Simulation", html = html)
}

# ---- SMM ----
#' @keywords internal
smm_tool <- function(mean_json, var_json, cor_mat_json = "null") {
  means <- as_numeric_input(mean_json)
  vars <- as_numeric_input(var_json)
  cor_mat <- as_r_input(cor_mat_json)
  if (!is.null(cor_mat)) cor_mat <- as.matrix(as.data.frame(cor_mat))
  result <- smm(means, vars, cor_mat)

  text <- paste0(
    "Second Moment Method Results:\n\n",
    format_result_table(
      `Total Mean` = result$total_mean,
      `Total Variance` = result$total_var,
      `Total Std Dev` = result$total_std
    )
  )

  html <- tryCatch(
    {
      html_result_table(
        `Total Mean` = result$total_mean,
        `Total Variance` = result$total_var,
        `Total Std Dev` = result$total_std
      )
    },
    error = function(e) NULL
  )

  tool_result(text, title = "Second Moment Method", html = html)
}

# ---- Contingency ----
#' @keywords internal
contingency_tool <- function(phigh = 0.95, pbase = 0.50) {
  phigh <- as.numeric(phigh)
  pbase <- as.numeric(pbase)
  if (is.null(.pra_agent_env$last_mcs)) {
    return("Error: Run mcs_tool first. Contingency requires Monte Carlo simulation results.")
  }
  result <- contingency(.pra_agent_env$last_mcs, phigh, pbase)

  text <- paste0(
    "Contingency Analysis:\n\n",
    format_result_table(
      `Base Percentile` = paste0("P", pbase * 100),
      `High Percentile` = paste0("P", phigh * 100),
      `Contingency Reserve` = result
    )
  )

  html <- tryCatch(
    {
      html_result_table(
        `Base Percentile` = paste0("P", pbase * 100),
        `High Percentile` = paste0("P", phigh * 100),
        `Contingency Reserve` = result
      )
    },
    error = function(e) NULL
  )

  tool_result(text, title = "Contingency Analysis", html = html)
}

# ---- Sensitivity ----
#' @keywords internal
sensitivity_tool <- function(task_dists_json, cor_mat_json = "null") {
  task_dists <- parse_task_dists(task_dists_json)
  validate_task_dists(task_dists, "sensitivity_tool")
  cor_mat <- as_r_input(cor_mat_json)
  if (!is.null(cor_mat)) cor_mat <- as.matrix(as.data.frame(cor_mat))
  result <- sensitivity(task_dists, cor_mat)
  names(result) <- paste("Task", seq_along(result))

  vals <- vapply(result, function(v) format(round(v, 4), big.mark = ","), character(1))
  lines <- paste0("  ", formatC(names(result), width = -10), vals)
  text <- paste0(
    "Sensitivity Analysis (variance contribution per task):\n\n",
    paste(lines, collapse = "\n")
  )

  html <- tryCatch(
    {
      # Table
      tbl_args <- as.list(result)
      names(tbl_args) <- names(result)
      tbl_html <- do.call(html_result_table, tbl_args)

      # Tornado chart
      plot_img <- plot_to_html(function() {
        sorted <- sort(result)
        graphics::barplot(sorted,
          horiz = TRUE, las = 1,
          main = "Sensitivity Analysis",
          xlab = "Variance Contribution",
          col = grDevices::colorRampPalette(c("#18bc9c", "#e74c3c"))(length(sorted)),
          border = "white"
        )
      })
      paste0(tbl_html, plot_img)
    },
    error = function(e) NULL
  )

  tool_result(text, title = "Sensitivity Analysis", html = html)
}

# ---- EVM: Composite analysis ----
#' @keywords internal
evm_analysis_tool <- function(bac, schedule_json, time_period, actual_per_complete,
                              actual_costs_json, cumulative = "true") {
  # Coerce scalar params - LLMs may send strings, arrays, or JSON fragments
  bac <- as.numeric(unlist(bac))[1]
  time_period <- as.integer(unlist(time_period))[1]
  actual_per_complete <- as.numeric(unlist(actual_per_complete))[1]
  # Auto-convert integer-like percentages to decimals (e.g. 35 -> 0.35)
  if (!is.na(actual_per_complete) && actual_per_complete > 1 &&
    actual_per_complete <= 100 && actual_per_complete == round(actual_per_complete)) {
    actual_per_complete <- actual_per_complete / 100
  }
  if (is.na(bac) || bac <= 0) {
    stop("evm_analysis_tool: bac must be a positive number.", call. = FALSE)
  }
  if (is.na(time_period) || time_period < 1) {
    stop("evm_analysis_tool: time_period must be a positive integer.", call. = FALSE)
  }
  if (is.na(actual_per_complete) || actual_per_complete < 0 || actual_per_complete > 1) {
    stop("evm_analysis_tool: actual_per_complete must be between 0 and 1.", call. = FALSE)
  }
  schedule <- as_numeric_input(schedule_json)
  if (time_period > length(schedule)) {
    stop(sprintf(
      "evm_analysis_tool: time_period (%d) exceeds schedule length (%d).",
      time_period, length(schedule)
    ), call. = FALSE)
  }
  actual_costs <- as_numeric_input(actual_costs_json)
  cum <- if (is.logical(cumulative)) cumulative else cumulative == "true"

  # Core metrics
  pv_val <- pv(bac, schedule, time_period)
  ev_val <- ev(bac, actual_per_complete)
  ac_val <- ac(actual_costs, time_period, cum)

  # Variances and indices
  sv_val <- sv(ev_val, pv_val)
  cv_val <- cv(ev_val, ac_val)
  spi_val <- spi(ev_val, pv_val)
  cpi_val <- cpi(ev_val, ac_val)

  # Forecasts
  eac_typical <- eac(bac, "typical", cpi = cpi_val)
  eac_atypical <- eac(bac, "atypical", ac = ac_val, ev = ev_val)
  eac_combined <- eac(bac, "combined", cpi = cpi_val, ac = ac_val, ev = ev_val, spi = spi_val)
  etc_val <- etc(bac, ev_val, cpi_val)
  vac_val <- vac(bac, eac_typical)
  tcpi_val <- tcpi(bac, ev_val, ac_val, "bac")

  text <- paste0(
    "Earned Value Management Analysis:\n\n",
    "Core Metrics:\n",
    format_result_table(`Planned Value (PV)` = pv_val, `Earned Value (EV)` = ev_val, `Actual Cost (AC)` = ac_val),
    "\n\nVariances:\n",
    format_result_table(`Schedule Variance (SV)` = sv_val, `Cost Variance (CV)` = cv_val),
    "\n\nPerformance Indices:\n",
    format_result_table(`Schedule Performance Index (SPI)` = spi_val, `Cost Performance Index (CPI)` = cpi_val),
    "\n\nForecasts:\n",
    format_result_table(
      `EAC (Typical)` = eac_typical, `EAC (Atypical)` = eac_atypical,
      `EAC (Combined)` = eac_combined, `Estimate to Complete (ETC)` = etc_val,
      `Variance at Completion (VAC)` = vac_val, `TCPI (to meet BAC)` = tcpi_val
    )
  )

  html <- tryCatch(
    {
      core_tbl <- html_result_table(
        `Planned Value (PV)` = pv_val, `Earned Value (EV)` = ev_val, `Actual Cost (AC)` = ac_val
      )
      perf_tbl <- html_result_table(
        `Schedule Variance (SV)` = sv_val, `Cost Variance (CV)` = cv_val,
        `SPI` = spi_val, `CPI` = cpi_val
      )
      forecast_tbl <- html_result_table(
        `EAC (Typical)` = eac_typical, `EAC (Atypical)` = eac_atypical,
        `EAC (Combined)` = eac_combined, `ETC` = etc_val,
        `VAC` = vac_val, `TCPI` = tcpi_val
      )

      # Bar chart comparing PV, EV, AC
      plot_img <- plot_to_html(function() {
        vals <- c(pv_val, ev_val, ac_val)
        nms <- c("PV", "EV", "AC")
        cols <- c("#3498db", "#18bc9c", "#e74c3c")
        graphics::barplot(vals,
          names.arg = nms, col = cols, border = "white",
          main = "EVM Core Metrics", ylab = "Value",
          ylim = c(0, max(vals) * 1.15)
        )
        graphics::text(seq(0.7, by = 1.2, length.out = 3), vals,
          labels = format(round(vals), big.mark = ","),
          pos = 3, cex = 0.85, font = 2
        )
      })

      paste0(
        "<h5>Core Metrics</h5>", core_tbl,
        "<h5>Performance</h5>", perf_tbl,
        "<h5>Forecasts</h5>", forecast_tbl,
        plot_img
      )
    },
    error = function(e) NULL
  )

  tool_result(text, title = "Earned Value Management", html = html)
}

# ---- EVM: Individual tools ----
#' @keywords internal
pv_tool <- function(bac, schedule_json, time_period) {
  schedule <- as_numeric_input(schedule_json)
  val <- pv(bac, schedule, time_period)
  format_result_table(`Planned Value (PV)` = val)
}

#' @keywords internal
ev_tool <- function(bac, actual_per_complete) {
  val <- ev(bac, actual_per_complete)
  format_result_table(`Earned Value (EV)` = val)
}

#' @keywords internal
ac_tool <- function(actual_costs_json, time_period, cumulative = "true") {
  actual_costs <- as_numeric_input(actual_costs_json)
  cum <- if (is.logical(cumulative)) cumulative else cumulative == "true"
  val <- ac(actual_costs, time_period, cum)
  format_result_table(`Actual Cost (AC)` = val)
}

#' @keywords internal
sv_tool <- function(ev, pv) {
  val <- sv(ev, pv)
  format_result_table(`Schedule Variance (SV)` = val)
}

#' @keywords internal
cv_tool <- function(ev, ac) {
  val <- cv(ev, ac)
  format_result_table(`Cost Variance (CV)` = val)
}

#' @keywords internal
spi_tool <- function(ev, pv) {
  val <- spi(ev, pv)
  format_result_table(`Schedule Performance Index (SPI)` = val)
}

#' @keywords internal
cpi_tool <- function(ev, ac) {
  val <- cpi(ev, ac)
  format_result_table(`Cost Performance Index (CPI)` = val)
}

#' @keywords internal
eac_tool <- function(bac, method = "typical", cpi = NULL, ac = NULL, ev = NULL, spi = NULL) {
  val <- eac(bac, method, cpi, ac, ev, spi)
  paste0(
    format_result_table(`Estimate at Completion (EAC)` = val),
    "\n  Method                          ", method
  )
}

#' @keywords internal
etc_tool <- function(bac, ev, cpi = NULL) {
  val <- etc(bac, ev, cpi)
  format_result_table(`Estimate to Complete (ETC)` = val)
}

#' @keywords internal
vac_tool <- function(bac, eac) {
  val <- vac(bac, eac)
  format_result_table(`Variance at Completion (VAC)` = val)
}

#' @keywords internal
tcpi_tool <- function(bac, ev, ac, target = "bac", eac = NULL) {
  val <- tcpi(bac, ev, ac, target, eac)
  paste0(
    format_result_table(`To-Complete Performance Index (TCPI)` = val),
    "\n  Target                                     ", target
  )
}

# ---- Bayesian tools ----
#' @keywords internal
risk_prob_tool <- function(cause_probs_json, risks_given_causes_json, risks_given_not_causes_json) {
  cause_probs <- as_numeric_input(cause_probs_json)
  rgc <- as_numeric_input(risks_given_causes_json)
  rgnc <- as_numeric_input(risks_given_not_causes_json)
  val <- risk_prob(cause_probs, rgc, rgnc)

  text <- paste0(
    "Bayesian Risk Analysis (Prior):\n\n",
    format_result_table(
      `Risk Probability` = val,
      `Risk Percentage` = paste0(round(val * 100, 2), "%"),
      `Number of Causes` = length(cause_probs)
    )
  )

  html <- tryCatch(
    {
      html_result_table(
        `Risk Probability` = val,
        `Risk Percentage` = paste0(round(val * 100, 2), "%"),
        `Number of Causes` = length(cause_probs)
      )
    },
    error = function(e) NULL
  )

  tool_result(text, title = "Bayesian Risk (Prior)", html = html)
}

#' @keywords internal
risk_post_prob_tool <- function(cause_probs_json, risks_given_causes_json,
                                risks_given_not_causes_json, observed_causes_json) {
  cause_probs <- as_numeric_input(cause_probs_json)
  rgc <- as_numeric_input(risks_given_causes_json)
  rgnc <- as_numeric_input(risks_given_not_causes_json)
  observed <- as_r_input(observed_causes_json)
  observed <- ifelse(is.na(observed), NA, as.logical(observed))
  val <- risk_post_prob(cause_probs, rgc, rgnc, observed)
  obs_labels <- ifelse(is.na(observed), "Unknown", ifelse(observed, "Occurred", "Did not occur"))
  obs_lines <- paste0("  Cause ", seq_along(observed), ": ", obs_labels)

  text <- paste0(
    "Bayesian Risk Analysis (Posterior):\n\n",
    format_result_table(
      `Posterior Risk Probability` = val,
      `Posterior Risk Percentage` = paste0(round(val * 100, 2), "%")
    ),
    "\n\nObservations:\n", paste(obs_lines, collapse = "\n")
  )

  html <- tryCatch(
    {
      obs_args <- as.list(stats::setNames(obs_labels, paste("Cause", seq_along(observed))))
      paste0(
        html_result_table(
          `Posterior Probability` = val,
          `Posterior Percentage` = paste0(round(val * 100, 2), "%")
        ),
        "<h5>Observations</h5>",
        do.call(html_result_table, obs_args)
      )
    },
    error = function(e) NULL
  )

  tool_result(text, title = "Bayesian Risk (Posterior)", html = html)
}

#' @keywords internal
cost_pdf_tool <- function(num_sims, risk_probs_json, means_given_risks_json,
                          sds_given_risks_json, base_cost = 0) {
  num_sims <- as.integer(unlist(num_sims))[1]
  base_cost <- as.numeric(unlist(base_cost))[1]
  risk_probs <- as_numeric_input(risk_probs_json)
  means <- as_numeric_input(means_given_risks_json)
  sds <- as_numeric_input(sds_given_risks_json)
  result <- cost_pdf(num_sims, risk_probs, means, sds, base_cost)
  .pra_agent_env$last_cost_pdf <- result

  text <- paste0(
    "Prior Cost Distribution (n = ", format(num_sims, big.mark = ","),
    ", base cost = ", format(base_cost, big.mark = ",", scientific = FALSE), "):\n\n",
    format_distribution(result)
  )

  html <- tryCatch(
    {
      s <- summarize_distribution(result)
      tbl <- html_result_table(
        Mean = s$mean, `Std Dev` = s$sd, P50 = s$percentiles$P50,
        P90 = s$percentiles$P90, P95 = s$percentiles$P95
      )
      tbl
    },
    error = function(e) NULL
  )

  tool_result(text, title = "Prior Cost Distribution", html = html)
}

#' @keywords internal
cost_post_pdf_tool <- function(num_sims, observed_risks_json, means_given_risks_json,
                               sds_given_risks_json, base_cost = 0) {
  num_sims <- as.integer(unlist(num_sims))[1]
  base_cost <- as.numeric(unlist(base_cost))[1]
  observed <- as_r_input(observed_risks_json)
  observed <- ifelse(is.na(observed), NA, as.logical(observed))
  means <- as_numeric_input(means_given_risks_json)
  sds <- as_numeric_input(sds_given_risks_json)
  result <- cost_post_pdf(num_sims, observed, means, sds, base_cost)
  .pra_agent_env$last_cost_post_pdf <- result

  text <- paste0(
    "Posterior Cost Distribution (n = ", format(num_sims, big.mark = ","),
    ", base cost = ", format(base_cost, big.mark = ",", scientific = FALSE), "):\n\n",
    format_distribution(result)
  )

  html <- tryCatch(
    {
      s <- summarize_distribution(result)
      html_result_table(
        Mean = s$mean, `Std Dev` = s$sd, P50 = s$percentiles$P50,
        P90 = s$percentiles$P90, P95 = s$percentiles$P95
      )
    },
    error = function(e) NULL
  )

  tool_result(text, title = "Posterior Cost Distribution", html = html)
}

# ---- Sigmoidal (composite) ----
#' @keywords internal
fit_and_predict_sigmoidal_tool <- function(x_json, y_json, model_type,
                                           predict_x_json = NULL, conf_level = NULL) {
  x <- as_numeric_input(x_json)
  y <- as_numeric_input(y_json)
  data <- data.frame(x = x, y = y)

  fit <- fit_sigmoidal(data, "x", "y", model_type)
  coefficients <- stats::coef(fit)

  predict_x <- as_numeric_input(predict_x_json)
  if (is.null(predict_x)) predict_x <- x

  if (is.null(conf_level) || is.na(conf_level)) {
    preds <- predict_sigmoidal(fit, predict_x, model_type)
  } else {
    preds <- predict_sigmoidal(fit, predict_x, model_type, conf_level)
  }

  # Format coefficients
  coef_lines <- paste0(
    "  ", formatC(names(coefficients), width = -10),
    format(round(unname(coefficients), 6), big.mark = ",")
  )

  # Format predictions table
  pred_df <- data.frame(x = predict_x, predicted = round(preds, 4))
  pred_lines <- paste(utils::capture.output(print(pred_df, row.names = FALSE)), collapse = "\n")

  text <- paste0(
    "Learning Curve Fit (", model_type, " model):\n\n",
    "Coefficients:\n", paste(coef_lines, collapse = "\n"),
    "\n\nPredictions:\n", pred_lines
  )

  html <- tryCatch(
    {
      # Coefficients table
      coef_args <- as.list(unname(coefficients))
      names(coef_args) <- names(coefficients)
      coef_tbl <- do.call(html_result_table, coef_args)

      # Learning curve plot with data + fitted curve
      plot_img <- plot_to_html(function() {
        graphics::plot(x, y,
          pch = 19, col = "#2c3e50",
          main = paste0("Learning Curve (", model_type, ")"),
          xlab = "x", ylab = "y"
        )
        fine_x <- seq(min(x), max(x), length.out = 200)
        fine_y <- predict_sigmoidal(fit, fine_x, model_type)
        graphics::lines(fine_x, fine_y, col = "#18bc9c", lwd = 2)
        if (length(predict_x) > 0) {
          graphics::points(predict_x, preds, pch = 4, col = "#e74c3c", cex = 1.2, lwd = 2)
        }
        graphics::legend("bottomright",
          legend = c("Data", "Fitted curve", "Predictions"),
          col = c("#2c3e50", "#18bc9c", "#e74c3c"),
          pch = c(19, NA, 4), lty = c(NA, 1, NA), lwd = c(NA, 2, 2),
          cex = 0.8, bg = "white"
        )
      })

      paste0("<h5>Coefficients</h5>", coef_tbl, plot_img)
    },
    error = function(e) NULL
  )

  tool_result(text, title = paste0("Learning Curve (", model_type, ")"), html = html)
}

# ---- DSM tools ----
#' @keywords internal
parent_dsm_tool <- function(matrix_json) {
  mat <- as_r_input(matrix_json)
  if (!is.matrix(mat)) mat <- as.matrix(as.data.frame(mat))
  result <- parent_dsm(mat)
  result_mat <- result$matrix
  lines <- paste(utils::capture.output(print(result_mat)), collapse = "\n")

  text <- paste0("Parent DSM (Resource-Task Dependencies):\n\n", lines)

  html <- tryCatch(
    {
      paste0(
        "<pre style='font-size:0.85em;background:#f8f9fa;padding:10px;border-radius:6px;'>",
        lines, "</pre>"
      )
    },
    error = function(e) NULL
  )

  tool_result(text, title = "Parent DSM", html = html)
}

#' @keywords internal
grandparent_dsm_tool <- function(s_matrix_json, r_matrix_json) {
  s_mat <- as_r_input(s_matrix_json)
  if (!is.matrix(s_mat)) s_mat <- as.matrix(as.data.frame(s_mat))
  r_mat <- as_r_input(r_matrix_json)
  if (!is.matrix(r_mat)) r_mat <- as.matrix(as.data.frame(r_mat))
  result <- grandparent_dsm(s_mat, r_mat)
  result_mat <- result$matrix
  lines <- paste(utils::capture.output(print(result_mat)), collapse = "\n")

  text <- paste0("Grandparent DSM (Risk-Resource-Task Dependencies):\n\n", lines)

  html <- tryCatch(
    {
      paste0(
        "<pre style='font-size:0.85em;background:#f8f9fa;padding:10px;border-radius:6px;'>",
        lines, "</pre>"
      )
    },
    error = function(e) NULL
  )

  tool_result(text, title = "Grandparent DSM", html = html)
}

# ===========================================================================
# /command framework — deterministic tool calls bypassing the LLM
# ===========================================================================

#' PRA Slash-Command Registry
#'
#' Returns a named list of command definitions. Each command has a title,
#' description, argument specs (name, type, required, help), usage examples,
#' and an executor function that calls the underlying PRA tool wrapper.
#'
#' @return A named list of command definitions.
#' @keywords internal
pra_command_registry <- function() {
  list(
    mcs = list(
      title = "Monte Carlo Simulation",
      description = "Run a Monte Carlo simulation for project cost/duration uncertainty.",
      args = list(
        list(
          name = "n", type = "integer", required = FALSE, default = 10000L,
          help = "Number of simulations (default: 10000)"
        ),
        list(
          name = "tasks", type = "json", required = TRUE, default = NULL,
          help = "JSON array of task distributions, e.g. [{\"type\":\"normal\",\"mean\":10,\"sd\":2}]"
        )
      ),
      examples = c(
        '/mcs tasks=[{"type":"normal","mean":10,"sd":2},{"type":"triangular","a":5,"b":10,"c":15}]',
        '/mcs n=5000 tasks=[{"type":"uniform","min":8,"max":12}]'
      ),
      fn = function(args) {
        n <- if (!is.null(args$n)) as.integer(args$n) else 10000L
        mcs_tool(num_sims = n, task_dists_json = args$tasks)
      }
    ),
    smm = list(
      title = "Second Moment Method",
      description = "Quick analytical estimate of total project mean and variance.",
      args = list(
        list(
          name = "means", type = "json", required = TRUE, default = NULL,
          help = "JSON array of task means, e.g. [10, 12, 8]"
        ),
        list(
          name = "vars", type = "json", required = TRUE, default = NULL,
          help = "JSON array of task variances, e.g. [4, 9, 2]"
        )
      ),
      examples = c(
        "/smm means=[10,12,8] vars=[4,9,2]"
      ),
      fn = function(args) {
        smm_tool(mean_json = args$means, var_json = args$vars)
      }
    ),
    contingency = list(
      title = "Contingency Reserve",
      description = "Calculate contingency reserve from the last MCS result. Run /mcs first.",
      args = list(
        list(
          name = "phigh", type = "number", required = FALSE, default = 0.95,
          help = "High confidence percentile (default: 0.95)"
        ),
        list(
          name = "pbase", type = "number", required = FALSE, default = 0.50,
          help = "Base percentile (default: 0.50)"
        )
      ),
      examples = c(
        "/contingency",
        "/contingency phigh=0.90 pbase=0.50"
      ),
      fn = function(args) {
        ph <- if (!is.null(args$phigh)) as.numeric(args$phigh) else 0.95
        pb <- if (!is.null(args$pbase)) as.numeric(args$pbase) else 0.50
        contingency_tool(phigh = ph, pbase = pb)
      }
    ),
    sensitivity = list(
      title = "Sensitivity Analysis",
      description = "Variance contribution per task (which tasks drive the most risk).",
      args = list(
        list(
          name = "tasks", type = "json", required = TRUE, default = NULL,
          help = "JSON array of task distributions (same format as /mcs)"
        )
      ),
      examples = c(
        '/sensitivity tasks=[{"type":"normal","mean":10,"sd":2},{"type":"triangular","a":5,"b":10,"c":15}]'
      ),
      fn = function(args) {
        sensitivity_tool(task_dists_json = args$tasks)
      }
    ),
    evm = list(
      title = "Earned Value Management",
      description = "Full EVM analysis: PV, EV, AC, variances, indices, and forecasts.",
      args = list(
        list(
          name = "bac", type = "number", required = TRUE, default = NULL,
          help = "Budget at Completion"
        ),
        list(
          name = "schedule", type = "json", required = TRUE, default = NULL,
          help = "JSON array of cumulative planned % complete, e.g. [0.2, 0.4, 0.6, 0.8, 1.0]"
        ),
        list(
          name = "period", type = "integer", required = TRUE, default = NULL,
          help = "Current time period (1-based index into schedule)"
        ),
        list(
          name = "complete", type = "number", required = TRUE, default = NULL,
          help = "Actual percent complete (0-1), e.g. 0.35"
        ),
        list(
          name = "costs", type = "json", required = TRUE, default = NULL,
          help = "JSON array of cumulative actual costs, e.g. [90000, 195000, 310000]"
        )
      ),
      examples = c(
        "/evm bac=500000 schedule=[0.2,0.4,0.6,0.8,1.0] period=3 complete=0.35 costs=[90000,195000,310000]"
      ),
      fn = function(args) {
        evm_analysis_tool(
          bac = as.numeric(args$bac),
          schedule_json = args$schedule,
          time_period = as.integer(args$period),
          actual_per_complete = as.numeric(args$complete),
          actual_costs_json = args$costs
        )
      }
    ),
    risk = list(
      title = "Bayesian Risk (Prior)",
      description = "Calculate prior risk probability from root causes using Bayes' theorem.",
      args = list(
        list(
          name = "causes", type = "json", required = TRUE, default = NULL,
          help = "JSON array of cause probabilities, e.g. [0.3, 0.2]"
        ),
        list(
          name = "given", type = "json", required = TRUE, default = NULL,
          help = "JSON array of P(Risk | Cause), e.g. [0.8, 0.6]"
        ),
        list(
          name = "not_given", type = "json", required = TRUE, default = NULL,
          help = "JSON array of P(Risk | not Cause), e.g. [0.2, 0.4]"
        )
      ),
      examples = c(
        "/risk causes=[0.3,0.2] given=[0.8,0.6] not_given=[0.2,0.4]"
      ),
      fn = function(args) {
        risk_prob_tool(
          cause_probs_json = args$causes,
          risks_given_causes_json = args$given,
          risks_given_not_causes_json = args$not_given
        )
      }
    ),
    risk_post = list(
      title = "Bayesian Risk (Posterior)",
      description = "Update risk probability after observing which causes occurred.",
      args = list(
        list(
          name = "causes", type = "json", required = TRUE, default = NULL,
          help = "JSON array of cause probabilities"
        ),
        list(
          name = "given", type = "json", required = TRUE, default = NULL,
          help = "JSON array of P(Risk | Cause)"
        ),
        list(
          name = "not_given", type = "json", required = TRUE, default = NULL,
          help = "JSON array of P(Risk | not Cause)"
        ),
        list(
          name = "observed", type = "json", required = TRUE, default = NULL,
          help = "JSON array of observations: 1 = occurred, 0 = did not, null = unknown. e.g. [1, null]"
        )
      ),
      examples = c(
        "/risk_post causes=[0.3,0.2] given=[0.8,0.6] not_given=[0.2,0.4] observed=[1,null]"
      ),
      fn = function(args) {
        risk_post_prob_tool(
          cause_probs_json = args$causes,
          risks_given_causes_json = args$given,
          risks_given_not_causes_json = args$not_given,
          observed_causes_json = args$observed
        )
      }
    ),
    learning = list(
      title = "Learning Curve Fit",
      description = "Fit a sigmoidal learning curve (logistic, Gompertz, or Pearl) and predict.",
      args = list(
        list(
          name = "x", type = "json", required = TRUE, default = NULL,
          help = "JSON array of x values (e.g. time periods)"
        ),
        list(
          name = "y", type = "json", required = TRUE, default = NULL,
          help = "JSON array of y values (e.g. cumulative output)"
        ),
        list(
          name = "model", type = "string", required = TRUE, default = NULL,
          help = "Model type: 'logistic', 'gompertz', or 'pearl'"
        ),
        list(
          name = "predict", type = "json", required = FALSE, default = NULL,
          help = "Optional JSON array of x values to predict at"
        )
      ),
      examples = c(
        "/learning x=[1,2,3,4,5] y=[5,15,40,70,90] model=logistic",
        "/learning x=[1,2,3,4,5] y=[5,15,40,70,90] model=gompertz predict=[6,7,8]"
      ),
      fn = function(args) {
        fit_and_predict_sigmoidal_tool(
          x_json = args$x, y_json = args$y,
          model_type = args$model, predict_x_json = args$predict
        )
      }
    ),
    dsm = list(
      title = "Design Structure Matrix",
      description = "Compute parent DSM (resource-task dependencies) from an input matrix.",
      args = list(
        list(
          name = "matrix", type = "json", required = TRUE, default = NULL,
          help = "JSON 2D array, e.g. [[1,0],[1,1]]"
        )
      ),
      examples = c(
        "/dsm matrix=[[1,1,0],[0,1,1],[1,0,1]]"
      ),
      fn = function(args) {
        parent_dsm_tool(matrix_json = args$matrix)
      }
    )
  )
}

#' Format Help for a Single Command
#'
#' @param cmd_name Character command name (without /).
#' @param cmd Command definition from the registry.
#' @return A markdown-formatted help string.
#' @keywords internal
format_command_help <- function(cmd_name, cmd) {
  # Arguments
  arg_lines <- vapply(cmd$args, function(a) {
    req_tag <- if (a$required) {
      " *(required)*"
    } else {
      paste0(" *(default: ", deparse(a$default), ")*")
    }
    paste0("- **", a$name, "**", req_tag, " \u2014 ", a$help)
  }, character(1))

  # Examples
  ex_lines <- paste0("  ", cmd$examples)

  paste0(
    "### /", cmd_name, " \u2014 ", cmd$title, "\n\n",
    cmd$description, "\n\n",
    "**Arguments:**\n", paste(arg_lines, collapse = "\n"), "\n\n",
    "**Examples:**\n", paste(ex_lines, collapse = "\n")
  )
}

#' Format Help Overview of All Commands
#'
#' @return A markdown-formatted overview of all available /commands.
#' @keywords internal
format_help_overview <- function() {
  registry <- pra_command_registry()
  lines <- vapply(names(registry), function(nm) {
    cmd <- registry[[nm]]
    paste0("- **/", nm, "** \u2014 ", cmd$title)
  }, character(1))

  paste0(
    "## PRA Commands\n\n",
    "Type a command to run an analysis directly. ",
    "Type `/help <command>` for detailed usage.\n\n",
    paste(lines, collapse = "\n"), "\n\n",
    "Type `/help` for this overview."
  )
}

#' Route User Input to the Appropriate Handler
#'
#' Classifies user input into one of three modes and returns a structured
#' routing decision. This function encapsulates the three-mode routing
#' architecture used by both the Shiny app and programmatic interfaces.
#'
#' @param input Character string of user input.
#' @return A list with:
#'   \describe{
#'     \item{mode}{Character: `"command"`, `"tool"`, or `"rag"`}
#'     \item{reason}{Character: brief explanation of the routing decision}
#'   }
#'   For `"command"` mode, also includes `command` (the command name).
#'
#' @details
#' Routing logic:
#' \enumerate{
#'   \item Input starting with `/` is routed to the **command** handler
#'         for deterministic execution (no LLM involved).
#'   \item Input containing numerical data patterns (distributions, arrays,
#'         dollar amounts, percentages with surrounding context) is routed
#'         to the **tool** handler where the LLM selects and calls tools.
#'   \item All other input (conceptual questions, explanations) is routed
#'         to the **rag** handler where the LLM answers from the knowledge
#'         base context.
#' }
#'
#' Note: modes `"tool"` and `"rag"` both go through the LLM, but the
#' distinction affects how the query is framed (tool-calling emphasis vs.
#' RAG-context emphasis). The LLM ultimately decides whether to call a
#' tool, but the routing hint improves reliability with smaller models.
#'
#' @examples
#' \dontrun{
#' route_input("/mcs tasks=[...]")
#' # list(mode = "command", reason = "...", command = "mcs")
#'
#' route_input("Simulate 3 tasks: Normal(10,2)...")
#' # list(mode = "tool", reason = "...")
#'
#' route_input("What is earned value?")
#' # list(mode = "rag", reason = "...")
#' }
#'
#' @keywords internal
route_input <- function(input) {
  input <- trimws(input)

  # Mode 1: Slash commands — deterministic
  if (grepl("^/", input)) {
    cmd_name <- sub("^/(\\S+).*", "\\1", input)
    return(list(
      mode = "command",
      reason = paste0("Input starts with /", cmd_name),
      command = tolower(cmd_name)
    ))
  }

  # Heuristic patterns for numerical/computational data
  numerical_patterns <- c(
    "\\b(normal|triangular|uniform)\\s*\\(",          # distribution specs
    "\\[\\s*[\\d.,\\s]+\\]",                          # numeric arrays [1, 2, 3]
    "\\$[\\d,]+",                                      # dollar amounts $500,000
    "\\bBAC\\b.*\\d",                                  # BAC with numbers
    "\\bP\\([A-Z]",                                    # P(C1), P(Risk|...)
    "\\bmean\\s*[=:]\\s*\\d",                          # mean=10 or mean: 10
    "\\bsd\\s*[=:]\\s*\\d",                            # sd=2
    "\\bvariance[s]?\\s*\\[",                          # variances [...]
    "\\bschedule\\s*\\[",                              # schedule [...]
    "\\bcosts?\\s*\\[",                                # costs [...]
    "\\bmatrix\\s*\\[",                                # matrix [...]
    "\\d+\\s*(simulations|iterations)"                 # 10000 simulations
  )

  for (pat in numerical_patterns) {
    if (grepl(pat, input, ignore.case = TRUE, perl = TRUE)) {
      return(list(
        mode = "tool",
        reason = paste0("Numerical data detected (pattern: ", pat, ")")
      ))
    }
  }

  # Mode 3: RAG — conceptual/explanatory
  list(
    mode = "rag",
    reason = "No numerical data or /command detected; routing to RAG"
  )
}

#' Parse key=value argument string with bracket-aware splitting
#'
#' Splits a string like `n=5000 tasks=[{"type": "normal", "mean": 10}]` into
#' a named list `list(n = "5000", tasks = '[{"type": "normal", "mean": 10}]')`.
#' Tracks bracket/brace/quote nesting so that spaces inside `[]`, `{}`, or `""`
#' are not treated as argument separators.
#'
#' @param arg_string Character string of arguments (without the command name).
#' @param known_args Character vector of recognized argument names.
#' @return A named list of parsed argument values (all character strings).
#' @keywords internal
parse_command_args <- function(arg_string, known_args) {
  if (is.null(arg_string) || nchar(trimws(arg_string)) == 0) return(list())

  chars <- strsplit(arg_string, "")[[1]]
  n <- length(chars)

  # State machine: find top-level `key=value` boundaries
  # A key= boundary is a known arg name followed by = at bracket depth 0, outside quotes
  depth_sq <- 0L  # [ ] nesting
  depth_cb <- 0L  # { } nesting
  in_quote <- FALSE

  # First pass: mark each character position with its nesting depth and quote state
  top_level <- logical(n)  # TRUE if character is at depth 0 and not in quotes
  for (i in seq_len(n)) {
    ch <- chars[i]
    if (in_quote) {
      if (ch == '"' && (i == 1L || chars[i - 1L] != "\\")) in_quote <- FALSE
    } else {
      if (ch == '"') {
        in_quote <- TRUE
      } else if (ch == '[') {
        depth_sq <- depth_sq + 1L
      } else if (ch == ']') {
        depth_sq <- max(0L, depth_sq - 1L)
      } else if (ch == '{') {
        depth_cb <- depth_cb + 1L
      } else if (ch == '}') {
        depth_cb <- max(0L, depth_cb - 1L)
      }
    }
    top_level[i] <- !in_quote && depth_sq == 0L && depth_cb == 0L
  }

  # Second pass: find `key=` patterns at top level
  # Sort known_args by length descending to match longest first
  known_args <- known_args[order(nchar(known_args), decreasing = TRUE)]
  boundaries <- list()  # list of list(name, value_start)

  for (nm in known_args) {
    nm_len <- nchar(nm)
    # Search for `nm=` at top level, preceded by start-of-string or whitespace
    for (pos in seq_len(n - nm_len)) {
      if (!top_level[pos]) next
      candidate <- paste0(chars[pos:(pos + nm_len - 1L)], collapse = "")
      if (candidate != nm) next
      eq_pos <- pos + nm_len
      if (eq_pos > n || chars[eq_pos] != "=") next
      # Check preceding character: must be start of string or whitespace
      if (pos > 1 && !grepl("\\s", chars[pos - 1L])) next
      boundaries[[length(boundaries) + 1L]] <- list(
        name = nm, key_start = pos, value_start = eq_pos + 1L
      )
    }
  }

  if (length(boundaries) == 0) return(list())

  # Sort boundaries by position
  starts <- vapply(boundaries, function(b) b$key_start, integer(1))
  boundaries <- boundaries[order(starts)]

  # Extract values: from value_start to just before the next boundary's key_start (or end)
  parsed <- list()
  for (i in seq_along(boundaries)) {
    b <- boundaries[[i]]
    val_start <- b$value_start
    if (i < length(boundaries)) {
      # Value ends just before the whitespace preceding the next key
      val_end <- boundaries[[i + 1L]]$key_start - 1L
      # Trim trailing whitespace
      while (val_end >= val_start && grepl("\\s", chars[val_end])) val_end <- val_end - 1L
    } else {
      val_end <- n
    }
    if (val_start <= val_end) {
      parsed[[b$name]] <- paste0(chars[val_start:val_end], collapse = "")
    } else {
      parsed[[b$name]] <- ""
    }
  }

  parsed
}

#' Parse and Execute a /command
#'
#' Parses user input starting with `/`, validates arguments, and executes
#' the corresponding tool function. Returns a list with `ok` (logical) and
#' `result` (character string — either the tool output or a help/error message).
#'
#' @param input Character string of user input (e.g. "/mcs n=5000 tasks=[...]").
#' @return A list with `ok` and `result`.
#' @keywords internal
execute_command <- function(input) {
  input <- trimws(input)
  registry <- pra_command_registry()

  # Split command name from the rest (everything after first whitespace)
  first_space <- regexpr("\\s", input)
  if (first_space > 0) {
    cmd_token <- substring(input, 1, first_space - 1)
    arg_string <- trimws(substring(input, first_space + 1))
  } else {
    cmd_token <- input
    arg_string <- ""
  }
  cmd_name <- sub("^/", "", tolower(cmd_token))

  # /help or /help <command>

  if (cmd_name == "help") {
    if (nchar(arg_string) > 0) {
      help_target <- sub("^/", "", tolower(trimws(arg_string)))
      if (help_target %in% names(registry)) {
        return(list(ok = TRUE, result = format_command_help(help_target, registry[[help_target]])))
      }
      return(list(ok = FALSE, result = paste0(
        "Unknown command: **", help_target, "**\n\n", format_help_overview()
      )))
    }
    return(list(ok = TRUE, result = format_help_overview()))
  }

  # Unknown command
  if (!cmd_name %in% names(registry)) {
    return(list(ok = FALSE, result = paste0(
      "Unknown command: **/", cmd_name, "**\n\n",
      "Did you mean one of these?\n",
      paste0("- /", names(registry), collapse = "\n"),
      "\n\nType `/help` for a list of all commands."
    )))
  }

  cmd <- registry[[cmd_name]]

  # No args — if command has required args, show help; otherwise execute with defaults
  has_required <- any(vapply(cmd$args, function(a) a$required, logical(1)))
  if (nchar(arg_string) == 0 && has_required) {
    return(list(ok = TRUE, result = format_command_help(cmd_name, cmd)))
  }

  # Parse key=value pairs with bracket-aware splitting.
  # Handles JSON values containing spaces, e.g. tasks=[{"type": "normal", "mean": 10}]
  parsed <- parse_command_args(arg_string, vapply(cmd$args, function(a) a$name, character(1)))

  # Check required arguments
  missing_args <- character(0)
  for (a in cmd$args) {
    if (a$required && is.null(parsed[[a$name]])) {
      missing_args <- c(missing_args, a$name)
    }
  }
  if (length(missing_args) > 0) {
    return(list(ok = FALSE, result = paste0(
      "**Missing required argument(s):** ", paste(missing_args, collapse = ", "), "\n\n",
      format_command_help(cmd_name, cmd)
    )))
  }

  # Execute
  tryCatch(
    {
      result <- cmd$fn(parsed)
      # Extract text from ContentToolResult if needed
      is_ctr <- any(grepl("ContentToolResult", class(result), fixed = TRUE))
      result_text <- if (is_ctr) {
        result@value
      } else if (is.character(result)) {
        result
      } else {
        as.character(result)
      }
      list(
        ok = TRUE, result = result_text,
        rich_result = if (is_ctr) result else NULL
      )
    },
    error = function(e) {
      list(ok = FALSE, result = paste0(
        "**Error running /", cmd_name, ":** ", e$message, "\n\n",
        "Check your inputs and try again. Type `/help ", cmd_name, "` for usage."
      ))
    }
  )
}

#' @srrstats {G5.2a} *Each error message produced by stop() is unique.*

# Helper: extract text value from tool results (either character or ContentToolResult)
tool_text <- function(result) {
  if (is.character(result)) {
    return(result)
  }
  # ContentToolResult S7 object
  tryCatch(result@value, error = function(e) as.character(result))
}

# ============================================================================
# JSON Parsing & Input Coercion
# ============================================================================

test_that("parse_json handles valid JSON", {
  skip_if_not_installed("jsonlite")
  expect_equal(PRA:::parse_json("[1, 2, 3]"), c(1, 2, 3))
  expect_equal(PRA:::parse_json("null"), NULL)
  expect_equal(PRA:::parse_json(""), NULL)
  expect_equal(PRA:::parse_json(NULL), NULL)
})

test_that("parse_json handles common LLM formatting mistakes", {
  skip_if_not_installed("jsonlite")
  # Bare comma-separated numbers (no brackets)
  expect_equal(PRA:::parse_json("0.3, 0.2"), c(0.3, 0.2))
  # Trailing comma
  expect_equal(PRA:::parse_json("[0.3, 0.2,]"), c(0.3, 0.2))
  # Single quotes instead of double
  expect_equal(PRA:::parse_json("['hello', 'world']"), c("hello", "world"))
  # Whitespace padding
  expect_equal(PRA:::parse_json("  [1, 2, 3]  "), c(1, 2, 3))
  # Smart quotes around value (LLM wraps in curly quotes)
  expect_equal(PRA:::parse_json("\u201c0.3\u201d"), "0.3")
})

test_that("as_numeric_input coerces to numeric", {
  skip_if_not_installed("jsonlite")
  expect_equal(PRA:::as_numeric_input("[0.3, 0.2]"), c(0.3, 0.2))
  expect_equal(PRA:::as_numeric_input("0.3, 0.2"), c(0.3, 0.2))
  expect_null(PRA:::as_numeric_input(NULL))
  # Already numeric
  expect_equal(PRA:::as_numeric_input(c(0.3, 0.2)), c(0.3, 0.2))
})

test_that("as_r_input passes R objects through unchanged", {
  expect_equal(PRA:::as_r_input(c(1, 2, 3)), c(1, 2, 3))
  expect_equal(PRA:::as_r_input(list(a = 1)), list(a = 1))
  expect_null(PRA:::as_r_input(NULL))
})

test_that("as_r_input parses JSON strings", {
  skip_if_not_installed("jsonlite")
  expect_equal(PRA:::as_r_input("[1, 2, 3]"), c(1, 2, 3))
  expect_null(PRA:::as_r_input("null"))
  expect_null(PRA:::as_r_input(""))
})

test_that("parse_task_dists converts JSON to list-of-lists", {
  skip_if_not_installed("jsonlite")
  json <- '[{"type":"normal","mean":10,"sd":2},{"type":"uniform","min":8,"max":12}]'
  result <- PRA:::parse_task_dists(json)
  expect_type(result, "list")
  expect_length(result, 2)
  expect_equal(result[[1]]$type, "normal")
  expect_equal(result[[1]]$mean, 10)
  expect_equal(result[[2]]$type, "uniform")
})

test_that("parse_json_param provides contextual errors", {
  skip_if_not_installed("jsonlite")
  expect_error(PRA:::parse_json_param("[invalid", "my_param", "my_tool"), "my_param.*my_tool")
  expect_null(PRA:::parse_json_param("null", "x", "y"))
  expect_null(PRA:::parse_json_param("", "x", "y"))
  expect_null(PRA:::parse_json_param(NULL, "x", "y"))
})

# ============================================================================
# Formatting Helpers
# ============================================================================

test_that("summarize_distribution returns correct structure", {
  x <- rnorm(1000, mean = 30, sd = 5)
  result <- PRA:::summarize_distribution(x)
  expect_type(result, "list")
  expect_true("mean" %in% names(result))
  expect_true("sd" %in% names(result))
  expect_true("percentiles" %in% names(result))
  expect_equal(names(result$percentiles), c("P5", "P10", "P25", "P50", "P75", "P90", "P95"))
  expect_true(result$percentiles$P5 < result$percentiles$P50)
  expect_true(result$percentiles$P50 < result$percentiles$P95)
})

test_that("format_result_table produces readable text", {
  result <- PRA:::format_result_table(Mean = 30.5, SD = 2.1)
  expect_type(result, "character")
  expect_true(grepl("Mean", result))
  expect_true(grepl("30.5", result))
  expect_true(grepl("SD", result))
})

test_that("format_distribution produces readable text", {
  x <- rnorm(1000, mean = 30, sd = 5)
  result <- PRA:::format_distribution(x)
  expect_type(result, "character")
  expect_true(grepl("Summary Statistics", result))
  expect_true(grepl("Percentiles", result))
  expect_true(grepl("P50", result))
  expect_true(grepl("P95", result))
})

test_that("html_result_table produces HTML table", {
  result <- PRA:::html_result_table(Mean = 30.5, SD = 2.1)
  expect_type(result, "character")
  expect_true(grepl("<table", result))
  expect_true(grepl("Mean", result))
  expect_true(grepl("30.5", result))
})

test_that("tool_result returns ContentToolResult with HTML when available", {
  skip_if_not_installed("ellmer")
  result <- PRA:::tool_result("plain text", title = "Test", html = "<p>rich</p>")
  expect_true(inherits(result, "ellmer::ContentToolResult") || is.character(result))
  text <- tool_text(result)
  expect_equal(text, "plain text")
})

test_that("tool_result falls back to plain text without HTML", {
  result <- PRA:::tool_result("plain text only")
  expect_type(result, "character")
  expect_equal(result, "plain text only")
})

# ============================================================================
# Tool Structure & Format Tests
# ============================================================================

test_that("pra_tools requires ellmer", {
  skip_if_not_installed("ellmer")
  tools <- pra_tools()
  expect_type(tools, "list")
  expect_true(length(tools) > 0)
})

test_that("validate_task_dists catches errors", {
  skip_if_not_installed("jsonlite")
  expect_error(PRA:::validate_task_dists(list(), "test"), "non-empty")
  expect_error(PRA:::validate_task_dists(list(list(type = "unknown")), "test"), "invalid type")
  expect_error(PRA:::validate_task_dists(list(list(type = "normal", mean = 10)), "test"), "missing required")
  expect_invisible(PRA:::validate_task_dists(list(list(type = "normal", mean = 10, sd = 2)), "test"))
})

test_that("get_ollama_models returns character vector", {
  result <- PRA:::get_ollama_models()
  expect_type(result, "character")
  expect_true(length(result) > 0)
})

# ============================================================================
# Input Validation Tests
# ============================================================================

test_that("mcs_tool validates num_sims", {
  skip_if_not_installed("jsonlite")
  task_json <- '[{"type":"normal","mean":10,"sd":2}]'
  expect_error(PRA:::mcs_tool(-1, task_json), "positive integer")
  expect_error(PRA:::mcs_tool(0, task_json), "positive integer")
})

test_that("mcs_tool validates distribution types", {
  skip_if_not_installed("jsonlite")
  expect_error(PRA:::mcs_tool(100, '[{"type":"beta","a":1,"b":2}]'), "invalid type")
  expect_error(PRA:::mcs_tool(100, '[{"type":"normal"}]'), "missing required parameters")
  expect_error(PRA:::mcs_tool(100, '[{"type":"triangular","a":5}]'), "missing required parameters")
})

test_that("evm_analysis_tool validates inputs", {
  skip_if_not_installed("jsonlite")
  expect_error(PRA:::evm_analysis_tool(
    bac = -100, schedule_json = "[0.5, 1.0]", time_period = 1,
    actual_per_complete = 0.5, actual_costs_json = "[1000]"
  ), "positive number")
  expect_error(PRA:::evm_analysis_tool(
    bac = 100000, schedule_json = "[0.5, 1.0]", time_period = 0,
    actual_per_complete = 0.5, actual_costs_json = "[1000]"
  ), "positive integer")
  expect_error(PRA:::evm_analysis_tool(
    bac = 100000, schedule_json = "[0.5, 1.0]", time_period = 5,
    actual_per_complete = 0.5, actual_costs_json = "[1000]"
  ), "exceeds schedule length")
  expect_error(PRA:::evm_analysis_tool(
    bac = 100000, schedule_json = "[0.5, 1.0]", time_period = 1,
    actual_per_complete = 1.5, actual_costs_json = "[1000]"
  ), "between 0 and 1")
})

test_that("contingency_tool errors without prior MCS", {
  skip_if_not_installed("jsonlite")
  env <- PRA:::.pra_agent_env
  old <- env$last_mcs
  env$last_mcs <- NULL
  result <- PRA:::contingency_tool(0.95, 0.50)
  expect_true(grepl("Run mcs_tool first", result))
  env$last_mcs <- old
})

# ============================================================================
# Numerical Correctness Tests — Tool wrappers produce correct values
# ============================================================================

test_that("smm_tool computes correct total mean, variance, and std dev", {
  skip_if_not_installed("jsonlite")
  result <- PRA:::smm_tool("[10, 15, 20, 8]", "[4, 9, 16, 2]")
  text <- tool_text(result)
  # total_mean = 10+15+20+8 = 53
  expect_true(grepl("53", text))
  # total_var = 4+9+16+2 = 31
  expect_true(grepl("31", text))
  # total_std = sqrt(31) = 5.5678
  expect_true(grepl("5\\.567[0-9]", text))
})

test_that("evm_analysis_tool computes correct metric values", {
  skip_if_not_installed("jsonlite")
  result <- PRA:::evm_analysis_tool(
    bac = 100000,
    schedule_json = "[0.1, 0.2, 0.4, 0.7, 1.0]",
    time_period = 3,
    actual_per_complete = 0.35,
    actual_costs_json = "[10000, 22000, 37000]",
    cumulative = "true"
  )
  text <- tool_text(result)
  # PV = 100000 * 0.4 = 40,000
  expect_true(grepl("40,000", text))
  # EV = 100000 * 0.35 = 35,000
  expect_true(grepl("35,000", text))
  # AC = 37,000 (cumulative at period 3)
  expect_true(grepl("37,000", text))
  # SV = EV - PV = 35000 - 40000 = -5,000
  expect_true(grepl("-5,000", text))
  # CV = EV - AC = 35000 - 37000 = -2,000
  expect_true(grepl("-2,000", text))
  # SPI = 35000/40000 = 0.875
  expect_true(grepl("0\\.875", text))
  # CPI = 35000/37000 = 0.9459
  expect_true(grepl("0\\.945[0-9]", text))
  # EAC(typical) = BAC/CPI = 100000/0.9459 = 105,714.29
  expect_true(grepl("105,714", text))
  # EAC(atypical) = AC + (BAC - EV) = 37000 + 65000 = 102,000
  expect_true(grepl("102,000", text))
})

test_that("evm_analysis_tool auto-converts integer percentages", {
  skip_if_not_installed("jsonlite")
  # 35 (integer-like) should be auto-converted to 0.35
  result <- PRA:::evm_analysis_tool(
    bac = 100000,
    schedule_json = "[0.5, 1.0]",
    time_period = 1,
    actual_per_complete = 35,
    actual_costs_json = "[30000]",
    cumulative = "true"
  )
  text <- tool_text(result)
  # EV = 100000 * 0.35 = 35,000
  expect_true(grepl("35,000", text))
})

test_that("evm_analysis_tool does not auto-convert decimal values", {
  skip_if_not_installed("jsonlite")
  # 0.55 should NOT be converted (it's already a decimal)
  result <- PRA:::evm_analysis_tool(
    bac = 100000,
    schedule_json = "[0.5, 1.0]",
    time_period = 1,
    actual_per_complete = 0.55,
    actual_costs_json = "[50000]",
    cumulative = "true"
  )
  text <- tool_text(result)
  # EV = 100000 * 0.55 = 55,000
  expect_true(grepl("55,000", text))
})

test_that("individual EVM tools compute correct values", {
  skip_if_not_installed("jsonlite")

  # PV = BAC * schedule[period] = 100000 * 0.4 = 40,000
  result_pv <- PRA:::pv_tool(100000, "[0.1, 0.2, 0.4, 0.7, 1.0]", 3)
  expect_true(grepl("40,000", result_pv))

  # EV = BAC * actual_per_complete = 100000 * 0.35 = 35,000
  result_ev <- PRA:::ev_tool(100000, 0.35)
  expect_true(grepl("35,000", result_ev))

  # AC = cumulative costs at period 3 = 37,000
  result_ac <- PRA:::ac_tool("[10000, 22000, 37000]", 3, "true")
  expect_true(grepl("37,000", result_ac))

  # SV = EV - PV = 35000 - 40000 = -5,000
  result_sv <- PRA:::sv_tool(35000, 40000)
  expect_true(grepl("-5,000", result_sv))

  # CV = EV - AC = 35000 - 37000 = -2,000
  result_cv <- PRA:::cv_tool(35000, 37000)
  expect_true(grepl("-2,000", result_cv))

  # SPI = EV/PV = 35000/40000 = 0.875
  result_spi <- PRA:::spi_tool(35000, 40000)
  expect_true(grepl("0\\.875", result_spi))

  # CPI = EV/AC = 35000/37000 = 0.9459...
  result_cpi <- PRA:::cpi_tool(35000, 37000)
  expect_true(grepl("0\\.945[0-9]", result_cpi))

  # EAC(atypical) = AC + (BAC - EV) = 37000 + 65000 = 102,000
  result_eac <- PRA:::eac_tool(100000, "atypical", ac = 37000, ev = 35000)
  expect_true(grepl("102,000", result_eac))

  # ETC = (BAC - EV)/CPI = 65000/0.9459 = 68,714.29
  result_etc <- PRA:::etc_tool(100000, 35000, 0.9459)
  expect_true(grepl("68,71[0-9]", result_etc))

  # VAC = BAC - EAC = 100000 - 110000 = -10,000
  result_vac <- PRA:::vac_tool(100000, 110000)
  expect_true(grepl("-10,000", result_vac))

  # TCPI = (BAC - EV)/(BAC - AC) = 65000/63000 = 1.0317...
  result_tcpi <- PRA:::tcpi_tool(100000, 35000, 37000, "bac")
  expect_true(grepl("1\\.031[0-9]", result_tcpi))
})

test_that("risk_prob_tool computes correct prior probability", {
  skip_if_not_installed("jsonlite")
  result <- PRA:::risk_prob_tool("[0.3, 0.2]", "[0.8, 0.6]", "[0.2, 0.4]")
  text <- tool_text(result)
  # P(R) = (0.8*0.3 + 0.2*0.7) + (0.6*0.2 + 0.4*0.8) = 0.38 + 0.44 = 0.82
  expect_true(grepl("0\\.82", text))
  expect_true(grepl("82", text)) # 82%
})

test_that("risk_post_prob_tool computes correct posterior probability", {
  skip_if_not_installed("jsonlite")
  result <- PRA:::risk_post_prob_tool("[0.3, 0.2]", "[0.8, 0.6]", "[0.2, 0.4]", "[1, null]")
  text <- tool_text(result)
  # Cause 1 observed (TRUE), Cause 2 unknown (NA)
  # numerator = 0.8 * 0.3 = 0.24
  # denominator = 0.8*0.3 + 0.2*0.7 = 0.38
  # posterior = 0.24/0.38 = 0.631578...
  expect_true(grepl("0\\.631[0-9]", text))
  expect_true(grepl("63\\.1[0-9]", text)) # 63.16%
  expect_true(grepl("Occurred", text))
  expect_true(grepl("Unknown", text))
})

test_that("risk_post_prob_tool with all causes observed", {
  skip_if_not_installed("jsonlite")
  result <- PRA:::risk_post_prob_tool("[0.3, 0.2]", "[0.8, 0.6]", "[0.2, 0.4]", "[1, 0]")
  text <- tool_text(result)
  # Cause 1 observed=1: num *= 0.8*0.3=0.24, denom *= 0.38
  # Cause 2 observed=0: num *= 0.4*0.8=0.32, denom *= 0.44
  # posterior = (0.24*0.32) / (0.38*0.44) = 0.0768 / 0.1672 = 0.4593...
  expect_true(grepl("0\\.459[0-9]", text))
  expect_true(grepl("Occurred", text))
  expect_true(grepl("Did not occur", text))
})

test_that("sensitivity_tool returns 1.0 for independent tasks", {
  skip_if_not_installed("jsonlite")
  # For independent tasks, sensitivity = 1 for all tasks
  task_json <- '[{"type":"normal","mean":10,"sd":2},{"type":"triangular","a":5,"b":15,"c":10},{"type":"uniform","min":8,"max":12}]'
  result <- PRA:::sensitivity_tool(task_json)
  text <- tool_text(result)
  expect_true(grepl("Task 1", text))
  expect_true(grepl("Task 2", text))
  expect_true(grepl("Task 3", text))
  # All sensitivities should be 1.0 for independent tasks
  # Extract the numeric values — each "1" or "1.0000" after task names
  lines <- strsplit(text, "\n")[[1]]
  task_lines <- lines[grepl("Task [0-9]", lines)]
  for (line in task_lines) {
    val <- as.numeric(trimws(sub(".*Task [0-9]+\\s+", "", line)))
    expect_equal(val, 1.0, tolerance = 0.001)
  }
})

test_that("parent_dsm_tool computes correct S * t(S)", {
  skip_if_not_installed("jsonlite")
  # S = [[1,0,1],[0,1,0],[1,0,1]]
  # S * t(S) = [[2,0,2],[0,1,0],[2,0,2]]
  result <- PRA:::parent_dsm_tool("[[1,0,1],[0,1,0],[1,0,1]]")
  text <- tool_text(result)
  expect_true(grepl("Parent DSM", text))
  # Check diagonal and off-diagonal values
  expect_true(grepl("2", text)) # diagonal elements [1,1] and [3,3]
  expect_true(grepl("1", text)) # diagonal element [2,2]
})

test_that("grandparent_dsm_tool computes correct result", {
  skip_if_not_installed("jsonlite")
  # S = I(2), R = I(2) => grandparent = S * R * t(R) * t(S) = I(2)
  result <- PRA:::grandparent_dsm_tool("[[1,0],[0,1]]", "[[1,0],[0,1]]")
  text <- tool_text(result)
  expect_true(grepl("Grandparent DSM", text))
})

# ============================================================================
# MCS Numerical Tests (stochastic, use tolerances)
# ============================================================================

test_that("mcs_tool produces statistically correct results", {
  skip_if_not_installed("jsonlite")
  set.seed(42)
  # Single normal task: Normal(100, 10)
  result <- PRA:::mcs_tool(50000, '[{"type":"normal","mean":100,"sd":10}]')
  text <- tool_text(result)
  # Mean should be close to 100
  s <- PRA:::summarize_distribution(PRA:::.pra_agent_env$last_mcs$total_distribution)
  expect_equal(s$mean, 100, tolerance = 1)
  expect_equal(s$sd, 10, tolerance = 1)
})

test_that("mcs_tool with multiple tasks sums correctly", {
  skip_if_not_installed("jsonlite")
  set.seed(42)
  # Normal(10,2) + Uniform(8,12): expected mean = 10 + 10 = 20
  task_json <- '[{"type":"normal","mean":10,"sd":2},{"type":"uniform","min":8,"max":12}]'
  PRA:::mcs_tool(50000, task_json)
  s <- PRA:::summarize_distribution(PRA:::.pra_agent_env$last_mcs$total_distribution)
  expect_equal(s$mean, 20, tolerance = 0.5)
  # Uniform(8,12) variance = (12-8)^2/12 = 1.333, Normal var = 4, total var = 5.333
  expect_equal(s$sd^2, 5.333, tolerance = 1)
})

test_that("contingency_tool computes correct reserve from MCS", {
  skip_if_not_installed("jsonlite")
  set.seed(42)
  task_json <- '[{"type":"normal","mean":100,"sd":10}]'
  PRA:::mcs_tool(50000, task_json)

  result <- PRA:::contingency_tool(0.95, 0.50)
  text <- tool_text(result)
  expect_true(grepl("Contingency", text))
  expect_true(grepl("P95", text))
  expect_true(grepl("P50", text))

  # For Normal(100,10): P95 - P50 ≈ 1.645*10 = 16.45
  dist <- PRA:::.pra_agent_env$last_mcs$total_distribution
  expected_contingency <- unname(quantile(dist, 0.95) - quantile(dist, 0.50))
  expect_true(grepl(format(round(expected_contingency, 4), big.mark = ","), text))
})

# ============================================================================
# Multi-Tool Chain Tests — Verify chaining state
# ============================================================================

test_that("MCS -> contingency chain produces consistent results", {
  skip_if_not_installed("jsonlite")
  set.seed(123)
  task_json <- '[{"type":"normal","mean":50,"sd":5},{"type":"triangular","a":10,"b":20,"c":30}]'

  # Step 1: Run MCS
  mcs_result <- PRA:::mcs_tool(10000, task_json)
  mcs_text <- tool_text(mcs_result)
  expect_true(grepl("Monte Carlo Simulation", mcs_text))

  # Verify MCS stored in environment
  expect_false(is.null(PRA:::.pra_agent_env$last_mcs))
  dist <- PRA:::.pra_agent_env$last_mcs$total_distribution

  # Step 2: Run contingency
  cont_result <- PRA:::contingency_tool(0.95, 0.50)
  cont_text <- tool_text(cont_result)

  # Verify contingency matches the stored MCS distribution
  p95 <- unname(quantile(dist, 0.95))
  p50 <- unname(quantile(dist, 0.50))
  expected_reserve <- p95 - p50
  expect_true(grepl(format(round(expected_reserve, 4), big.mark = ","), cont_text))
})

test_that("MCS -> sensitivity chain uses same distributions", {
  skip_if_not_installed("jsonlite")
  task_json <- '[{"type":"normal","mean":50,"sd":5},{"type":"uniform","min":10,"max":30}]'

  # Step 1: MCS
  PRA:::mcs_tool(5000, task_json)
  expect_false(is.null(PRA:::.pra_agent_env$last_mcs))

  # Step 2: Sensitivity on same distributions
  sens_result <- PRA:::sensitivity_tool(task_json)
  sens_text <- tool_text(sens_result)
  expect_true(grepl("Task 1", sens_text))
  expect_true(grepl("Task 2", sens_text))
})

test_that("cost_pdf -> cost_post_pdf chain works", {
  skip_if_not_installed("jsonlite")
  set.seed(42)

  # Step 1: Prior cost distribution
  prior_result <- PRA:::cost_pdf_tool(10000, "[0.3, 0.2]", "[50000, 30000]", "[10000, 5000]", 100000)
  prior_text <- tool_text(prior_result)
  expect_true(grepl("Prior Cost Distribution", prior_text))
  expect_false(is.null(PRA:::.pra_agent_env$last_cost_pdf))

  # Step 2: Posterior cost (risk 1 occurred, risk 2 unknown)
  post_result <- PRA:::cost_post_pdf_tool(10000, "[true, null]", "[50000, 30000]", "[10000, 5000]", 100000)
  post_text <- tool_text(post_result)
  expect_true(grepl("Posterior Cost Distribution", post_text))

  # Posterior mean should be higher than prior mean (risk 1 definitely occurred)
  prior_dist <- PRA:::.pra_agent_env$last_cost_pdf
  post_dist <- PRA:::.pra_agent_env$last_cost_post_pdf
  expect_true(mean(post_dist) > mean(prior_dist))
})

# ============================================================================
# LLM-Style Input Pipeline Tests — Simulate realistic LLM tool calls
# ============================================================================

test_that("pipeline: LLM sends string-wrapped numbers for EVM", {
  skip_if_not_installed("jsonlite")
  # LLMs often send numbers as strings in JSON
  result <- PRA:::evm_analysis_tool(
    bac = "100000",
    schedule_json = "[0.25, 0.5, 0.75, 1.0]",
    time_period = "2",
    actual_per_complete = "0.40",
    actual_costs_json = "[22000, 48000]",
    cumulative = "true"
  )
  text <- tool_text(result)
  # PV = 100000 * 0.5 = 50,000
  expect_true(grepl("50,000", text))
  # EV = 100000 * 0.40 = 40,000
  expect_true(grepl("40,000", text))
  # AC = 48,000
  expect_true(grepl("48,000", text))
  # SV = 40000 - 50000 = -10,000
  expect_true(grepl("-10,000", text))
})

test_that("pipeline: LLM sends integer percentage for EVM", {
  skip_if_not_installed("jsonlite")
  result <- PRA:::evm_analysis_tool(
    bac = 500000,
    schedule_json = "[0.2, 0.4, 0.6, 0.8, 1.0]",
    time_period = 3,
    actual_per_complete = 55, # 55% as integer
    actual_costs_json = "[95000, 200000, 310000]",
    cumulative = "true"
  )
  text <- tool_text(result)
  # EV = 500000 * 0.55 = 275,000
  expect_true(grepl("275,000", text))
  # PV = 500000 * 0.6 = 300,000
  expect_true(grepl("300,000", text))
  # SV = 275000 - 300000 = -25,000
  expect_true(grepl("-25,000", text))
})

test_that("pipeline: LLM sends bare numbers for risk_prob", {
  skip_if_not_installed("jsonlite")
  # LLM sends "0.3, 0.2" instead of "[0.3, 0.2]"
  result <- PRA:::risk_prob_tool("0.3, 0.2", "0.8, 0.6", "0.2, 0.4")
  text <- tool_text(result)
  expect_true(grepl("0\\.82", text))
})

test_that("pipeline: LLM sends trailing-comma JSON for SMM", {
  skip_if_not_installed("jsonlite")
  result <- PRA:::smm_tool("[10, 15, 20, 8,]", "[4, 9, 16, 2,]")
  text <- tool_text(result)
  expect_true(grepl("53", text)) # total mean
  expect_true(grepl("31", text)) # total variance
})

test_that("pipeline: LLM sends R vectors directly (programmatic use)", {
  # Users calling tools from R code, not via LLM
  result <- PRA:::smm_tool(
    mean_json = c(10, 12, 8),
    var_json = c(4, 9, 2)
  )
  text <- tool_text(result)
  # total_mean = 30, total_var = 15, total_std = sqrt(15) = 3.873 (rounded)
  expect_true(grepl("30", text))
  expect_true(grepl("15", text))
  expect_true(grepl("3\\.873", text))
})

test_that("pipeline: LLM sends R lists for MCS", {
  result <- PRA:::mcs_tool(
    num_sims = 1000,
    task_dists_json = list(
      list(type = "normal", mean = 10, sd = 2),
      list(type = "uniform", min = 8, max = 12)
    )
  )
  text <- tool_text(result)
  expect_true(grepl("Monte Carlo Simulation", text))
})

test_that("pipeline: LLM sends R vectors for risk_prob", {
  result <- PRA:::risk_prob_tool(
    cause_probs_json = c(0.3, 0.2),
    risks_given_causes_json = c(0.8, 0.6),
    risks_given_not_causes_json = c(0.2, 0.4)
  )
  text <- tool_text(result)
  expect_true(grepl("0\\.82", text))
})

test_that("pipeline: LLM sends R vectors for EVM", {
  result <- PRA:::evm_analysis_tool(
    bac = 100000,
    schedule_json = c(0.1, 0.2, 0.4, 0.7, 1.0),
    time_period = 3,
    actual_per_complete = 0.35,
    actual_costs_json = c(10000, 22000, 37000),
    cumulative = TRUE
  )
  text <- tool_text(result)
  expect_true(grepl("Earned Value Management", text))
  expect_true(grepl("40,000", text)) # PV
  expect_true(grepl("-5,000", text)) # SV
})

test_that("pipeline: LLM sends R matrix for DSM", {
  result <- PRA:::parent_dsm_tool(matrix(c(1, 0, 1, 0, 1, 0, 1, 0, 1), nrow = 3))
  text <- tool_text(result)
  expect_true(grepl("Parent DSM", text))
})

test_that("pipeline: LLM sends R lists for sensitivity", {
  result <- PRA:::sensitivity_tool(
    task_dists_json = list(
      list(type = "normal", mean = 10, sd = 2),
      list(type = "triangular", a = 5, b = 15, c = 10)
    )
  )
  text <- tool_text(result)
  expect_true(grepl("Sensitivity", text))
})

# ============================================================================
# Learning Curve Numerical Tests
# ============================================================================

test_that("fit_and_predict_sigmoidal_tool fits logistic model correctly", {
  skip_if_not_installed("jsonlite")
  x_json <- "[1,2,3,4,5,6,7,8,9,10]"
  y_json <- "[5,15,40,60,70,75,80,85,90,95]"
  result <- PRA:::fit_and_predict_sigmoidal_tool(x_json, y_json, "logistic")
  text <- tool_text(result)
  expect_true(grepl("Learning Curve Fit", text))
  expect_true(grepl("logistic", text))
  expect_true(grepl("Coefficients", text))
  expect_true(grepl("Predictions", text))
})

test_that("fit_and_predict_sigmoidal_tool predictions at new x values", {
  skip_if_not_installed("jsonlite")
  x_json <- "[1,2,3,4,5,6,7,8,9,10]"
  y_json <- "[5,15,40,60,70,75,80,85,90,95]"
  result <- PRA:::fit_and_predict_sigmoidal_tool(x_json, y_json, "logistic", predict_x_json = "[12, 15]")
  text <- tool_text(result)
  # Predictions at x=12 and x=15 should exist
  expect_true(grepl("12", text))
  expect_true(grepl("15", text))
})

# ============================================================================
# Cost PDF Numerical Tests
# ============================================================================

test_that("cost_pdf_tool baseline cost is reflected in output", {
  skip_if_not_installed("jsonlite")
  set.seed(42)
  # With base_cost = 100000, mean should be >= 100000
  result <- PRA:::cost_pdf_tool(10000, "[0.3, 0.2]", "[50000, 30000]", "[10000, 5000]", 100000)
  text <- tool_text(result)
  expect_true(grepl("100,000", text))

  s <- PRA:::summarize_distribution(PRA:::.pra_agent_env$last_cost_pdf)
  # Mean should be > base cost (risks add cost)
  expect_true(s$mean > 100000)
  # Expected additional cost = 0.3*50000 + 0.2*30000 = 21000
  # So mean should be around 121000
  expect_equal(s$mean, 121000, tolerance = 5000)
})

test_that("cost_post_pdf_tool with known risk produces higher mean", {
  skip_if_not_installed("jsonlite")
  set.seed(42)
  # Risk 1 occurred (TRUE), risk 2 did not (FALSE)
  result <- PRA:::cost_post_pdf_tool(10000, "[true, false]", "[50000, 30000]", "[10000, 5000]", 100000)
  text <- tool_text(result)

  s <- PRA:::summarize_distribution(PRA:::.pra_agent_env$last_cost_post_pdf)
  # Only risk 1 impact: mean should be around 100000 + 50000 = 150000
  expect_equal(s$mean, 150000, tolerance = 2000)
})

# ============================================================================
# MCS with Correlation
# ============================================================================

test_that("mcs_tool with correlation matrix produces valid results", {
  skip_if_not_installed("jsonlite")
  set.seed(42)
  task_json <- '[{"type":"normal","mean":10,"sd":2},{"type":"normal","mean":12,"sd":3}]'
  cor_json <- "[[1,0.5],[0.5,1]]"
  result <- PRA:::mcs_tool(10000, task_json, cor_json)
  text <- tool_text(result)
  expect_true(grepl("Monte Carlo Simulation", text))

  # Mean should still be 22 (correlation doesn't affect mean)
  s <- PRA:::summarize_distribution(PRA:::.pra_agent_env$last_mcs$total_distribution)
  expect_equal(s$mean, 22, tolerance = 0.5)
  # But variance should be higher than independent case (4 + 9 = 13)
  # With cor=0.5: var = 4 + 9 + 2*0.5*2*3 = 19
  expect_true(s$sd^2 > 14) # Should be around 19
})

# ============================================================================
# Stores / Environment State Tests
# ============================================================================

test_that("mcs_tool stores result for downstream tools", {
  skip_if_not_installed("jsonlite")
  task_json <- '[{"type":"normal","mean":10,"sd":2},{"type":"uniform","min":8,"max":12}]'
  PRA:::mcs_tool(1000, task_json)
  expect_false(is.null(PRA:::.pra_agent_env$last_mcs))
  expect_s3_class(PRA:::.pra_agent_env$last_mcs, "mcs")
})

test_that("cost_pdf_tool stores result for chaining", {
  skip_if_not_installed("jsonlite")
  PRA:::cost_pdf_tool(1000, "[0.3, 0.2]", "[50000, 30000]", "[10000, 5000]", 100000)
  expect_false(is.null(PRA:::.pra_agent_env$last_cost_pdf))
})

# ============================================================================
# /Command Framework Tests
# ============================================================================

test_that("command registry returns all 9 commands", {
  registry <- PRA:::pra_command_registry()
  expect_type(registry, "list")
  expect_length(registry, 9)
  expected <- c(
    "mcs", "smm", "contingency", "sensitivity", "evm",
    "risk", "risk_post", "learning", "dsm"
  )
  expect_equal(sort(names(registry)), sort(expected))
})

test_that("each command has required fields", {
  registry <- PRA:::pra_command_registry()
  for (nm in names(registry)) {
    cmd <- registry[[nm]]
    expect_true(!is.null(cmd$title), info = paste(nm, "missing title"))
    expect_true(!is.null(cmd$description), info = paste(nm, "missing description"))
    expect_true(!is.null(cmd$args), info = paste(nm, "missing args"))
    expect_true(!is.null(cmd$examples), info = paste(nm, "missing examples"))
    expect_true(is.function(cmd$fn), info = paste(nm, "missing fn"))
  }
})

test_that("/help returns overview of all commands", {
  r <- PRA:::execute_command("/help")
  expect_true(r$ok)
  expect_true(grepl("PRA Commands", r$result))
  expect_true(grepl("/mcs", r$result, fixed = TRUE))
  expect_true(grepl("/evm", r$result, fixed = TRUE))
  expect_true(grepl("/risk", r$result, fixed = TRUE))
})

test_that("/help <command> returns detailed help", {
  r <- PRA:::execute_command("/help mcs")
  expect_true(r$ok)
  expect_true(grepl("Monte Carlo Simulation", r$result))
  expect_true(grepl("tasks", r$result))
  expect_true(grepl("Examples", r$result))
})

test_that("unknown command returns suggestions", {
  r <- PRA:::execute_command("/foobar")
  expect_false(r$ok)
  expect_true(grepl("Unknown command", r$result))
  expect_true(grepl("/mcs", r$result, fixed = TRUE))
})

test_that("command with no args shows help when args are required", {
  r <- PRA:::execute_command("/evm")
  expect_true(r$ok)
  expect_true(grepl("bac", r$result))
  expect_true(grepl("required", r$result, ignore.case = TRUE))
})

test_that("missing required args returns guidance", {
  r <- PRA:::execute_command("/risk causes=[0.3]")
  expect_false(r$ok)
  expect_true(grepl("Missing required", r$result))
  expect_true(grepl("given", r$result))
  expect_true(grepl("not_given", r$result))
})

test_that("/smm executes correctly", {
  skip_if_not_installed("jsonlite")
  r <- PRA:::execute_command("/smm means=[10,20,30] vars=[4,9,16]")
  expect_true(r$ok)
  expect_true(grepl("60", r$result)) # total mean
  expect_true(grepl("29", r$result)) # total variance
})

test_that("/risk executes correctly with numerical verification", {
  skip_if_not_installed("jsonlite")
  r <- PRA:::execute_command("/risk causes=[0.3,0.2] given=[0.8,0.6] not_given=[0.2,0.4]")
  expect_true(r$ok)
  expect_true(grepl("0\\.82", r$result))
})

test_that("/evm executes correctly with numerical verification", {
  skip_if_not_installed("jsonlite")
  r <- PRA:::execute_command("/evm bac=500000 schedule=[0.2,0.4,0.6,0.8,1.0] period=3 complete=0.35 costs=[90000,195000,310000]")
  expect_true(r$ok)
  expect_true(grepl("300,000", r$result)) # PV
  expect_true(grepl("175,000", r$result)) # EV
  expect_true(grepl("-125,000", r$result)) # SV
  expect_true(grepl("0\\.5645", r$result)) # CPI
})

test_that("/mcs executes and returns rich result", {
  skip_if_not_installed("jsonlite")
  r <- PRA:::execute_command('/mcs n=1000 tasks=[{"type":"normal","mean":10,"sd":2}]')
  expect_true(r$ok)
  expect_true(grepl("Monte Carlo", r$result))
  expect_false(is.null(r$rich_result))
})

test_that("/contingency executes with defaults after /mcs", {
  skip_if_not_installed("jsonlite")
  # Run MCS first to populate last_mcs
  PRA:::execute_command('/mcs n=1000 tasks=[{"type":"normal","mean":10,"sd":2}]')
  r <- PRA:::execute_command("/contingency")
  expect_true(r$ok)
  expect_true(grepl("Contingency", r$result))
})

test_that("error in tool execution returns friendly message", {
  skip_if_not_installed("jsonlite")
  # Invalid JSON for tasks
  r <- PRA:::execute_command("/mcs tasks=not_valid_json")
  expect_false(r$ok)
  expect_true(grepl("Error running /mcs", r$result))
  expect_true(grepl("/help mcs", r$result, fixed = TRUE))
})

test_that("format_command_help produces valid markdown", {
  registry <- PRA:::pra_command_registry()
  help_text <- PRA:::format_command_help("mcs", registry$mcs)
  expect_true(grepl("### /mcs", help_text))
  expect_true(grepl("Arguments", help_text))
  expect_true(grepl("Examples", help_text))
})

test_that("format_help_overview lists all commands", {
  overview <- PRA:::format_help_overview()
  registry <- PRA:::pra_command_registry()
  for (nm in names(registry)) {
    expect_true(grepl(paste0("/", nm), overview, fixed = TRUE),
      info = paste("Missing", nm, "in overview")
    )
  }
})

# ============================================================================
# Parser Edge Cases: bracket-aware argument parsing
# ============================================================================

test_that("parse_command_args handles JSON with spaces", {
  skip_if_not_installed("jsonlite")
  parsed <- PRA:::parse_command_args(
    'tasks=[{"type": "normal", "mean": 10, "sd": 2}]',
    "tasks"
  )
  expect_equal(names(parsed), "tasks")
  # Value should contain the full JSON including spaces
  expect_true(grepl("normal", parsed$tasks))
  expect_true(grepl("mean", parsed$tasks))
  # Should parse as valid JSON
  obj <- jsonlite::fromJSON(parsed$tasks)
  expect_equal(obj$type, "normal")
  expect_equal(obj$mean, 10)
})

test_that("parse_command_args handles multiple args with JSON spaces", {
  parsed <- PRA:::parse_command_args(
    'n=5000 tasks=[{"type": "normal", "mean": 10, "sd": 2}]',
    c("n", "tasks")
  )
  expect_equal(parsed$n, "5000")
  expect_true(grepl("normal", parsed$tasks))
})

test_that("parse_command_args handles nested brackets", {
  parsed <- PRA:::parse_command_args(
    'matrix=[[1, 0, 1], [0, 1, 0], [1, 0, 1]]',
    "matrix"
  )
  expect_true(grepl("\\[\\[1", parsed$matrix))
})

test_that("parse_command_args handles multiple JSON args with spaces", {
  parsed <- PRA:::parse_command_args(
    'causes=[0.3, 0.2] given=[0.8, 0.6] not_given=[0.2, 0.4]',
    c("causes", "given", "not_given")
  )
  expect_equal(length(parsed), 3)
  expect_true(grepl("0.3", parsed$causes))
  expect_true(grepl("0.8", parsed$given))
  expect_true(grepl("0.2", parsed$not_given))
})

test_that("parse_command_args handles empty string", {
  parsed <- PRA:::parse_command_args("", c("x", "y"))
  expect_equal(length(parsed), 0)
})

test_that("parse_command_args handles NULL", {
  parsed <- PRA:::parse_command_args(NULL, c("x"))
  expect_equal(length(parsed), 0)
})

test_that("parse_command_args handles quoted strings in JSON", {
  parsed <- PRA:::parse_command_args(
    'tasks=[{"type": "normal", "mean": 10}] n=500',
    c("tasks", "n")
  )
  expect_equal(parsed$n, "500")
  expect_true(grepl('"type"', parsed$tasks))
})

test_that("parse_command_args handles args appearing after JSON", {
  # n comes after the JSON arg
  parsed <- PRA:::parse_command_args(
    'tasks=[{"type": "uniform", "min": 5, "max": 15}] n=1000',
    c("n", "tasks")
  )
  expect_equal(parsed$n, "1000")
  expect_true(grepl("uniform", parsed$tasks))
})

# ============================================================================
# /command execution with spaces in JSON (integration)
# ============================================================================

test_that("/mcs works with spaces in JSON values", {
  skip_if_not_installed("jsonlite")
  r <- PRA:::execute_command(
    '/mcs tasks=[{"type": "normal", "mean": 10, "sd": 2}]'
  )
  expect_true(r$ok)
  expect_true(grepl("Monte Carlo", r$result))
})

test_that("/mcs works with multiple tasks and spaces", {
  skip_if_not_installed("jsonlite")
  r <- PRA:::execute_command(
    '/mcs n=1000 tasks=[{"type": "normal", "mean": 10, "sd": 2}, {"type": "uniform", "min": 5, "max": 15}]'
  )
  expect_true(r$ok)
  expect_true(grepl("Monte Carlo", r$result))
})

test_that("/evm works with spaces in JSON arrays", {
  skip_if_not_installed("jsonlite")
  r <- PRA:::execute_command(
    "/evm bac=500000 schedule=[0.2, 0.4, 0.6, 0.8, 1.0] period=3 complete=0.35 costs=[90000, 195000, 310000]"
  )
  expect_true(r$ok)
  expect_true(grepl("300,000", r$result))
})

test_that("/risk works with spaces in arrays", {
  skip_if_not_installed("jsonlite")
  r <- PRA:::execute_command(
    "/risk causes=[0.3, 0.2] given=[0.8, 0.6] not_given=[0.2, 0.4]"
  )
  expect_true(r$ok)
  expect_true(grepl("0\\.82", r$result))
})

test_that("/smm works with spaces in arrays", {
  skip_if_not_installed("jsonlite")
  r <- PRA:::execute_command("/smm means=[10, 12, 8] vars=[4, 9, 2]")
  expect_true(r$ok)
  expect_true(grepl("30", r$result))
})

test_that("/dsm works with nested brackets and spaces", {
  skip_if_not_installed("jsonlite")
  r <- PRA:::execute_command("/dsm matrix=[[1, 1, 0], [0, 1, 1], [1, 0, 1]]")
  expect_true(r$ok)
  expect_true(grepl("Parent DSM", r$result))
})

test_that("/sensitivity works with spaces in task JSON", {
  skip_if_not_installed("jsonlite")
  r <- PRA:::execute_command(
    '/sensitivity tasks=[{"type": "normal", "mean": 10, "sd": 2}, {"type": "triangular", "a": 5, "b": 10, "c": 15}]'
  )
  expect_true(r$ok)
  expect_true(grepl("Sensitivity", r$result))
})

# ============================================================================
# Multi-turn state / chaining tests
# ============================================================================

test_that("contingency chains correctly after mcs via /commands", {
  skip_if_not_installed("jsonlite")
  env <- PRA:::.pra_agent_env
  old_mcs <- env$last_mcs
  env$last_mcs <- NULL

  # Run MCS
  r1 <- PRA:::execute_command('/mcs n=1000 tasks=[{"type": "normal", "mean": 20, "sd": 3}]')
  expect_true(r1$ok)
  expect_false(is.null(env$last_mcs))

  # Chain to contingency
  r2 <- PRA:::execute_command("/contingency phigh=0.90 pbase=0.50")
  expect_true(r2$ok)
  expect_true(grepl("Contingency", r2$result))
  expect_true(grepl("P90", r2$result))
  expect_true(grepl("P50", r2$result))

  env$last_mcs <- old_mcs
})

test_that("contingency returns guidance without prior mcs", {
  skip_if_not_installed("jsonlite")
  env <- PRA:::.pra_agent_env
  old_mcs <- env$last_mcs
  env$last_mcs <- NULL

  r <- PRA:::execute_command("/contingency")
  # Tool returns a friendly error message (not a crash)
  expect_true(grepl("mcs_tool", r$result, ignore.case = TRUE))

  env$last_mcs <- old_mcs
})

test_that("cost_pdf chains to cost_post_pdf via tool wrappers", {
  skip_if_not_installed("jsonlite")
  env <- PRA:::.pra_agent_env
  old_cost <- env$last_cost_pdf
  env$last_cost_pdf <- NULL

  # Prior cost distribution
  PRA:::cost_pdf_tool(1000, "[0.3, 0.2]", "[50000, 30000]", "[10000, 5000]", 100000)
  expect_false(is.null(env$last_cost_pdf))

  # Posterior cost distribution
  result <- PRA:::cost_post_pdf_tool(1000, "[1, 0]", "[50000, 30000]", "[10000, 5000]", 100000)
  text <- tool_text(result)
  expect_true(grepl("Posterior", text))

  env$last_cost_pdf <- old_cost
})

# ============================================================================
# Routing logic tests
# ============================================================================

test_that("execute_command routes /help correctly", {
  r <- PRA:::execute_command("/help")
  expect_true(r$ok)
  expect_true(grepl("PRA Commands", r$result))
})

test_that("execute_command routes /help <cmd> correctly", {
  r <- PRA:::execute_command("/help evm")
  expect_true(r$ok)
  expect_true(grepl("Earned Value", r$result))
  expect_true(grepl("bac", r$result))
})

test_that("execute_command routes /help <unknown> correctly", {
  r <- PRA:::execute_command("/help foobar")
  expect_false(r$ok)
  expect_true(grepl("Unknown command", r$result))
  expect_true(grepl("PRA Commands", r$result))
})

test_that("execute_command handles leading/trailing whitespace", {
  r <- PRA:::execute_command("  /smm means=[10,20] vars=[4,9]  ")
  expect_true(r$ok)
  expect_true(grepl("30", r$result))
})

test_that("execute_command is case-insensitive for command names", {
  r <- PRA:::execute_command("/SMM means=[10,20] vars=[4,9]")
  expect_true(r$ok)
  expect_true(grepl("30", r$result))
})

test_that("all 9 commands have consistent registry structure", {
  registry <- PRA:::pra_command_registry()
  for (nm in names(registry)) {
    cmd <- registry[[nm]]
    # Each arg must have all required fields
    for (a in cmd$args) {
      expect_true(!is.null(a$name), info = paste(nm, "arg missing name"))
      expect_true(!is.null(a$type), info = paste(nm, a$name, "missing type"))
      expect_true(!is.null(a$required), info = paste(nm, a$name, "missing required"))
      expect_true(!is.null(a$help), info = paste(nm, a$name, "missing help"))
      expect_true(a$type %in% c("integer", "number", "string", "json"),
        info = paste(nm, a$name, "invalid type:", a$type))
    }
    # Examples must be non-empty and start with /
    expect_true(length(cmd$examples) > 0, info = paste(nm, "missing examples"))
    for (ex in cmd$examples) {
      expect_true(grepl("^/", ex), info = paste(nm, "example doesn't start with /:", ex))
    }
  }
})

# ============================================================================
# Three-Mode Routing Tests
# ============================================================================

test_that("route_input classifies /commands as 'command' mode", {
  r <- PRA:::route_input("/mcs tasks=[...]")
  expect_equal(r$mode, "command")
  expect_equal(r$command, "mcs")

  r <- PRA:::route_input("/help")
  expect_equal(r$mode, "command")
  expect_equal(r$command, "help")

  r <- PRA:::route_input("/evm bac=500000")
  expect_equal(r$mode, "command")
  expect_equal(r$command, "evm")
})

test_that("route_input classifies numerical data as 'tool' mode", {
  # Distribution specifications
  r <- PRA:::route_input("Run a simulation with Normal(10, 2) and Triangular(5, 10, 15)")
  expect_equal(r$mode, "tool")

  # Numeric arrays
  r <- PRA:::route_input("Means are [10, 15, 20] and variances are [4, 9, 16]")
  expect_equal(r$mode, "tool")

  # Dollar amounts with BAC
  r <- PRA:::route_input("My project has BAC = $500,000 with schedule [0.2, 0.4]")
  expect_equal(r$mode, "tool")

  # Simulation count
  r <- PRA:::route_input("Run 10000 simulations for a 3-task project")
  expect_equal(r$mode, "tool")

  # Probabilities
  r <- PRA:::route_input("P(C1) = 0.3, P(Risk|C1) = 0.8")
  expect_equal(r$mode, "tool")

  # Schedule/costs arrays
  r <- PRA:::route_input("Actual costs [90000, 195000, 310000]")
  expect_equal(r$mode, "tool")
})

test_that("route_input classifies conceptual questions as 'rag' mode", {
  r <- PRA:::route_input("What is earned value management?")
  expect_equal(r$mode, "rag")

  r <- PRA:::route_input("Explain the difference between SPI and CPI")
  expect_equal(r$mode, "rag")

  r <- PRA:::route_input("How does Monte Carlo simulation work?")
  expect_equal(r$mode, "rag")

  r <- PRA:::route_input("What are the benefits of Bayesian risk analysis?")
  expect_equal(r$mode, "rag")

  r <- PRA:::route_input("Tell me about contingency reserves")
  expect_equal(r$mode, "rag")
})

test_that("route_input returns required fields", {
  for (input in c("/help", "Normal(10,2)", "What is EVM?")) {
    r <- PRA:::route_input(input)
    expect_true("mode" %in% names(r))
    expect_true("reason" %in% names(r))
    expect_true(r$mode %in% c("command", "tool", "rag"))
    expect_true(nchar(r$reason) > 0)
  }
})

test_that("route_input handles whitespace", {
  r <- PRA:::route_input("  /mcs tasks=[]  ")
  expect_equal(r$mode, "command")
})

# ============================================================================
# Utility functions: check_package, save_pra_plot, tool_result, html helpers
# ============================================================================

test_that("check_package errors with purpose message", {
  expect_error(
    PRA:::check_package("nonexistent_fake_pkg", purpose = "testing"),
    "required.*testing"
  )
  expect_error(
    PRA:::check_package("nonexistent_fake_pkg"),
    "required"
  )
})

test_that("save_pra_plot creates a PNG file", {
  plot_fn <- function() plot(1:5, 1:5, main = "test")
  path <- PRA:::save_pra_plot(plot_fn, "test_coverage")
  expect_true(file.exists(path))
  expect_true(grepl("\\.png$", path))
  expect_true(file.info(path)$size > 0)
  unlink(path)
})

test_that("plot_to_html returns base64 img tag", {
  skip_if_not_installed("base64enc")
  plot_fn <- function() plot(1:5, 1:5, main = "test")
  html <- PRA:::plot_to_html(plot_fn)
  expect_true(grepl("^<img src=\"data:image/png;base64,", html))
  expect_true(grepl("max-width", html))
})

test_that("html_result_table builds valid HTML table", {
  tbl <- PRA:::html_result_table(Mean = 42.5, SD = 3.2, N = 100L)
  expect_true(grepl("<table", tbl))
  expect_true(grepl("Mean", tbl))
  expect_true(grepl("42.5", tbl))
  expect_true(grepl("3.2", tbl))
})

test_that("tool_result returns ContentToolResult when html provided", {
  skip_if_not_installed("ellmer")
  result <- PRA:::tool_result("plain text", title = "Test", html = "<p>Hello</p>")
  # Should return a ContentToolResult (S7 object)
  expect_true(any(grepl("ContentToolResult", class(result), fixed = TRUE)))
})

test_that("tool_result returns plain text when html is NULL", {
  result <- PRA:::tool_result("plain text", title = "Test", html = NULL)
  expect_equal(result, "plain text")
})

# ============================================================================
# Tool functions: HTML output branches
# ============================================================================

test_that("mcs_tool returns rich result with HTML", {
  skip_if_not_installed("ellmer")
  result <- PRA:::mcs_tool(
    num_sims = 1000L,
    task_dists_json = '[{"type":"normal","mean":10,"sd":2},{"type":"uniform","min":5,"max":15}]'
  )
  expect_true(any(grepl("ContentToolResult", class(result), fixed = TRUE)))
})

test_that("smm_tool returns rich result with HTML", {
  skip_if_not_installed("ellmer")
  result <- PRA:::smm_tool(
    mean_json = "[10, 15, 20]",
    var_json = "[4, 9, 16]"
  )
  expect_true(any(grepl("ContentToolResult", class(result), fixed = TRUE)))
})

test_that("sensitivity_tool returns rich result with HTML", {
  skip_if_not_installed("ellmer")
  # Need to run MCS first
  PRA:::mcs_tool(
    num_sims = 1000L,
    task_dists_json = '[{"type":"normal","mean":10,"sd":2},{"type":"triangular","a":5,"b":10,"c":15}]'
  )
  result <- PRA:::sensitivity_tool(
    task_dists_json = '[{"type":"normal","mean":10,"sd":2},{"type":"triangular","a":5,"b":10,"c":15}]'
  )
  expect_true(any(grepl("ContentToolResult", class(result), fixed = TRUE)))
})

test_that("contingency_tool returns rich result with HTML", {
  skip_if_not_installed("ellmer")
  # Need to run MCS first
  PRA:::mcs_tool(
    num_sims = 1000L,
    task_dists_json = '[{"type":"normal","mean":10,"sd":2},{"type":"uniform","min":5,"max":15}]'
  )
  result <- PRA:::contingency_tool(phigh = 0.95, pbase = 0.50)
  expect_true(any(grepl("ContentToolResult", class(result), fixed = TRUE)))
})

test_that("parent_dsm_tool returns rich result with HTML", {
  skip_if_not_installed("ellmer")
  result <- PRA:::parent_dsm_tool(matrix_json = "[[1,1,0],[0,1,1],[1,0,1]]")
  expect_true(any(grepl("ContentToolResult", class(result), fixed = TRUE)))
})

test_that("grandparent_dsm_tool returns rich result with HTML", {
  skip_if_not_installed("ellmer")
  # grandparent_dsm_tool needs square S matrix and R matrix where ncol(S) == ncol(R)
  result <- PRA:::grandparent_dsm_tool(
    s_matrix_json = "[[1,0],[0,1]]",
    r_matrix_json = "[[1,2],[3,4]]"
  )
  expect_true(any(grepl("ContentToolResult", class(result), fixed = TRUE)))
})

test_that("fit_and_predict_sigmoidal_tool returns result with predictions", {
  result <- PRA:::fit_and_predict_sigmoidal_tool(
    x_json = "[1,2,3,4,5,6,7,8,9,10]",
    y_json = "[5,15,40,60,70,75,80,85,90,95]",
    model_type = "logistic",
    predict_x_json = "[11,12]"
  )
  # May return ContentToolResult or plain text depending on plot rendering
  result_text <- if (is.character(result)) result else result@value
  expect_true(grepl("Learning Curve|logistic|Coefficients", result_text, ignore.case = TRUE))
  expect_true(grepl("K", result_text))
})

# ============================================================================
# Command registry fn wrappers: risk_post, learning, dsm
# ============================================================================

test_that("/risk_post command executes via registry fn", {
  registry <- PRA:::pra_command_registry()
  result <- registry$risk_post$fn(list(
    causes = "[0.3,0.2]",
    given = "[0.8,0.6]",
    not_given = "[0.2,0.4]",
    observed = "[1,null]"
  ))
  # Should return text or ContentToolResult
  result_text <- if (is.character(result)) result else result@value
  expect_true(grepl("posterior|Posterior|updated|Updated", result_text, ignore.case = TRUE))
})

test_that("/learning command executes via registry fn", {
  registry <- PRA:::pra_command_registry()
  result <- registry$learning$fn(list(
    x = "[1,2,3,4,5,6,7,8,9,10]",
    y = "[5,15,40,60,70,75,80,85,90,95]",
    model = "logistic",
    predict = "[11,12]"
  ))
  result_text <- if (is.character(result)) result else result@value
  expect_true(grepl("K|logistic|Learning|predict", result_text, ignore.case = TRUE))
})

test_that("parse_command_args handles empty boundaries", {
  # Test with a simple key=value with no complex nesting
  result <- PRA:::parse_command_args("", list("key"))
  expect_equal(result, list())
})

test_that("parse_command_args handles value with trailing whitespace", {
  result <- PRA:::parse_command_args("key=value  ", list("key"))
  # Values are trimmed by execute_command, not parse_command_args
  expect_equal(trimws(result$key), "value")
})

#' Bayesian Inference for Risk Probability.
#'
#' This function calculates the overall probability of a risk event 'R' occurring
#' based on the probabilities of multiple root causes and their associated conditional probabilities.
#'
#' @srrstats {G1.0} *Software lists primary reference from published academic literature.*
#' @srrstats {G1.1} *Software is the first implementation within **R** of the algorithm which has previously been implemented in other languages or contexts.*
#' @srrstats {G1.4} *Software uses [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {G2.0} *Implements assertions on lengths of inputs - all vectors must have same length.*
#' @srrstats {G2.0a} *Parameter documentation explicitly states expected input structure.*
#' @srrstats {G2.1} *Implements assertions on types of inputs - probability values must be numeric between 0 and 1.*
#' @srrstats {G2.1a} *Parameter documentation explicitly states data types expected (vectors of probabilities).*
#' @srrstats {G2.15} *Implements checks for NaN values via is.nan() prior to processing.*
#' @srrstats {G2.16} *Implements checks for Inf/-Inf values via is.infinite() prior to processing.*
#' @srrstats {G5.2a} *Each error message produced by stop() is unique.*
#'
#' @param cause_probs A vector of probabilities for each root cause 'C_i'.
#' @param risks_given_causes A vector of conditional probabilities of the risk event 'R' given each cause 'C_i'.
#' @param risks_given_not_causes A vector of conditional probabilities of the risk event 'R' given not each cause 'C_i'.
#' @return The function returns a numeric value for the probability of risk event 'R'.
#' @references
#' Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
#' construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
#' @examples
#' cause_probs <- c(0.3, 0.2)
#' risks_given_causes <- c(0.8, 0.6)
#' risks_given_not_causes <- c(0.2, 0.4)
#' risk_prob_value <- risk_prob(cause_probs, risks_given_causes, risks_given_not_causes)
#' print(risk_prob_value)
#'
#' @export
risk_prob <- function(cause_probs, risks_given_causes, risks_given_not_causes) {
  # Validate inputs
  if (any(is.nan(cause_probs)) || any(is.nan(risks_given_causes)) || any(is.nan(risks_given_not_causes))) {
    stop("Input vectors must not contain NaN values.")
  }
  if (anyNA(cause_probs) || anyNA(risks_given_causes) || anyNA(risks_given_not_causes)) {
    stop("Input vectors must not contain NA values.")
  }
  if (any(is.infinite(cause_probs)) || any(is.infinite(risks_given_causes)) || any(is.infinite(risks_given_not_causes))) {
    stop("Input vectors must not contain infinite values.")
  }
  if (length(cause_probs) != length(risks_given_causes) || length(cause_probs) != length(risks_given_not_causes)) {
    stop("All input vectors must have the same length.")
  }
  if (any(cause_probs < 0 | cause_probs > 1)) {
    stop("All values in cause_probs must be between 0 and 1.")
  }
  if (any(risks_given_causes < 0 | risks_given_causes > 1)) {
    stop("All values in risks_given_causes must be between 0 and 1.")
  }
  if (any(risks_given_not_causes < 0 | risks_given_not_causes > 1)) {
    stop("All values in risks_given_not_causes must be between 0 and 1.")
  }

  # Calculate P(R) using the law of total probability
  total_risk_prob <- 0
  for (i in seq_along(cause_probs)) {
    not_cause_prob <- 1 - cause_probs[i]
    total_risk_prob <- total_risk_prob +
      (risks_given_causes[i] * cause_probs[i]) +
      (risks_given_not_causes[i] * not_cause_prob)
  }

  return(total_risk_prob)
}

#' Cost Probability Density.
#'
#' This function generates random samples from a mixture model representing the cost 'A'
#' associated with multiple risk events 'R_i'. Each risk event has its own probability,
#' mean, and standard deviation for the cost distribution. The function also accounts for a baseline cost
#' when no risk event occurs.
#'
#' @srrstats {G1.0} *Software lists primary reference from published academic literature.*
#' @srrstats {G1.1} *Software is the first implementation within **R** of the algorithm which has previously been implemented in other languages or contexts.*
#' @srrstats {G1.4} *Software uses [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {G2.0} *Implements assertions on lengths of inputs - all parameter vectors must have same length.*
#' @srrstats {G2.0a} *Parameter documentation explicitly states expected input structure.*
#' @srrstats {G2.1} *Implements assertions on types of inputs via is.numeric() checks.*
#' @srrstats {G2.1a} *Parameter documentation explicitly states data types expected.*
#' @srrstats {G2.2} *Prohibits multivariate input for num_sims which must be a positive integer.*
#' @srrstats {G2.15} *Implements checks for NaN values via is.nan() prior to processing.*
#' @srrstats {G2.16} *Implements checks for Inf/-Inf values via is.infinite() prior to processing.*
#' @srrstats {G5.2a} *Each error message produced by stop() is unique.*
#'
#' @param num_sims Number of random samples to draw from the mixture model.
#' @param risk_probs A vector of probabilities for each risk event 'R_i'.
#' @param means_given_risks A vector of means of the normal distribution for cost 'A' given each risk event 'R_i'.
#' @param sds_given_risks A vector of standard deviations of the normal distribution for cost 'A' given each risk event 'R_i'.
#' @param base_cost The baseline cost given no risk event occurs.
#' @return A numeric vector of random samples from the mixture model.
#' @references
#' Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
#' construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
#' @examples
#' # Example with three risk events
#' num_sims <- 1000
#' risk_probs <- c(0.3, 0.5, 0.2)
#' means_given_risks <- c(10000, 15000, 5000)
#' sds_given_risks <- c(2000, 1000, 1000)
#' base_cost <- 2000
#' samples <- cost_pdf(
#'   num_sims = num_sims,
#'   risk_probs = risk_probs,
#'   means_given_risks = means_given_risks,
#'   sds_given_risks = sds_given_risks,
#'   base_cost = base_cost
#' )
#' hist(samples, breaks = 30, col = "skyblue", main = "Histogram of Cost", xlab = "Cost")
#' @importFrom stats rbinom rnorm
#' @export
cost_pdf <- function(num_sims, risk_probs, means_given_risks, sds_given_risks, base_cost = 0) {
  # Validate inputs
  if (num_sims <= 0 || !is.numeric(num_sims)) stop("num_sims must be a positive integer.")
  if (any(is.nan(risk_probs)) || any(is.nan(means_given_risks)) || any(is.nan(sds_given_risks))) {
    stop("risk_probs, means_given_risks, and sds_given_risks must not contain NaN values.")
  }
  if (anyNA(risk_probs) || anyNA(means_given_risks) || anyNA(sds_given_risks)) {
    stop("risk_probs, means_given_risks, and sds_given_risks must not contain NA values.")
  }
  if (any(is.infinite(risk_probs)) || any(is.infinite(means_given_risks)) || any(is.infinite(sds_given_risks))) {
    stop("risk_probs, means_given_risks, and sds_given_risks must not contain infinite values.")
  }
  if (is.nan(base_cost)) stop("base_cost must not be NaN.")
  if (is.na(base_cost)) stop("base_cost must not be NA.")
  if (is.infinite(base_cost)) stop("base_cost must not be infinite.")
  if (any(risk_probs < 0 | risk_probs > 1)) stop("All risk_probs must be between 0 and 1.")
  if (sum(risk_probs) > 1) stop("Sum of risk_probs must not exceed 1.")
  if (length(risk_probs) != length(means_given_risks) || length(risk_probs) != length(sds_given_risks)) {
    stop("risk_probs, means_given_risks, and sds_given_risks must have the same length.")
  }
  if (any(sds_given_risks < 0)) stop("Standard deviations must be non-negative.")

  # Number of risk events
  num_risks <- length(risk_probs)

  # Initialize cost samples with base cost
  samples <- rep(base_cost, num_sims)

  # Iterate over each risk event
  for (i in seq_len(num_risks)) {
    # Simulate risk outcomes for the current risk event
    risk_event <- rbinom(num_sims, size = 1, prob = risk_probs[i])

    # Add cost samples for the current risk event
    samples <- samples + ifelse(
      risk_event == 1,
      rnorm(num_sims, mean = means_given_risks[i], sd = sds_given_risks[i]),
      0
    )
  }

  return(samples)
}

#' @srrstats {G5.2} *Error and warning behaviour is explicitly demonstrated through tests.*
#' @srrstats {G5.2a} *Every error message is unique and tested.*
#' @srrstats {G5.2b} *Tests trigger every error message and compare with expected values.*
#' @srrstats {G5.3} *Return objects tested for absence of NA, NaN, Inf.*
#' @srrstats {G5.6} *Parameter recovery tests verify implementations produce expected results given data with known properties.*
#' @srrstats {G5.6a} *Parameter recovery tests succeed within defined tolerance rather than exact values.*
#' @srrstats {G5.6b} *Parameter recovery tests run with multiple random seeds when randomness is involved.*
#' @srrstats {G5.7} *Algorithm performance tests verify implementations perform correctly as data properties change.*
#' @srrstats {G5.8} *Edge condition tests verify appropriate behavior with extreme data properties.*
#' @srrstats {G5.8a} *Zero-length data tests trigger clear errors.*
#' @srrstats {G5.8b} *Unsupported data type tests trigger clear errors.*
#' @srrstats {G5.8c} *All-NA and all-identical data tests trigger clear errors or warnings.*
#' @srrstats {G5.8d} *Out-of-scope data tests verify appropriate behavior.*
#' @srrstats {G5.9} *Noise susceptibility tests verify stochastic behavior stability.*
#' @srrstats {G5.9a} *Trivial noise tests show results are stable at machine epsilon scale.*
#' @srrstats {G5.9b} *Random seed stability tests show consistent behavior across different seeds.*

# Unit tests for risk_prob function
test_that("risk_prob calculates correct risk probabilities", {
  cause_probs <- c(0.3, 0.2)
  risks_given_causes <- c(0.8, 0.6)
  risks_given_not_causes <- c(0.2, 0.4)

  result <- risk_prob(cause_probs, risks_given_causes, risks_given_not_causes)
  expected <- (0.3 * 0.8) + ((1 - 0.3) * 0.2) + (0.2 * 0.6) + ((1 - 0.2) * 0.4)
  expect_equal(result, expected)
})

test_that("risk_prob handles input validation correctly", {
  expect_error(risk_prob(c(0.3, 1.2), c(0.8, 0.6), c(0.2, 0.4)), "All values in cause_probs must be between 0 and 1.")
  expect_error(risk_prob(c(0.3, 0.2), c(0.8, 1.6), c(0.2, 0.4)), "All values in risks_given_causes must be between 0 and 1.")
  expect_error(risk_prob(c(0.3, 0.2), c(0.8, 0.6), c(0.2, -0.4)), "All values in risks_given_not_causes must be between 0 and 1.")
  expect_error(risk_prob(c(0.3, 0.2), c(0.8), c(0.2, 0.4)), "All input vectors must have the same length.")
})

# Unit tests for cost_pdf function
test_that("cost_pdf generates correct sample sizes", {
  num_sims <- 1000
  risk_probs <- c(0.3, 0.5)
  means_given_risks <- c(10000, 15000)
  sds_given_risks <- c(2000, 1000)
  base_cost <- 2000

  samples <- cost_pdf(num_sims, risk_probs, means_given_risks, sds_given_risks, base_cost)
  expect_equal(length(samples), num_sims)
})

test_that("cost_pdf handles input validation correctly", {
  expect_error(cost_pdf(-1000, c(0.3, 0.5), c(10000, 15000), c(2000, 1000), 2000), "num_sims must be a positive integer.")
  expect_error(cost_pdf(1000, c(0.3, 1.5), c(10000, 15000), c(2000, 1000), 2000), "All risk_probs must be between 0 and 1.")
  expect_error(cost_pdf(1000, c(0.3, 0.5), c(10000), c(2000, 1000), 2000), "risk_probs, means_given_risks, and sds_given_risks must have the same length.")
  expect_error(cost_pdf(1000, c(0.3, 0.5), c(10000, 15000), c(2000, -1000), 2000), "Standard deviations must be non-negative.")
})

test_that("cost_pdf generates realistic samples", {
  num_sims <- 1000
  risk_probs <- c(0.3)
  means_given_risks <- c(10000)
  sds_given_risks <- c(2000)
  base_cost <- 2000

  samples <- cost_pdf(num_sims, risk_probs, means_given_risks, sds_given_risks, base_cost)
  expect_true(all(samples >= base_cost))
  expect_gt(mean(samples), base_cost)
})

# Additional risk_prob tests
test_that("risk_prob handles single cause correctly", {
  cause_probs <- c(0.5)
  risks_given_causes <- c(0.8)
  risks_given_not_causes <- c(0.2)

  result <- risk_prob(cause_probs, risks_given_causes, risks_given_not_causes)
  expected <- (0.5 * 0.8) + (0.5 * 0.2)
  expect_equal(result, expected)
})

test_that("risk_prob handles extreme probabilities correctly", {
  # All causes certain
  cause_probs <- c(1.0, 1.0)
  risks_given_causes <- c(0.8, 0.6)
  risks_given_not_causes <- c(0.2, 0.4)

  result <- risk_prob(cause_probs, risks_given_causes, risks_given_not_causes)
  expect_true(is.numeric(result))

  # No causes
  cause_probs <- c(0.0, 0.0)
  result <- risk_prob(cause_probs, risks_given_causes, risks_given_not_causes)
  expect_true(is.numeric(result))
})

# Additional cost_pdf tests
test_that("cost_pdf handles zero risk probabilities correctly", {
  num_sims <- 1000
  risk_probs <- c(0, 0)
  means_given_risks <- c(10000, 15000)
  sds_given_risks <- c(2000, 1000)
  base_cost <- 2000

  samples <- cost_pdf(num_sims, risk_probs, means_given_risks, sds_given_risks, base_cost)
  expect_equal(length(samples), num_sims)
  expect_true(all(samples == base_cost))
})

test_that("cost_pdf validates sum of risk_probs", {
  expect_error(
    cost_pdf(1000, c(0.6, 0.6), c(10000, 15000), c(2000, 1000), 2000),
    "Sum of risk_probs must not exceed 1."
  )
})

test_that("cost_pdf handles default base_cost correctly", {
  num_sims <- 100
  risk_probs <- c(0)
  means_given_risks <- c(10000)
  sds_given_risks <- c(2000)

  samples <- cost_pdf(num_sims, risk_probs, means_given_risks, sds_given_risks)
  expect_true(all(samples == 0))
})

# ============================================================================
# NaN/NA/Inf Error Tests (G5.2, G5.2b)
# ============================================================================
test_that("risk_prob rejects NaN inputs", {
  expect_error(risk_prob(c(NaN, 0.2), c(0.8, 0.6), c(0.2, 0.4)), "Input vectors must not contain NaN values.")
  expect_error(risk_prob(c(0.3, 0.2), c(NaN, 0.6), c(0.2, 0.4)), "Input vectors must not contain NaN values.")
  expect_error(risk_prob(c(0.3, 0.2), c(0.8, 0.6), c(NaN, 0.4)), "Input vectors must not contain NaN values.")
})

test_that("risk_prob rejects NA inputs", {
  expect_error(risk_prob(c(NA_real_, 0.2), c(0.8, 0.6), c(0.2, 0.4)), "Input vectors must not contain NA values.")
  expect_error(risk_prob(c(0.3, 0.2), c(NA_real_, 0.6), c(0.2, 0.4)), "Input vectors must not contain NA values.")
  expect_error(risk_prob(c(0.3, 0.2), c(0.8, 0.6), c(NA_real_, 0.4)), "Input vectors must not contain NA values.")
})

test_that("risk_prob rejects Inf inputs", {
  expect_error(risk_prob(c(Inf, 0.2), c(0.8, 0.6), c(0.2, 0.4)), "Input vectors must not contain infinite values.")
  expect_error(risk_prob(c(0.3, 0.2), c(Inf, 0.6), c(0.2, 0.4)), "Input vectors must not contain infinite values.")
  expect_error(risk_prob(c(0.3, 0.2), c(0.8, 0.6), c(Inf, 0.4)), "Input vectors must not contain infinite values.")
})

test_that("cost_pdf rejects NaN in risk params", {
  expect_error(
    cost_pdf(1000, c(NaN, 0.5), c(10000, 15000), c(2000, 1000), 2000),
    "risk_probs, means_given_risks, and sds_given_risks must not contain NaN values."
  )
  expect_error(
    cost_pdf(1000, c(0.3, 0.5), c(NaN, 15000), c(2000, 1000), 2000),
    "risk_probs, means_given_risks, and sds_given_risks must not contain NaN values."
  )
  expect_error(
    cost_pdf(1000, c(0.3, 0.5), c(10000, 15000), c(NaN, 1000), 2000),
    "risk_probs, means_given_risks, and sds_given_risks must not contain NaN values."
  )
})

test_that("cost_pdf rejects NA in risk params", {
  expect_error(
    cost_pdf(1000, c(NA_real_, 0.5), c(10000, 15000), c(2000, 1000), 2000),
    "risk_probs, means_given_risks, and sds_given_risks must not contain NA values."
  )
})

test_that("cost_pdf rejects Inf in risk params", {
  expect_error(
    cost_pdf(1000, c(Inf, 0.5), c(10000, 15000), c(2000, 1000), 2000),
    "risk_probs, means_given_risks, and sds_given_risks must not contain infinite values."
  )
})

test_that("cost_pdf rejects NaN base_cost", {
  expect_error(
    cost_pdf(1000, c(0.3, 0.5), c(10000, 15000), c(2000, 1000), NaN),
    "base_cost must not be NaN."
  )
})

test_that("cost_pdf rejects NA base_cost", {
  expect_error(
    cost_pdf(1000, c(0.3, 0.5), c(10000, 15000), c(2000, 1000), NA_real_),
    "base_cost must not be NA."
  )
})

test_that("cost_pdf rejects Inf base_cost", {
  expect_error(
    cost_pdf(1000, c(0.3, 0.5), c(10000, 15000), c(2000, 1000), Inf),
    "base_cost must not be infinite."
  )
})

# ============================================================================
# G5.3: Return value tests
# ============================================================================
test_that("risk_prob result contains no NA, NaN, or Inf", {
  result <- risk_prob(c(0.3, 0.2), c(0.8, 0.6), c(0.2, 0.4))
  expect_false(is.na(result))
  expect_false(is.nan(result))
  expect_false(is.infinite(result))
})

test_that("cost_pdf result contains no NA, NaN, or Inf", {
  set.seed(42)
  samples <- cost_pdf(100, c(0.3, 0.5), c(10000, 15000), c(2000, 1000), 2000)
  expect_false(anyNA(samples))
  expect_false(any(is.nan(samples)))
  expect_false(any(is.infinite(samples)))
})

# ============================================================================
# Parameter Recovery Tests (G5.6, G5.6a, G5.6b)
# ============================================================================

test_that("risk_prob recovers known probability with certain cause", {
  # If cause is certain (prob=1) and risk given cause is certain (prob=1)
  cause_probs <- c(1.0)
  risks_given_causes <- c(1.0)
  risks_given_not_causes <- c(0.0)

  result <- risk_prob(cause_probs, risks_given_causes, risks_given_not_causes)

  # Expected: P(R) = 1.0 * 1.0 + 0.0 * 0.0 = 1.0
  expect_equal(result, 1.0, tolerance = 1e-10)
})

test_that("risk_prob recovers known probability with no risk", {
  # If all conditional probabilities are zero
  cause_probs <- c(0.5, 0.3)
  risks_given_causes <- c(0.0, 0.0)
  risks_given_not_causes <- c(0.0, 0.0)

  result <- risk_prob(cause_probs, risks_given_causes, risks_given_not_causes)

  # Expected: P(R) = 0
  expect_equal(result, 0.0, tolerance = 1e-10)
})

test_that("risk_prob recovers known probability from manual calculation", {
  cause_probs <- c(0.3, 0.2)
  risks_given_causes <- c(0.8, 0.6)
  risks_given_not_causes <- c(0.2, 0.4)

  result <- risk_prob(cause_probs, risks_given_causes, risks_given_not_causes)

  # Manual calculation using law of total probability:
  # P(R) = P(R|C1)*P(C1) + P(R|¬C1)*P(¬C1) + P(R|C2)*P(C2) + P(R|¬C2)*P(¬C2)
  #      = 0.8*0.3 + 0.2*0.7 + 0.6*0.2 + 0.4*0.8
  #      = 0.24 + 0.14 + 0.12 + 0.32 = 0.82
  expected <- 0.8 * 0.3 + 0.2 * 0.7 + 0.6 * 0.2 + 0.4 * 0.8

  expect_equal(result, expected, tolerance = 1e-10)
})

test_that("cost_pdf converges to theoretical mean with single certain risk (seed 123)", {
  set.seed(123)

  num_sims <- 100000
  risk_probs <- c(1.0) # Certain risk
  means_given_risks <- c(10000)
  sds_given_risks <- c(0) # No variance
  base_cost <- 5000

  samples <- cost_pdf(num_sims, risk_probs, means_given_risks, sds_given_risks, base_cost)

  # Expected: all samples = base_cost + mean_risk = 15000
  expected_mean <- 5000 + 10000

  expect_equal(mean(samples), expected_mean, tolerance = 1)
})

test_that("cost_pdf converges to theoretical mean with uncertain risk (seed 42)", {
  set.seed(42)

  num_sims <- 100000
  risk_probs <- c(0.5)
  means_given_risks <- c(10000)
  sds_given_risks <- c(1000)
  base_cost <- 5000

  samples <- cost_pdf(num_sims, risk_probs, means_given_risks, sds_given_risks, base_cost)

  # Expected mean: base_cost + risk_prob * mean_risk
  expected_mean <- 5000 + 0.5 * 10000 # 10000

  expect_equal(mean(samples), expected_mean, tolerance = 100)
})

test_that("cost_pdf with zero risk probability equals base cost (seed 123)", {
  set.seed(123)

  num_sims <- 1000
  risk_probs <- c(0.0)
  means_given_risks <- c(10000)
  sds_given_risks <- c(1000)
  base_cost <- 5000

  samples <- cost_pdf(num_sims, risk_probs, means_given_risks, sds_given_risks, base_cost)

  # All samples should equal base_cost
  expect_true(all(samples == base_cost))
})

# ============================================================================
# Algorithm Performance Tests (G5.7)
# ============================================================================

test_that("cost_pdf mean estimate improves with sample size", {
  risk_probs <- c(0.5)
  means_given_risks <- c(10000)
  sds_given_risks <- c(1000)
  base_cost <- 5000

  # Expected mean: base_cost + risk_prob * mean_risk = 5000 + 0.5*10000 = 10000
  expected_mean <- 10000

  # Test with increasing sample sizes
  set.seed(123)
  result_1k <- mean(cost_pdf(1000, risk_probs, means_given_risks, sds_given_risks, base_cost))

  set.seed(123)
  result_10k <- mean(cost_pdf(10000, risk_probs, means_given_risks, sds_given_risks, base_cost))

  set.seed(123)
  result_100k <- mean(cost_pdf(100000, risk_probs, means_given_risks, sds_given_risks, base_cost))

  # Error should decrease with sample size
  error_1k <- abs(result_1k - expected_mean)
  error_10k <- abs(result_10k - expected_mean)
  error_100k <- abs(result_100k - expected_mean)

  expect_true(error_100k < error_10k)
  expect_true(error_10k <= error_1k)
})

# ============================================================================
# Noise Susceptibility Tests (G5.9a) - Trivial Noise
# ============================================================================

test_that("risk_prob is stable to trivial noise in probabilities", {
  cause_probs <- c(0.3, 0.2)
  risks_given_causes <- c(0.8, 0.6)
  risks_given_not_causes <- c(0.2, 0.4)

  result_clean <- risk_prob(cause_probs, risks_given_causes, risks_given_not_causes)

  # Add machine epsilon noise
  cause_probs_noisy <- cause_probs + runif(2, -.Machine$double.eps, .Machine$double.eps)

  result_noisy <- risk_prob(cause_probs_noisy, risks_given_causes, risks_given_not_causes)

  # Results should be essentially identical
  expect_equal(result_clean, result_noisy, tolerance = 10 * .Machine$double.eps)
})

# ============================================================================
# Noise Susceptibility Tests (G5.9b) - Random Seed Stability
# ============================================================================

test_that("cost_pdf produces consistent distributions across seeds", {
  num_sims <- 10000
  risk_probs <- c(0.5)
  means_given_risks <- c(10000)
  sds_given_risks <- c(1000)
  base_cost <- 5000

  # Run with different seeds
  set.seed(111)
  samples_1 <- cost_pdf(num_sims, risk_probs, means_given_risks, sds_given_risks, base_cost)

  set.seed(222)
  samples_2 <- cost_pdf(num_sims, risk_probs, means_given_risks, sds_given_risks, base_cost)

  set.seed(333)
  samples_3 <- cost_pdf(num_sims, risk_probs, means_given_risks, sds_given_risks, base_cost)

  # Means should be similar
  expect_equal(mean(samples_1), mean(samples_2), tolerance = 100)
  expect_equal(mean(samples_2), mean(samples_3), tolerance = 100)

  # Standard deviations should be similar
  expect_equal(sd(samples_1), sd(samples_2), tolerance = 100)
  expect_equal(sd(samples_2), sd(samples_3), tolerance = 100)
})

#' Generate Correlation Matrix from Random Samples.
#'
#' This function generates random samples from specified probability distributions
#' and computes the correlation matrix for the generated samples.
#'
#' @srrstats {G1.0} *Software lists primary reference from published academic literature.*
#' @srrstats {G1.1} *Software is the first implementation within **R** of the algorithm which has previously been implemented in other languages or contexts.*
#' @srrstats {G1.4} *Software uses [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {G2.0} *Implements assertions on lengths of inputs - validates num_vars does not exceed length of dists.*
#' @srrstats {G2.0a} *Parameter documentation explicitly states expected input types.*
#' @srrstats {G2.1} *Implements assertions on types of inputs via is.numeric() and is.list() checks.*
#' @srrstats {G2.1a} *Parameter documentation explicitly states data types expected (positive integers, list of functions).*
#' @srrstats {G2.4a} *Uses as.integer() for integer comparison of num_samples and num_vars.*
#' @srrstats {G2.15} *Implements checks for NaN values via is.nan() prior to processing.*
#' @srrstats {G2.16} *Implements checks for Inf/-Inf values via is.infinite() prior to processing.*
#' @srrstats {G5.2a} *Each error message produced by stop() is unique.*
#'
#' @param num_samples The number of samples to generate.
#' @param num_vars The number of distributions to sample.
#' @param dists A list describing each distribution. Each element should be a function
#' that generates random samples. The names of the list elements will be used to
#' identify the distributions.
#'
#' @return The function returns the correlation matrix for the distributions.
#' @references
#' Govan, Paul, and Ivan Damnjanovic. "The resource-based view on project risk management."
#' Journal of construction engineering and management 142.9 (2016): 04016034.
#' @examples
#' # List of probability distributions
#' dists <- list(
#'   normal = function(n) rnorm(n, mean = 0, sd = 1),
#'   uniform = function(n) runif(n, min = 0, max = 1),
#'   exponential = function(n) rexp(n, rate = 1),
#'   poisson = function(n) rpois(n, lambda = 1),
#'   binomial = function(n) rbinom(n, size = 10, prob = 0.5)
#' )
#'
#' # Generate correlation matrix
#' cor_matrix <- cor_matrix(num_samples = 100, num_vars = 5, dists = dists)
#'
#' # Print correlation matrix
#' print(cor_matrix)
#'
#' @importFrom stats cor
#' @export
# Function to generate random samples and calculate correlation matrix
cor_matrix <- function(num_samples = 100, num_vars = 5, dists) {
  # Error handling
  if (!is.list(dists) || length(dists) == 0) {
    stop("dists must be a non-empty list.")
  }
  if (!all(sapply(dists, is.function))) {
    stop("All elements in dists must be functions.")
  }

  # Error checks for num_samples
  if (is.numeric(num_samples) && length(num_samples) == 1 && is.nan(num_samples)) {
    stop("num_samples must not be NaN.")
  }
  if (is.numeric(num_samples) && length(num_samples) == 1 && !is.nan(num_samples) && is.na(num_samples)) {
    stop("num_samples must not be NA.")
  }
  if (is.numeric(num_samples) && length(num_samples) == 1 && is.infinite(num_samples)) {
    stop("num_samples must not be infinite.")
  }
  if (!is.numeric(num_samples) || num_samples <= 0 || num_samples != as.integer(num_samples)) {
    stop("num_samples must be a positive integer.")
  }

  # Error checks for num_vars
  if (is.numeric(num_vars) && length(num_vars) == 1 && is.nan(num_vars)) {
    stop("num_vars must not be NaN.")
  }
  if (is.numeric(num_vars) && length(num_vars) == 1 && !is.nan(num_vars) && is.na(num_vars)) {
    stop("num_vars must not be NA.")
  }
  if (is.numeric(num_vars) && length(num_vars) == 1 && is.infinite(num_vars)) {
    stop("num_vars must not be infinite.")
  }
  if (!is.numeric(num_vars) || num_vars <= 0 || num_vars != as.integer(num_vars)) {
    stop("num_vars must be a positive integer.")
  }
  if (num_vars > length(dists)) {
    stop("num_vars must not exceed the number of distributions in dists.")
  }

  # Initialize a matrix to store the samples
  samples <- matrix(0, nrow = num_samples, ncol = num_vars)

  # Randomly select distributions and generate samples
  for (i in seq_len(num_vars)) {
    dist_name <- sample(names(dists), 1)
    samples[, i] <- dists[[dist_name]](num_samples)
  }

  # Calculate the correlation matrix
  cor_matrix <- stats::cor(samples)

  return(cor_matrix)
}

#' @srrstats {G5.2} *Error and warning behaviour is explicitly demonstrated through tests.*
#' @srrstats {G5.2a} *Every error message is unique and tested.*
#' @srrstats {G5.2b} *Tests trigger every error message and compare with expected values.*
#' @srrstats {G5.3} *Return objects tested for absence of NA, NaN, Inf.*
#' @srrstats {G5.6} *Parameter recovery tests verify implementations produce expected results given data with known properties.*
#' @srrstats {G5.6a} *Parameter recovery tests succeed within defined tolerance rather than exact values.*
#' @srrstats {G5.6b} *Parameter recovery tests run with multiple random seeds when randomness is involved.*
#' @srrstats {G5.7} *Algorithm performance tests verify implementations perform correctly as data properties change.*
#' @srrstats {G5.8} *Edge condition tests verify appropriate behavior with extreme data properties.*
#' @srrstats {G5.8a} *Zero-length data tests trigger clear errors.*
#' @srrstats {G5.8b} *Unsupported data type tests trigger clear errors.*
#' @srrstats {G5.8c} *All-NA and all-identical data tests trigger clear errors or warnings.*
#' @srrstats {G5.8d} *Out-of-scope data tests verify appropriate behavior.*
#' @srrstats {G5.9} *Noise susceptibility tests verify stochastic behavior stability.*
#' @srrstats {G5.9a} *Trivial noise tests show results are stable at machine epsilon scale.*
#' @srrstats {G5.9b} *Random seed stability tests show consistent behavior across different seeds.*

# Define a set of distributions
dists <- list(
  normal = function(n) rnorm(n, mean = 0, sd = 1),
  uniform = function(n) runif(n, min = 0, max = 1),
  exponential = function(n) rexp(n, rate = 1),
  poisson = function(n) rpois(n, lambda = 1),
  binomial = function(n) rbinom(n, size = 10, prob = 0.5)
)

test_that("cor_matrix returns correct dimensions for valid inputs", {
  result <- cor_matrix(num_samples = 100, num_vars = 5, dists = dists)
  expect_true(is.matrix(result))
  expect_equal(nrow(result), 5)
  expect_equal(ncol(result), 5)
})

test_that("cor_matrix throws error for invalid num_samples", {
  expect_error(cor_matrix(num_samples = -1, num_vars = 5, dists = dists), "num_samples must be a positive integer.")
  expect_error(cor_matrix(num_samples = "a", num_vars = 5, dists = dists), "num_samples must be a positive integer.")
  expect_error(cor_matrix(num_samples = 0, num_vars = 5, dists = dists), "num_samples must be a positive integer.")
})

test_that("cor_matrix throws error for invalid num_vars", {
  expect_error(cor_matrix(num_samples = 100, num_vars = -1, dists = dists), "num_vars must be a positive integer.")
  expect_error(cor_matrix(num_samples = 100, num_vars = "a", dists = dists), "num_vars must be a positive integer.")
  expect_error(cor_matrix(num_samples = 100, num_vars = 0, dists = dists), "num_vars must be a positive integer.")
  expect_error(cor_matrix(num_samples = 100, num_vars = 10, dists = dists), "num_vars must not exceed the number of distributions in dists.")
})

test_that("cor_matrix throws error for invalid dists", {
  expect_error(cor_matrix(num_samples = 100, num_vars = 5, dists = "not a list"), "dists must be a non-empty list.")
  expect_error(cor_matrix(num_samples = 100, num_vars = 5, dists = list()), "dists must be a non-empty list.")
  expect_error(cor_matrix(num_samples = 100, num_vars = 5, dists = list("not a function")), "All elements in dists must be functions.")
})

test_that("cor_matrix works correctly with edge cases", {
  result <- cor_matrix(num_samples = 1, num_vars = 1, dists = dists)
  expect_true(is.matrix(result))
  expect_equal(nrow(result), 1)
  expect_equal(ncol(result), 1)
})

# ============================================================================
# NaN/NA/Inf Error Tests (G5.2, G5.2b)
# ============================================================================
test_that("cor_matrix rejects NaN num_samples", {
  expect_error(cor_matrix(num_samples = NaN, num_vars = 5, dists = dists), "num_samples must not be NaN.")
})

test_that("cor_matrix rejects NA num_samples", {
  expect_error(cor_matrix(num_samples = NA_real_, num_vars = 5, dists = dists), "num_samples must not be NA.")
})

test_that("cor_matrix rejects Inf num_samples", {
  expect_error(cor_matrix(num_samples = Inf, num_vars = 5, dists = dists), "num_samples must not be infinite.")
})

test_that("cor_matrix rejects NaN num_vars", {
  expect_error(cor_matrix(num_samples = 100, num_vars = NaN, dists = dists), "num_vars must not be NaN.")
})

test_that("cor_matrix rejects NA num_vars", {
  expect_error(cor_matrix(num_samples = 100, num_vars = NA_real_, dists = dists), "num_vars must not be NA.")
})

test_that("cor_matrix rejects Inf num_vars", {
  expect_error(cor_matrix(num_samples = 100, num_vars = Inf, dists = dists), "num_vars must not be infinite.")
})

# ============================================================================
# G5.3: Return value tests
# ============================================================================
test_that("cor_matrix result contains no NA, NaN, or Inf", {
  set.seed(42)
  result <- cor_matrix(num_samples = 100, num_vars = 3, dists = dists)
  expect_false(anyNA(result))
  expect_false(any(is.nan(result)))
  expect_false(any(is.infinite(result)))
})

# ============================================================================
# Parameter Recovery Tests (G5.6, G5.6a)
# ============================================================================

test_that("cor_matrix has unit diagonal", {
  set.seed(123)

  # cor_matrix expects named list of distributions
  dist_list <- list(
    norm1 = function(n) rnorm(n, 10, 2),
    norm2 = function(n) rnorm(n, 15, 3)
  )

  result <- cor_matrix(num_samples = 10000, num_vars = 2, dists = dist_list)

  # Diagonal elements must be 1 (correlation with self)
  expect_equal(diag(result), c(1, 1), tolerance = 1e-10)
})

test_that("cor_matrix is symmetric", {
  set.seed(42)

  # cor_matrix expects named list of distributions
  dist_list <- list(
    norm1 = function(n) rnorm(n, 10, 2),
    unif1 = function(n) runif(n, 5, 15),
    norm2 = function(n) rnorm(n, 20, 5)
  )

  result <- cor_matrix(num_samples = 10000, num_vars = 3, dists = dist_list)

  # Correlation matrix must be symmetric
  expect_equal(result, t(result), tolerance = 1e-10)
})

# ============================================================================
# Algorithm Performance Tests (G5.7)
# ============================================================================

test_that("cor_matrix estimates improve with sample size", {
  dist_list <- list(
    norm1 = function(n) rnorm(n, 10, 2),
    norm2 = function(n) rnorm(n, 15, 3)
  )

  # Test with increasing sample sizes
  set.seed(123)
  result_100 <- cor_matrix(num_samples = 100, num_vars = 2, dists = dist_list)

  set.seed(123)
  result_1000 <- cor_matrix(num_samples = 1000, num_vars = 2, dists = dist_list)

  set.seed(123)
  result_10000 <- cor_matrix(num_samples = 10000, num_vars = 2, dists = dist_list)

  # Diagonal should always be 1
  expect_equal(diag(result_100), c(1, 1), tolerance = 1e-10)
  expect_equal(diag(result_1000), c(1, 1), tolerance = 1e-10)
  expect_equal(diag(result_10000), c(1, 1), tolerance = 1e-10)

  # Off-diagonal correlation should stabilize (be more consistent)
  # For independent normals, correlation should be near 0
  expect_true(abs(result_10000[1, 2]) < abs(result_100[1, 2]) + 0.1)
})

# ============================================================================
# Edge Condition Tests (G5.8b) - Unsupported Types
# ============================================================================

test_that("cor_matrix rejects non-function elements in dists", {
  bad_dists <- list(
    norm1 = function(n) rnorm(n, 0, 1),
    not_a_function = "this is a string"
  )
  expect_error(cor_matrix(100, 2, bad_dists), "All elements in dists must be functions")
})

# ============================================================================
# Noise Susceptibility Tests (G5.9b) - Random Seed Stability
# ============================================================================

test_that("cor_matrix produces consistent estimates across seeds", {
  dist_list <- list(
    norm1 = function(n) rnorm(n, 10, 2),
    norm2 = function(n) rnorm(n, 15, 3)
  )

  # Run with different seeds
  set.seed(111)
  result_1 <- cor_matrix(num_samples = 5000, num_vars = 2, dists = dist_list)

  set.seed(222)
  result_2 <- cor_matrix(num_samples = 5000, num_vars = 2, dists = dist_list)

  set.seed(333)
  result_3 <- cor_matrix(num_samples = 5000, num_vars = 2, dists = dist_list)

  # Correlation estimates should be similar
  # For independent normals, should all be near 0
  expect_equal(result_1[1, 2], result_2[1, 2], tolerance = 0.1)
  expect_equal(result_2[1, 2], result_3[1, 2], tolerance = 0.1)
})

#' Sensitivity Analysis.
#'
#' This function performs sensitivity analysis on a project with multiple tasks,
#' each having its own cost distribution. It calculates the sensitivity of the
#' variance in total project cost with respect to the variance in each task's cost.
#' It can also account for correlations between task costs if a correlation matrix
#' is provided.
#'
#' @srrstats {G1.0} *Software lists primary reference from published academic literature.*
#' @srrstats {G1.1} *Software is the first implementation within **R** of the algorithm which has previously been implemented in other languages or contexts.*
#' @srrstats {G1.4} *Software uses [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {G2.0} *Implements assertions on dimensions - validates cor_mat dimensions match number of tasks.*
#' @srrstats {G2.0a} *Parameter documentation explicitly states expected input structure.*
#' @srrstats {G2.1} *Implements assertions on types of inputs via is.list() and is.matrix() checks.*
#' @srrstats {G2.1a} *Parameter documentation explicitly states data types expected.*
#' @srrstats {G2.15} *Implements checks for NaN values via is.nan() prior to processing.*
#' @srrstats {G2.16} *Implements checks for Inf/-Inf values via is.infinite() prior to processing.*
#' @srrstats {G3.1} *Correlation handling is user-controlled via optional cor_mat parameter.*
#' @srrstats {G3.1a} *Documentation describes usage of correlation matrix in examples.*
#' @srrstats {G5.2a} *Each error message produced by stop() is unique.*
#'
#' @param task_dists A list of lists describing each task distribution. Each inner
#' list should contain the type of distribution and its parameters. Supported distributions
#' are "normal", "triangular", and "uniform".
#' @param cor_mat The correlation matrix for the tasks (Optional). If provided, it
#' should be a square matrix with dimensions equal to the number of tasks. If not
#' provided, tasks are assumed to be independent.
#' @return The function returns a vector of sensitivity results with respect to
#' each task. Each element in the vector corresponds to the sensitivity of the variance
#' in total project cost with respect to the variance in the respective task's cost.
#' @references
#' Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
#' construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
#' @examples
#' # Set the task distributions for a toy project.
#' task_dists <- list(
#'   list(type = "normal", mean = 10, sd = 2), # Task A: Normal distribution
#'   list(type = "triangular", a = 5, b = 15, c = 10), # Task B: Triangular distribution
#'   list(type = "uniform", min = 8, max = 12) # Task C: Uniform distribution
#' )
#'
#' # Set the correlation matrix between the tasks.
#' cor_mat <- matrix(c(
#'   1, 0.5, 0.3,
#'   0.5, 1, 0.4,
#'   0.3, 0.4, 1
#' ), nrow = 3, byrow = TRUE)
#'
#' # Calculate the sensitivity of each task and print the results
#' sensitivity_results <- sensitivity(task_dists, cor_mat)
#' print(sensitivity_results)
#'
#' # Build a vertical barchart and display the results.
#' data <- data.frame(
#'   Tasks = c("A", "B", "C"),
#'   Sensitivity = sensitivity_results
#' )
#' barplot(
#'   height = data$Sensitivity, names = data$Tasks, col = "skyblue",
#'   horiz = TRUE, xlab = "Sensitivity", ylab = "Tasks"
#' )
#' title("Sensitivity Analysis of Project Tasks")
#'
#' # Without correlation matrix
#' sensitivity_results_indep <- sensitivity(task_dists)
#' print(sensitivity_results_indep)
#'
#' # Build a vertical barchart and display the results.
#' data_indep <- data.frame(
#'   Tasks = c("A", "B", "C"),
#'   Sensitivity = sensitivity_results_indep
#' )
#' barplot(
#'   height = data_indep$Sensitivity, names = data_indep$Tasks,
#'   col = "lightgreen",
#'   horiz = TRUE, xlab = "Sensitivity", ylab = "Tasks"
#' )
#' title("Sensitivity Analysis of Project Tasks (Independent)")
#'
#' @export
# Define the sensitivity analysis function
sensitivity <- function(task_dists, cor_mat = NULL) {
  # Error handling
  if (is.null(task_dists)) {
    stop("task_dists must not be NULL")
  }
  if (!is.list(task_dists) || length(task_dists) == 0) {
    stop("task_dists must be a non-empty list")
  }

  num_tasks <- length(task_dists)

  # Extract variances from task distributions
  task_variances <- sapply(task_dists, function(dist) {
    if (dist$type == "normal") {
      return(dist$sd^2)
    } else if (dist$type == "triangular") {
      return((dist$a^2 + dist$b^2 + dist$c^2 - dist$a * dist$b - dist$a * dist$c - dist$b * dist$c) / 18)
    } else if (dist$type == "uniform") {
      return((dist$max - dist$min)^2 / 12)
    } else {
      stop("Unsupported distribution type.")
    }
  })

  # Create the covariance matrix
  if (!is.null(cor_mat)) {
    if (!is.matrix(cor_mat) || nrow(cor_mat) != num_tasks || ncol(cor_mat) != num_tasks) {
      stop("The correlation matrix must be square and match the number of tasks.")
    }
    if (any(is.nan(cor_mat))) {
      stop("cor_mat must not contain NaN values")
    }
    if (anyNA(cor_mat)) {
      stop("cor_mat must not contain NA values")
    }
    if (any(is.infinite(cor_mat))) {
      stop("cor_mat must not contain infinite values")
    }

    cov_matrix <- matrix(0, nrow = num_tasks, ncol = num_tasks)
    for (i in seq_len(num_tasks)) {
      for (j in seq_len(num_tasks)) {
        cov_matrix[i, j] <- cor_mat[i, j] * sqrt(task_variances[i] * task_variances[j])
      }
    }
  } else {
    cov_matrix <- diag(task_variances)
  }

  # Calculate the total variance of the project cost
  total_variance <- sum(task_variances) + sum(cov_matrix[upper.tri(cov_matrix)] * 2)

  # Initialize sensitivity vector
  sensitivity <- numeric(num_tasks)

  # Calculate the sensitivity of the total variance with respect to each task's variance
  for (i in seq_len(num_tasks)) {
    sensitivity[i] <- 1 + 2 * sum(cov_matrix[i, -i] / sqrt(task_variances[i] * task_variances[-i]))
  }

  # Return the sensitivity vector
  return(sensitivity)
}

#' @srrstats {G5.2} *Error and warning behaviour is explicitly demonstrated through tests.*
#' @srrstats {G5.2a} *Every error message is unique and tested.*
#' @srrstats {G5.2b} *Tests trigger every error message and compare with expected values.*
#' @srrstats {G5.3} *Return objects tested for absence of NA, NaN, Inf.*
#' @srrstats {G5.6} *Parameter recovery tests verify implementations produce expected results given data with known properties.*
#' @srrstats {G5.6a} *Parameter recovery tests succeed within defined tolerance rather than exact values.*
#' @srrstats {G5.6b} *Parameter recovery tests run with multiple random seeds when randomness is involved.*
#' @srrstats {G5.7} *Algorithm performance tests verify implementations perform correctly as data properties change.*
#' @srrstats {G5.8} *Edge condition tests verify appropriate behavior with extreme data properties.*
#' @srrstats {G5.8a} *Zero-length data tests trigger clear errors.*
#' @srrstats {G5.8b} *Unsupported data type tests trigger clear errors.*
#' @srrstats {G5.8c} *All-NA and all-identical data tests trigger clear errors or warnings.*
#' @srrstats {G5.8d} *Out-of-scope data tests verify appropriate behavior.*
#' @srrstats {G5.9} *Noise susceptibility tests verify stochastic behavior stability.*
#' @srrstats {G5.9a} *Trivial noise tests show results are stable at machine epsilon scale.*
#' @srrstats {G5.9b} *Random seed stability tests show consistent behavior across different seeds.*

test_that("sensitivity function calculates correctly for normal distributions", {
  normal_dists <- list(
    list(type = "normal", mean = 10, sd = 2),
    list(type = "normal", mean = 20, sd = 3)
  )
  result <- sensitivity(normal_dists)
  expect_true(is.numeric(result))
  expect_length(result, 2)

  # Expected variances
  expected_variances <- sapply(normal_dists, function(dist) dist$sd^2)
  expected_sensitivity <- 1 + 2 * 0 # Since there is no correlation matrix provided

  expect_equal(result[1], expected_sensitivity)
  expect_equal(result[2], expected_sensitivity)
})

test_that("sensitivity function calculates correctly for triangular distributions", {
  triangular_dists <- list(
    list(type = "triangular", a = 1, b = 3, c = 5),
    list(type = "triangular", a = 2, b = 4, c = 6)
  )
  result <- sensitivity(triangular_dists)
  expect_true(is.numeric(result))
  expect_length(result, 2)
})

test_that("sensitivity function calculates correctly for uniform distributions", {
  uniform_dists <- list(
    list(type = "uniform", min = 1, max = 5),
    list(type = "uniform", min = 2, max = 6)
  )
  result <- sensitivity(uniform_dists)
  expect_true(is.numeric(result))
  expect_length(result, 2)
})

test_that("sensitivity function handles correlation matrix correctly", {
  normal_dists <- list(
    list(type = "normal", mean = 10, sd = 2),
    list(type = "normal", mean = 20, sd = 3)
  )
  cor_mat <- matrix(c(1, 0.8, 0.8, 1), nrow = 2)
  result <- sensitivity(normal_dists, cor_mat)
  expect_true(is.numeric(result))
  expect_length(result, 2)
})

test_that("sensitivity function handles errors correctly", {
  normal_dists <- list(
    list(type = "normal", mean = 10, sd = 2),
    list(type = "normal", mean = 20, sd = 3)
  )

  # Unsupported distribution type
  unsupported_dists <- list(list(type = "unsupported", mean = 10, sd = 2))
  expect_error(sensitivity(unsupported_dists), "Unsupported distribution type.")

  # Incorrect correlation matrix
  incorrect_cor_mat <- matrix(c(1, 0.8), nrow = 1)
  expect_error(sensitivity(normal_dists, incorrect_cor_mat), "The correlation matrix must be square and match the number of tasks.")
})

test_that("sensitivity function validates NULL input correctly", {
  expect_error(sensitivity(NULL), "task_dists must not be NULL")
})

test_that("sensitivity function validates empty list correctly", {
  expect_error(sensitivity(list()), "task_dists must be a non-empty list")
})

test_that("sensitivity function validates non-list input correctly", {
  expect_error(sensitivity("not a list"), "task_dists must be a non-empty list")
})

test_that("sensitivity function handles mixed distribution types", {
  mixed_dists <- list(
    list(type = "normal", mean = 10, sd = 2),
    list(type = "triangular", a = 5, b = 10, c = 15),
    list(type = "uniform", min = 8, max = 12)
  )
  result <- sensitivity(mixed_dists)
  expect_true(is.numeric(result))
  expect_length(result, 3)
})

test_that("sensitivity function handles single task correctly", {
  single_dist <- list(list(type = "normal", mean = 10, sd = 2))
  result <- sensitivity(single_dist)
  expect_true(is.numeric(result))
  expect_length(result, 1)
  expect_equal(result[1], 1) # No correlation, sensitivity should be 1
})

# ============================================================================
# NaN/NA/Inf Error Tests (G5.2, G5.2b)
# ============================================================================
test_that("sensitivity rejects NaN in cor_mat", {
  normal_dists <- list(
    list(type = "normal", mean = 10, sd = 2),
    list(type = "normal", mean = 20, sd = 3)
  )
  cor_mat <- matrix(c(1, NaN, NaN, 1), nrow = 2)
  expect_error(sensitivity(normal_dists, cor_mat), "cor_mat must not contain NaN values")
})

test_that("sensitivity rejects NA in cor_mat", {
  normal_dists <- list(
    list(type = "normal", mean = 10, sd = 2),
    list(type = "normal", mean = 20, sd = 3)
  )
  cor_mat <- matrix(c(1, NA, NA, 1), nrow = 2)
  expect_error(sensitivity(normal_dists, cor_mat), "cor_mat must not contain NA values")
})

test_that("sensitivity rejects Inf in cor_mat", {
  normal_dists <- list(
    list(type = "normal", mean = 10, sd = 2),
    list(type = "normal", mean = 20, sd = 3)
  )
  cor_mat <- matrix(c(1, Inf, Inf, 1), nrow = 2)
  expect_error(sensitivity(normal_dists, cor_mat), "cor_mat must not contain infinite values")
})

# ============================================================================
# G5.3: Return value tests
# ============================================================================
test_that("sensitivity result contains no NA, NaN, or Inf", {
  normal_dists <- list(
    list(type = "normal", mean = 10, sd = 2),
    list(type = "normal", mean = 20, sd = 3)
  )
  result <- sensitivity(normal_dists)
  expect_false(anyNA(result))
  expect_false(any(is.nan(result)))
  expect_false(any(is.infinite(result)))
})

# ============================================================================
# Parameter Recovery Tests (G5.6, G5.6a)
# ============================================================================

test_that("sensitivity recovers known values for independent tasks", {
  task_dists <- list(
    list(type = "normal", mean = 10, sd = 2),
    list(type = "normal", mean = 15, sd = 3)
  )
  # No correlation matrix = independent

  result <- sensitivity(task_dists)

  # For independent tasks, sensitivity should be 1.0 for all tasks
  expect_equal(result, c(1, 1), tolerance = 1e-10)
})

# ============================================================================
# Edge Condition Tests (G5.8c) - All-Identical Distributions
# ============================================================================

test_that("sensitivity handles all identical distributions", {
  # All tasks with same distribution
  task_dists <- list(
    list(type = "normal", mean = 10, sd = 2),
    list(type = "normal", mean = 10, sd = 2),
    list(type = "normal", mean = 10, sd = 2)
  )

  result <- sensitivity(task_dists)

  # All should have same sensitivity
  expect_equal(result[1], result[2], tolerance = 1e-10)
  expect_equal(result[2], result[3], tolerance = 1e-10)
})

#' Build the PRA Knowledge Base for RAG Retrieval
#'
#' Reads the curated risk analysis knowledge files bundled with the PRA package,
#' chunks them, generates embeddings via Ollama, and stores them in a DuckDB-backed
#' ragnar knowledge base for retrieval-augmented generation.
#'
#' The knowledge base is built once and cached to disk. Subsequent calls with the
#' same `store_path` load the existing store.
#'
#' @param store_path Path to store the DuckDB knowledge base. Defaults to a
#'   cache directory under [tools::R_user_dir()].
#' @param embed_model Ollama embedding model name (default `"nomic-embed-text"`).
#' @param overwrite Logical. If `TRUE`, rebuild the knowledge base even if a
#'   cached version exists. Default `FALSE`.
#'
#' @return A ragnar store object that can be passed to [retrieve_context()].
#'
#' @examples
#' \dontrun{
#' store <- build_knowledge_base()
#' context <- retrieve_context(store, "How do I run a Monte Carlo simulation?")
#' }
#'
#' @export
build_knowledge_base <- function(store_path = NULL,
                                 embed_model = "nomic-embed-text",
                                 overwrite = FALSE) {
  if (!requireNamespace("ragnar", quietly = TRUE)) {
    stop("Package 'ragnar' is required. Install with: install.packages('ragnar')")
  }
  if (!requireNamespace("ellmer", quietly = TRUE)) {
    stop("Package 'ellmer' is required for embeddings. Install with: install.packages('ellmer')")
  }


  # Default store path in user cache directory
  if (is.null(store_path)) {
    cache_dir <- tools::R_user_dir("PRA", "cache")
    if (!dir.exists(cache_dir)) dir.create(cache_dir, recursive = TRUE)
    store_path <- file.path(cache_dir, "pra_knowledge.duckdb")
  }

  # Return cached store if it exists, is valid, and overwrite is FALSE
  if (file.exists(store_path) && !overwrite) {
    message("Loading cached knowledge base from: ", store_path)
    store <- tryCatch({
      s <- ragnar::ragnar_store_connect(store_path)
      # Validate the store works with current duckdb version
      ragnar::ragnar_retrieve(s, "test", top_k = 1L)
      s
    }, error = function(e) {
      message("Cached knowledge base is incompatible (", conditionMessage(e),
              "). Rebuilding...")
      if (file.exists(store_path)) unlink(store_path)
      NULL
    })
    if (!is.null(store)) return(store)
  }

  # Read knowledge files
  knowledge_dir <- system.file("knowledge", package = "PRA")
  if (knowledge_dir == "") {
    stop("Knowledge directory not found. Is the PRA package installed correctly?")
  }

  md_files <- list.files(knowledge_dir, pattern = "\\.md$", full.names = TRUE)
  if (length(md_files) == 0) {
    stop("No knowledge files found in: ", knowledge_dir)
  }

  message("Building PRA knowledge base from ", length(md_files), " files...")

  # Create store with Ollama embeddings and a source column for attribution
  embed_fn <- ragnar::embed_ollama(model = embed_model)
  store <- ragnar::ragnar_store_create(
    store_path,
    embed = embed_fn,
    overwrite = overwrite,
    extra_cols = data.frame(source = character(0))
  )

  # Read, chunk, and insert each file individually (ragnar v2 requires

  # MarkdownDocumentChunks objects, not plain data frames)
  total_chunks <- 0L
  for (f in md_files) {
    text <- paste(readLines(f, warn = FALSE), collapse = "\n")
    chunks <- ragnar::markdown_chunk(text)
    chunks$source <- basename(f)
    ragnar::ragnar_store_insert(store, chunks)
    total_chunks <- total_chunks + nrow(chunks)
  }

  message("Inserted ", total_chunks, " chunks from ", length(md_files), " knowledge files.")

  # Build retrieval index (BM25 + VSS)
  ragnar::ragnar_store_build_index(store)

  message("Knowledge base built and saved to: ", store_path)
  store
}

#' Add Custom Documents to the PRA Knowledge Base
#'
#' Ingests additional markdown or text documents into an existing PRA knowledge
#' base. Use this to extend the agent's knowledge with your own project
#' documentation, standards, templates, or lessons learned.
#'
#' @param store A ragnar store object from [build_knowledge_base()].
#' @param path Character. Path to a file or directory. If a directory, all
#'   `.md` and `.txt` files are ingested recursively.
#' @param embed_model Character. Ollama embedding model (default
#'   `"nomic-embed-text"`).
#'
#' @return The store object (invisibly), updated with the new documents.
#'
#' @examples
#' \dontrun{
#' store <- build_knowledge_base()
#'
#' # Add a single file
#' add_documents(store, "path/to/my_risk_register.md")
#'
#' # Add all .md and .txt files in a directory
#' add_documents(store, "path/to/project_docs/")
#' }
#'
#' @export
add_documents <- function(store, path, embed_model = "nomic-embed-text") {
  if (!requireNamespace("ragnar", quietly = TRUE)) {
    stop("Package 'ragnar' is required. Install with: install.packages('ragnar')")
  }

  # Collect files
  if (dir.exists(path)) {
    files <- list.files(path,
      pattern = "\\.(md|txt)$", full.names = TRUE,
      recursive = TRUE
    )
    if (length(files) == 0) {
      stop("No .md or .txt files found in: ", path)
    }
  } else if (file.exists(path)) {
    files <- path
  } else {
    stop("Path does not exist: ", path)
  }

  message("Adding ", length(files), " document(s) to knowledge base...")

  total_chunks <- 0L
  for (f in files) {
    text <- paste(readLines(f, warn = FALSE), collapse = "\n")
    chunks <- ragnar::markdown_chunk(text)
    chunks$source <- basename(f)
    ragnar::ragnar_store_insert(store, chunks)
    total_chunks <- total_chunks + nrow(chunks)
  }

  ragnar::ragnar_store_build_index(store)

  message("Added ", total_chunks, " chunks from ", length(files), " file(s).")
  invisible(store)
}

#' Retrieve Relevant Context for a Query
#'
#' Searches the PRA knowledge base using combined vector similarity search (VSS)
#' and BM25 full-text search to find the most relevant chunks for a user query.
#'
#' @param store A ragnar store object from [build_knowledge_base()].
#' @param query Character string. The user's question or query.
#' @param top_k Integer. Number of chunks to retrieve (default 5).
#'
#' @return A character vector of relevant text chunks with source attribution,
#'   suitable for injecting into an LLM prompt as additional context.
#'
#' @examples
#' \dontrun{
#' store <- build_knowledge_base()
#' chunks <- retrieve_context(store, "What is earned value management?")
#' cat(chunks, sep = "\n---\n")
#' }
#'
#' @export
retrieve_context <- function(store, query, top_k = 3) {
  if (!requireNamespace("ragnar", quietly = TRUE)) {
    stop("Package 'ragnar' is required. Install with: install.packages('ragnar')")
  }

  results <- ragnar::ragnar_retrieve(store, query, top_k = top_k)

  if (nrow(results) == 0) {
    return(character(0))
  }


  # Append source attribution to each chunk so LLM can cite references
  if ("source" %in% names(results)) {
    # ragnar v2 may return list columns; unlist for safety
    sources <- unlist(results$source)
  } else {
    sources <- rep("PRA knowledge base", nrow(results))
  }
  paste0(results$text, "\n[Source: ", sources, "]")
}

#' @srrstats {G5.2a} *Each error message produced by stop() is unique.*

test_that("knowledge files exist in inst/knowledge", {
  knowledge_dir <- system.file("knowledge", package = "PRA")
  # If running in dev mode, use inst/ directly

  if (knowledge_dir == "") {
    knowledge_dir <- file.path("../../inst/knowledge")
  }
  expect_true(dir.exists(knowledge_dir))
  md_files <- list.files(knowledge_dir, pattern = "\\.md$")
  expect_true(length(md_files) >= 6)
  expected_files <- c(
    "mcs_methods.md", "evm_standards.md", "bayesian_risk.md",
    "learning_curves.md", "sensitivity_contingency.md", "pra_functions.md"
  )
  for (f in expected_files) {
    expect_true(f %in% md_files, info = paste("Missing knowledge file:", f))
  }
})

test_that("knowledge files are non-empty", {
  knowledge_dir <- system.file("knowledge", package = "PRA")
  if (knowledge_dir == "") {
    knowledge_dir <- file.path("../../inst/knowledge")
  }
  md_files <- list.files(knowledge_dir, pattern = "\\.md$", full.names = TRUE)
  for (f in md_files) {
    content <- readLines(f, warn = FALSE)
    expect_true(length(content) > 10,
      info = paste("File too short:", basename(f))
    )
  }
})

test_that("build_knowledge_base requires ragnar", {
  skip_if_not_installed("ragnar")
  skip_if_not_installed("ellmer")
  expect_true(is.function(build_knowledge_base))
})

test_that("retrieve_context requires ragnar", {
  skip_if_not_installed("ragnar")
  expect_true(is.function(retrieve_context))
})

# ============================================================================
# Error path tests (always run, no Ollama needed)
# ============================================================================

test_that("build_knowledge_base errors when ragnar missing", {
  # Mock requireNamespace to return FALSE for ragnar
  mockr_available <- requireNamespace("mockr", quietly = TRUE)
  if (!mockr_available) {
    # Test the error message text without mocking
    expect_match(
      tryCatch(
        withr::with_package("PRA", {
          # We can verify error message format is correct
          stop("Package 'ragnar' is required. Install with: install.packages('ragnar')")
        }),
        error = function(e) e$message
      ),
      "ragnar"
    )
  }
})

test_that("add_documents errors on nonexistent path", {
  skip_if_not_installed("ragnar")
  expect_error(
    add_documents(NULL, "/nonexistent/path/that/doesnt/exist"),
    "Path does not exist"
  )
})

test_that("add_documents errors on directory with no md/txt files", {
  skip_if_not_installed("ragnar")
  # Create a temp directory with a non-md file
  td <- tempdir()
  tmp_subdir <- file.path(td, "pra_test_empty_dir")
  dir.create(tmp_subdir, showWarnings = FALSE)
  file.create(file.path(tmp_subdir, "data.csv"))
  on.exit(unlink(tmp_subdir, recursive = TRUE), add = TRUE)

  expect_error(
    add_documents(NULL, tmp_subdir),
    "No .md or .txt files found"
  )
})

# ============================================================================
# retrieve_context output formatting
# ============================================================================

test_that("retrieve_context formats sources with attribution", {
  skip_if_not_installed("ragnar")
  skip_if_not_installed("ellmer")
  skip_on_cran()
  # Skip if Ollama isn't running
  ollama_ok <- tryCatch({
    con <- url("http://localhost:11434/api/tags", open = "rb")
    on.exit(close(con))
    TRUE
  }, error = function(e) FALSE)
  skip_if(!ollama_ok, "Ollama not running")

  store <- build_knowledge_base()
  chunks <- retrieve_context(store, "Monte Carlo simulation", top_k = 2)
  expect_true(length(chunks) > 0)
  # Each chunk should end with [Source: filename.md]
  for (chunk in chunks) {
    expect_true(grepl("\\[Source: .+\\.md\\]$", chunk),
      info = paste("Missing source attribution in chunk:", substr(chunk, 1, 50)))
  }
})

test_that("retrieve_context returns expected number of chunks", {
  skip_if_not_installed("ragnar")
  skip_if_not_installed("ellmer")
  skip_on_cran()
  ollama_ok <- tryCatch({
    con <- url("http://localhost:11434/api/tags", open = "rb")
    on.exit(close(con))
    TRUE
  }, error = function(e) FALSE)
  skip_if(!ollama_ok, "Ollama not running")

  store <- build_knowledge_base()
  chunks <- retrieve_context(store, "Monte Carlo simulation", top_k = 1)
  expect_true(length(chunks) >= 1)
  expect_type(chunks, "character")
})

# ============================================================================
# build_knowledge_base caching and overwrite behavior
# ============================================================================

test_that("build_knowledge_base returns store with overwrite=FALSE when cache exists", {
  skip_if_not_installed("ragnar")
  skip_if_not_installed("ellmer")
  skip_on_cran()
  ollama_ok <- tryCatch({
    con <- url("http://localhost:11434/api/tags", open = "rb")
    on.exit(close(con))
    TRUE
  }, error = function(e) FALSE)
  skip_if(!ollama_ok, "Ollama not running")

  # First call builds the store
  store1 <- build_knowledge_base()
  expect_true(!is.null(store1))

  # Second call with default overwrite=FALSE loads from cache
  msg <- capture.output(store2 <- build_knowledge_base(), type = "message")
  expect_true(any(grepl("Loading cached", msg)))
})

test_that("build_knowledge_base rebuilds with overwrite=TRUE", {
  skip_if_not_installed("ragnar")
  skip_if_not_installed("ellmer")
  skip_on_cran()
  ollama_ok <- tryCatch({
    con <- url("http://localhost:11434/api/tags", open = "rb")
    on.exit(close(con))
    TRUE
  }, error = function(e) FALSE)
  skip_if(!ollama_ok, "Ollama not running")

  # Build to temp location so we don't break the main cache
  tmp_store <- tempfile(fileext = ".duckdb")
  on.exit(unlink(tmp_store), add = TRUE)

  store1 <- build_knowledge_base(store_path = tmp_store)
  expect_true(!is.null(store1))

  # Overwrite should rebuild
  msg <- capture.output(
    store2 <- build_knowledge_base(store_path = tmp_store, overwrite = TRUE),
    type = "message"
  )
  expect_true(any(grepl("Building PRA knowledge base", msg)))
})

test_that("build_knowledge_base custom store_path works", {
  skip_if_not_installed("ragnar")
  skip_if_not_installed("ellmer")
  skip_on_cran()
  ollama_ok <- tryCatch({
    con <- url("http://localhost:11434/api/tags", open = "rb")
    on.exit(close(con))
    TRUE
  }, error = function(e) FALSE)
  skip_if(!ollama_ok, "Ollama not running")

  tmp_store <- tempfile(fileext = ".duckdb")
  on.exit(unlink(tmp_store), add = TRUE)

  store <- build_knowledge_base(store_path = tmp_store)
  expect_true(!is.null(store))
  expect_true(file.exists(tmp_store))
})

# ============================================================================
# add_documents integration
# ============================================================================

test_that("add_documents works with a single markdown file", {
  skip_if_not_installed("ragnar")
  skip_if_not_installed("ellmer")
  skip_on_cran()
  ollama_ok <- tryCatch({
    con <- url("http://localhost:11434/api/tags", open = "rb")
    on.exit(close(con))
    TRUE
  }, error = function(e) FALSE)
  skip_if(!ollama_ok, "Ollama not running")

  tmp_store <- tempfile(fileext = ".duckdb")
  on.exit(unlink(tmp_store), add = TRUE)

  store <- build_knowledge_base(store_path = tmp_store)

  # Create a temp markdown file
  tmp_md <- tempfile(fileext = ".md")
  writeLines(c(
    "# Custom Risk Process",
    "",
    "Our organization uses a 5-step risk process.",
    "Step 1: Identify risks from the work breakdown structure.",
    "Step 2: Assess probability and impact using subject matter experts.",
    "Step 3: Run Monte Carlo simulation for schedule risk.",
    "Step 4: Calculate contingency reserves at P80 confidence.",
    "Step 5: Monitor risks weekly using earned value metrics."
  ), tmp_md)
  on.exit(unlink(tmp_md), add = TRUE)

  msg <- capture.output(add_documents(store, tmp_md), type = "message")
  expect_true(any(grepl("Added", msg)))

  # Verify the new content is retrievable
  chunks <- retrieve_context(store, "custom risk process steps", top_k = 3)
  expect_true(length(chunks) > 0)
})

test_that("add_documents works with a directory", {
  skip_if_not_installed("ragnar")
  skip_if_not_installed("ellmer")
  skip_on_cran()
  ollama_ok <- tryCatch({
    con <- url("http://localhost:11434/api/tags", open = "rb")
    on.exit(close(con))
    TRUE
  }, error = function(e) FALSE)
  skip_if(!ollama_ok, "Ollama not running")

  tmp_store <- tempfile(fileext = ".duckdb")
  on.exit(unlink(tmp_store), add = TRUE)

  store <- build_knowledge_base(store_path = tmp_store)

  # Create temp dir with multiple files
  tmp_dir <- file.path(tempdir(), "pra_test_docs")
  dir.create(tmp_dir, showWarnings = FALSE)
  writeLines("# Doc 1\nFirst document about risk identification.", file.path(tmp_dir, "doc1.md"))
  writeLines("# Doc 2\nSecond document about risk response planning.", file.path(tmp_dir, "doc2.txt"))
  on.exit(unlink(tmp_dir, recursive = TRUE), add = TRUE)

  msg <- capture.output(add_documents(store, tmp_dir), type = "message")
  expect_true(any(grepl("2 document", msg)))
})

# ============================================================================
# RAG Retrieval Quality Tests (require Ollama)
# ============================================================================

test_that("RAG retrieves correct source for MCS queries", {
  skip_if_not_installed("ragnar")
  skip_if_not_installed("ellmer")
  skip_on_cran()
  ollama_ok <- tryCatch({
    con <- url("http://localhost:11434/api/tags", open = "rb")
    on.exit(close(con))
    TRUE
  }, error = function(e) FALSE)
  skip_if(!ollama_ok, "Ollama not running")

  store <- build_knowledge_base()
  chunks <- retrieve_context(store, "How do I run a Monte Carlo simulation?", top_k = 3)
  expect_true(length(chunks) > 0)
  expect_true(any(grepl("mcs_methods\\.md", chunks)),
    info = paste("Sources:", paste(chunks, collapse = " | ")))
})

test_that("RAG retrieves correct source for EVM queries", {
  skip_if_not_installed("ragnar")
  skip_if_not_installed("ellmer")
  skip_on_cran()
  ollama_ok <- tryCatch({
    con <- url("http://localhost:11434/api/tags", open = "rb")
    on.exit(close(con))
    TRUE
  }, error = function(e) FALSE)
  skip_if(!ollama_ok, "Ollama not running")

  store <- build_knowledge_base()
  chunks <- retrieve_context(store, "What is earned value management and how is CPI calculated?", top_k = 3)
  expect_true(length(chunks) > 0)
  expect_true(any(grepl("evm_standards\\.md", chunks)),
    info = paste("Sources:", paste(chunks, collapse = " | ")))
})

test_that("RAG retrieves correct source for Bayesian queries", {
  skip_if_not_installed("ragnar")
  skip_if_not_installed("ellmer")
  skip_on_cran()
  ollama_ok <- tryCatch({
    con <- url("http://localhost:11434/api/tags", open = "rb")
    on.exit(close(con))
    TRUE
  }, error = function(e) FALSE)
  skip_if(!ollama_ok, "Ollama not running")

  store <- build_knowledge_base()
  chunks <- retrieve_context(store, "How does Bayesian risk analysis work for root cause analysis?", top_k = 3)
  expect_true(length(chunks) > 0)
  expect_true(any(grepl("bayesian_risk\\.md", chunks)),
    info = paste("Sources:", paste(chunks, collapse = " | ")))
})

test_that("RAG retrieves correct source for learning curve queries", {
  skip_if_not_installed("ragnar")
  skip_if_not_installed("ellmer")
  skip_on_cran()
  ollama_ok <- tryCatch({
    con <- url("http://localhost:11434/api/tags", open = "rb")
    on.exit(close(con))
    TRUE
  }, error = function(e) FALSE)
  skip_if(!ollama_ok, "Ollama not running")

  store <- build_knowledge_base()
  chunks <- retrieve_context(store, "What sigmoidal learning curve models are available?", top_k = 3)
  expect_true(length(chunks) > 0)
  expect_true(any(grepl("learning_curves\\.md", chunks)),
    info = paste("Sources:", paste(chunks, collapse = " | ")))
})

test_that("RAG retrieves correct source for sensitivity queries", {
  skip_if_not_installed("ragnar")
  skip_if_not_installed("ellmer")
  skip_on_cran()
  ollama_ok <- tryCatch({
    con <- url("http://localhost:11434/api/tags", open = "rb")
    on.exit(close(con))
    TRUE
  }, error = function(e) FALSE)
  skip_if(!ollama_ok, "Ollama not running")

  store <- build_knowledge_base()
  chunks <- retrieve_context(store, "How is contingency reserve calculated from simulation results?", top_k = 3)
  expect_true(length(chunks) > 0)
  expect_true(any(grepl("sensitivity_contingency\\.md", chunks)),
    info = paste("Sources:", paste(chunks, collapse = " | ")))
})

# ============================================================================
# Knowledge coverage audit
# ============================================================================

test_that("each tool domain has a corresponding knowledge file", {
  tool_knowledge_map <- list(
    mcs = "mcs_methods.md",
    smm = "mcs_methods.md",
    evm = "evm_standards.md",
    bayesian = "bayesian_risk.md",
    learning = "learning_curves.md",
    sensitivity = "sensitivity_contingency.md",
    contingency = "sensitivity_contingency.md"
  )

  knowledge_dir <- system.file("knowledge", package = "PRA")
  if (knowledge_dir == "") knowledge_dir <- file.path("../../inst/knowledge")
  md_files <- list.files(knowledge_dir, pattern = "\\.md$")

  for (domain in names(tool_knowledge_map)) {
    expected_file <- tool_knowledge_map[[domain]]
    expect_true(expected_file %in% md_files,
      info = paste("Missing knowledge coverage for", domain, ":", expected_file))
  }
})

#' Second Moment Method Analysis.
#'
#' This function performs the Second Moment Method (SMM) analysis to estimate the
#' total mean, variance, and standard deviation of a project based on individual
#' task means, variances, and an optional correlation matrix.
#'
#' @srrstats {G1.0} *Software lists primary reference from published academic literature.*
#' @srrstats {G1.1} *Software is the first implementation within **R** of the algorithm which has previously been implemented in other languages or contexts.*
#' @srrstats {G1.4} *Software uses [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {G2.0} *Implements assertions on lengths of inputs - mean and var vectors must have same length.*
#' @srrstats {G2.0a} *Parameter documentation explicitly states expected input structure.*
#' @srrstats {G2.1} *Implements assertions on types of inputs via is.numeric() and is.matrix() checks.*
#' @srrstats {G2.1a} *Parameter documentation explicitly states data types expected.*
#' @srrstats {G2.13} *Implements checks for missing data via anyNA() prior to processing.*
#' @srrstats {G2.14a} *Errors on missing data with informative message.*
#' @srrstats {G2.15} *Implements checks for NaN values via is.nan() prior to processing.*
#' @srrstats {G2.16} *Implements checks for Inf/-Inf values via is.infinite() prior to processing.*
#' @srrstats {G3.1} *Correlation handling is user-controlled via optional cor_mat parameter.*
#' @srrstats {G3.1a} *Documentation describes usage of correlation matrix in examples.*
#' @srrstats {G5.2a} *Each error message produced by stop() is unique.*
#'
#' @param mean  The mean vector.
#' @param var The variance vector.
#' @param cor_mat The correlation matrix (optional). If not provided, tasks are assumed to be independent.
#' @return The function returns a list of the total mean, variance, and standard deviation for the project.
#' @references
#' Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
#' construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
#' @examples
#'
#' # Set the mean vector, variance vector, and correlation matrix for a toy project.
#' mean <- c(10, 15, 20)
#' var <- c(4, 9, 16)
#' cor_mat <- matrix(c(
#'   1, 0.5, 0.3,
#'   0.5, 1, 0.4,
#'   0.3, 0.4, 1
#' ), nrow = 3, byrow = TRUE)
#'
#' # Use the Second Moment Method to estimate the results for the project.
#' result <- smm(mean, var, cor_mat)
#' print(result)
#'
#' # Without correlation matrix (independent tasks)
#' result <- smm(mean, var)
#' print(result)
#'
#' # When certain tasks are discrete and others are continuous, the SMM can still
#' # be applied as long as the variance values accurately reflect the variability of each task.
#'
#' discrete_mean <- c(5, 10)
#' discrete_var <- c(0, 0)
#' continuous_mean <- c(15, 20)
#' continuous_var <- c(4, 5)
#' mean <- c(discrete_mean, continuous_mean)
#' var <- c(discrete_var, continuous_var)
#' cor_mat <- matrix(c(
#'   1, 0, 0.2, 0.3,
#'   0, 1, 0.1, 0.2,
#'   0.2, 0.1, 1, 0.4,
#'   0.3, 0.2, 0.4,
#'   1
#' ), nrow = 4, byrow = TRUE)
#' result <- smm(mean, var, cor_mat)
#' print(result)
#'
#' @export
# Second Moment Method
smm <- function(mean, var, cor_mat = NULL) {
  # Error handling
  if (is.null(mean) || is.null(var)) {
    stop("mean and var must not be NULL")
  }
  if (!is.numeric(mean) || !is.numeric(var)) {
    stop("mean and var must be numeric vectors")
  }
  if (length(mean) == 0 || length(var) == 0) {
    stop("mean and var must not be empty")
  }
  if (any(is.nan(mean)) || any(is.nan(var))) {
    stop("mean and var must not contain NaN values")
  }
  if (anyNA(mean) || anyNA(var)) {
    stop("mean and var must not contain NA values")
  }
  if (any(is.infinite(mean)) || any(is.infinite(var))) {
    stop("mean and var must not contain infinite values")
  }
  if (any(var < 0)) {
    stop("var values must be non-negative")
  }
  # Check if the mean and variance vectors have the same length
  if (length(mean) != length(var)) {
    stop("The mean and variance vectors must have the same length.")
  }

  num_tasks <- length(mean)

  # Calculate the covariance matrix
  if (!is.null(cor_mat)) {
    if (!is.matrix(cor_mat) || nrow(cor_mat) != num_tasks || ncol(cor_mat) != num_tasks) {
      stop("The correlation matrix must be square and match the number of tasks.")
    }
    if (any(is.nan(cor_mat))) {
      stop("cor_mat must not contain NaN values")
    }
    if (anyNA(cor_mat)) {
      stop("cor_mat must not contain NA values")
    }
    if (any(is.infinite(cor_mat))) {
      stop("cor_mat must not contain infinite values")
    }

    cov_matrix <- matrix(0, nrow = num_tasks, ncol = num_tasks)
    for (i in seq_len(num_tasks)) {
      for (j in seq_len(num_tasks)) {
        cov_matrix[i, j] <- cor_mat[i, j] * sqrt(var[i] * var[j])
      }
    }
  } else {
    cov_matrix <- diag(var)
  }

  # Calculate the total mean
  total_mean <- sum(mean)

  # Calculate the total variance
  total_var <- sum(var) + sum(cov_matrix[upper.tri(cov_matrix)] * 2)

  # Return a list with the results
  result <- list(
    total_mean = total_mean,
    total_var = total_var,
    total_std = sqrt(total_var)
  )

  class(result) <- "smm"
  return(result)
}

#' Print method for SMM results.
#'
#' This function defines how to print the results of the Second Moment Method
#' (SMM) analysis. It formats the output to display the total mean, variance,
#' and standard deviation in a readable manner.
#' @param x An object of class "smm" containing the SMM results.
#' @param ... Additional arguments (not used).
#' @return None. The function prints the SMM results to the console.
#' @examples
#' mean <- c(10, 15, 20)
#' var <- c(4, 9, 16)
#' cor_mat <- matrix(c(
#'   1, 0.5, 0.3,
#'   0.5, 1, 0.4,
#'   0.3, 0.4, 1
#' ), nrow = 3, byrow = TRUE)
#' result <- smm(mean, var, cor_mat)
#' print(result)
#'
#' # Without correlation matrix (independent tasks)
#' result <- smm(mean, var)
#' print(result)
#'
#' @export
#' @method print smm
print.smm <- function(x, ...) {
  cat("Second Moment Method Results:\n")
  cat("------------------------------\n")
  cat("Total Mean: ", x$total_mean, "\n")
  cat("Total Variance: ", x$total_var, "\n")
  cat("Total Standard Deviation: ", x$total_std, "\n")
}

#' @srrstats {G5.2} *Error and warning behaviour is explicitly demonstrated through tests.*
#' @srrstats {G5.2a} *Every error message is unique and tested.*
#' @srrstats {G5.2b} *Tests trigger every error message and compare with expected values.*
#' @srrstats {G5.3} *Return objects tested for absence of NA, NaN, Inf.*
#' @srrstats {G5.6} *Parameter recovery tests verify implementations produce expected results given data with known properties.*
#' @srrstats {G5.6a} *Parameter recovery tests succeed within defined tolerance rather than exact values.*
#' @srrstats {G5.6b} *Parameter recovery tests run with multiple random seeds when randomness is involved.*
#' @srrstats {G5.7} *Algorithm performance tests verify implementations perform correctly as data properties change.*
#' @srrstats {G5.8} *Edge condition tests verify appropriate behavior with extreme data properties.*
#' @srrstats {G5.8a} *Zero-length data tests trigger clear errors.*
#' @srrstats {G5.8b} *Unsupported data type tests trigger clear errors.*
#' @srrstats {G5.8c} *All-NA and all-identical data tests trigger clear errors or warnings.*
#' @srrstats {G5.8d} *Out-of-scope data tests verify appropriate behavior.*
#' @srrstats {G5.9} *Noise susceptibility tests verify stochastic behavior stability.*
#' @srrstats {G5.9a} *Trivial noise tests show results are stable at machine epsilon scale.*
#' @srrstats {G5.9b} *Random seed stability tests show consistent behavior across different seeds.*

test_that("second moment analysis works without correlation matrix", {
  mean <- c(10, 15, 20)
  var <- c(4, 9, 16)
  results <- smm(mean, var)

  expect_equal(results$total_mean, 10 + 15 + 20)
  expect_equal(results$total_std, sqrt(4 + 9 + 16))
})

test_that("second moment analysis works with correlation matrix", {
  mean <- c(10, 15, 20)
  var <- c(4, 9, 16)

  cor_mat <- matrix(c(
    1, 0.5, 0.3,
    0.5, 1, 0.4,
    0.3, 0.4, 1
  ), nrow = 3, byrow = TRUE)

  results <- smm(mean, var, cor_mat)

  task_variances <- c(4, 9, 16)

  cov_matrix <- matrix(0, nrow = 3, ncol = 3)
  for (i in 1:3) {
    for (j in 1:3) {
      cov_matrix[i, j] <- cor_mat[i, j] * sqrt(task_variances[i] * task_variances[j])
    }
  }

  total_variance <- sum(task_variances) + sum(cov_matrix[upper.tri(cov_matrix)] * 2)

  expect_equal(results$total_mean, 10 + 15 + 20)
  expect_equal(results$total_std, sqrt(total_variance))
})

test_that("smm validates NULL inputs correctly", {
  var <- c(4, 9, 16)
  mean <- c(10, 15, 20)

  expect_error(smm(NULL, var), "mean and var must not be NULL")
  expect_error(smm(mean, NULL), "mean and var must not be NULL")
})

test_that("smm validates numeric inputs correctly", {
  mean <- c(10, 15, 20)
  var <- c(4, 9, 16)

  expect_error(smm("not numeric", var), "mean and var must be numeric vectors")
  expect_error(smm(mean, "not numeric"), "mean and var must be numeric vectors")
})

test_that("smm validates empty vectors correctly", {
  # Note: c() returns NULL in R, so it triggers the NULL check first
  expect_error(smm(c(), c(1, 2, 3)), "mean and var must not be NULL")
  expect_error(smm(c(1, 2, 3), c()), "mean and var must not be NULL")

  # Test with numeric(0) which is truly empty but not NULL
  expect_error(smm(numeric(0), c(1, 2, 3)), "mean and var must not be empty")
  expect_error(smm(c(1, 2, 3), numeric(0)), "mean and var must not be empty")
})

test_that("smm validates NA values correctly", {
  expect_error(smm(c(10, NA, 20), c(4, 9, 16)), "mean and var must not contain NA values")
  expect_error(smm(c(10, 15, 20), c(4, NA, 16)), "mean and var must not contain NA values")
})

test_that("smm validates non-negative variance correctly", {
  expect_error(smm(c(10, 15, 20), c(4, -9, 16)), "var values must be non-negative")
})

test_that("smm validates vector length match correctly", {
  expect_error(smm(c(10, 15), c(4, 9, 16)), "The mean and variance vectors must have the same length.")
})

test_that("smm validates correlation matrix correctly", {
  mean <- c(10, 15, 20)
  var <- c(4, 9, 16)

  # Non-square correlation matrix
  bad_cor_mat <- matrix(c(1, 0.5), nrow = 1)
  expect_error(smm(mean, var, bad_cor_mat), "The correlation matrix must be square and match the number of tasks.")

  # Wrong size correlation matrix
  wrong_size_cor_mat <- matrix(c(1, 0.5, 0.5, 1), nrow = 2)
  expect_error(smm(mean, var, wrong_size_cor_mat), "The correlation matrix must be square and match the number of tasks.")
})

test_that("smm print method works correctly", {
  mean <- c(10, 15, 20)
  var <- c(4, 9, 16)
  result <- smm(mean, var)

  expect_output(print(result), "Second Moment Method Results:")
  expect_output(print(result), "Total Mean:")
  expect_output(print(result), "Total Variance:")
  expect_output(print(result), "Total Standard Deviation:")
})

test_that("smm returns correct class", {
  mean <- c(10, 15, 20)
  var <- c(4, 9, 16)
  result <- smm(mean, var)

  expect_s3_class(result, "smm")
})

# ============================================================================
# NaN/NA/Inf Error Tests (G5.2, G5.2b)
# ============================================================================
test_that("smm rejects NaN in mean/var", {
  expect_error(smm(c(10, NaN, 20), c(4, 9, 16)), "mean and var must not contain NaN values")
  expect_error(smm(c(10, 15, 20), c(4, NaN, 16)), "mean and var must not contain NaN values")
})

test_that("smm rejects Inf in mean/var", {
  expect_error(smm(c(10, Inf, 20), c(4, 9, 16)), "mean and var must not contain infinite values")
  expect_error(smm(c(10, 15, 20), c(4, Inf, 16)), "mean and var must not contain infinite values")
})

test_that("smm rejects NaN in cor_mat", {
  mean <- c(10, 15, 20)
  var <- c(4, 9, 16)
  cor_mat <- matrix(c(1, NaN, 0.3, NaN, 1, 0.4, 0.3, 0.4, 1), nrow = 3)
  expect_error(smm(mean, var, cor_mat), "cor_mat must not contain NaN values")
})

test_that("smm rejects NA in cor_mat", {
  mean <- c(10, 15, 20)
  var <- c(4, 9, 16)
  cor_mat <- matrix(c(1, NA, 0.3, NA, 1, 0.4, 0.3, 0.4, 1), nrow = 3)
  expect_error(smm(mean, var, cor_mat), "cor_mat must not contain NA values")
})

test_that("smm rejects Inf in cor_mat", {
  mean <- c(10, 15, 20)
  var <- c(4, 9, 16)
  cor_mat <- matrix(c(1, Inf, 0.3, Inf, 1, 0.4, 0.3, 0.4, 1), nrow = 3)
  expect_error(smm(mean, var, cor_mat), "cor_mat must not contain infinite values")
})

# ============================================================================
# G5.3: Return value tests
# ============================================================================
test_that("smm result components contain no NA, NaN, or Inf", {
  mean <- c(10, 15, 20)
  var <- c(4, 9, 16)
  result <- smm(mean, var)

  expect_false(is.na(result$total_mean))
  expect_false(is.nan(result$total_mean))
  expect_false(is.infinite(result$total_mean))

  expect_false(is.na(result$total_var))
  expect_false(is.nan(result$total_var))
  expect_false(is.infinite(result$total_var))

  expect_false(is.na(result$total_std))
  expect_false(is.nan(result$total_std))
  expect_false(is.infinite(result$total_std))
})

# ============================================================================
# Parameter Recovery Tests (G5.6, G5.6a)
# ============================================================================

test_that("smm recovers known variance for independent tasks", {
  mean <- c(10, 15, 20)
  var <- c(4, 9, 16)
  # No correlation matrix = independent tasks
  result <- smm(mean, var)

  # Expected: total_mean = sum of means, total_var = sum of variances
  expected_mean <- 10 + 15 + 20
  expected_var <- 4 + 9 + 16

  expect_equal(result$total_mean, expected_mean, tolerance = 1e-10)
  expect_equal(result$total_var, expected_var, tolerance = 1e-10)
  expect_equal(result$total_std, sqrt(expected_var), tolerance = 1e-10)
})

test_that("smm recovers known variance with perfect correlation", {
  mean <- c(10, 15, 20)
  var <- c(4, 9, 16)
  cor_mat <- matrix(1, nrow = 3, ncol = 3) # Perfect correlation
  result <- smm(mean, var, cor_mat)

  # Expected: total_var = (sum of standard deviations)^2
  expected_mean <- 10 + 15 + 20
  expected_var <- (2 + 3 + 4)^2 # (sqrt(4) + sqrt(9) + sqrt(16))^2 = 81

  expect_equal(result$total_mean, expected_mean, tolerance = 1e-10)
  expect_equal(result$total_var, expected_var, tolerance = 1e-10)
})

test_that("smm recovers known variance with partial correlation", {
  mean <- c(10, 20)
  var <- c(4, 9)
  cor_mat <- matrix(c(1, 0.5, 0.5, 1), nrow = 2)
  result <- smm(mean, var, cor_mat)

  # Manual calculation:
  # total_var = var[1] + var[2] + 2*cov[1,2]
  # cov[1,2] = cor[1,2] * sqrt(var[1]) * sqrt(var[2]) = 0.5 * 2 * 3 = 3
  expected_mean <- 10 + 20
  expected_var <- 4 + 9 + 2 * 3 # = 19

  expect_equal(result$total_mean, expected_mean, tolerance = 1e-10)
  expect_equal(result$total_var, expected_var, tolerance = 1e-10)
})

# ============================================================================
# Edge Condition Tests (G5.8a) - Zero-Length Data
# ============================================================================

test_that("smm rejects zero-length vectors", {
  # numeric(0) is truly empty (not NULL)
  expect_error(smm(numeric(0), c(1, 2, 3)), "mean and var must not be empty")
  expect_error(smm(c(1, 2, 3), numeric(0)), "mean and var must not be empty")
})

# ============================================================================
# Edge Condition Tests (G5.8c) - All-Identical Data
# ============================================================================

test_that("smm handles all identical means and variances", {
  # All tasks identical
  mean <- c(10, 10, 10)
  var <- c(4, 4, 4)

  result <- smm(mean, var)

  # Total mean = 30, total var = 12 (assuming independence)
  expect_equal(result$total_mean, 30, tolerance = 1e-10)
  expect_equal(result$total_var, 12, tolerance = 1e-10)
})

test_that("smm handles zero variance case", {
  mean <- c(10, 15, 20)
  var <- c(0, 0, 0) # All zero variance

  result <- smm(mean, var)

  # Total mean should still be correct
  expect_equal(result$total_mean, 45, tolerance = 1e-10)
  # Total variance should be 0
  expect_equal(result$total_var, 0, tolerance = 1e-10)
  expect_equal(result$total_std, 0, tolerance = 1e-10)
})

# ============================================================================
# Noise Susceptibility Tests (G5.9a) - Trivial Noise
# ============================================================================

test_that("smm is stable to machine epsilon noise in inputs", {
  mean <- c(10, 15, 20)
  var <- c(4, 9, 16)

  result_clean <- smm(mean, var)

  # Add trivial noise at machine epsilon scale
  mean_noisy <- mean + runif(3, -.Machine$double.eps, .Machine$double.eps)
  var_noisy <- var + runif(3, -.Machine$double.eps, .Machine$double.eps)

  result_noisy <- smm(mean_noisy, var_noisy)

  # Results should be essentially identical
  expect_equal(result_clean$total_mean, result_noisy$total_mean,
    tolerance = 10 * .Machine$double.eps
  )
  expect_equal(result_clean$total_var, result_noisy$total_var,
    tolerance = 10 * .Machine$double.eps
  )
})

#' Resource-based 'Parent' Design Structure Matrix (DSM).
#'
#' This function computes the Resource-based 'Parent' Design Structure Matrix (DSM)
#' from a given Resource-Task Matrix 'S'. The 'Parent' DSM indicates the number of resources
#' shared between each pair of tasks in a project.
#'
#' @srrstats {G1.0} *Software lists primary reference from published academic literature.*
#' @srrstats {G1.1} *Software is the first implementation within **R** of the algorithm which has previously been implemented in other languages or contexts.*
#' @srrstats {G1.4} *Software uses [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {G2.0} *Implements assertions on dimensions and types of matrix input.*
#' @srrstats {G2.0a} *Parameter documentation explicitly states expected input structure (resources x tasks matrix).*
#' @srrstats {G2.1} *Implements assertions on types of inputs via is.matrix(), is.data.frame(), and is.numeric() checks.*
#' @srrstats {G2.1a} *Parameter documentation explicitly states data types expected (matrix or data frame).*
#' @srrstats {G2.7} *Accepts both matrix and data.frame as input.*
#' @srrstats {G2.15} *Implements checks for NaN values via is.nan() prior to processing.*
#' @srrstats {G2.16} *Implements checks for Inf/-Inf values via is.infinite() prior to processing.*
#' @srrstats {G5.2a} *Each error message produced by stop() is unique.*
#'
#' @param S Resource-Task Matrix 'S' giving the links (arcs) between resources and tasks.
#' Rows represent resources and columns represent tasks.
#' @return An S3 object of class `"dsm"` with the following components:
#' \describe{
#'   \item{matrix}{The Resource-based 'Parent' DSM giving the number of resources shared between each task.}
#'   \item{type}{Character string `"parent"`.}
#'   \item{n_tasks}{Number of tasks (columns in S).}
#'   \item{n_resources}{Number of resources (rows in S).}
#' }
#' @references
#' Govan, Paul, and Ivan Damnjanovic. "The resource-based view on project risk management."
#' Journal of construction engineering and management 142.9 (2016): 04016034.
#' @examples
#' # Set the S matrix for a toy project (3 resources x 4 tasks).
#' s <- matrix(c(1, 1, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1), nrow = 3, ncol = 4)
#' cat("Resource-Task Matrix:\n")
#' print(s)
#'
#' # Calculate the Resource-based Parent DSM and print the results.
#' resource_dsm <- parent_dsm(s)
#' print(resource_dsm)
#'
#' @export
# Parent DSM function
parent_dsm <- function(S) {
  # Error handling
  if (is.null(S)) {
    stop("S must not be NULL")
  }
  if (!is.matrix(S) && !is.data.frame(S)) {
    stop("S must be a matrix or data frame")
  }
  if (!is.numeric(as.matrix(S))) {
    stop("S must contain numeric values")
  }
  if (any(is.nan(as.matrix(S)))) {
    stop("S must not contain NaN values")
  }
  if (anyNA(as.matrix(S))) {
    stop("S must not contain NA values")
  }
  if (any(is.infinite(as.matrix(S)))) {
    stop("S must not contain infinite values")
  }

  S <- as.matrix(S)

  # Compute the tasks x tasks DSM: t(S) %*% S
  dsm_matrix <- t(S) %*% S

  result <- list(
    matrix = dsm_matrix,
    type = "parent",
    n_tasks = ncol(S),
    n_resources = nrow(S)
  )
  class(result) <- "dsm"
  return(result)
}


#' Risk-based 'Grandparent' Design Structure Matrix (DSM).
#'
#' This function computes the Risk-based 'Grandparent' Design Structure Matrix (DSM)
#' from given Resource-Task Matrix 'S' and Risk-Resource Matrix 'R'.
#' The 'Grandparent' DSM indicates the number of risks shared between each pair of
#' tasks in a project.
#'
#' @srrstats {G1.0} *Software lists primary reference from published academic literature.*
#' @srrstats {G1.1} *Software is the first implementation within **R** of the algorithm which has previously been implemented in other languages or contexts.*
#' @srrstats {G1.4} *Software uses [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {G2.0} *Implements assertions on dimensions of matrix inputs - validates compatible dimensions for multiplication.*
#' @srrstats {G2.0a} *Parameter documentation explicitly states expected input structure (resources x tasks for S, risks x resources for R).*
#' @srrstats {G2.1} *Implements assertions on types of inputs via is.matrix(), is.data.frame(), and is.numeric() checks.*
#' @srrstats {G2.1a} *Parameter documentation explicitly states data types expected (matrix or data frame).*
#' @srrstats {G2.7} *Accepts both matrix and data.frame as input.*
#' @srrstats {G2.15} *Implements checks for NaN values via is.nan() prior to processing.*
#' @srrstats {G2.16} *Implements checks for Inf/-Inf values via is.infinite() prior to processing.*
#' @srrstats {G5.2a} *Each error message produced by stop() is unique.*
#'
#' @param S Resource-Task Matrix 'S' giving the links (arcs) between resources and tasks.
#' Rows represent resources and columns represent tasks.
#' @param R Risk-Resource Matrix 'R' giving the links (arcs) between risks and resources.
#' Rows represent risks and columns represent resources.
#' @return An S3 object of class `"dsm"` with the following components:
#' \describe{
#'   \item{matrix}{The Risk-based 'Grandparent' DSM giving the number of risks shared between each task.}
#'   \item{type}{Character string `"grandparent"`.}
#'   \item{n_tasks}{Number of tasks (columns in S).}
#'   \item{n_resources}{Number of resources (rows in S).}
#'   \item{n_risks}{Number of risks (rows in R).}
#' }
#' @references
#' Govan, Paul, and Ivan Damnjanovic. "The resource-based view on project risk management."
#' Journal of construction engineering and management 142.9 (2016): 04016034.
#' @examples
#' # Set the S and R matrices and print the results.
#' S <- matrix(c(1, 1, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1), nrow = 3, ncol = 4)
#' R <- matrix(c(1, 1, 0, 1, 0, 0), nrow = 2, ncol = 3)
#' cat("Resource-Task Matrix (3 resources x 4 tasks):\n")
#' print(S)
#' cat("\nRisk-Resource Matrix (2 risks x 3 resources):\n")
#' print(R)
#' # Calculate the Risk-based Grandparent Matrix and print the results.
#' risk_dsm <- grandparent_dsm(S, R)
#' print(risk_dsm)
#'
#' @export

# Grandparent DSM function
grandparent_dsm <- function(S, R) {
  # Error handling
  if (is.null(S) || is.null(R)) {
    stop("S and R must not be NULL")
  }
  if (!is.matrix(S) && !is.data.frame(S)) {
    stop("S must be a matrix or data frame")
  }
  if (!is.matrix(R) && !is.data.frame(R)) {
    stop("R must be a matrix or data frame")
  }
  if (!is.numeric(as.matrix(S)) || !is.numeric(as.matrix(R))) {
    stop("S and R must contain numeric values")
  }
  if (any(is.nan(as.matrix(S))) || any(is.nan(as.matrix(R)))) {
    stop("S and R must not contain NaN values")
  }
  if (anyNA(as.matrix(S)) || anyNA(as.matrix(R))) {
    stop("S and R must not contain NA values")
  }
  if (any(is.infinite(as.matrix(S))) || any(is.infinite(as.matrix(R)))) {
    stop("S and R must not contain infinite values")
  }

  S <- as.matrix(S)
  R <- as.matrix(R)

  # Check dimension compatibility: R (risks x resources) %*% S (resources x tasks)
  # requires ncol(R) == nrow(S)
  if (nrow(S) != ncol(R)) {
    stop("Number of rows in S (resources) must equal the number of columns in R (resources).")
  }

  # Compute the risk-task matrix: R %*% S (risks x tasks)
  risk_task <- R %*% S

  # Compute the tasks x tasks DSM: t(risk_task) %*% risk_task
  dsm_matrix <- t(risk_task) %*% risk_task

  result <- list(
    matrix = dsm_matrix,
    type = "grandparent",
    n_tasks = ncol(S),
    n_resources = nrow(S),
    n_risks = nrow(R)
  )
  class(result) <- "dsm"
  return(result)
}


#' Print a DSM object.
#'
#' @param x A `dsm` object returned by [parent_dsm()] or [grandparent_dsm()].
#' @param ... Additional arguments passed to [print.default()].
#' @return Invisibly returns `x`.
#' @export
#' @method print dsm
print.dsm <- function(x, ...) {
  type_label <- if (x$type == "parent") {
    "Resource-based 'Parent'"
  } else {
    "Risk-based 'Grandparent'"
  }
  cat(type_label, "Design Structure Matrix\n")
  cat("Tasks:", x$n_tasks, " Resources:", x$n_resources)
  if (!is.null(x$n_risks)) cat("  Risks:", x$n_risks)
  cat("\n\n")
  print(x$matrix, ...)
  invisible(x)
}


#' Plot a DSM heatmap.
#'
#' Displays the Design Structure Matrix as a heatmap where color intensity
#' represents the number of shared resources (parent) or risks (grandparent)
#' between task pairs.
#'
#' @param x A `dsm` object returned by [parent_dsm()] or [grandparent_dsm()].
#' @param main Optional plot title. If `NULL`, a default title is generated.
#' @param col Color palette vector. If `NULL`, uses [grDevices::heat.colors()].
#' @param ... Additional arguments passed to [graphics::image()].
#' @return Invisibly returns `x`.
#' @importFrom graphics image axis
#' @importFrom grDevices heat.colors
#' @export
#' @method plot dsm
plot.dsm <- function(x, main = NULL, col = NULL, ...) {
  m <- x$matrix
  n <- nrow(m)
  if (n == 0) {
    message("Nothing to plot: DSM has zero dimensions.")
    return(invisible(x))
  }
  if (is.null(main)) {
    type_label <- if (x$type == "parent") "Parent" else "Grandparent"
    main <- paste(type_label, "DSM")
  }
  if (is.null(col)) {
    col <- grDevices::heat.colors(12, rev = TRUE)
  }
  labels <- colnames(m)
  if (is.null(labels)) labels <- seq_len(n)
  # image() plots bottom-to-top, so reverse row order for conventional DSM layout
  graphics::image(seq_len(n), seq_len(n), t(m[n:1, , drop = FALSE]),
                  main = main, xlab = "Task", ylab = "Task",
                  axes = FALSE, col = col, ...)
  graphics::axis(1, at = seq_len(n), labels = labels)
  graphics::axis(2, at = seq_len(n), labels = rev(labels))
  invisible(x)
}

#' @srrstats {G5.2} *Error and warning behaviour is explicitly demonstrated through tests.*
#' @srrstats {G5.2a} *Every error message is unique and tested.*
#' @srrstats {G5.2b} *Tests trigger every error message and compare with expected values.*
#' @srrstats {G5.3} *Return objects tested for absence of NA, NaN, Inf.*
#' @srrstats {G5.6} *Parameter recovery tests verify implementations produce expected results given data with known properties.*
#' @srrstats {G5.6a} *Parameter recovery tests succeed within defined tolerance rather than exact values.*
#' @srrstats {G5.6b} *Parameter recovery tests run with multiple random seeds when randomness is involved.*
#' @srrstats {G5.7} *Algorithm performance tests verify implementations perform correctly as data properties change.*
#' @srrstats {G5.8} *Edge condition tests verify appropriate behavior with extreme data properties.*
#' @srrstats {G5.8a} *Zero-length data tests trigger clear errors.*
#' @srrstats {G5.8b} *Unsupported data type tests trigger clear errors.*
#' @srrstats {G5.8c} *All-NA and all-identical data tests trigger clear errors or warnings.*
#' @srrstats {G5.8d} *Out-of-scope data tests verify appropriate behavior.*
#' @srrstats {G5.9} *Noise susceptibility tests verify stochastic behavior stability.*
#' @srrstats {G5.9a} *Trivial noise tests show results are stable at machine epsilon scale.*
#' @srrstats {G5.9b} *Random seed stability tests show consistent behavior across different seeds.*

# ============================================================================
# parent_dsm: Basic functionality
# ============================================================================

test_that("parent_dsm function works correctly with valid input", {
  # Identity matrix (2 resources x 2 tasks)
  sm <- matrix(c(1, 0, 0, 1), nrow = 2, ncol = 2)

  result <- parent_dsm(sm)

  expect_s3_class(result, "dsm")
  expect_equal(result$type, "parent")
  expect_true(is.matrix(result$matrix))
  expect_equal(dim(result$matrix), c(2, 2))

  # t(I) %*% I = I
expected_result <- matrix(c(1, 0, 0, 1), nrow = 2, ncol = 2)
  expect_equal(result$matrix, expected_result)
})

test_that("parent_dsm handles non-square matrix", {
  # 2 resources x 3 tasks
  sm <- matrix(c(1, 0, 0, 1, 1, 1), nrow = 2, ncol = 3)

  result <- parent_dsm(sm)

  expect_s3_class(result, "dsm")
  # Output should be tasks x tasks (3x3)
  expect_equal(dim(result$matrix), c(3, 3))
  expect_equal(result$n_tasks, 3)
  expect_equal(result$n_resources, 2)

  # t(S) %*% S: col1=(1,0), col2=(0,1), col3=(1,1)
  # [1,1] = 1*1+0*0 = 1, [1,2] = 1*0+0*1 = 0, [1,3] = 1*1+0*1 = 1
  # [2,2] = 0*0+1*1 = 1, [2,3] = 0*1+1*1 = 1
  # [3,3] = 1*1+1*1 = 2
  expected <- matrix(c(1, 0, 1, 0, 1, 1, 1, 1, 2), nrow = 3, ncol = 3)
  expect_equal(result$matrix, expected)
})

test_that("parent_dsm handles larger matrices", {
  sm <- matrix(c(1, 2, 3, 4, 5, 6, 7, 8, 9), nrow = 3, ncol = 3)

  result <- parent_dsm(sm)

  expect_s3_class(result, "dsm")
  expect_true(is.matrix(result$matrix))
  expect_equal(dim(result$matrix), c(3, 3))

  # t(S) %*% S where S columns are (1,2,3), (4,5,6), (7,8,9)
  # [1,1] = 1+4+9 = 14, [1,2] = 4+10+18 = 32, [1,3] = 7+16+27 = 50
  # [2,2] = 16+25+36 = 77, [2,3] = 28+40+54 = 122
  # [3,3] = 49+64+81 = 194
  expected_result <- matrix(c(14, 32, 50, 32, 77, 122, 50, 122, 194), nrow = 3, ncol = 3)
  expect_equal(result$matrix, expected_result)
})

# ============================================================================
# grandparent_dsm: Basic functionality
# ============================================================================

test_that("grandparent_dsm function works correctly with valid input", {
  sm <- matrix(c(1, 0, 0, 1), nrow = 2, ncol = 2)
  rm <- matrix(c(1, 2, 3, 4), nrow = 2, ncol = 2)

  result <- grandparent_dsm(sm, rm)

  expect_s3_class(result, "dsm")
  expect_equal(result$type, "grandparent")
  expect_true(is.matrix(result$matrix))
  expect_equal(dim(result$matrix), c(2, 2))

  # R %*% S = [[1,3],[2,4]] %*% I = [[1,3],[2,4]]
  # t(RS) %*% RS = [[1,2],[3,4]] %*% [[1,3],[2,4]] = [[5,11],[11,25]]
  expected_result <- matrix(c(5, 11, 11, 25), nrow = 2, ncol = 2)
  expect_equal(result$matrix, expected_result)
})

test_that("grandparent_dsm handles non-square S matrix", {
  # S: 3 resources x 4 tasks, R: 2 risks x 3 resources
  sm <- matrix(c(1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1), nrow = 3, ncol = 4)
  rm <- matrix(c(1, 1, 0, 1, 0, 0), nrow = 2, ncol = 3)

  result <- grandparent_dsm(sm, rm)

  expect_s3_class(result, "dsm")
  # Output should be tasks x tasks (4x4)
  expect_equal(dim(result$matrix), c(4, 4))
  expect_equal(result$n_tasks, 4)
  expect_equal(result$n_resources, 3)
  expect_equal(result$n_risks, 2)
})

test_that("grandparent_dsm function handles incompatible dimensions", {
  # S: 2 resources x 2 tasks, R: 1 risk x 3 resources
  sm <- matrix(c(1, 0, 0, 1), nrow = 2, ncol = 2)
  rm <- matrix(c(1, 2, 3), nrow = 1, ncol = 3)

  expect_error(grandparent_dsm(sm, rm), "Number of rows in S.*must equal.*number of columns in R")
})

test_that("grandparent_dsm function handles larger matrices", {
  sm <- matrix(c(1, 2, 3, 4, 5, 6, 7, 8, 9), nrow = 3, ncol = 3)
  rm <- matrix(c(1, 2, 3, 4, 5, 6, 7, 8, 9), nrow = 3, ncol = 3)

  result <- grandparent_dsm(sm, rm)

  expect_s3_class(result, "dsm")
  expect_true(is.matrix(result$matrix))
  expect_equal(dim(result$matrix), c(3, 3))
})

test_that("grandparent_dsm function handles non-square R matrix", {
  # S: 2 resources x 2 tasks, R: 2 risks x 3 resources (incompatible)
  sm <- matrix(c(1, 0, 0, 1), nrow = 2, ncol = 2)
  rm <- matrix(c(1, 2, 3, 4, 5, 6), nrow = 2, ncol = 3)

  expect_error(grandparent_dsm(sm, rm), "Number of rows in S.*must equal.*number of columns in R")
})

# ============================================================================
# NULL input validation
# ============================================================================

test_that("parent_dsm handles NULL input correctly", {
  expect_error(parent_dsm(NULL), "S must not be NULL")
})

test_that("grandparent_dsm handles NULL inputs correctly", {
  sm <- matrix(c(1, 0, 0, 1), nrow = 2, ncol = 2)
  rm <- matrix(c(1, 2, 3, 4), nrow = 2, ncol = 2)

  expect_error(grandparent_dsm(NULL, rm), "S and R must not be NULL")
  expect_error(grandparent_dsm(sm, NULL), "S and R must not be NULL")
})

# ============================================================================
# Non-matrix input validation
# ============================================================================

test_that("parent_dsm handles non-matrix input correctly", {
  expect_error(parent_dsm("not a matrix"), "S must be a matrix or data frame")
  expect_error(parent_dsm(c(1, 2, 3, 4)), "S must be a matrix or data frame")
})

test_that("grandparent_dsm handles non-matrix inputs correctly", {
  sm <- matrix(c(1, 0, 0, 1), nrow = 2, ncol = 2)

  expect_error(grandparent_dsm("not a matrix", sm), "S must be a matrix or data frame")
  expect_error(grandparent_dsm(sm, "not a matrix"), "R must be a matrix or data frame")
})

# ============================================================================
# Non-numeric input validation
# ============================================================================

test_that("parent_dsm handles non-numeric matrix correctly", {
  char_matrix <- matrix(c("a", "b", "c", "d"), nrow = 2, ncol = 2)
  expect_error(parent_dsm(char_matrix), "S must contain numeric values")
})

test_that("grandparent_dsm handles non-numeric matrices correctly", {
  sm <- matrix(c(1, 0, 0, 1), nrow = 2, ncol = 2)
  char_matrix <- matrix(c("a", "b", "c", "d"), nrow = 2, ncol = 2)

  expect_error(grandparent_dsm(char_matrix, sm), "S and R must contain numeric values")
  expect_error(grandparent_dsm(sm, char_matrix), "S and R must contain numeric values")
})

# ============================================================================
# Data frame input (should work directly)
# ============================================================================

test_that("parent_dsm handles data frame input correctly", {
  df <- data.frame(a = c(1, 0), b = c(0, 1))
  result <- parent_dsm(df)
  expect_s3_class(result, "dsm")
  expect_true(is.matrix(result$matrix))
})

test_that("grandparent_dsm handles data frame inputs correctly", {
  sm <- data.frame(a = c(1, 0), b = c(0, 1))
  rm <- data.frame(a = c(1, 2), b = c(3, 4))
  result <- grandparent_dsm(sm, rm)
  expect_s3_class(result, "dsm")
  expect_true(is.matrix(result$matrix))
})

# ============================================================================
# NaN/NA/Inf Error Tests (G5.2, G5.2b)
# ============================================================================

test_that("parent_dsm rejects NaN values in S", {
  sm <- matrix(c(1, NaN, 0, 1), nrow = 2, ncol = 2)
  expect_error(parent_dsm(sm), "S must not contain NaN values")
})

test_that("parent_dsm rejects NA values in S", {
  sm <- matrix(c(1, NA_real_, 0, 1), nrow = 2, ncol = 2)
  expect_error(parent_dsm(sm), "S must not contain NA values")
})

test_that("parent_dsm rejects Inf values in S", {
  sm <- matrix(c(1, Inf, 0, 1), nrow = 2, ncol = 2)
  expect_error(parent_dsm(sm), "S must not contain infinite values")
})

test_that("grandparent_dsm rejects NaN values in S or R", {
  sm <- matrix(c(1, NaN, 0, 1), nrow = 2, ncol = 2)
  rm <- matrix(c(1, 2, 3, 4), nrow = 2, ncol = 2)
  expect_error(grandparent_dsm(sm, rm), "S and R must not contain NaN values")
})

test_that("grandparent_dsm rejects NA values in S or R", {
  sm <- matrix(c(1, NA_real_, 0, 1), nrow = 2, ncol = 2)
  rm <- matrix(c(1, 2, 3, 4), nrow = 2, ncol = 2)
  expect_error(grandparent_dsm(sm, rm), "S and R must not contain NA values")
})

test_that("grandparent_dsm rejects Inf values in S or R", {
  sm <- matrix(c(1, Inf, 0, 1), nrow = 2, ncol = 2)
  rm <- matrix(c(1, 2, 3, 4), nrow = 2, ncol = 2)
  expect_error(grandparent_dsm(sm, rm), "S and R must not contain infinite values")
})

# ============================================================================
# G5.3: Return value tests
# ============================================================================

test_that("parent_dsm result contains no NA, NaN, or Inf", {
  sm <- matrix(c(1, 2, 3, 4, 5, 6, 7, 8, 9), nrow = 3, ncol = 3)
  result <- parent_dsm(sm)
  expect_false(anyNA(result$matrix))
  expect_false(any(is.nan(result$matrix)))
  expect_false(any(is.infinite(result$matrix)))
})

test_that("grandparent_dsm result contains no NA, NaN, or Inf", {
  sm <- matrix(c(1, 2, 3, 4, 5, 6, 7, 8, 9), nrow = 3, ncol = 3)
  rm <- matrix(c(1, 2, 3, 4, 5, 6, 7, 8, 9), nrow = 3, ncol = 3)
  result <- grandparent_dsm(sm, rm)
  expect_false(anyNA(result$matrix))
  expect_false(any(is.nan(result$matrix)))
  expect_false(any(is.infinite(result$matrix)))
})

# ============================================================================
# Parameter Recovery Tests (G5.6, G5.6a)
# ============================================================================

test_that("parent_dsm recovers known matrix properties", {
  # Identity matrix: t(I) %*% I = I
  S <- matrix(c(1, 0, 0, 1), nrow = 2)
  result <- parent_dsm(S)

  expected <- matrix(c(1, 0, 0, 1), nrow = 2)
  expect_equal(result$matrix, expected, tolerance = 1e-10)
})

test_that("parent_dsm diagonal equals column sums of squares", {
  # S: 3 resources x 3 tasks
  S <- matrix(c(1, 0, 1, 0, 1, 0, 1, 0, 1), nrow = 3, ncol = 3)
  result <- parent_dsm(S)

  # Diagonal of t(S) %*% S equals column sums of squares (resources per task)
  # Col 1: (1,0,1) -> 1+0+1 = 2, Col 2: (0,1,0) -> 0+1+0 = 1, Col 3: (1,0,1) -> 1+0+1 = 2
  expect_equal(diag(result$matrix), c(2, 1, 2), tolerance = 1e-10)
})

test_that("parent_dsm is symmetric", {
  S <- matrix(c(1, 1, 0, 0, 1, 1, 1, 0, 1), nrow = 3, ncol = 3)
  result <- parent_dsm(S)

  # t(S) %*% S is always symmetric
  expect_equal(result$matrix, t(result$matrix), tolerance = 1e-10)
})

test_that("grandparent_dsm recovers known shared risks", {
  S <- matrix(c(1, 0, 0, 1), nrow = 2, ncol = 2)
  R <- matrix(c(1, 1, 0, 0), nrow = 2, ncol = 2)
  result <- grandparent_dsm(S, R)

  expect_s3_class(result, "dsm")
  expect_true(is.matrix(result$matrix))
  expect_equal(nrow(result$matrix), 2)
  expect_equal(ncol(result$matrix), 2)
})

test_that("grandparent_dsm is symmetric", {
  S <- matrix(c(1, 1, 0, 0, 1, 1, 1, 0, 1), nrow = 3, ncol = 3)
  R <- matrix(c(1, 0, 1, 0, 1, 0, 1, 0, 1), nrow = 3, ncol = 3)
  result <- grandparent_dsm(S, R)

  # Result must be symmetric
  expect_equal(result$matrix, t(result$matrix), tolerance = 1e-10)
})

# ============================================================================
# Edge Condition Tests (G5.8a) - Zero-Length Data
# ============================================================================

test_that("parent_dsm handles zero-dimension matrix", {
  empty_matrix <- matrix(numeric(0), nrow = 0, ncol = 0)
  result <- parent_dsm(empty_matrix)

  expect_s3_class(result, "dsm")
  expect_true(is.matrix(result$matrix))
  expect_equal(nrow(result$matrix), 0)
  expect_equal(ncol(result$matrix), 0)
})

test_that("grandparent_dsm handles zero-dimension matrices", {
  empty_s <- matrix(numeric(0), nrow = 0, ncol = 0)
  empty_r <- matrix(numeric(0), nrow = 0, ncol = 0)
  result <- grandparent_dsm(empty_s, empty_r)

  expect_s3_class(result, "dsm")
  expect_equal(nrow(result$matrix), 0)
  expect_equal(ncol(result$matrix), 0)
})

# ============================================================================
# Edge Condition Tests (G5.8c) - All-Zero/All-One Matrices
# ============================================================================

test_that("parent_dsm handles all-zero matrix", {
  S <- matrix(0, nrow = 3, ncol = 3)
  result <- parent_dsm(S)

  expect_true(all(result$matrix == 0))
})

test_that("parent_dsm handles all-one matrix", {
  S <- matrix(1, nrow = 3, ncol = 3)
  result <- parent_dsm(S)

  expect_s3_class(result, "dsm")
  expect_true(is.matrix(result$matrix))
  expect_equal(nrow(result$matrix), 3)
  expect_equal(ncol(result$matrix), 3)
})

# ============================================================================
# Edge Condition Tests (G5.8d) - 1x1 Matrix
# ============================================================================

test_that("parent_dsm handles 1x1 matrix", {
  S <- matrix(5, nrow = 1, ncol = 1)
  result <- parent_dsm(S)

  expect_s3_class(result, "dsm")
  expect_equal(result$matrix, matrix(25, 1, 1))
})

test_that("grandparent_dsm requires compatible dimensions", {
  S <- matrix(c(1, 0, 0, 1), nrow = 2, ncol = 2)
  R_wrong <- matrix(c(1, 0, 1), nrow = 1, ncol = 3)

  expect_error(
    grandparent_dsm(S, R_wrong),
    "Number of rows in S.*must equal.*number of columns in R"
  )
})

# ============================================================================
# Noise Susceptibility Tests (G5.9a) - Trivial Noise
# ============================================================================

test_that("parent_dsm is stable to trivial noise in matrix values", {
  S <- matrix(c(1, 2, 3, 4, 5, 6, 7, 8, 9), nrow = 3, ncol = 3)
  result_clean <- parent_dsm(S)

  # Add trivial noise
  S_noisy <- S + matrix(runif(9, -.Machine$double.eps, .Machine$double.eps), nrow = 3)
  result_noisy <- parent_dsm(S_noisy)

  expect_equal(result_clean$matrix, result_noisy$matrix, tolerance = 100 * .Machine$double.eps)
})

test_that("grandparent_dsm is stable to trivial noise in matrix values", {
  S <- matrix(c(1, 2, 3, 4, 5, 6, 7, 8, 9), nrow = 3, ncol = 3)
  R <- matrix(c(1, 0, 1, 0, 1, 0, 1, 0, 1), nrow = 3, ncol = 3)
  result_clean <- grandparent_dsm(S, R)

  S_noisy <- S + matrix(runif(9, -.Machine$double.eps, .Machine$double.eps), nrow = 3)
  R_noisy <- R + matrix(runif(9, -.Machine$double.eps, .Machine$double.eps), nrow = 3)
  result_noisy <- grandparent_dsm(S_noisy, R_noisy)

  expect_equal(result_clean$matrix, result_noisy$matrix, tolerance = 100 * .Machine$double.eps)
})

# ============================================================================
# S3 class and methods
# ============================================================================

test_that("parent_dsm returns dsm S3 class with correct fields", {
  S <- matrix(c(1, 0, 1, 0, 1, 0), nrow = 2, ncol = 3)
  result <- parent_dsm(S)

  expect_s3_class(result, "dsm")
  expect_equal(result$type, "parent")
  expect_equal(result$n_tasks, 3)
  expect_equal(result$n_resources, 2)
  expect_null(result$n_risks)
})

test_that("grandparent_dsm returns dsm S3 class with correct fields", {
  S <- matrix(c(1, 0, 1, 0, 1, 0), nrow = 2, ncol = 3)
  R <- matrix(c(1, 1, 0, 1), nrow = 2, ncol = 2)
  result <- grandparent_dsm(S, R)

  expect_s3_class(result, "dsm")
  expect_equal(result$type, "grandparent")
  expect_equal(result$n_tasks, 3)
  expect_equal(result$n_resources, 2)
  expect_equal(result$n_risks, 2)
})

test_that("print.dsm works for parent DSM", {
  S <- matrix(c(1, 0, 0, 1), nrow = 2, ncol = 2)
  result <- parent_dsm(S)

  output <- capture.output(print(result))
  expect_true(any(grepl("Parent", output)))
  expect_true(any(grepl("Tasks:", output)))
})

test_that("print.dsm works for grandparent DSM", {
  S <- matrix(c(1, 0, 0, 1), nrow = 2, ncol = 2)
  R <- matrix(c(1, 1, 0, 1), nrow = 2, ncol = 2)
  result <- grandparent_dsm(S, R)

  output <- capture.output(print(result))
  expect_true(any(grepl("Grandparent", output)))
  expect_true(any(grepl("Risks:", output)))
})

test_that("plot.dsm works for parent DSM", {
  S <- matrix(c(1, 0, 1, 0, 1, 0, 1, 0, 1), nrow = 3, ncol = 3)
  result <- parent_dsm(S)

  expect_no_error(plot(result))
})

test_that("plot.dsm works for grandparent DSM", {
  S <- matrix(c(1, 0, 0, 1), nrow = 2, ncol = 2)
  R <- matrix(c(1, 1, 0, 1), nrow = 2, ncol = 2)
  result <- grandparent_dsm(S, R)

  expect_no_error(plot(result))
})

test_that("plot.dsm handles zero-dimension DSM", {
  empty_matrix <- matrix(numeric(0), nrow = 0, ncol = 0)
  result <- parent_dsm(empty_matrix)

  expect_message(plot(result), "Nothing to plot")
})

#' Monte Carlo Simulation.
#'
#' This function performs a Monte Carlo simulation to estimate the total duration of a project
#' based on individual task distributions and an optional correlation matrix.
#'
#' @srrstats {G1.0} *Software lists primary reference from published academic literature.*
#' @srrstats {G1.1} *Software is the first implementation within **R** of the algorithm which has previously been implemented in other languages or contexts.*
#' @srrstats {G1.4} *Software uses [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {G2.0} *Implements assertions on lengths of inputs - validates num_sims is single value and cor_mat dimensions match tasks.*
#' @srrstats {G2.0a} *Parameter documentation explicitly states expected input structure.*
#' @srrstats {G2.1} *Implements assertions on types of inputs via is.numeric(), is.list(), and is.matrix() checks.*
#' @srrstats {G2.1a} *Parameter documentation explicitly states data types expected.*
#' @srrstats {G2.2} *Prohibits multivariate input for num_sims which must be a single positive integer.*
#' @srrstats {G2.4a} *Uses as.integer() for integer comparison of num_sims.*
#' @srrstats {G2.13} *Implements checks for NA values in num_sims.*
#' @srrstats {G2.14a} *Errors on missing data with informative message.*
#' @srrstats {G2.15} *Implements checks for NaN values via is.nan() prior to processing.*
#' @srrstats {G2.16} *Implements checks for Inf/-Inf values via is.infinite() prior to processing.*
#' @srrstats {G3.1} *Correlation handling is user-controlled via optional cor_mat parameter.*
#' @srrstats {G3.1a} *Documentation describes usage of correlation matrix in examples.*
#' @srrstats {G5.2a} *Each error message produced by stop() is unique.*
#'
#' @param num_sims The number of simulations to run.
#' @param task_dists A list of lists describing each task distribution with its parameters.
#' Each task distribution should be specified as a list with a "type" field (indicating
#' the distribution type: "normal", "triangular", or "uniform") and the corresponding
#' parameters: for "normal" (mean, sd), for "triangular" (a, b, c), and for "uniform"
#' (min, max). For example:
#' list(
#'  list(type = "normal", mean = 10, sd = 2),
#'  list(type = "triangular", a = 5, b = 10, c = 15),
#'  list(type = "uniform", min = 8, max = 12)
#'  )
#' @param cor_mat The correlation matrix for the tasks (Optional). If not provided,
#' tasks are assumed to be independent.
#' @return The function returns a list of the total mean, variance, standard deviation,
#' and percentiles for the project.
#' @references
#' Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
#' construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
#' @examples
#' # Set the number of simulations and task distributions for a toy project.
#' num_sims <- 10000
#' task_dists <- list(
#'   list(type = "normal", mean = 10, sd = 2), # Task A: Normal distribution
#'   list(type = "triangular", a = 5, b = 10, c = 15), # Task B: Triangular distribution
#'   list(type = "uniform", min = 8, max = 12) # Task C: Uniform distribution
#' )
#'
#' # Set the correlation matrix for the correlations between tasks.
#' cor_mat <- matrix(c(
#'   1, 0.5, 0.3,
#'   0.5, 1, 0.4,
#'   0.3, 0.4, 1
#' ), nrow = 3, byrow = TRUE)
#'
#' # Run the Monte Carlo sumulation and print the results.
#' results <- mcs(num_sims, task_dists, cor_mat)
#' cat("Mean Total Duration:", results$total_mean, "\n")
#' cat("Variance of Total Variance:", results$total_variance, "\n")
#' cat("Standard Deviation of Total Duration:", results$total_sd, "\n")
#' cat("5th Percentile:", results$percentiles[1], "\n")
#' cat("Median (50th Percentile):", results$percentiles[2], "\n")
#' cat("95th Percentile:", results$percentiles[3], "\n")
#' hist(results$total_distribution,
#'   breaks = 50, main = "Distribution of Total Project Duration",
#'   xlab = "Total Duration", col = "skyblue", border = "white"
#' )
#' legend("topright", legend = c("Total Duration Distribution"), fill = c("skyblue"))
#'
#' @importFrom mc2d rtriang
#' @importFrom stats rnorm runif var sd quantile
#' @export

# Monte Carlo Simulation
mcs <- function(num_sims, task_dists, cor_mat = NULL) {
  # Error handling
  if (is.null(num_sims) || is.null(task_dists)) {
    stop("num_sims and task_dists must not be NULL")
  }
  if (!is.numeric(num_sims) || length(num_sims) != 1) {
    stop("num_sims must be a single positive integer")
  }
  if (is.nan(num_sims)) {
    stop("num_sims must not be NaN")
  }
  if (is.na(num_sims)) {
    stop("num_sims must not be NA")
  }
  if (is.infinite(num_sims)) {
    stop("num_sims must not be infinite")
  }
  if (num_sims <= 0 || num_sims != as.integer(num_sims)) {
    stop("num_sims must be a positive integer")
  }
  if (!is.list(task_dists) || length(task_dists) == 0) {
    stop("task_dists must be a non-empty list")
  }

  num_tasks <- length(task_dists)

  # Generate uncorrelated random samples for each task based on the specified distributions
  uncorrelated_samples <- matrix(NA, nrow = num_sims, ncol = num_tasks)
  for (i in seq_along(task_dists)) {
    dist <- task_dists[[i]]
    if (dist$type == "normal") {
      uncorrelated_samples[, i] <- stats::rnorm(num_sims, mean = dist$mean, sd = dist$sd)
    } else if (dist$type == "triangular") {
      uncorrelated_samples[, i] <- mc2d::rtriang(num_sims, min = dist$a, mode = dist$b, max = dist$c)
    } else if (dist$type == "uniform") {
      uncorrelated_samples[, i] <- stats::runif(num_sims, min = dist$min, max = dist$max)
    } else {
      stop("Unsupported distribution type.")
    }
  }

  # Apply Cholesky decomposition to the correlation matrix if provided
  if (!is.null(cor_mat)) {
    if (!is.matrix(cor_mat) || nrow(cor_mat) != num_tasks || ncol(cor_mat) != num_tasks) {
      stop("The correlation matrix must be square and match the number of tasks.")
    }
    if (any(is.nan(cor_mat))) {
      stop("cor_mat must not contain NaN values")
    }
    if (anyNA(cor_mat)) {
      stop("cor_mat must not contain NA values")
    }
    if (any(is.infinite(cor_mat))) {
      stop("cor_mat must not contain infinite values")
    }
    cholesky_decomp <- chol(cor_mat)
    correlated_samples <- uncorrelated_samples %*% cholesky_decomp
  } else {
    correlated_samples <- uncorrelated_samples
  }

  # Calculate total project duration for each simulation
  total_distribution <- rowSums(correlated_samples)

  # Analyze the results
  total_mean <- mean(total_distribution)
  total_variance <- stats::var(total_distribution)
  total_sd <- stats::sd(total_distribution)
  percentiles <- stats::quantile(total_distribution, probs = c(0.05, 0.50, 0.95))

  # Create a list to return the results
  result <- list(
    total_mean = total_mean,
    total_variance = total_variance,
    total_sd = total_sd,
    percentiles = percentiles,
    total_distribution = total_distribution
  )

  class(result) <- "mcs"
  return(result)
}

#' Print method for Monte Carlo Simulation results.
#'
#' Displays the total mean, variance, standard deviation, and percentiles of the
#' Monte Carlo Simulation results in a readable format.
#' @param x An object of class "mcs".
#' @param ... Additional arguments (not used).
#' @return None. Prints the results to the console.
#' @examples
#' # Set the number of simulations and task distributions for a toy project.
#' num_sims <- 10000
#' task_dists <- list(
#'   list(type = "normal", mean = 10, sd = 2), # Task A: Normal distribution
#'   list(type = "triangular", a = 5, b = 10, c = 15), # Task B: Triangular distribution
#'   list(type = "uniform", min = 8, max = 12) # Task C: Uniform distribution
#' )
#'
#' # Set the correlation matrix for the correlations between tasks.
#' cor_mat <- matrix(c(
#'   1, 0.5, 0.3,
#'   0.5, 1, 0.4,
#'   0.3, 0.4, 1
#' ), nrow = 3, byrow = TRUE)
#'
#' # Run the Monte Carlo sumulation and print the results.
#' results <- mcs(num_sims, task_dists, cor_mat)
#' # print(results)
#' @export
print.mcs <- function(x, ...) {
  cat("Monte Carlo Simulation Results:\n")
  cat("Total Mean:", x$total_mean, "\n")
  cat("Total Variance:", x$total_variance, "\n")
  cat("Total Standard Deviation:", x$total_sd, "\n")
  cat("Percentiles:\n")
  print(x$percentiles)
}

#' @srrstats {G5.2} *Error and warning behaviour is explicitly demonstrated through tests.*
#' @srrstats {G5.2a} *Every error message is unique and tested.*
#' @srrstats {G5.2b} *Tests trigger every error message and compare with expected values.*
#' @srrstats {G5.3} *Return objects tested for absence of NA, NaN, Inf.*
#' @srrstats {G5.6} *Parameter recovery tests verify implementations produce expected results given data with known properties.*
#' @srrstats {G5.6a} *Parameter recovery tests succeed within defined tolerance rather than exact values.*
#' @srrstats {G5.6b} *Parameter recovery tests run with multiple random seeds when randomness is involved.*
#' @srrstats {G5.7} *Algorithm performance tests verify implementations perform correctly as data properties change.*
#' @srrstats {G5.8} *Edge condition tests verify appropriate behavior with extreme data properties.*
#' @srrstats {G5.8a} *Zero-length data tests trigger clear errors.*
#' @srrstats {G5.8b} *Unsupported data type tests trigger clear errors.*
#' @srrstats {G5.8c} *All-NA and all-identical data tests trigger clear errors or warnings.*
#' @srrstats {G5.8d} *Out-of-scope data tests verify appropriate behavior.*
#' @srrstats {G5.9} *Noise susceptibility tests verify stochastic behavior stability.*
#' @srrstats {G5.9a} *Trivial noise tests show results are stable at machine epsilon scale.*
#' @srrstats {G5.9b} *Random seed stability tests show consistent behavior across different seeds.*

# Define a helper function to generate simulation results for testing
generate_simulation_results <- function() {
  set.seed(123) # Set seed for reproducibility
  num_sims <- 1000
  total_distribution <- rnorm(num_sims, mean = 100, sd = 15)
  list(total_distribution = total_distribution)
}

test_that("contingency function calculates correctly with default percentiles", {
  sims <- generate_simulation_results()
  result <- contingency(sims)
  expect_true(is.numeric(result))

  # Calculate expected values
  expected_phigh_value <- quantile(sims$total_distribution, probs = 0.95)
  expected_pbase_value <- quantile(sims$total_distribution, probs = 0.50)
  expected_contingency <- expected_phigh_value - expected_pbase_value

  expect_equal(result, expected_contingency)
})

test_that("contingency function calculates correctly with custom percentiles", {
  sims <- generate_simulation_results()
  result <- contingency(sims, phigh = 0.90, pbase = 0.10)
  expect_true(is.numeric(result))

  # Calculate expected values
  expected_phigh_value <- quantile(sims$total_distribution, probs = 0.90)
  expected_pbase_value <- quantile(sims$total_distribution, probs = 0.10)
  expected_contingency <- expected_phigh_value - expected_pbase_value

  expect_equal(result, expected_contingency)
})

test_that("contingency function handles edge cases correctly", {
  sims <- generate_simulation_results()

  # phigh is less than pbase (uncommon but should be tested)
  expect_error(contingency(sims, phigh = 0.25, pbase = 0.75), "phigh must be greater than pbase")
})

test_that("contingency function handles invalid input correctly", {
  sims <- generate_simulation_results()

  # Invalid percentile values
  expect_error(contingency(sims, phigh = 1.5, pbase = 0.50), "phigh must be between 0 and 1")
  expect_error(contingency(sims, phigh = 0.95, pbase = -0.5), "pbase must be between 0 and 1")

  # Non-numeric percentiles
  expect_error(contingency(sims, phigh = "high", pbase = 0.50), "phigh must be between 0 and 1")
  expect_error(contingency(sims, phigh = 0.95, pbase = "base"), "pbase must be between 0 and 1")
})

# ============================================================================
# NaN/NA/Inf Error Tests (G5.2, G5.2b)
# ============================================================================
test_that("contingency rejects NaN inputs", {
  sims <- generate_simulation_results()
  expect_error(contingency(sims, phigh = NaN, pbase = 0.50), "phigh must not be NaN")
  expect_error(contingency(sims, phigh = 0.95, pbase = NaN), "pbase must not be NaN")
})

test_that("contingency rejects NA inputs", {
  sims <- generate_simulation_results()
  expect_error(contingency(sims, phigh = NA_real_, pbase = 0.50), "phigh must not be NA")
  expect_error(contingency(sims, phigh = 0.95, pbase = NA_real_), "pbase must not be NA")
})

test_that("contingency rejects Inf inputs", {
  sims <- generate_simulation_results()
  expect_error(contingency(sims, phigh = Inf, pbase = 0.50), "phigh must not be infinite")
  expect_error(contingency(sims, phigh = 0.95, pbase = Inf), "pbase must not be infinite")
})

# ============================================================================
# G5.3: Return value tests
# ============================================================================
test_that("contingency result contains no NA, NaN, or Inf", {
  sims <- generate_simulation_results()
  result <- contingency(sims)
  expect_false(is.na(result))
  expect_false(is.nan(result))
  expect_false(is.infinite(result))
})

# ============================================================================
# Parameter Recovery Tests (G5.6, G5.6a)
# ============================================================================

test_that("contingency recovers known percentile difference", {
  set.seed(123)

  # Create known MCS result with specific percentiles
  num_sims <- 10000
  task_dists <- list(list(type = "normal", mean = 100, sd = 10))
  mcs_result <- mcs(num_sims, task_dists)

  result <- contingency(mcs_result, phigh = 0.95, pbase = 0.50)

  # Should return difference between 95th and 50th percentiles
  expected <- mcs_result$percentiles[3] - mcs_result$percentiles[2]

  expect_equal(result, expected, tolerance = 1e-10)
})

# ============================================================================
# Edge Condition Tests (G5.8c) - All-Identical Data
# ============================================================================

test_that("contingency handles all identical values in distribution", {
  set.seed(123)

  # Create MCS result with constant values
  num_sims <- 1000
  constant_dist <- rep(100, num_sims)
  sims <- list(total_distribution = constant_dist)
  class(sims) <- "mcs"

  result <- contingency(sims, phigh = 0.95, pbase = 0.50)

  # Contingency should be 0 since all values are identical
  expect_equal(as.numeric(result), 0, tolerance = 1e-10)
})

#' @srrstats {G5.2} *Error and warning behaviour is explicitly demonstrated through tests.*
#' @srrstats {G5.2a} *Every error message is unique and tested.*
#' @srrstats {G5.2b} *Tests trigger every error message and compare with expected values.*
#' @srrstats {G5.3} *Return objects tested for absence of NA, NaN, Inf.*
#' @srrstats {G5.6} *Parameter recovery tests verify implementations produce expected results given data with known properties.*
#' @srrstats {G5.6a} *Parameter recovery tests succeed within defined tolerance rather than exact values.*
#' @srrstats {G5.6b} *Parameter recovery tests run with multiple random seeds when randomness is involved.*
#' @srrstats {G5.7} *Algorithm performance tests verify implementations perform correctly as data properties change.*
#' @srrstats {G5.8} *Edge condition tests verify appropriate behavior with extreme data properties.*
#' @srrstats {G5.8a} *Zero-length data tests trigger clear errors.*
#' @srrstats {G5.8b} *Unsupported data type tests trigger clear errors.*
#' @srrstats {G5.8c} *All-NA and all-identical data tests trigger clear errors or warnings.*
#' @srrstats {G5.8d} *Out-of-scope data tests verify appropriate behavior.*
#' @srrstats {G5.9} *Noise susceptibility tests verify stochastic behavior stability.*
#' @srrstats {G5.9a} *Trivial noise tests show results are stable at machine epsilon scale.*
#' @srrstats {G5.9b} *Random seed stability tests show consistent behavior across different seeds.*

test_that("mcs function works correctly with different distribution types", {
  set.seed(123) # Set seed for reproducibility

  # Test with normal distribution
  normal_dist <- list(list(type = "normal", mean = 10, sd = 2))
  result <- mcs(1000, normal_dist)
  expect_true(is.list(result))

  # Test with triangular distribution
  triangular_dist <- list(list(type = "triangular", a = 5, b = 10, c = 15))
  result <- mcs(1000, triangular_dist)
  expect_true(is.list(result))

  # Test with uniform distribution
  uniform_dist <- list(list(type = "uniform", min = 5, max = 15))
  result <- mcs(1000, uniform_dist)
  expect_true(is.list(result))
})

test_that("mcs function handles correlation matrix correctly", {
  set.seed(123) # Set seed for reproducibility

  # Test with correlation matrix
  normal_dist <- list(
    list(type = "normal", mean = 10, sd = 2),
    list(type = "normal", mean = 20, sd = 3)
  )
  cor_mat <- matrix(c(1, 0.8, 0.8, 1), nrow = 2)
  result <- mcs(1000, normal_dist, cor_mat)
  expect_true(is.list(result))
})

test_that("mcs function calculates statistics correctly", {
  set.seed(123) # Set seed for reproducibility

  # Test with normal distribution
  normal_dist <- list(list(type = "normal", mean = 10, sd = 2))
  result <- mcs(1000, normal_dist)
  expect_true(is.numeric(result$total_mean))
  expect_true(is.numeric(result$total_variance))
  expect_true(is.numeric(result$total_sd))
  expect_true(is.numeric(result$percentiles))
})

test_that("mcs function handles errors correctly", {
  set.seed(123) # Set seed for reproducibility

  # Test unsupported distribution type
  unsupported_dist <- list(list(type = "unsupported", mean = 10, sd = 2))
  expect_error(mcs(1000, unsupported_dist), "Unsupported distribution type.")

  # Test incorrect correlation matrix
  normal_dist <- list(
    list(type = "normal", mean = 10, sd = 2),
    list(type = "normal", mean = 20, sd = 3)
  )
  incorrect_cor_mat <- matrix(c(1, 0.8), nrow = 1)
  expect_error(mcs(1000, normal_dist, incorrect_cor_mat), "The correlation matrix must be square and match the number of tasks.")
})

test_that("mcs function validates num_sims correctly", {
  normal_dist <- list(list(type = "normal", mean = 10, sd = 2))

  # Test NULL num_sims
  expect_error(mcs(NULL, normal_dist), "num_sims and task_dists must not be NULL")

  # Test non-numeric num_sims
  expect_error(mcs("1000", normal_dist), "num_sims must be a single positive integer")

  # Test negative num_sims
  expect_error(mcs(-100, normal_dist), "num_sims must be a positive integer")

  # Test zero num_sims
  expect_error(mcs(0, normal_dist), "num_sims must be a positive integer")

  # Test non-integer num_sims
  expect_error(mcs(100.5, normal_dist), "num_sims must be a positive integer")

  # Test NA num_sims
  expect_error(mcs(NA, normal_dist), "num_sims must be a single positive integer")
})

test_that("mcs function validates task_dists correctly", {
  # Test NULL task_dists
  expect_error(mcs(1000, NULL), "num_sims and task_dists must not be NULL")

  # Test empty task_dists
  expect_error(mcs(1000, list()), "task_dists must be a non-empty list")

  # Test non-list task_dists
  expect_error(mcs(1000, "not a list"), "task_dists must be a non-empty list")
})

# ============================================================================
# NaN/NA/Inf Error Tests (G5.2, G5.2b)
# ============================================================================
test_that("mcs rejects NaN num_sims", {
  normal_dist <- list(list(type = "normal", mean = 10, sd = 2))
  expect_error(mcs(NaN, normal_dist), "num_sims must not be NaN")
})

test_that("mcs rejects NA_real_ num_sims", {
  normal_dist <- list(list(type = "normal", mean = 10, sd = 2))
  expect_error(mcs(NA_real_, normal_dist), "num_sims must not be NA")
})

test_that("mcs rejects Inf num_sims", {
  normal_dist <- list(list(type = "normal", mean = 10, sd = 2))
  expect_error(mcs(Inf, normal_dist), "num_sims must not be infinite")
})

test_that("mcs rejects NaN in cor_mat", {
  normal_dist <- list(
    list(type = "normal", mean = 10, sd = 2),
    list(type = "normal", mean = 20, sd = 3)
  )
  cor_mat <- matrix(c(1, NaN, NaN, 1), nrow = 2)
  expect_error(mcs(100, normal_dist, cor_mat), "cor_mat must not contain NaN values")
})

test_that("mcs rejects NA in cor_mat", {
  normal_dist <- list(
    list(type = "normal", mean = 10, sd = 2),
    list(type = "normal", mean = 20, sd = 3)
  )
  cor_mat <- matrix(c(1, NA, NA, 1), nrow = 2)
  expect_error(mcs(100, normal_dist, cor_mat), "cor_mat must not contain NA values")
})

test_that("mcs rejects Inf in cor_mat", {
  normal_dist <- list(
    list(type = "normal", mean = 10, sd = 2),
    list(type = "normal", mean = 20, sd = 3)
  )
  cor_mat <- matrix(c(1, Inf, Inf, 1), nrow = 2)
  expect_error(mcs(100, normal_dist, cor_mat), "cor_mat must not contain infinite values")
})

# ============================================================================
# G5.3: Return value tests
# ============================================================================
test_that("mcs result components contain no NA, NaN, or Inf", {
  set.seed(42)
  normal_dist <- list(list(type = "normal", mean = 10, sd = 2))
  result <- mcs(100, normal_dist)

  expect_false(is.na(result$total_mean))
  expect_false(is.nan(result$total_mean))
  expect_false(is.infinite(result$total_mean))

  expect_false(is.na(result$total_variance))
  expect_false(is.nan(result$total_variance))
  expect_false(is.infinite(result$total_variance))

  expect_false(is.na(result$total_sd))
  expect_false(is.nan(result$total_sd))
  expect_false(is.infinite(result$total_sd))

  expect_false(anyNA(result$percentiles))
  expect_false(any(is.nan(result$percentiles)))
  expect_false(any(is.infinite(result$percentiles)))

  expect_false(anyNA(result$total_distribution))
  expect_false(any(is.nan(result$total_distribution)))
  expect_false(any(is.infinite(result$total_distribution)))
})

test_that("mcs print method works correctly", {
  set.seed(123)
  normal_dist <- list(list(type = "normal", mean = 10, sd = 2))
  result <- mcs(100, normal_dist)

  expect_output(print(result), "Monte Carlo Simulation Results:")
  expect_output(print(result), "Total Mean:")
  expect_output(print(result), "Total Variance:")
})

# ============================================================================
# Parameter Recovery Tests (G5.6, G5.6a, G5.6b)
# ============================================================================

test_that("mcs converges to theoretical mean for normal distribution (seed 123)", {
  set.seed(123)

  # Large sample size for convergence
  num_sims <- 100000
  task_dists <- list(
    list(type = "normal", mean = 10, sd = 2),
    list(type = "normal", mean = 15, sd = 3)
  )

  result <- mcs(num_sims, task_dists)

  # Expected mean = sum of individual means (no correlation)
  expected_mean <- 10 + 15

  # With large N, sample mean should converge to theoretical mean
  expect_equal(result$total_mean, expected_mean, tolerance = 0.1)
})

test_that("mcs converges to theoretical mean for normal distribution (seed 42)", {
  set.seed(42)

  # Same test with different seed (G5.6b)
  num_sims <- 100000
  task_dists <- list(
    list(type = "normal", mean = 10, sd = 2),
    list(type = "normal", mean = 15, sd = 3)
  )

  result <- mcs(num_sims, task_dists)
  expected_mean <- 10 + 15

  expect_equal(result$total_mean, expected_mean, tolerance = 0.1)
})

test_that("mcs converges to theoretical variance for independent tasks (seed 123)", {
  set.seed(123)

  num_sims <- 100000
  task_dists <- list(
    list(type = "normal", mean = 10, sd = 2),
    list(type = "normal", mean = 15, sd = 3)
  )

  result <- mcs(num_sims, task_dists)

  # Expected variance = sum of individual variances (independent)
  expected_var <- 2^2 + 3^2 # 4 + 9 = 13

  expect_equal(result$total_variance, expected_var, tolerance = 0.2)
})

test_that("mcs converges to theoretical variance for independent tasks (seed 789)", {
  set.seed(789)

  # Third seed for G5.6b coverage
  num_sims <- 100000
  task_dists <- list(
    list(type = "normal", mean = 10, sd = 2),
    list(type = "normal", mean = 15, sd = 3)
  )

  result <- mcs(num_sims, task_dists)
  expected_var <- 2^2 + 3^2

  expect_equal(result$total_variance, expected_var, tolerance = 0.2)
})

test_that("mcs with high correlation produces higher variance", {
  set.seed(123)

  num_sims <- 10000
  task_dists <- list(
    list(type = "normal", mean = 10, sd = 2),
    list(type = "normal", mean = 15, sd = 3)
  )
  # High correlation (0.99 instead of 1.0 to avoid singularity)
  cor_mat <- matrix(c(1, 0.99, 0.99, 1), nrow = 2)

  result <- mcs(num_sims, task_dists, cor_mat)

  # With high correlation, variance should be larger than independent case
  # Var(X + Y) = Var(X) + Var(Y) + 2*Cov(X,Y)
  # When cor=0.99: Cov(X,Y) ≈ SD(X)*SD(Y)*0.99 = 2*3*0.99 = 5.94
  expected_var_approx <- 4 + 9 + 2 * 5.94 # ≈ 24.88

  expect_equal(result$total_variance, expected_var_approx, tolerance = 0.5)
})

# ============================================================================
# Algorithm Performance Tests (G5.7)
# ============================================================================

test_that("mcs convergence improves with increased sample size", {
  # Use same seed for all to ensure consistent comparison
  task_dists <- list(list(type = "normal", mean = 100, sd = 10))

  set.seed(123)
  result_1k <- mcs(1000, task_dists)

  set.seed(123)
  result_10k <- mcs(10000, task_dists)

  set.seed(123)
  result_100k <- mcs(100000, task_dists)

  # Calculate error from theoretical mean
  error_1k <- abs(result_1k$total_mean - 100)
  error_10k <- abs(result_10k$total_mean - 100)
  error_100k <- abs(result_100k$total_mean - 100)

  # Error should generally decrease as sample size increases
  # Using <= to account for random variation
  expect_true(error_100k <= error_10k)
  expect_true(error_10k <= error_1k)
})

test_that("mcs variance estimates stabilize with sample size", {
  set.seed(42)

  task_dists <- list(list(type = "normal", mean = 50, sd = 5))

  # Multiple runs at different sample sizes
  n_sizes <- c(1000, 5000, 10000, 50000)
  variances <- sapply(n_sizes, function(n) {
    mcs(n, task_dists)$total_variance
  })

  # Variance estimates should converge to theoretical value (25)
  # Later estimates should be more stable
  expect_true(abs(variances[4] - 25) < abs(variances[1] - 25))
})

test_that("mcs percentile accuracy improves with sample size", {
  set.seed(789)

  task_dists <- list(list(type = "normal", mean = 0, sd = 1))

  result_1k <- mcs(1000, task_dists)
  result_100k <- mcs(100000, task_dists)

  # 50th percentile should be closer to 0 with larger sample
  error_1k <- abs(result_1k$percentiles[2] - 0)
  error_100k <- abs(result_100k$percentiles[2] - 0)

  expect_true(error_100k < error_1k)
})

# ============================================================================
# Edge Condition Tests (G5.8c) - All-Identical Data
# ============================================================================

test_that("mcs handles all identical distribution parameters", {
  set.seed(123)

  # All tasks with identical distributions
  task_dists <- list(
    list(type = "normal", mean = 10, sd = 2),
    list(type = "normal", mean = 10, sd = 2),
    list(type = "normal", mean = 10, sd = 2)
  )

  result <- mcs(1000, task_dists)

  # Should still produce valid results
  expect_true(is.numeric(result$total_mean))
  expect_false(is.na(result$total_mean))

  # Mean should be 3*10 = 30
  expect_equal(result$total_mean, 30, tolerance = 1)
})

# ============================================================================
# Noise Susceptibility Tests (G5.9b) - Random Seed Stability
# ============================================================================

test_that("mcs produces similar results across different seeds", {
  task_dists <- list(list(type = "normal", mean = 100, sd = 10))

  # Run with multiple different seeds
  set.seed(123)
  result_1 <- mcs(50000, task_dists)

  set.seed(456)
  result_2 <- mcs(50000, task_dists)

  set.seed(789)
  result_3 <- mcs(50000, task_dists)

  # All should converge to similar mean (within reasonable tolerance)
  expect_equal(result_1$total_mean, result_2$total_mean, tolerance = 1)
  expect_equal(result_2$total_mean, result_3$total_mean, tolerance = 1)

  # Variance estimates should also be similar
  expect_equal(result_1$total_variance, result_2$total_variance, tolerance = 10)
  expect_equal(result_2$total_variance, result_3$total_variance, tolerance = 10)
})

#' Launch the PRA Risk Analysis Agent App
#'
#' Starts a Shiny app that provides an interactive chat interface to the PRA
#' risk analysis agent. The agent can select and execute PRA tools (Monte Carlo
#' simulation, EVM, Bayesian inference, etc.) in response to natural language
#' questions. Uses shinychat for a polished streaming chat experience with
#' inline tool result display.
#'
#' Requires Ollama to be running locally with the specified model downloaded.
#'
#' @param model Character. Ollama model name (default `"llama3.2"`).
#' @param rag Logical. Whether to enable RAG context retrieval (default `TRUE`).
#' @param embed_model Character. Ollama embedding model for RAG (default
#'   `"nomic-embed-text"`).
#' @param port Integer. Port for the Shiny app (default `NULL` lets Shiny choose).
#' @param launch.browser Logical. Whether to open a browser (default `TRUE`).
#' @return None. This function is called to launch the shiny app.
#'
#' @examples
#' \dontrun{
#' # Ensure Ollama is running, then:
#' pra_app()
#'
#' # With a specific model:
#' pra_app(model = "qwen2.5")
#' }
#'
#' @export
pra_app <- function(model = "llama3.2", rag = TRUE, embed_model = "nomic-embed-text",
                    port = NULL, launch.browser = TRUE) {
  check_package("shiny")
  check_package("bslib")
  check_package("ellmer")
  check_package("shinychat")

  # Check Ollama connectivity with a lightweight HTTP ping
  ollama_ok <- tryCatch(
    {
      con <- url("http://localhost:11434/api/tags", open = "rb")
      on.exit(close(con))
      TRUE
    },
    error = function(e) FALSE
  )

  if (!ollama_ok) {
    stop(
      "Cannot connect to Ollama. Please ensure:\n",
      "1. Ollama is installed (https://ollama.com)\n",
      "2. Ollama is running (run 'ollama serve' in a terminal)\n",
      "3. The model '", model, "' is downloaded (run 'ollama pull ", model, "')"
    )
  }

  app <- pra_shiny_app(model = model, rag = rag, embed_model = embed_model)
  shiny::runApp(app, port = port, launch.browser = launch.browser)
}

#' Fetch Available Ollama Models
#'
#' Queries the local Ollama API for available models. Falls back to a default
#' list if the API is unreachable.
#'
#' @return A character vector of model names.
#' @keywords internal
get_ollama_models <- function() {
  # 1. Try the Ollama HTTP API
  models <- tryCatch({
    con <- url("http://localhost:11434/api/tags", open = "rb")
    on.exit(close(con))
    raw <- readLines(con, warn = FALSE)
    parsed <- jsonlite::fromJSON(paste(raw, collapse = ""))
    m <- sort(parsed$models$name)
    if (length(m) > 0) m else NULL
  }, error = function(e) NULL)

  if (!is.null(models)) return(models)

  # 2. Fall back to the ollama CLI (more reliable on some systems)
  models <- tryCatch({
    out <- system2("ollama", "list", stdout = TRUE, stderr = FALSE)
    if (length(out) > 1) {
      # Skip the header row; first whitespace-delimited token is the model name
      lines <- out[-1]
      lines <- lines[nchar(trimws(lines)) > 0]
      m <- trimws(sub("\\s+.*", "", lines))
      m <- sort(m[nchar(m) > 0])
      if (length(m) > 0) m else NULL
    } else NULL
  }, error = function(e) NULL)

  if (!is.null(models)) return(models)

  # 3. Hard-coded fallback
  c("llama3.2", "llama3.1", "qwen2.5")
}

#' Create the PRA Shiny App Object
#'
#' @param model Ollama model name.
#' @param rag Whether to enable RAG.
#' @param embed_model Ollama embedding model.
#' @return A Shiny app object.
#' @keywords internal
pra_shiny_app <- function(model = "llama3.2", rag = TRUE, embed_model = "nomic-embed-text") {
  # Fetch models at startup
  available_models <- get_ollama_models()
  if (!model %in% available_models) {
    available_models <- c(model, available_models)
  }

  # Example prompts users can click — use /commands for deterministic execution
  example_prompts <- list(
    list(
      icon = "dice",
      label = "Monte Carlo simulation",
      prompt = '/mcs tasks=[{"type":"normal","mean":10,"sd":2},{"type":"triangular","a":5,"b":10,"c":15},{"type":"uniform","min":8,"max":12}]'
    ),
    list(
      icon = "chart-line",
      label = "Earned Value Management",
      prompt = "/evm bac=500000 schedule=[0.2,0.4,0.6,0.8,1.0] period=3 complete=0.35 costs=[90000,195000,310000]"
    ),
    list(
      icon = "diagram-project",
      label = "Bayesian risk probability",
      prompt = "/risk causes=[0.3,0.2] given=[0.8,0.6] not_given=[0.2,0.4]"
    ),
    list(
      icon = "magnifying-glass-chart",
      label = "Sensitivity analysis",
      prompt = '/sensitivity tasks=[{"type":"normal","mean":10,"sd":2},{"type":"triangular","a":5,"b":10,"c":15},{"type":"uniform","min":8,"max":12}]'
    )
  )

  # Build clickable prompt chip HTML with data attributes (onclick gets stripped by Shiny)
  prompt_chips_html <- paste(vapply(seq_along(example_prompts), function(i) {
    ex <- example_prompts[[i]]
    # Encode prompt as base64 to avoid quote escaping issues
    prompt_b64 <- base64enc::base64encode(charToRaw(ex$prompt))
    paste0(
      '<button class="example-chip" data-prompt="', prompt_b64, '">',
      '<i class="fa-solid fa-', ex$icon, '"></i> ', ex$label, "</button>"
    )
  }, character(1)), collapse = "\n")

  # JS to attach click handlers via event delegation (survives Shiny sanitizer)
  chip_click_js <- shiny::tags$script(shiny::HTML("
    document.addEventListener('click', function(e) {
      var chip = e.target.closest('.example-chip');
      if (chip && chip.dataset.prompt) {
        var prompt = atob(chip.dataset.prompt);
        Shiny.setInputValue('example_click', {prompt: prompt, ts: Date.now()}, {priority: 'event'});
      }
    });
  "))

  ui <- bslib::page_fillable(
    title = "PRA - Risk Analysis Agent",
    theme = bslib::bs_theme(
      version = 5, bootswatch = "flatly",
      primary = "#2c3e50",
      success = "#18bc9c"
    ),
    shiny::tags$head(
      shiny::tags$link(
        rel = "stylesheet",
        href = "https://cdnjs.cloudflare.com/ajax/libs/font-awesome/6.5.0/css/all.min.css"
      ),
      chip_click_js,
      shiny::tags$style(shiny::HTML("
        body { background: #f5f6fa; }
        .data-preview { max-height: 180px; overflow-y: auto; font-size: 0.82em; }

        .sidebar-header {
          text-align: center;
          padding-bottom: 10px; border-bottom: 1px solid #dee2e6; margin-bottom: 10px;
        }
        .sidebar-section { margin-bottom: 12px; }
        .sidebar-section-label {
          font-size: 0.7em; font-weight: 600; text-transform: uppercase;
          letter-spacing: 0.05em; color: #95a5a6; margin-bottom: 4px;
        }
        .sidebar .form-group { margin-bottom: 6px; }
        .status-badge {
          display: inline-block; font-size: 0.75em; padding: 2px 8px;
          border-radius: 10px; font-weight: 600;
        }
        .status-connected { background: #d5f5e3; color: #1e8449; }
        .status-error { background: #fadbd8; color: #c0392b; }

        .example-chip {
          display: flex; align-items: center; gap: 8px;
          width: 100%; padding: 10px 14px;
          background: white; border: 1px solid #dee2e6; border-radius: 10px;
          font-size: 0.88em; color: #2c3e50; text-align: left;
          cursor: pointer; transition: all 0.15s ease;
        }
        .example-chip:hover {
          border-color: #18bc9c; background: #f0faf7; color: #18bc9c;
        }
        .example-chip i { font-size: 1em; width: 18px; text-align: center; color: #95a5a6; }
        .example-chip:hover i { color: #18bc9c; }
        .example-chips { display: flex; flex-direction: column; gap: 8px; margin-top: 8px; }
      "))
    ),
    bslib::layout_sidebar(
      fillable = TRUE,
      sidebar = bslib::sidebar(
        id = "sidebar",
        open = "desktop",
        width = 280,
        padding = c(12, 14),

        # Header — compact logo + status
        shiny::div(
          class = "sidebar-header",
          shiny::tags$img(
            src = "https://raw.githubusercontent.com/paulgovan/PRA/main/inst/logo.png",
            height = "70px",
            style = "display: block; margin: 0 auto;",
            onerror = "this.style.display='none'"
          ),
          shiny::uiOutput("status_badge")
        ),

        # Model
        shiny::div(
          class = "sidebar-section",
          shiny::div(
            class = "sidebar-section-label",
            style = "display: flex; justify-content: space-between; align-items: center;",
            "Model",
            shiny::actionLink("refresh_models", shiny::icon("arrows-rotate"),
              style = "font-size: 0.85em; color: #95a5a6;"
            )
          ),
          shiny::selectizeInput(
            "model", NULL,
            choices = available_models,
            selected = model,
            width = "100%",
            options = list(create = TRUE, placeholder = "Select or type model...")
          )
        ),

        # Options
        shiny::div(
          class = "sidebar-section",
          shiny::div(class = "sidebar-section-label", "Options"),
          shiny::checkboxInput("rag_enabled", "RAG Context", value = rag)
        ),

        # Actions
        shiny::div(
          class = "sidebar-section",
          shiny::div(class = "sidebar-section-label", "Actions"),
          shiny::div(
            style = "display: flex; gap: 6px;",
            shiny::actionButton("new_chat_btn", "New Chat",
              class = "btn-outline-secondary btn-sm flex-fill",
              icon = shiny::icon("rotate-right")
            ),
            shiny::downloadButton("download_chat", "Export",
              class = "btn-outline-secondary btn-sm flex-fill",
              icon = shiny::icon("download")
            )
          )
        ),

        # Data upload
        shiny::div(
          class = "sidebar-section",
          shiny::div(class = "sidebar-section-label", "Data"),
          shiny::fileInput("data_file", NULL,
            accept = c(".csv", ".tsv", ".txt"),
            buttonLabel = "Browse",
            placeholder = "Upload CSV..."
          ),
          shiny::uiOutput("data_preview_ui")
        ),

        # Footer
        shiny::div(
          style = "margin-top: auto; padding-top: 8px; border-top: 1px solid #dee2e6;",
          shiny::p(
            class = "text-muted",
            style = "font-size: 0.7em; margin: 0;",
            "Powered by ellmer + shinychat + Ollama"
          )
        )
      ),

      # Main content: chat
      shinychat::chat_ui("chat",
        placeholder = "Ask a risk analysis question...",
        fill = TRUE
      )
    )
  )

  server <- function(input, output, session) {
    # Reactive values
    chat_obj <- shiny::reactiveVal(NULL)
    is_connected <- shiny::reactiveVal(FALSE)

    # Status badge
    output$status_badge <- shiny::renderUI({
      if (is_connected()) {
        shiny::span(class = "status-badge status-connected", "\u2713 Connected")
      } else {
        shiny::span(class = "status-badge status-error", "\u2717 Disconnected")
      }
    })

    # Welcome message with clickable example chips in a single message
    send_welcome <- function(cur_model, cur_rag) {
      welcome_html <- paste0(
        "<p><strong>Welcome to PRA!</strong> Connected to <strong>", cur_model, "</strong>",
        if (cur_rag) " with RAG context" else "", ".</p>",
        "<p>Use <strong>/commands</strong> for instant results or ask questions in natural language. ",
        "Type <code>/help</code> for all commands.</p>",
        '<div class="example-chips">', prompt_chips_html, "</div>"
      )
      shinychat::chat_append("chat", shiny::HTML(welcome_html), role = "assistant")
    }

    # Initialize chat on startup and when model/RAG changes
    init_chat <- function() {
      cur_model <- input$model
      cur_rag <- input$rag_enabled
      shiny::req(cur_model)

      tryCatch(
        {
          chat <- pra_chat(
            model = cur_model,
            rag = cur_rag,
            embed_model = embed_model
          )
          chat_obj(chat)
          is_connected(TRUE)
          send_welcome(cur_model, cur_rag)
        },
        error = function(e) {
          is_connected(FALSE)
          shinychat::chat_append("chat",
            paste0(
              "**Connection error:** ", e$message,
              "\n\nMake sure Ollama is running (`ollama serve`) and the model is pulled (`ollama pull ",
              cur_model, "`)."
            ),
            role = "assistant"
          )
        }
      )
    }

    # Fire on startup
    shiny::observe({
      init_chat()
    }) |>
      shiny::bindEvent(TRUE, once = TRUE)

    # Reinitialize when model or RAG setting changes
    shiny::observeEvent(list(input$model, input$rag_enabled),
      {
        shinychat::chat_clear("chat")
        is_connected(FALSE)
        chat_obj(NULL)
        .pra_agent_env$user_data_context <- NULL
        .pra_agent_env$user_data <- NULL
        init_chat()
      },
      ignoreInit = TRUE
    )

    # Refresh model list
    shiny::observeEvent(input$refresh_models, {
      models <- get_ollama_models()
      cur <- input$model
      if (!is.null(cur) && nchar(cur) > 0 && !cur %in% models) {
        models <- c(cur, models)
      }
      shiny::updateSelectizeInput(session, "model", choices = models,
                                  selected = cur, server = TRUE)
    })

    # New chat button
    shiny::observeEvent(input$new_chat_btn, {
      shinychat::chat_clear("chat")
      chat_obj(NULL)
      is_connected(FALSE)
      .pra_agent_env$user_data_context <- NULL
      .pra_agent_env$user_data <- NULL
      init_chat()
    })

    # Handle example prompt clicks
    shiny::observeEvent(input$example_click, {
      prompt <- input$example_click$prompt
      shiny::req(nchar(trimws(prompt)) > 0)

      # Show the prompt as a user message
      shinychat::chat_append("chat", prompt, role = "user")

      # Route input
      routing <- route_input(prompt)
      if (routing$mode == "command") {
        result <- execute_command(prompt)
        if (!is.null(result$rich_result) && any(grepl("ContentToolResult", class(result$rich_result), fixed = TRUE))) {
          display <- result$rich_result@extra$display
          if (!is.null(display) && !is.null(display$html)) {
            shinychat::chat_append("chat", shiny::HTML(display$html), role = "assistant")
            return()
          }
        }
        shinychat::chat_append("chat", result$result, role = "assistant")
        return()
      }

      shiny::req(chat_obj())

      if (routing$mode == "rag") {
        handle_rag_query(prompt)
        return()
      }

      # Tool mode: LLM with tools, no RAG context
      query <- build_query(prompt, rag_enabled = FALSE)
      stream <- chat_obj()$stream_async(query, stream = "content")
      shinychat::chat_append("chat", stream)
    })

    # Handle /commands — deterministic tool execution without LLM
    handle_slash_command <- function(user_msg) {
      result <- execute_command(user_msg)
      # If we got a rich ContentToolResult, display the HTML version
      if (!is.null(result$rich_result) && any(grepl("ContentToolResult", class(result$rich_result), fixed = TRUE))) {
        display <- result$rich_result@extra$display
        if (!is.null(display) && !is.null(display$html)) {
          shinychat::chat_append("chat", shiny::HTML(display$html), role = "assistant")
          return()
        }
      }
      # Otherwise display the text/markdown result
      shinychat::chat_append("chat", result$result, role = "assistant")
    }

    # Handle RAG queries — conceptual questions answered from knowledge base
    # Uses a separate chat WITHOUT tools to prevent the LLM from calling
    # analysis functions on questions like "what is earned value?"
    handle_rag_query <- function(user_msg) {
      rag_enabled <- shiny::isolate(input$rag_enabled)

      # Retrieve RAG context
      context_chunks <- character(0)
      sources <- character(0)
      if (rag_enabled && !is.null(.pra_agent_env$rag_store)) {
        tryCatch({
          context_chunks <- retrieve_context(.pra_agent_env$rag_store, user_msg, top_k = 3)
          if (length(context_chunks) > 0) {
            sources <- unique(gsub(
              ".*\\[Source: (.+?)\\]$", "\\1",
              context_chunks[grepl("\\[Source:", context_chunks)]
            ))
          }
        }, error = function(e) NULL)
      }

      if (length(context_chunks) > 0) {
        # Build query with RAG context for a tool-free chat
        context_text <- paste(context_chunks, collapse = "\n\n---\n\n")
        source_label <- paste(unique(sources), collapse = ", ")
        query <- paste0(
          "Use the following reference material to answer the question. ",
          "Be concise and cite sources at the end.\n\n",
          "Reference material:\n\n", context_text,
          "\n\n---\n\nQuestion: ", user_msg
        )

        # Create a lightweight chat WITHOUT tools for RAG answers
        rag_chat <- tryCatch({
          ellmer::chat_ollama(
            model = shiny::isolate(input$model),
            system_prompt = paste0(
              "You are a Project Risk Analysis expert. ",
              "Answer the user's question using ONLY the reference material provided. ",
              "Do NOT attempt to run any computations or call any tools. ",
              "Be concise. At the end, cite sources like: Sources: filename.md"
            ),
            api_args = list(options = list(num_ctx = 16384L))
          )
        }, error = function(e) NULL)

        if (!is.null(rag_chat)) {
          stream <- rag_chat$stream_async(query, stream = "content")
          shinychat::chat_append("chat", stream)
        } else {
          # Fallback: display raw RAG context
          shinychat::chat_append("chat", paste0(
            paste(context_chunks, collapse = "\n\n---\n\n"),
            "\n\n*Sources: ", paste(unique(sources), collapse = ", "), "*"
          ), role = "assistant")
        }
      } else {
        # No RAG context available — fall back to LLM with tools
        query <- build_query(user_msg, rag_enabled = FALSE)
        stream <- chat_obj()$stream_async(query, stream = "content")
        shinychat::chat_append("chat", stream)
      }
    }

    # Handle user messages — three-mode routing via route_input()
    shiny::observeEvent(input$chat_user_input, {
      user_msg <- input$chat_user_input
      shiny::req(nchar(trimws(user_msg)) > 0)

      routing <- route_input(user_msg)
      message("[PRA routing] mode=", routing$mode, " | ", routing$reason)

      if (routing$mode == "command") {
        handle_slash_command(user_msg)
        return()
      }

      shiny::req(chat_obj())

      if (routing$mode == "rag") {
        # RAG mode: answer conceptual questions using a dedicated chat without
        # tools, so the LLM cannot accidentally call analysis functions.
        handle_rag_query(user_msg)
        return()
      }

      # Tool mode: LLM has full tool access for numerical computation
      query <- build_query(user_msg, rag_enabled = FALSE)

      # Stream response — shinychat handles progressive display + tool cards
      stream <- chat_obj()$stream_async(query, stream = "content")
      shinychat::chat_append("chat", stream)
    })

    # Build enriched query with optional RAG context and uploaded data
    build_query <- function(user_msg, rag_enabled) {
      query <- user_msg

      # Include uploaded data context
      if (!is.null(.pra_agent_env$user_data_context)) {
        query <- paste0(
          "Available data:\n", .pra_agent_env$user_data_context,
          "\n\n---\n\n", query
        )
      }

      # Optionally prepend RAG context with source attribution
      if (rag_enabled && !is.null(.pra_agent_env$rag_store)) {
        tryCatch(
          {
            context_chunks <- retrieve_context(.pra_agent_env$rag_store, user_msg, top_k = 3)
            if (length(context_chunks) > 0) {
              # Extract source filenames for citation reminder
              sources <- unique(gsub(
                ".*\\[Source: (.+?)\\]$", "\\1",
                context_chunks[grepl("\\[Source:", context_chunks)]
              ))
              context_text <- paste(context_chunks, collapse = "\n\n---\n\n")
              source_note <- if (length(sources) > 0) {
                paste0(
                  "\n\nIMPORTANT: Use this knowledge base context to answer the question. ",
                  "Cite these sources: ", paste(sources, collapse = ", "), ". ",
                  "Only call a tool if the user provides specific numerical data to compute."
                )
              } else {
                ""
              }
              query <- paste0(
                "Reference material from PRA knowledge base:\n\n",
                context_text, source_note, "\n\n---\n\nUser question: ", query
              )
            }
          },
          error = function(e) NULL
        )
      }

      query
    }

    # Handle file upload
    shiny::observeEvent(input$data_file, {
      shiny::req(input$data_file)
      tryCatch(
        {
          df <- utils::read.csv(input$data_file$datapath)

          col_info <- paste(names(df), collapse = ", ")
          preview_lines <- paste(utils::capture.output(
            print(utils::head(df, 5), row.names = FALSE)
          ), collapse = "\n")
          data_ctx <- paste0(
            "User uploaded file: ", input$data_file$name, "\n",
            "Columns: ", col_info, "\n",
            "Rows: ", nrow(df), "\n\n",
            "Preview:\n", preview_lines
          )
          .pra_agent_env$user_data_context <- data_ctx
          .pra_agent_env$user_data <- df

          shinychat::chat_append("chat", paste0(
            "Uploaded **", input$data_file$name, "** (",
            nrow(df), " rows, ", ncol(df), " columns: ", col_info, ")."
          ), role = "assistant")
        },
        error = function(e) {
          shinychat::chat_append("chat",
            paste0("**File error:** ", e$message),
            role = "assistant"
          )
        }
      )
    })

    # Data preview (XSS-safe)
    output$data_preview_ui <- shiny::renderUI({
      shiny::req(input$data_file)
      df <- .pra_agent_env$user_data
      if (is.null(df)) {
        return(NULL)
      }
      preview <- utils::head(df, 5)
      esc <- function(x) htmltools::htmlEscape(as.character(x))
      header <- paste0("<th>", vapply(names(preview), esc, character(1)), "</th>", collapse = "")
      rows <- apply(preview, 1, function(r) {
        paste0("<tr>", paste0("<td>", vapply(r, esc, character(1)), "</td>", collapse = ""), "</tr>")
      })
      html_table <- paste0(
        "<div class='data-preview'><table class='table table-sm table-striped'>",
        "<thead><tr>", header, "</tr></thead><tbody>",
        paste(rows, collapse = ""), "</tbody></table></div>"
      )
      shiny::HTML(html_table)
    })

    # Chat export as markdown
    output$download_chat <- shiny::downloadHandler(
      filename = function() {
        paste0("pra-chat-", format(Sys.time(), "%Y%m%d-%H%M%S"), ".md")
      },
      content = function(file) {
        chat <- chat_obj()
        header <- paste0(
          "# PRA Chat Log\n\n",
          "**Date:** ", format(Sys.time(), "%Y-%m-%d %H:%M:%S"), "\n\n---\n\n"
        )
        if (is.null(chat)) {
          writeLines(paste0(header, "*No conversation to export.*"), file)
        } else {
          turns <- tryCatch(chat$get_turns(), error = function(e) list())
          lines <- vapply(turns, function(turn) {
            role <- tryCatch(turn@role, error = function(e) "unknown")
            text <- tryCatch(turn@text, error = function(e) "")
            if (role == "user") {
              paste0("**User:** ", text)
            } else if (role == "assistant") {
              paste0("**Assistant:**\n\n", text)
            } else {
              text
            }
          }, character(1))
          writeLines(paste0(header, paste(lines, collapse = "\n\n---\n\n")), file)
        }
      }
    )
  }

  shiny::shinyApp(ui = ui, server = server)
}

#' @srrstats {G5.2a} *Each error message produced by stop() is unique.*

test_that("pra_shiny_app creates a Shiny app object", {
  skip_if_not_installed("shiny")
  skip_if_not_installed("bslib")
  skip_if_not_installed("ellmer")
  skip_if_not_installed("shinychat")
  app <- PRA:::pra_shiny_app()
  expect_s3_class(app, "shiny.appobj")
})

test_that("pra_shiny_app accepts model parameter", {
  skip_if_not_installed("shiny")
  skip_if_not_installed("bslib")
  skip_if_not_installed("ellmer")
  skip_if_not_installed("shinychat")
  app <- PRA:::pra_shiny_app(model = "qwen2.5")
  expect_s3_class(app, "shiny.appobj")
})

test_that("pra_shiny_app accepts rag parameter", {
  skip_if_not_installed("shiny")
  skip_if_not_installed("bslib")
  skip_if_not_installed("ellmer")
  skip_if_not_installed("shinychat")
  app <- PRA:::pra_shiny_app(rag = FALSE)
  expect_s3_class(app, "shiny.appobj")
})

test_that("pra_app function exists and has correct parameters", {
  expect_true(is.function(pra_app))
  args <- formals(pra_app)
  expect_true("model" %in% names(args))
  expect_true("rag" %in% names(args))
  expect_true("embed_model" %in% names(args))
  expect_true("port" %in% names(args))
  expect_true("launch.browser" %in% names(args))
})

test_that("pra_app default model is llama3.2", {
  args <- formals(pra_app)
  expect_equal(args$model, "llama3.2")
})

test_that("pra_app default rag is TRUE", {
  args <- formals(pra_app)
  expect_true(args$rag)
})

test_that("get_ollama_models returns character vector", {
  skip_if_not_installed("shiny")
  skip_if_not_installed("bslib")
  skip_if_not_installed("ellmer")
  skip_if_not_installed("shinychat")
  models <- PRA:::get_ollama_models()
  expect_type(models, "character")
  expect_true(length(models) >= 1)
})

test_that("get_ollama_models falls back to defaults when Ollama unreachable", {
  # Temporarily overwrite the URL by mocking — but since it's a simple function
  # we can test that it returns a reasonable fallback.
  # The function always returns at least the default models on error.
  models <- PRA:::get_ollama_models()
  expect_type(models, "character")
  expect_true(length(models) >= 1)
})

test_that("pra_app errors when Ollama is not running on wrong port", {
  skip_if_not_installed("shiny")
  skip_if_not_installed("ellmer")
  skip_if_not_installed("bslib")
  skip_if_not_installed("shinychat")
  skip_on_cran()

  # pra_app checks connectivity — verify it has the Ollama check
  fn_body <- deparse(body(pra_app))
  expect_true(any(grepl("ollama_ok|Cannot connect to Ollama", fn_body)))
})

test_that("pra_shiny_app includes httpHandler and serverFuncSource", {
  skip_if_not_installed("shiny")
  skip_if_not_installed("bslib")
  skip_if_not_installed("ellmer")
  skip_if_not_installed("shinychat")
  skip_if_not_installed("base64enc")

  app <- PRA:::pra_shiny_app(model = "llama3.2", rag = FALSE)
  expect_true(is.function(app$httpHandler))
  expect_true(is.function(app$serverFuncSource))
})

test_that("pra_shiny_app has server function", {
  skip_if_not_installed("shiny")
  skip_if_not_installed("bslib")
  skip_if_not_installed("ellmer")
  skip_if_not_installed("shinychat")

  app <- PRA:::pra_shiny_app(model = "llama3.2", rag = FALSE)
  expect_true(is.function(app$serverFuncSource))
})

# ============================================================================
# route_input integration tests (exercised via app routing logic)
# ============================================================================

test_that("route_input correctly routes /help as command", {
  r <- PRA:::route_input("/help")
  expect_equal(r$mode, "command")
  expect_equal(r$command, "help")
})

test_that("route_input correctly routes conceptual questions as rag", {
  rag_queries <- c(
    "What is earned value management?",
    "How does Monte Carlo simulation work?",
    "Explain CPI and SPI in project management",
    "What are the differences between SMM and MCS?"
  )
  for (q in rag_queries) {
    r <- PRA:::route_input(q)
    expect_equal(r$mode, "rag", info = paste("Query should route to RAG:", q))
  }
})

test_that("route_input correctly routes numerical data as tool", {
  tool_queries <- c(
    "BAC = $500,000 with schedule [0.2, 0.4, 0.6, 0.8, 1.0]",
    "P(C1) = 0.3, P(C2) = 0.2, P(R|C1) = 0.8",
    "Task A has a normal(10, 2) distribution"
  )
  for (q in tool_queries) {
    r <- PRA:::route_input(q)
    expect_equal(r$mode, "tool", info = paste("Query should route to tool:", q))
  }
})

#' @srrstats {G5.2a} *Each error message produced by stop() is unique.*

test_that("pra_system_prompt returns a non-empty string", {
  prompt <- PRA:::pra_system_prompt()
  expect_type(prompt, "character")
  expect_true(nchar(prompt) > 50)
  # Should mention key tool names
  expect_true(grepl("mcs_tool", prompt))
  expect_true(grepl("evm_analysis_tool", prompt))
  expect_true(grepl("risk_prob_tool", prompt))
  expect_true(grepl("fit_and_predict_sigmoidal_tool", prompt))
  # Should instruct citation of RAG sources
  expect_true(grepl("cite", prompt, ignore.case = TRUE))
  # Should distinguish conceptual questions from computation
  expect_true(grepl("CONCEPTUAL", prompt))
  expect_true(grepl("NUMERICAL DATA", prompt))
  # Should tell the LLM not to output /commands
  expect_true(grepl("Do not use /commands", prompt, ignore.case = TRUE))
})

test_that("pra_system_prompt is concise for small models", {
  prompt <- PRA:::pra_system_prompt()
  # Should be under 2000 chars
  expect_true(nchar(prompt) < 2000)
})

test_that("pra_chat requires ellmer", {
  skip_if_not_installed("ellmer")
  expect_true(is.function(pra_chat))
})

test_that("pra_chat accepts chat parameter", {
  skip_if_not_installed("ellmer")
  # Function should accept a chat parameter
  args <- formals(pra_chat)
  expect_true("chat" %in% names(args))
  expect_null(args$chat)
})

test_that("pra_chat default model is llama3.2", {
  args <- formals(pra_chat)
  expect_equal(args$model, "llama3.2")
})

test_that("pra_app requires shiny and ellmer", {
  skip_if_not_installed("shiny")
  skip_if_not_installed("ellmer")
  skip_if_not_installed("bslib")
  skip_if_not_installed("shinychat")
  expect_true(is.function(pra_app))
})

# ============================================================================
# pra_chat() constructor tests (require Ollama)
# ============================================================================

test_that("pra_chat creates a chat with tools registered", {
  skip_if_not_installed("ellmer")
  skip_on_cran()
  ollama_ok <- tryCatch({
    con <- url("http://localhost:11434/api/tags", open = "rb")
    on.exit(close(con))
    TRUE
  }, error = function(e) FALSE)
  skip_if(!ollama_ok, "Ollama not running")

  chat <- pra_chat(model = "llama3.2", rag = FALSE)
  expect_true(!is.null(chat))
  # Chat should be an R6 or S7 object with a $chat method
  expect_true(is.function(chat$chat) || is.function(chat[["chat"]]))
})

test_that("pra_chat with rag=TRUE initializes RAG store", {
  skip_if_not_installed("ellmer")
  skip_if_not_installed("ragnar")
  skip_on_cran()
  ollama_ok <- tryCatch({
    con <- url("http://localhost:11434/api/tags", open = "rb")
    on.exit(close(con))
    TRUE
  }, error = function(e) FALSE)
  skip_if(!ollama_ok, "Ollama not running")

  chat <- pra_chat(model = "llama3.2", rag = TRUE)
  expect_true(!is.null(chat))
  # RAG store should be set in the agent env
  expect_true(!is.null(PRA:::.pra_agent_env$rag_store))
})

test_that("pra_chat with rag=FALSE does not build knowledge base", {
  skip_if_not_installed("ellmer")
  skip_on_cran()
  ollama_ok <- tryCatch({
    con <- url("http://localhost:11434/api/tags", open = "rb")
    on.exit(close(con))
    TRUE
  }, error = function(e) FALSE)
  skip_if(!ollama_ok, "Ollama not running")

  # Clear the rag_store first
  env <- PRA:::.pra_agent_env
  old_store <- env$rag_store
  env$rag_store <- NULL

  chat <- pra_chat(model = "llama3.2", rag = FALSE)
  expect_true(!is.null(chat))
  # RAG store should remain NULL when rag=FALSE
  expect_null(env$rag_store)

  # Restore
  env$rag_store <- old_store
})

test_that("pra_chat accepts pre-configured chat object", {
  skip_if_not_installed("ellmer")
  skip_on_cran()
  ollama_ok <- tryCatch({
    con <- url("http://localhost:11434/api/tags", open = "rb")
    on.exit(close(con))
    TRUE
  }, error = function(e) FALSE)
  skip_if(!ollama_ok, "Ollama not running")

  # Create a bare Ollama chat, then pass it to pra_chat
  base_chat <- ellmer::chat_ollama(model = "llama3.2")
  result <- pra_chat(chat = base_chat, rag = FALSE)
  expect_true(!is.null(result))
  # The returned object should be the same chat, now with tools
  expect_identical(result, base_chat)
})

test_that("pra_system_prompt includes tool routing guidance", {
  prompt <- PRA:::pra_system_prompt()
  # Should have tool selection section
  expect_true(grepl("Tool selection", prompt))
  # Should reference key tool types
  expect_true(grepl("smm_tool", prompt))
  expect_true(grepl("parent_dsm_tool", prompt))
  expect_true(grepl("cost_pdf_tool", prompt))
})

# ============================================================================
# pra_chat() with mock chat object (no Ollama needed, always runs in covr)
# ============================================================================

test_that("pra_chat with pre-configured chat sets system prompt and registers tools", {
  skip_if_not_installed("ellmer")

  # Create a minimal mock chat using R6-like interface
  mock_chat <- local({
    prompt_set <- NULL
    tools_registered <- list()
    env <- new.env(parent = emptyenv())
    env$set_system_prompt <- function(prompt) {
      prompt_set <<- prompt
    }
    env$register_tool <- function(tool) {
      tools_registered[[length(tools_registered) + 1L]] <<- tool
    }
    env$chat <- function(msg) "mock response"
    env$get_turns <- function() list()
    env$.prompt_set <- function() prompt_set
    env$.tools_registered <- function() tools_registered
    env
  })

  result <- pra_chat(chat = mock_chat, rag = FALSE)

  # Should return the same object
  expect_identical(result, mock_chat)
  # System prompt should have been set
  expect_true(!is.null(mock_chat$.prompt_set()))
  expect_true(nchar(mock_chat$.prompt_set()) > 50)
  # Tools should have been registered
  expect_true(length(mock_chat$.tools_registered()) > 0)
})

test_that("pra_chat with rag=TRUE and mock chat attempts RAG setup", {
  skip_if_not_installed("ellmer")
  skip_if_not_installed("ragnar")

  mock_chat <- local({
    env <- new.env(parent = emptyenv())
    env$set_system_prompt <- function(prompt) NULL
    env$register_tool <- function(tool) NULL
    env$chat <- function(msg) "mock response"
    env
  })

  # Clear RAG store first
  env <- getFromNamespace(".pra_agent_env", "PRA")
  old_store <- env$rag_store
  env$rag_store <- NULL

  # This will try to build knowledge base — may succeed or warn depending on Ollama
  result <- tryCatch(
    pra_chat(chat = mock_chat, rag = TRUE),
    warning = function(w) {
      # If Ollama not available, RAG setup warns but continues
      mock_chat
    }
  )
  expect_true(!is.null(result))

  # Restore
  env$rag_store <- old_store
})

# ============================================================================
# pra_app() constructor tests
# ============================================================================

test_that("pra_app errors when Ollama is not running", {
  skip_if_not_installed("shiny")
  skip_if_not_installed("ellmer")
  skip_if_not_installed("bslib")
  skip_if_not_installed("shinychat")
  skip_on_cran()

  # This will only work if Ollama is NOT running on a non-standard port
  # The function checks connectivity before starting
  expect_true(is.function(pra_app))
  args <- formals(pra_app)
  expect_equal(args$model, "llama3.2")
  expect_true(args$rag)
})

test_that("get_ollama_models returns a character vector", {
  skip_if_not_installed("jsonlite")
  models <- PRA:::get_ollama_models()
  expect_type(models, "character")
  expect_true(length(models) > 0)
})

test_that("pra_shiny_app returns a shiny app object", {
  skip_if_not_installed("shiny")
  skip_if_not_installed("ellmer")
  skip_if_not_installed("bslib")
  skip_if_not_installed("shinychat")
  skip_if_not_installed("base64enc")

  app <- PRA:::pra_shiny_app(model = "llama3.2", rag = FALSE)
  expect_s3_class(app, "shiny.appobj")
})

#' Contingency Calculation.
#'
#' This function calculates the contingency required for a project based on the results
#' of a Monte Carlo simulation. The contingency is determined by the difference between
#' the specified high percentile (phigh) and the base percentile (pbase) of the total
#' project duration distribution.
#'
#' @srrstats {G1.0} *Software lists primary reference from published academic literature.*
#' @srrstats {G1.1} *Software is the first implementation within **R** of the algorithm which has previously been implemented in other languages or contexts.*
#' @srrstats {G1.4} *Software uses [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {G2.0} *Implements assertions on lengths of inputs - phigh and pbase are validated as single values.*
#' @srrstats {G2.0a} *Parameter documentation explicitly states expected input types (single numeric values between 0 and 1).*
#' @srrstats {G2.1} *Implements assertions on types of inputs via is.numeric() checks.*
#' @srrstats {G2.1a} *Parameter documentation explicitly states data types expected.*
#' @srrstats {G2.2} *Prohibits multivariate input for phigh and pbase parameters which are expected to be univariate.*
#' @srrstats {G2.15} *Implements checks for NaN values via is.nan() prior to processing.*
#' @srrstats {G2.16} *Implements checks for Inf/-Inf values via is.infinite() prior to processing.*
#' @srrstats {G5.2a} *Each error message produced by stop() is unique.*
#'
#' @param sims List of results from a Monte Carlo simulation containing the total
#' project duration distribution.
#' @param phigh Percentile level for contingency calculation. Default is 0.95 (95th percentile).
#' @param pbase Base level for contingency calculation. Default is 0.5 (50th percentile).
#' @return The function returns the value of calculated contingency based on the
#' specified percentiles.
#' @references
#' Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
#' construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
#' @examples
#' # Set the number os simulations and the task distributions for a toy project.
#' num_sims <- 10000
#' task_dists <- list(
#'   list(type = "normal", mean = 10, sd = 2), # Task A: Normal distribution
#'   list(type = "triangular", a = 5, b = 10, c = 15), # Task B: Triangular distribution
#'   list(type = "uniform", min = 8, max = 12) # Task C: Uniform distribution
#' )
#'
#' # Set the correlation matrix for the correlations between tasks.
#' cor_mat <- matrix(c(
#'   1, 0.5, 0.3,
#'   0.5, 1, 0.4,
#'   0.3, 0.4, 1
#' ), nrow = 3, byrow = TRUE)
#'
#' # Run the Monte Carlo simulation.
#' results <- mcs(num_sims, task_dists, cor_mat)
#'
#' # Calculate the contingency and print the results.
#' contingency <- contingency(results, phigh = 0.95, pbase = 0.50)
#' cat("Contingency based on 95th percentile and 50th percentile:", contingency)
#'
#' # Without correlation matrix
#' results_indep <- mcs(num_sims, task_dists)
#' contingency_indep <- contingency(results_indep,
#'   phigh = 0.95,
#'   pbase = 0.50
#' )
#' cat("Contingency based on 95th percentile and 50th percentile (
#' independent tasks):", contingency_indep)
#'
#' # Build a barplot to visualize the contingency results.
#' contingency_data <- data.frame(
#'   Scenario = c("With Correlation", "Independent Tasks"),
#'   Contingency = c(contingency, contingency_indep)
#' )
#' barplot(
#'   height = contingency_data$Contingency,
#'   names = contingency_data$Scenario,
#'   col = c("orange", "purple"),
#'   horiz = TRUE,
#'   xlab = "Contingency",
#'   ylab = "Scenario"
#' )
#' title("Contingency Calculation for Project Scenarios")
#'
#' @importFrom stats quantile
#' @export
contingency <- function(sims, phigh = 0.95, pbase = 0.50) {
  # Check for NaN, NA, and Inf values
  if (is.numeric(phigh) && is.nan(phigh)) {
    stop("phigh must not be NaN")
  }
  if (is.numeric(pbase) && is.nan(pbase)) {
    stop("pbase must not be NaN")
  }
  if (is.numeric(phigh) && is.na(phigh)) {
    stop("phigh must not be NA")
  }
  if (is.numeric(pbase) && is.na(pbase)) {
    stop("pbase must not be NA")
  }
  if (is.numeric(phigh) && is.infinite(phigh)) {
    stop("phigh must not be infinite")
  }
  if (is.numeric(pbase) && is.infinite(pbase)) {
    stop("pbase must not be infinite")
  }
  # Check for valid p-values
  if (!is.numeric(phigh) || phigh < 0 || phigh > 1) {
    stop("phigh must be between 0 and 1")
  }
  if (!is.numeric(pbase) || pbase < 0 || pbase > 1) {
    stop("pbase must be between 0 and 1")
  }
  if (phigh < pbase) {
    stop("phigh must be greater than pbase.")
  }



  # Extract the relevant percentiles from the simulation results
  phigh_value <- stats::quantile(sims$total_distribution, probs = phigh)
  pbase_value <- stats::quantile(sims$total_distribution, probs = pbase)

  # Calculate the contingency
  contingency <- phigh_value - pbase_value

  # Return the contingency value
  return(contingency)
}

#' PRA: Project Risk Analysis
#'
#' @description
#' Data analysis for Project Risk Management via the Second Moment Method,
#' Monte Carlo Simulation, Contingency Analysis, Sensitivity Analysis,
#' Earned Value Management, Learning Curves, Bayesian Methods, and more.
#'
#' @section Key modules:
#' \describe{
#'   \item{**Monte Carlo Simulation** (`mcs()`)}{
#'     Propagates schedule or cost uncertainty through a triangular distribution
#'     model using `mc2d`. Returns an S3 object with a `print` method.}
#'   \item{**Second Moment Method** (`smm()`)}{
#'     Analytical first- and second-moment propagation of uncertainty without
#'     simulation. Returns an S3 object with a `print` method.}
#'   \item{**Earned Value Management** (`pv()`, `ev()`, `ac()`, `sv()`, `cv()`,
#'     `spi()`, `cpi()`, `eac()`, `etc()`, `tcpi()`, `vac()`)}{
#'     Full suite of EVM performance metrics and forecasting functions.}
#'   \item{**Contingency Analysis** (`contingency()`)}{
#'     Derives cost or schedule contingency from simulation output at a
#'     user-specified confidence level.}
#'   \item{**Sensitivity Analysis** (`sensitivity()`)}{
#'     Ranks risk drivers by their contribution to overall project uncertainty.}
#'   \item{**Learning Curves** (`fit_sigmoidal()`, `predict_sigmoidal()`,
#'     `plot_sigmoidal()`)}{
#'     Fits Pearl, Gompertz, or Logistic sigmoidal growth models to
#'     cumulative cost or progress data using nonlinear least squares
#'     (`minpack.lm`).}
#'   \item{**Bayesian Risk Inference** (`risk_prob()`, `risk_post_prob()`)}{
#'     Beta-Binomial conjugate updating for risk event probabilities.}
#'   \item{**Correlation Matrices** (`cor_matrix()`)}{
#'     Constructs valid positive-definite correlation matrices for use in
#'     multivariate simulations.}
#'   \item{**Design Structure Matrix** (`parent_dsm()`, `grandparent_dsm()`)}{
#'     Derives direct (parent) and indirect (grandparent) dependency structure
#'     matrices from a binary adjacency matrix.}
#' }
#'
#' @section References:
#' Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
#' construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
#'
#' @srrstats {G1.2} *Life Cycle Statement: This package is in active
#'   development. The API is considered stable for all exported functions.
#'   Future development will focus on additional risk analysis methods and
#'   performance improvements. Users can expect backward compatibility for
#'   existing functions.*
#'
#' @srrstats {G1.3} *All statistical terminology is defined in individual
#'   function documentation. Key terms include: Monte Carlo Simulation,
#'   Earned Value Management (EVM), Bayesian inference, sensitivity analysis,
#'   and sigmoidal growth models (Pearl, Gompertz, Logistic).*
#'
#' @author Paul Govan \email{paul.govan2@@gmail.com}
#'   (ORCID: \href{https://orcid.org/0000-0002-1821-8492}{0000-0002-1821-8492})
#'
#' @keywords internal
"_PACKAGE"

## usethis namespace: start
## usethis namespace: end
NULL

#' Planned Value (PV).
#'
#' Calculates the Planned Value (PV) of work completed based on the Budget at Completion
#' (BAC) and the planned schedule.
#'
#' @srrstats {G1.0} *Software lists primary reference from published academic literature.*
#' @srrstats {G1.1} *Software is the first implementation within **R** of the algorithm which has previously been implemented in other languages or contexts.*
#' @srrstats {G1.4} *Software uses [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {G2.0} *Implements assertions on lengths of inputs - validates single values and non-empty vectors.*
#' @srrstats {G2.0a} *Parameter documentation explicitly states expected input structure.*
#' @srrstats {G2.1} *Implements assertions on types of inputs via is.numeric() checks.*
#' @srrstats {G2.1a} *Parameter documentation explicitly states data types expected.*
#' @srrstats {G2.2} *Prohibits multivariate input for bac and time_period which must be single values.*
#' @srrstats {G2.13} *Implements checks for missing data via anyNA() prior to processing.*
#' @srrstats {G2.14a} *Errors on missing data with informative message.*
#' @srrstats {G2.15} *Implements checks for NaN values via is.nan() prior to processing.*
#' @srrstats {G2.16} *Implements checks for Inf/-Inf values via is.infinite() prior to processing.*
#' @srrstats {G5.2a} *Each error message produced by stop() is unique.*
#'
#' @param bac Budget at Completion (BAC) (total planned budget).
#' @param schedule Vector of planned work completion (in terms of percentage) at each
#' time period.
#' @param time_period Current time period.
#' @return The function returns the Planned Value (PV) of work completed.
#' @references
#' Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
#' construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
#' @examples
#' # Set the BAC, schedule, and current time period for a toy project.
#' bac <- 100000
#' schedule <- c(0.1, 0.2, 0.4, 0.7, 1.0)
#' time_period <- 3
#'
#' # Calculate the PV and print the results.
#' pv <- pv(bac, schedule, time_period)
#' cat("Planned Value (PV):", pv, "\n")
#'
#' @seealso \code{\link{ev}}, \code{\link{ac}}, \code{\link{sv}}, \code{\link{cv}},
#' \code{\link{spi}}, \code{\link{cpi}}, \code{\link{eac}}
#'
#' @export
pv <- function(bac, schedule, time_period) {
  # Error handling
  if (is.null(bac) || is.null(schedule) || is.null(time_period)) {
    stop("bac, schedule, and time_period must not be NULL")
  }
  if (!is.numeric(bac) || !is.numeric(schedule) || !is.numeric(time_period)) {
    stop("bac, schedule, and time_period must be numeric")
  }
  if (length(bac) != 1) {
    stop("bac must be a single numeric value")
  }
  if (length(time_period) != 1) {
    stop("time_period must be a single numeric value")
  }
  if (any(is.nan(bac)) || any(is.nan(schedule)) || any(is.nan(time_period))) {
    stop("bac, schedule, and time_period must not contain NaN values")
  }
  if (anyNA(bac) || anyNA(schedule) || anyNA(time_period)) {
    stop("bac, schedule, and time_period must not contain NA values")
  }
  if (any(is.infinite(bac)) || any(is.infinite(schedule)) || any(is.infinite(time_period))) {
    stop("bac, schedule, and time_period must not contain infinite values")
  }
  if (length(schedule) == 0) {
    stop("schedule must not be empty")
  }
  if (time_period < 1 || time_period > length(schedule)) {
    stop("time_period must be within the range of the schedule vector")
  }
  if (any(schedule < 0) || any(schedule > 1)) {
    stop("schedule values must be between 0 and 1")
  }
  if (bac < 0) {
    stop("bac must be non-negative")
  }

  pv <- bac * schedule[time_period]
  return(pv)
}

#' Earned Value (EV).
#'
#' Calculates the Earned Value (EV) of work completed based on the Budget at Completion
#' (BAC) and the actual work completion percentage.
#'
#' @srrstats {G1.0} *Software lists primary reference from published academic literature.*
#' @srrstats {G1.1} *Software is the first implementation within **R** of the algorithm which has previously been implemented in other languages or contexts.*
#' @srrstats {G1.4} *Software uses [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {G2.0} *Implements assertions on lengths of inputs - validates single values.*
#' @srrstats {G2.0a} *Parameter documentation explicitly states expected input structure (single numeric values).*
#' @srrstats {G2.1} *Implements assertions on types of inputs via is.numeric() checks.*
#' @srrstats {G2.1a} *Parameter documentation explicitly states data types expected.*
#' @srrstats {G2.2} *Prohibits multivariate input for bac and actual_per_complete which must be single values.*
#' @srrstats {G2.13} *Implements checks for missing data via anyNA() prior to processing.*
#' @srrstats {G2.14a} *Errors on missing data with informative message.*
#' @srrstats {G2.15} *Implements checks for NaN values via is.nan() prior to processing.*
#' @srrstats {G2.16} *Implements checks for Inf/-Inf values via is.infinite() prior to processing.*
#' @srrstats {G5.2a} *Each error message produced by stop() is unique.*
#'
#' @param bac Budget at Completion (BAC) (total planned budget).
#' @param actual_per_complete Actual work completion percentage.
#' @return The function returns the Earned Value (EV) of work completed.
#' @references
#' Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
#' construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
#' @examples
#' # Set the BAC and actual % complete for a toy project.
#' bac <- 100000
#' actual_per_complete <- 0.35
#'
#' # Calculate the EV and print the results.
#' ev <- ev(bac, actual_per_complete)
#' cat("Earned Value (EV):", ev, "\n")
#' @seealso \code{\link{pv}}, \code{\link{ac}}, \code{\link{sv}}, \code{\link{cv}},
#' \code{\link{spi}}, \code{\link{cpi}}, \code{\link{eac}}
#'
#' @export
ev <- function(bac, actual_per_complete) {
  # Error handling
  if (is.null(bac) || is.null(actual_per_complete)) {
    stop("bac and actual_per_complete must not be NULL")
  }
  if (!is.numeric(bac) || !is.numeric(actual_per_complete)) {
    stop("bac and actual_per_complete must be numeric")
  }
  if (length(bac) != 1) {
    stop("bac must be a single numeric value")
  }
  if (length(actual_per_complete) != 1) {
    stop("actual_per_complete must be a single numeric value")
  }
  if (is.nan(bac) || is.nan(actual_per_complete)) {
    stop("bac and actual_per_complete must not be NaN")
  }
  if (anyNA(bac) || anyNA(actual_per_complete)) {
    stop("bac and actual_per_complete must not be NA")
  }
  if (is.infinite(bac) || is.infinite(actual_per_complete)) {
    stop("bac and actual_per_complete must not be infinite")
  }
  if (actual_per_complete < 0 || actual_per_complete > 1) {
    stop("actual_per_complete must be between 0 and 1")
  }
  if (bac < 0) {
    stop("bac must be non-negative")
  }

  ev <- bac * actual_per_complete
  return(ev)
}

#' Actual Cost (AC).
#'
#' Calculates the Actual Cost (AC) of work completed based on the actual costs incurred
#' at each time period.
#'
#' @srrstats {G1.0} *Software lists primary reference from published academic literature.*
#' @srrstats {G1.1} *Software is the first implementation within **R** of the algorithm which has previously been implemented in other languages or contexts.*
#' @srrstats {G1.4} *Software uses [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {G2.0} *Implements assertions on lengths of inputs - validates single values and non-empty vectors.*
#' @srrstats {G2.0a} *Parameter documentation explicitly states expected input structure.*
#' @srrstats {G2.1} *Implements assertions on types of inputs via is.numeric() and is.logical() checks.*
#' @srrstats {G2.1a} *Parameter documentation explicitly states data types expected.*
#' @srrstats {G2.2} *Prohibits multivariate input for time_period and cumulative which must be single values.*
#' @srrstats {G2.13} *Implements checks for missing data via anyNA() prior to processing.*
#' @srrstats {G2.14a} *Errors on missing data with informative message.*
#' @srrstats {G2.15} *Implements checks for NaN values via is.nan() prior to processing.*
#' @srrstats {G2.16} *Implements checks for Inf/-Inf values via is.infinite() prior to processing.*
#' @srrstats {G5.2a} *Each error message produced by stop() is unique.*
#'
#' @param actual_costs Vector of actual costs incurred at each time period. Can be either
#'   period costs (cost per period) or cumulative costs depending on the cumulative parameter.
#' @param time_period Current time period.
#' @param cumulative Logical. If TRUE (default), actual_costs are already cumulative and
#'   the value at time_period is returned directly. If FALSE, actual_costs are period costs
#'   and will be summed up to time_period.
#' @return The function returns the Actual Cost (AC) of work completed to date.
#' @references
#' Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
#' construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
#' @examples
#' # Using cumulative costs (default)
#' cumulative_costs <- c(9000, 27000, 63000, 133000, 233000)
#' time_period <- 3
#' ac <- ac(cumulative_costs, time_period)
#' cat("Actual Cost (AC):", ac, "\n")
#'
#' # Using period costs
#' period_costs <- c(9000, 18000, 36000, 70000, 100000)
#' ac <- ac(period_costs, time_period, cumulative = FALSE)
#' cat("Actual Cost (AC):", ac, "\n")
#'
#' @seealso \code{\link{pv}}, \code{\link{ev}}, \code{\link{sv}}, \code{\link{cv}},
#' \code{\link{spi}}, \code{\link{cpi}}, \code{\link{eac}}
#'
#' @export
ac <- function(actual_costs, time_period, cumulative = TRUE) {
  # Error handling
  if (is.null(actual_costs) || is.null(time_period)) {
    stop("actual_costs and time_period must not be NULL")
  }
  if (!is.numeric(actual_costs) || !is.numeric(time_period)) {
    stop("actual_costs and time_period must be numeric")
  }
  if (length(time_period) != 1) {
    stop("time_period must be a single numeric value")
  }
  if (!is.logical(cumulative) || length(cumulative) != 1) {
    stop("cumulative must be a single logical value")
  }
  if (any(is.nan(actual_costs)) || any(is.nan(time_period))) {
    stop("actual_costs and time_period must not contain NaN values")
  }
  if (anyNA(actual_costs) || anyNA(time_period)) {
    stop("actual_costs and time_period must not contain NA values")
  }
  if (any(is.infinite(actual_costs)) || any(is.infinite(time_period))) {
    stop("actual_costs and time_period must not contain infinite values")
  }
  if (length(actual_costs) == 0) {
    stop("actual_costs must not be empty")
  }
  if (time_period < 1 || time_period > length(actual_costs)) {
    stop("time_period must be within the range of the actual_costs vector")
  }
  if (any(actual_costs < 0)) {
    stop("actual_costs must be non-negative")
  }

  if (cumulative) {
    ac <- actual_costs[time_period]
  } else {
    ac <- sum(actual_costs[1:time_period])
  }
  return(ac)
}

#' Schedule Variance (SV).
#'
#' Calculates the Schedule Variance (SV) of work completed based on the Earned Value (EV)
#' and Planned Value (PV).
#'
#' @srrstats {G1.0} *Software lists primary reference from published academic literature.*
#' @srrstats {G1.1} *Software is the first implementation within **R** of the algorithm which has previously been implemented in other languages or contexts.*
#' @srrstats {G1.4} *Software uses [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {G2.0} *Implements assertions on lengths of inputs - validates single values.*
#' @srrstats {G2.0a} *Parameter documentation explicitly states expected input structure (single numeric values).*
#' @srrstats {G2.1} *Implements assertions on types of inputs via is.numeric() checks.*
#' @srrstats {G2.1a} *Parameter documentation explicitly states data types expected.*
#' @srrstats {G2.2} *Prohibits multivariate input for ev and pv which must be single values.*
#' @srrstats {G2.13} *Implements checks for missing data via anyNA() prior to processing.*
#' @srrstats {G2.14a} *Errors on missing data with informative message.*
#' @srrstats {G2.15} *Implements checks for NaN values via is.nan() prior to processing.*
#' @srrstats {G2.16} *Implements checks for Inf/-Inf values via is.infinite() prior to processing.*
#' @srrstats {G5.2a} *Each error message produced by stop() is unique.*
#'
#' @param ev Earned Value.
#' @param pv Planned Value.
#' @return The function returns the Schedule Variance (SV) of work completed.
#' @references
#' Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
#' construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
#' @examples
#' # Set the BAC, schedule, and current time period for an example project.
#' bac <- 100000
#' schedule <- c(0.1, 0.2, 0.4, 0.7, 1.0)
#' time_period <- 3
#'
#' # Calculate the PV.
#' pv <- pv(bac, schedule, time_period)
#'
#' # Set the actual % complete and calculate the EV.
#' actual_per_complete <- 0.35
#' ev <- ev(bac, actual_per_complete)
#'
#' # Calculate the SV and print the results.
#' sv <- sv(ev, pv)
#' cat("Schedule Variance (SV):", sv, "\n")
#'
#' @seealso \code{\link{pv}}, \code{\link{ev}}, \code{\link{ac}}, \code{\link{cv}},
#' \code{\link{spi}}, \code{\link{cpi}}, \code{\link{eac}}
#'
#' @export
sv <- function(ev, pv) {
  # Error handling
  if (is.null(ev) || is.null(pv)) {
    stop("ev and pv must not be NULL")
  }
  if (!is.numeric(ev) || !is.numeric(pv)) {
    stop("ev and pv must be numeric")
  }
  if (length(ev) != 1 || length(pv) != 1) {
    stop("ev and pv must be single numeric values")
  }
  if (is.nan(ev) || is.nan(pv)) {
    stop("ev and pv must not be NaN")
  }
  if (anyNA(ev) || anyNA(pv)) {
    stop("ev and pv must not be NA")
  }
  if (is.infinite(ev) || is.infinite(pv)) {
    stop("ev and pv must not be infinite")
  }
  if (ev < 0) {
    stop("ev must be non-negative")
  }
  if (pv < 0) {
    stop("pv must be non-negative")
  }

  sv <- ev - pv
  return(sv)
}

#' Cost Variance (CV).
#'
#' Calculates the Cost Variance (CV) of work completed based on the Earned Value (EV)
#' and Actual Cost (AC).
#'
#' @srrstats {G1.0} *Software lists primary reference from published academic literature.*
#' @srrstats {G1.1} *Software is the first implementation within **R** of the algorithm which has previously been implemented in other languages or contexts.*
#' @srrstats {G1.4} *Software uses [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {G2.0} *Implements assertions on lengths of inputs - validates single values.*
#' @srrstats {G2.0a} *Parameter documentation explicitly states expected input structure (single numeric values).*
#' @srrstats {G2.1} *Implements assertions on types of inputs via is.numeric() checks.*
#' @srrstats {G2.1a} *Parameter documentation explicitly states data types expected.*
#' @srrstats {G2.2} *Prohibits multivariate input for ev and ac which must be single values.*
#' @srrstats {G2.13} *Implements checks for missing data via anyNA() prior to processing.*
#' @srrstats {G2.14a} *Errors on missing data with informative message.*
#' @srrstats {G2.15} *Implements checks for NaN values via is.nan() prior to processing.*
#' @srrstats {G2.16} *Implements checks for Inf/-Inf values via is.infinite() prior to processing.*
#' @srrstats {G5.2a} *Each error message produced by stop() is unique.*
#'
#' @param ev Earned Value.
#' @param ac Actual Cost.
#' @return The function returns the Cost Variance (CV) of work completed.
#' @references
#' Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
#' construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
#' @examples
#' # Set the BAC and actual % complete for an example project.
#' bac <- 100000
#' actual_per_complete <- 0.35
#'
#' # Calcualte the EV
#' ev <- ev(bac, actual_per_complete)
#'
#' # Set the actual costs and current time period and calculate the AC.
#' actual_costs <- c(9000, 18000, 36000, 70000, 100000)
#' time_period <- 3
#' ac <- ac(actual_costs, time_period)
#'
#' # Calculate the CV and print the results.
#' cv <- cv(ev, ac)
#' cat("Cost Variance (CV):", cv, "\n")
#' @seealso \code{\link{pv}}, \code{\link{ev}}, \code{\link{ac}}, \code{\link{sv}},
#' \code{\link{spi}}, \code{\link{cpi}}, \code{\link{eac}}
#'
#' @export
cv <- function(ev, ac) {
  # Error handling
  if (is.null(ev) || is.null(ac)) {
    stop("ev and ac must not be NULL")
  }
  if (!is.numeric(ev) || !is.numeric(ac)) {
    stop("ev and ac must be numeric")
  }
  if (length(ev) != 1 || length(ac) != 1) {
    stop("ev and ac must be single numeric values")
  }
  if (is.nan(ev) || is.nan(ac)) {
    stop("ev and ac must not be NaN")
  }
  if (anyNA(ev) || anyNA(ac)) {
    stop("ev and ac must not be NA")
  }
  if (is.infinite(ev) || is.infinite(ac)) {
    stop("ev and ac must not be infinite")
  }
  if (ev < 0) {
    stop("ev must be non-negative")
  }
  if (ac < 0) {
    stop("ac must be non-negative")
  }

  cv <- ev - ac
  return(cv)
}

#' Schedule Performance Index (SPI).
#'
#' Calculates the Schedule Performance Index (SPI) of work completed based on the Earned Value (EV)
#' and Planned Value (PV).
#'
#' @srrstats {G1.0} *Software lists primary reference from published academic literature.*
#' @srrstats {G1.1} *Software is the first implementation within **R** of the algorithm which has previously been implemented in other languages or contexts.*
#' @srrstats {G1.4} *Software uses [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {G2.0} *Implements assertions on lengths of inputs - validates single values.*
#' @srrstats {G2.0a} *Parameter documentation explicitly states expected input structure (single numeric values).*
#' @srrstats {G2.1} *Implements assertions on types of inputs via is.numeric() checks.*
#' @srrstats {G2.1a} *Parameter documentation explicitly states data types expected.*
#' @srrstats {G2.2} *Prohibits multivariate input for ev and pv which must be single values.*
#' @srrstats {G2.13} *Implements checks for missing data via anyNA() prior to processing.*
#' @srrstats {G2.14a} *Errors on missing data with informative message.*
#' @srrstats {G2.15} *Implements checks for NaN values via is.nan() prior to processing.*
#' @srrstats {G2.16} *Implements checks for Inf/-Inf values via is.infinite() prior to processing.*
#' @srrstats {G5.2a} *Each error message produced by stop() is unique.*
#'
#' @param ev Earned Value.
#' @param pv Planned Value.
#' @return The function returns the Schedule Performance Index (SPI) of work completed.
#' @references
#' Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
#' construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
#' @examples
#' # Set the BAC, schedule, and current time period for an example project.
#' bac <- 100000
#' schedule <- c(0.1, 0.2, 0.4, 0.7, 1.0)
#' time_period <- 3
#'
#' # Calculate the PV.
#' pv <- pv(bac, schedule, time_period)
#'
#' # Set the actual % complete and calculate the EV.
#' actual_per_complete <- 0.35
#' ev <- ev(bac, actual_per_complete)
#'
#' # Calculate the SPI and print the results.
#' spi <- spi(ev, pv)
#' cat("Schedule Performance Index (SPI):", spi, "\n")
#' @seealso \code{\link{pv}}, \code{\link{ev}}, \code{\link{ac}}, \code{\link{sv}},
#' \code{\link{cv}}, \code{\link{cpi}}, \code{\link{eac}}
#'
#' @export
spi <- function(ev, pv) {
  # Error handling
  if (is.null(ev) || is.null(pv)) {
    stop("ev and pv must not be NULL")
  }
  if (!is.numeric(ev) || !is.numeric(pv)) {
    stop("ev and pv must be numeric")
  }
  if (length(ev) != 1 || length(pv) != 1) {
    stop("ev and pv must be single numeric values")
  }
  if (is.nan(ev) || is.nan(pv)) {
    stop("ev and pv must not be NaN")
  }
  if (anyNA(ev) || anyNA(pv)) {
    stop("ev and pv must not be NA")
  }
  if (is.infinite(ev) || is.infinite(pv)) {
    stop("ev and pv must not be infinite")
  }
  if (ev < 0) {
    stop("ev must be non-negative")
  }
  if (pv <= 0) {
    stop("pv must be greater than zero")
  }

  spi <- ev / pv
  return(spi)
}

#' Cost Performance Index (CPI).
#'
#' Calculates the Cost Performance Index (CPI) of work completed based on the Earned Value (EV)
#' and Actual Cost (AC).
#'
#' @srrstats {G1.0} *Software lists primary reference from published academic literature.*
#' @srrstats {G1.1} *Software is the first implementation within **R** of the algorithm which has previously been implemented in other languages or contexts.*
#' @srrstats {G1.4} *Software uses [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {G2.0} *Implements assertions on lengths of inputs - validates single values.*
#' @srrstats {G2.0a} *Parameter documentation explicitly states expected input structure (single numeric values).*
#' @srrstats {G2.1} *Implements assertions on types of inputs via is.numeric() checks.*
#' @srrstats {G2.1a} *Parameter documentation explicitly states data types expected.*
#' @srrstats {G2.2} *Prohibits multivariate input for ev and ac which must be single values.*
#' @srrstats {G2.13} *Implements checks for missing data via anyNA() prior to processing.*
#' @srrstats {G2.14a} *Errors on missing data with informative message.*
#' @srrstats {G2.15} *Implements checks for NaN values via is.nan() prior to processing.*
#' @srrstats {G2.16} *Implements checks for Inf/-Inf values via is.infinite() prior to processing.*
#' @srrstats {G5.2a} *Each error message produced by stop() is unique.*
#'
#' @param ev Earned Value.
#' @param ac Actual Cost.
#' @return The function returns the Cost Performance Index (CPI) of work completed.
#' @references
#' Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
#' construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
#' @examples
#' # Set the BAC and actual % complete for an example project.
#' bac <- 100000
#' actual_per_complete <- 0.35
#'
#' # Calcualte the EV
#' ev <- ev(bac, actual_per_complete)
#'
#' # Set the actual costs and current time period and calculate the AC.
#' actual_costs <- c(9000, 18000, 36000, 70000, 100000)
#' time_period <- 3
#' ac <- ac(actual_costs, time_period)
#'
#' # Calculate the CPI and print the results.
#' cpi <- cpi(ev, ac)
#' cat("Cost Performance Index (CPI):", cpi, "\n")
#'
#' @seealso \code{\link{pv}}, \code{\link{ev}}, \code{\link{ac}}, \code{\link{sv}},
#' \code{\link{cv}}, \code{\link{spi}}, \code{\link{eac}}
#'
#' @export
cpi <- function(ev, ac) {
  # Error handling
  if (is.null(ev) || is.null(ac)) {
    stop("ev and ac must not be NULL")
  }
  if (!is.numeric(ev) || !is.numeric(ac)) {
    stop("ev and ac must be numeric")
  }
  if (length(ev) != 1 || length(ac) != 1) {
    stop("ev and ac must be single numeric values")
  }
  if (is.nan(ev) || is.nan(ac)) {
    stop("ev and ac must not be NaN")
  }
  if (anyNA(ev) || anyNA(ac)) {
    stop("ev and ac must not be NA")
  }
  if (is.infinite(ev) || is.infinite(ac)) {
    stop("ev and ac must not be infinite")
  }
  if (ev < 0) {
    stop("ev must be non-negative")
  }
  if (ac <= 0) {
    stop("ac must be greater than zero")
  }

  cpi <- ev / ac
  return(cpi)
}

#' Estimate at Completion (EAC).
#'
#' Calculates the Estimate at Completion (EAC) using various methods based on
#' project performance assumptions.
#'
#' @srrstats {G1.0} *Software lists primary reference from published academic literature.*
#' @srrstats {G1.1} *Software is the first implementation within **R** of the algorithm which has previously been implemented in other languages or contexts.*
#' @srrstats {G1.4} *Software uses [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {G2.0} *Implements assertions on lengths of inputs - validates single values.*
#' @srrstats {G2.0a} *Parameter documentation explicitly states expected input structure (single numeric values).*
#' @srrstats {G2.1} *Implements assertions on types of inputs via is.numeric() checks.*
#' @srrstats {G2.1a} *Parameter documentation explicitly states data types expected.*
#' @srrstats {G2.2} *Prohibits multivariate input for all parameters which must be single values.*
#' @srrstats {G2.3a} *Uses explicit validation against valid_methods vector for method parameter.*
#' @srrstats {G2.3b} *Uses tolower() to ensure method parameter is not case sensitive.*
#' @srrstats {G2.13} *Implements checks for missing data via is.na() prior to processing.*
#' @srrstats {G2.14a} *Errors on missing data with informative message.*
#' @srrstats {G2.15} *Implements checks for NaN values via is.nan() prior to processing.*
#' @srrstats {G2.16} *Implements checks for Inf/-Inf values via is.infinite() prior to processing.*
#' @srrstats {G5.2a} *Each error message produced by stop() is unique.*
#'
#' @param bac Budget at Completion (BAC) (total planned budget).
#' @param method The EAC calculation method. One of:
#'   \itemize{
#'     \item "typical" (default): EAC = BAC / CPI. Assumes future work performed at current cost efficiency.
#'     \item "atypical": EAC = AC + (BAC - EV). Assumes future work performed at planned rate.
#'     \item "combined": EAC = AC + (BAC - EV) / (CPI * SPI). Considers both cost and schedule performance.
#'   }
#' @param cpi Cost Performance Index (CPI). Required for "typical" and "combined" methods.
#' @param ac Actual Cost. Required for "atypical" and "combined" methods.
#' @param ev Earned Value. Required for "atypical" and "combined" methods.
#' @param spi Schedule Performance Index. Required for "combined" method.
#' @return The function returns the Estimate at Completion (EAC).
#' @references
#' Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
#' construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
#' @examples
#' # Method 1: Typical - assumes current CPI continues
#' bac <- 100000
#' cpi <- 0.83
#' eac <- eac(bac, cpi = cpi)
#' cat("EAC (typical):", round(eac, 2), "\n")
#'
#' # Method 2: Atypical - assumes future work at planned rate
#' ac <- 63000
#' ev <- 35000
#' eac <- eac(bac, method = "atypical", ac = ac, ev = ev)
#' cat("EAC (atypical):", round(eac, 2), "\n")
#'
#' # Method 3: Combined - considers both CPI and SPI
#' spi <- 0.875
#' eac <- eac(bac, method = "combined", cpi = cpi, ac = ac, ev = ev, spi = spi)
#' cat("EAC (combined):", round(eac, 2), "\n")
#'
#' @seealso \code{\link{pv}}, \code{\link{ev}}, \code{\link{ac}}, \code{\link{sv}},
#' \code{\link{cv}}, \code{\link{spi}}, \code{\link{cpi}}, \code{\link{etc}},
#' \code{\link{vac}}, \code{\link{tcpi}}
#'
#' @export
eac <- function(bac, method = "typical", cpi = NULL, ac = NULL, ev = NULL, spi = NULL) {
  # Validate method
  method <- tolower(method)
  valid_methods <- c("typical", "atypical", "combined")
  if (!method %in% valid_methods) {
    stop("method must be one of: ", paste(valid_methods, collapse = ", "))
  }

  # Error handling for bac
  if (is.null(bac)) {
    stop("bac must not be NULL")
  }
  if (!is.numeric(bac) || length(bac) != 1 || is.nan(bac)) {
    stop("bac must be a single non-NaN numeric value")
  }
  if (is.na(bac)) {
    stop("bac must be a single numeric value")
  }
  if (is.infinite(bac)) {
    stop("bac must not be infinite")
  }
  if (bac < 0) {
    stop("bac must be non-negative")
  }

  # Calculate EAC based on method
  if (method == "typical") {
    # EAC = BAC / CPI
    if (is.null(cpi)) {
      stop("cpi is required for the 'typical' method")
    }
    if (!is.numeric(cpi) || length(cpi) != 1 || is.nan(cpi)) {
      stop("cpi must be a single non-NaN numeric value")
    }
    if (is.na(cpi)) {
      stop("cpi must be a single numeric value")
    }
    if (is.infinite(cpi)) {
      stop("cpi must not be infinite")
    }
    if (cpi <= 0) {
      stop("cpi must be greater than zero")
    }
    eac <- bac / cpi
  } else if (method == "atypical") {
    # EAC = AC + (BAC - EV)
    if (is.null(ac) || is.null(ev)) {
      stop("ac and ev are required for the 'atypical' method")
    }
    if (!is.numeric(ac) || length(ac) != 1 || is.nan(ac)) {
      stop("ac must be a single non-NaN numeric value")
    }
    if (is.na(ac)) {
      stop("ac must be a single numeric value")
    }
    if (is.infinite(ac)) {
      stop("ac must not be infinite")
    }
    if (!is.numeric(ev) || length(ev) != 1 || is.nan(ev)) {
      stop("ev must be a single non-NaN numeric value")
    }
    if (is.na(ev)) {
      stop("ev must be a single numeric value")
    }
    if (is.infinite(ev)) {
      stop("ev must not be infinite")
    }
    if (ac < 0 || ev < 0) {
      stop("ac and ev must be non-negative")
    }
    eac <- ac + (bac - ev)
  } else if (method == "combined") {
    # EAC = AC + (BAC - EV) / (CPI * SPI)
    if (is.null(cpi) || is.null(ac) || is.null(ev) || is.null(spi)) {
      stop("cpi, ac, ev, and spi are required for the 'combined' method")
    }
    if (!is.numeric(cpi) || length(cpi) != 1 || is.nan(cpi)) {
      stop("cpi must be a single non-NaN numeric value for 'combined' method")
    }
    if (is.na(cpi)) {
      stop("cpi must be a single numeric value")
    }
    if (is.infinite(cpi)) {
      stop("cpi must not be infinite for 'combined' method")
    }
    if (!is.numeric(spi) || length(spi) != 1 || is.nan(spi)) {
      stop("spi must be a single non-NaN numeric value")
    }
    if (is.na(spi)) {
      stop("spi must be a single numeric value")
    }
    if (is.infinite(spi)) {
      stop("spi must not be infinite")
    }
    if (!is.numeric(ac) || length(ac) != 1 || is.nan(ac)) {
      stop("ac must be a single non-NaN numeric value for 'combined' method")
    }
    if (is.na(ac)) {
      stop("ac must be a single numeric value")
    }
    if (is.infinite(ac)) {
      stop("ac must not be infinite for 'combined' method")
    }
    if (!is.numeric(ev) || length(ev) != 1 || is.nan(ev)) {
      stop("ev must be a single non-NaN numeric value for 'combined' method")
    }
    if (is.na(ev)) {
      stop("ev must be a single numeric value")
    }
    if (is.infinite(ev)) {
      stop("ev must not be infinite for 'combined' method")
    }
    if (cpi <= 0 || spi <= 0) {
      stop("cpi and spi must be greater than zero")
    }
    if (ac < 0 || ev < 0) {
      stop("ac and ev must be non-negative")
    }
    eac <- ac + (bac - ev) / (cpi * spi)
  }

  return(eac)
}

#' Estimate to Complete (ETC).
#'
#' Calculates the Estimate to Complete (ETC), which is the expected cost to finish
#' the remaining work.
#'
#' @srrstats {G1.0} *Software lists primary reference from published academic literature.*
#' @srrstats {G1.1} *Software is the first implementation within **R** of the algorithm which has previously been implemented in other languages or contexts.*
#' @srrstats {G1.4} *Software uses [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {G2.0} *Implements assertions on lengths of inputs - validates single values.*
#' @srrstats {G2.0a} *Parameter documentation explicitly states expected input structure (single numeric values).*
#' @srrstats {G2.1} *Implements assertions on types of inputs via is.numeric() checks.*
#' @srrstats {G2.1a} *Parameter documentation explicitly states data types expected.*
#' @srrstats {G2.2} *Prohibits multivariate input for bac, ev, and cpi which must be single values.*
#' @srrstats {G2.13} *Implements checks for missing data via anyNA() and is.na() prior to processing.*
#' @srrstats {G2.14a} *Errors on missing data with informative message.*
#' @srrstats {G2.15} *Implements checks for NaN values via is.nan() prior to processing.*
#' @srrstats {G2.16} *Implements checks for Inf/-Inf values via is.infinite() prior to processing.*
#' @srrstats {G5.2a} *Each error message produced by stop() is unique.*
#'
#' @param bac Budget at Completion (BAC) (total planned budget).
#' @param ev Earned Value.
#' @param cpi Cost Performance Index. If NULL, assumes remaining work will be completed
#'   at planned cost (ETC = BAC - EV). If provided, adjusts for current performance
#'   (ETC = (BAC - EV) / CPI).
#' @return The function returns the Estimate to Complete (ETC).
#' @references
#' Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
#' construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
#' @examples
#' bac <- 100000
#' ev <- 35000
#' cpi <- 0.83
#'
#' # ETC assuming remaining work at planned rate
#' etc <- etc(bac, ev)
#' cat("ETC (planned rate):", etc, "\n")
#'
#' # ETC assuming remaining work at current CPI
#' etc <- etc(bac, ev, cpi)
#' cat("ETC (current CPI):", round(etc, 2), "\n")
#'
#' @seealso \code{\link{eac}}, \code{\link{vac}}, \code{\link{tcpi}}
#'
#' @export
etc <- function(bac, ev, cpi = NULL) {
  # Error handling
  if (is.null(bac) || is.null(ev)) {
    stop("bac and ev must not be NULL")
  }
  if (!is.numeric(bac) || !is.numeric(ev)) {
    stop("bac and ev must be numeric")
  }
  if (length(bac) != 1 || length(ev) != 1) {
    stop("bac and ev must be single numeric values")
  }
  if (is.nan(bac) || is.nan(ev)) {
    stop("bac and ev must not be NaN")
  }
  if (anyNA(bac) || anyNA(ev)) {
    stop("bac and ev must not be NA")
  }
  if (is.infinite(bac) || is.infinite(ev)) {
    stop("bac and ev must not be infinite")
  }
  if (bac < 0 || ev < 0) {
    stop("bac and ev must be non-negative")
  }

  if (is.null(cpi)) {
    # Remaining work at planned rate
    etc <- bac - ev
  } else {
    if (!is.numeric(cpi) || length(cpi) != 1 || is.nan(cpi)) {
      stop("cpi must be a single non-NaN numeric value")
    }
    if (is.na(cpi)) {
      stop("cpi must be a single numeric value")
    }
    if (is.infinite(cpi)) {
      stop("cpi must not be infinite")
    }
    if (cpi <= 0) {
      stop("cpi must be greater than zero")
    }
    # Remaining work at current CPI
    etc <- (bac - ev) / cpi
  }

  return(etc)
}

#' Variance at Completion (VAC).
#'
#' Calculates the Variance at Completion (VAC), which is the difference between
#' the budget and the expected final cost. Positive VAC indicates under budget,
#' negative indicates over budget.
#'
#' @srrstats {G1.0} *Software lists primary reference from published academic literature.*
#' @srrstats {G1.1} *Software is the first implementation within **R** of the algorithm which has previously been implemented in other languages or contexts.*
#' @srrstats {G1.4} *Software uses [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {G2.0} *Implements assertions on lengths of inputs - validates single values.*
#' @srrstats {G2.0a} *Parameter documentation explicitly states expected input structure (single numeric values).*
#' @srrstats {G2.1} *Implements assertions on types of inputs via is.numeric() checks.*
#' @srrstats {G2.1a} *Parameter documentation explicitly states data types expected.*
#' @srrstats {G2.2} *Prohibits multivariate input for bac and eac which must be single values.*
#' @srrstats {G2.13} *Implements checks for missing data via anyNA() prior to processing.*
#' @srrstats {G2.14a} *Errors on missing data with informative message.*
#' @srrstats {G2.15} *Implements checks for NaN values via is.nan() prior to processing.*
#' @srrstats {G2.16} *Implements checks for Inf/-Inf values via is.infinite() prior to processing.*
#' @srrstats {G5.2a} *Each error message produced by stop() is unique.*
#'
#' @param bac Budget at Completion (BAC) (total planned budget).
#' @param eac Estimate at Completion.
#' @return The function returns the Variance at Completion (VAC).
#' @references
#' Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
#' construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
#' @examples
#' bac <- 100000
#' eac <- 120482 # From EAC calculation
#'
#' vac <- vac(bac, eac)
#' cat("Variance at Completion (VAC):", round(vac, 2), "\n")
#' cat("Project is expected to be", abs(round(vac, 2)), ifelse(vac < 0, "over", "under"), "budget\n")
#'
#' @seealso \code{\link{eac}}, \code{\link{etc}}, \code{\link{tcpi}}
#'
#' @export
vac <- function(bac, eac) {
  # Error handling
  if (is.null(bac) || is.null(eac)) {
    stop("bac and eac must not be NULL")
  }
  if (!is.numeric(bac) || !is.numeric(eac)) {
    stop("bac and eac must be numeric")
  }
  if (length(bac) != 1 || length(eac) != 1) {
    stop("bac and eac must be single numeric values")
  }
  if (is.nan(bac) || is.nan(eac)) {
    stop("bac and eac must not be NaN")
  }
  if (anyNA(bac) || anyNA(eac)) {
    stop("bac and eac must not be NA")
  }
  if (is.infinite(bac) || is.infinite(eac)) {
    stop("bac and eac must not be infinite")
  }
  if (bac < 0 || eac < 0) {
    stop("bac and eac must be non-negative")
  }

  vac <- bac - eac
  return(vac)
}

#' To-Complete Performance Index (TCPI).
#'
#' Calculates the To-Complete Performance Index (TCPI), which indicates the cost
#' performance required on remaining work to meet a target (BAC or EAC).
#' TCPI > 1 means efficiency must improve; TCPI < 1 means efficiency can decrease.
#'
#' @srrstats {G1.0} *Software lists primary reference from published academic literature.*
#' @srrstats {G1.1} *Software is the first implementation within **R** of the algorithm which has previously been implemented in other languages or contexts.*
#' @srrstats {G1.4} *Software uses [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {G2.0} *Implements assertions on lengths of inputs - validates single values.*
#' @srrstats {G2.0a} *Parameter documentation explicitly states expected input structure (single numeric values).*
#' @srrstats {G2.1} *Implements assertions on types of inputs via is.numeric() checks.*
#' @srrstats {G2.1a} *Parameter documentation explicitly states data types expected.*
#' @srrstats {G2.2} *Prohibits multivariate input for all parameters which must be single values.*
#' @srrstats {G2.3} *Uses tolower() to ensure target parameter is not case sensitive.*
#' @srrstats {G2.3a} *Uses explicit validation for target parameter to only permit expected values.*
#' @srrstats {G2.13} *Implements checks for missing data via anyNA() and is.na() prior to processing.*
#' @srrstats {G2.14a} *Errors on missing data with informative message.*
#' @srrstats {G2.15} *Implements checks for NaN values via is.nan() prior to processing.*
#' @srrstats {G2.16} *Implements checks for Inf/-Inf values via is.infinite() prior to processing.*
#' @srrstats {G5.2a} *Each error message produced by stop() is unique.*
#'
#' @param bac Budget at Completion (BAC) (total planned budget).
#' @param ev Earned Value.
#' @param ac Actual Cost.
#' @param target The target to calculate TCPI against. Either "bac" (default) to meet
#'   original budget, or "eac" to meet revised estimate. If "eac", the eac parameter
#'   must be provided.
#' @param eac Estimate at Completion. Required when target = "eac".
#' @return The function returns the To-Complete Performance Index (TCPI).
#' @references
#' Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
#' construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
#' @examples
#' bac <- 100000
#' ev <- 35000
#' ac <- 63000
#'
#' # TCPI to complete within original budget
#' tcpi_bac <- tcpi(bac, ev, ac)
#' cat("TCPI (to meet BAC):", round(tcpi_bac, 2), "\n")
#'
#' # TCPI to complete within revised estimate
#' eac <- 120482
#' tcpi_eac <- tcpi(bac, ev, ac, target = "eac", eac = eac)
#' cat("TCPI (to meet EAC):", round(tcpi_eac, 2), "\n")
#'
#' @seealso \code{\link{eac}}, \code{\link{etc}}, \code{\link{vac}}, \code{\link{cpi}}
#'
#' @export
tcpi <- function(bac, ev, ac, target = "bac", eac = NULL) {
  # Convert target to lowercase for case-insensitive matching
  target <- tolower(target)

  # Error handling
  if (is.null(bac) || is.null(ev) || is.null(ac)) {
    stop("bac, ev, and ac must not be NULL")
  }
  if (!is.numeric(bac) || !is.numeric(ev) || !is.numeric(ac)) {
    stop("bac, ev, and ac must be numeric")
  }
  if (length(bac) != 1 || length(ev) != 1 || length(ac) != 1) {
    stop("bac, ev, and ac must be single numeric values")
  }
  if (is.nan(bac) || is.nan(ev) || is.nan(ac)) {
    stop("bac, ev, and ac must not be NaN")
  }
  if (anyNA(bac) || anyNA(ev) || anyNA(ac)) {
    stop("bac, ev, and ac must not be NA")
  }
  if (is.infinite(bac) || is.infinite(ev) || is.infinite(ac)) {
    stop("bac, ev, and ac must not be infinite")
  }
  if (bac < 0 || ev < 0 || ac < 0) {
    stop("bac, ev, and ac must be non-negative")
  }

  if (!target %in% c("bac", "eac")) {
    stop("target must be either 'bac' or 'eac'")
  }

  if (target == "bac") {
    # TCPI = (BAC - EV) / (BAC - AC)
    denominator <- bac - ac
    if (denominator <= 0) {
      stop("Cannot calculate TCPI: actual cost already meets or exceeds budget")
    }
    tcpi <- (bac - ev) / denominator
  } else {
    # TCPI = (BAC - EV) / (EAC - AC)
    if (is.null(eac)) {
      stop("eac is required when target = 'eac'")
    }
    if (!is.numeric(eac) || length(eac) != 1 || is.nan(eac)) {
      stop("eac must be a single non-NaN numeric value")
    }
    if (is.na(eac)) {
      stop("eac must be a single numeric value")
    }
    if (is.infinite(eac)) {
      stop("eac must not be infinite")
    }
    if (eac < 0) {
      stop("eac must be non-negative")
    }
    denominator <- eac - ac
    if (denominator <= 0) {
      stop("Cannot calculate TCPI: actual cost already meets or exceeds EAC")
    }
    tcpi <- (bac - ev) / denominator
  }

  return(tcpi)
}

#' @srrstats {G5.2} *Error and warning behaviour is explicitly demonstrated through tests.*
#' @srrstats {G5.2a} *Every error message is unique and tested.*
#' @srrstats {G5.2b} *Tests trigger every error message and compare with expected values.*
#' @srrstats {G5.3} *Return objects tested for absence of NA, NaN, Inf.*
#' @srrstats {G5.6} *Parameter recovery tests verify implementations produce expected results given data with known properties.*
#' @srrstats {G5.6a} *Parameter recovery tests succeed within defined tolerance rather than exact values.*
#' @srrstats {G5.6b} *Parameter recovery tests run with multiple random seeds when randomness is involved.*
#' @srrstats {G5.7} *Algorithm performance tests verify implementations perform correctly as data properties change.*
#' @srrstats {G5.8} *Edge condition tests verify appropriate behavior with extreme data properties.*
#' @srrstats {G5.8a} *Zero-length data tests trigger clear errors.*
#' @srrstats {G5.8b} *Unsupported data type tests trigger clear errors.*
#' @srrstats {G5.8c} *All-NA and all-identical data tests trigger clear errors or warnings.*
#' @srrstats {G5.8d} *Out-of-scope data tests verify appropriate behavior.*
#' @srrstats {G5.9} *Noise susceptibility tests verify stochastic behavior stability.*
#' @srrstats {G5.9a} *Trivial noise tests show results are stable at machine epsilon scale.*
#' @srrstats {G5.9b} *Random seed stability tests show consistent behavior across different seeds.*

test_that("Planned Value (PV) calculation is correct", {
  bac <- 100000
  schedule <- c(0.1, 0.2, 0.4, 0.7, 1.0)
  time_period <- 3

  pv_value <- pv(bac, schedule, time_period)
  expect_equal(pv_value, 40000)
})

test_that("Earned Value (EV) calculation is correct", {
  bac <- 100000
  actual_per_complete <- 0.35

  ev_value <- ev(bac, actual_per_complete)
  expect_equal(ev_value, 35000)
})

test_that("Actual Cost (AC) calculation is correct", {
  actual_costs <- c(9000, 18000, 36000, 70000, 100000)
  time_period <- 3

  ac_value <- ac(actual_costs, time_period)
  expect_equal(ac_value, 36000)
})

test_that("Schedule Variance (SV) calculation is correct", {
  ev_value <- 35000
  pv_value <- 40000

  sv_value <- sv(ev_value, pv_value)
  expect_equal(sv_value, -5000)
})

test_that("Cost Variance (CV) calculation is correct", {
  ev_value <- 35000
  ac_value <- 36000

  cv_value <- cv(ev_value, ac_value)
  expect_equal(cv_value, -1000)
})

test_that("Schedule Performance Index (SPI) calculation is correct", {
  ev_value <- 35000
  pv_value <- 40000

  spi_value <- spi(ev_value, pv_value)
  expect_equal(spi_value, 0.875)
})

test_that("Cost Performance Index (CPI) calculation is correct", {
  ev_value <- 35000
  ac_value <- 36000

  cpi_value <- cpi(ev_value, ac_value)
  expect_equal(cpi_value, 0.9722222, tolerance = 1e-6)
})

# ============================================================================
# PV Error Handling Tests
# ============================================================================
test_that("pv validates NULL inputs", {
  expect_error(pv(NULL, c(0.1, 0.2), 1), "bac, schedule, and time_period must not be NULL")
  expect_error(pv(100000, NULL, 1), "bac, schedule, and time_period must not be NULL")
  expect_error(pv(100000, c(0.1, 0.2), NULL), "bac, schedule, and time_period must not be NULL")
})

test_that("pv validates numeric inputs", {
  expect_error(pv("100000", c(0.1, 0.2), 1), "bac, schedule, and time_period must be numeric")
  expect_error(pv(100000, "schedule", 1), "bac, schedule, and time_period must be numeric")
  expect_error(pv(100000, c(0.1, 0.2), "1"), "bac, schedule, and time_period must be numeric")
})

test_that("pv validates single value inputs", {
  expect_error(pv(c(100000, 200000), c(0.1, 0.2), 1), "bac must be a single numeric value")
  expect_error(pv(100000, c(0.1, 0.2), c(1, 2)), "time_period must be a single numeric value")
})

test_that("pv validates NA inputs", {
  # NA alone is logical, so it fails numeric check first; use NA_real_ for numeric NA
  expect_error(pv(NA_real_, c(0.1, 0.2), 1), "bac, schedule, and time_period must not contain NA values")
  expect_error(pv(100000, c(0.1, NA), 1), "bac, schedule, and time_period must not contain NA values")
  expect_error(pv(100000, c(0.1, 0.2), NA_real_), "bac, schedule, and time_period must not contain NA values")
})

test_that("pv rejects NaN inputs", {
  expect_error(pv(NaN, c(0.1, 0.2), 1), "bac, schedule, and time_period must not contain NaN values")
  expect_error(pv(100000, c(0.1, NaN), 1), "bac, schedule, and time_period must not contain NaN values")
  expect_error(pv(100000, c(0.1, 0.2), NaN), "bac, schedule, and time_period must not contain NaN values")
})

test_that("pv rejects Inf inputs", {
  expect_error(pv(Inf, c(0.1, 0.2), 1), "bac, schedule, and time_period must not contain infinite values")
  expect_error(pv(100000, c(0.1, Inf), 1), "bac, schedule, and time_period must not contain infinite values")
  expect_error(pv(100000, c(0.1, 0.2), Inf), "bac, schedule, and time_period must not contain infinite values")
})

test_that("pv validates schedule range", {
  expect_error(pv(100000, c(0.1, 1.5), 1), "schedule values must be between 0 and 1")
  expect_error(pv(100000, c(-0.1, 0.5), 1), "schedule values must be between 0 and 1")
})

test_that("pv validates time_period range", {
  expect_error(pv(100000, c(0.1, 0.2, 0.5), 0), "time_period must be within the range of the schedule vector")
  expect_error(pv(100000, c(0.1, 0.2, 0.5), 5), "time_period must be within the range of the schedule vector")
})

test_that("pv validates non-negative bac", {
  expect_error(pv(-100000, c(0.1, 0.2), 1), "bac must be non-negative")
})

test_that("pv validates non-empty schedule", {
  expect_error(pv(100000, numeric(0), 1), "schedule must not be empty")
})

# ============================================================================
# EV Error Handling Tests
# ============================================================================
test_that("ev validates NULL inputs", {
  expect_error(ev(NULL, 0.5), "bac and actual_per_complete must not be NULL")
  expect_error(ev(100000, NULL), "bac and actual_per_complete must not be NULL")
})

test_that("ev validates numeric inputs", {
  expect_error(ev("100000", 0.5), "bac and actual_per_complete must be numeric")
  expect_error(ev(100000, "0.5"), "bac and actual_per_complete must be numeric")
})

test_that("ev validates single value inputs", {
  expect_error(ev(c(100000, 200000), 0.5), "bac must be a single numeric value")
  expect_error(ev(100000, c(0.3, 0.5)), "actual_per_complete must be a single numeric value")
})

test_that("ev rejects NaN inputs", {
  expect_error(ev(NaN, 0.5), "bac and actual_per_complete must not be NaN")
  expect_error(ev(100000, NaN), "bac and actual_per_complete must not be NaN")
})

test_that("ev rejects NA inputs", {
  expect_error(ev(NA_real_, 0.5), "bac and actual_per_complete must not be NA")
  expect_error(ev(100000, NA_real_), "bac and actual_per_complete must not be NA")
})

test_that("ev rejects Inf inputs", {
  expect_error(ev(Inf, 0.5), "bac and actual_per_complete must not be infinite")
  expect_error(ev(100000, Inf), "bac and actual_per_complete must not be infinite")
})

test_that("ev validates percentage range", {
  expect_error(ev(100000, 1.5), "actual_per_complete must be between 0 and 1")
  expect_error(ev(100000, -0.1), "actual_per_complete must be between 0 and 1")
})

test_that("ev validates non-negative bac", {
  expect_error(ev(-100000, 0.5), "bac must be non-negative")
})

# ============================================================================
# AC Tests
# ============================================================================
test_that("ac works with cumulative costs (default)", {
  cumulative_costs <- c(9000, 27000, 63000)
  expect_equal(ac(cumulative_costs, 2), 27000)
  expect_equal(ac(cumulative_costs, 3), 63000)
})

test_that("ac works with period costs", {
  period_costs <- c(9000, 18000, 36000)
  expect_equal(ac(period_costs, 2, cumulative = FALSE), 27000)
  expect_equal(ac(period_costs, 3, cumulative = FALSE), 63000)
})

test_that("ac validates NULL inputs", {
  expect_error(ac(NULL, 1), "actual_costs and time_period must not be NULL")
  expect_error(ac(c(1000, 2000), NULL), "actual_costs and time_period must not be NULL")
})

test_that("ac validates cumulative parameter", {
  expect_error(ac(c(1000, 2000), 1, cumulative = "yes"), "cumulative must be a single logical value")
  expect_error(ac(c(1000, 2000), 1, cumulative = c(TRUE, FALSE)), "cumulative must be a single logical value")
})

test_that("ac validates numeric inputs", {
  expect_error(ac("not numeric", 1), "actual_costs and time_period must be numeric")
  expect_error(ac(c(1000, 2000), "1"), "actual_costs and time_period must be numeric")
})

test_that("ac validates single value time_period", {
  expect_error(ac(c(1000, 2000), c(1, 2)), "time_period must be a single numeric value")
})

test_that("ac rejects NaN inputs", {
  expect_error(ac(c(NaN, 2000), 1), "actual_costs and time_period must not contain NaN values")
  expect_error(ac(c(1000, 2000), NaN), "actual_costs and time_period must not contain NaN values")
})

test_that("ac rejects NA inputs", {
  expect_error(ac(c(NA_real_, 2000), 1), "actual_costs and time_period must not contain NA values")
  expect_error(ac(c(1000, 2000), NA_real_), "actual_costs and time_period must not contain NA values")
})

test_that("ac rejects Inf inputs", {
  expect_error(ac(c(Inf, 2000), 1), "actual_costs and time_period must not contain infinite values")
  expect_error(ac(c(1000, 2000), Inf), "actual_costs and time_period must not contain infinite values")
})

test_that("ac validates non-empty costs", {
  expect_error(ac(numeric(0), 1), "actual_costs must not be empty")
})

test_that("ac validates time_period range", {
  expect_error(ac(c(1000, 2000), 0), "time_period must be within the range of the actual_costs vector")
  expect_error(ac(c(1000, 2000), 5), "time_period must be within the range of the actual_costs vector")
})

test_that("ac validates non-negative costs", {
  expect_error(ac(c(1000, -2000), 1), "actual_costs must be non-negative")
})

# ============================================================================
# SV Error Handling Tests
# ============================================================================
test_that("sv validates NULL inputs", {
  expect_error(sv(NULL, 40000), "ev and pv must not be NULL")
  expect_error(sv(35000, NULL), "ev and pv must not be NULL")
})

test_that("sv validates numeric inputs", {
  expect_error(sv("35000", 40000), "ev and pv must be numeric")
  expect_error(sv(35000, "40000"), "ev and pv must be numeric")
})

test_that("sv validates single value inputs", {
  expect_error(sv(c(35000, 36000), 40000), "ev and pv must be single numeric values")
  expect_error(sv(35000, c(40000, 41000)), "ev and pv must be single numeric values")
})

test_that("sv rejects NaN inputs", {
  expect_error(sv(NaN, 40000), "ev and pv must not be NaN")
  expect_error(sv(35000, NaN), "ev and pv must not be NaN")
})

test_that("sv rejects NA inputs", {
  expect_error(sv(NA_real_, 40000), "ev and pv must not be NA")
  expect_error(sv(35000, NA_real_), "ev and pv must not be NA")
})

test_that("sv rejects Inf inputs", {
  expect_error(sv(Inf, 40000), "ev and pv must not be infinite")
  expect_error(sv(35000, Inf), "ev and pv must not be infinite")
})

test_that("sv validates non-negative inputs", {
  expect_error(sv(-35000, 40000), "ev must be non-negative")
  expect_error(sv(35000, -40000), "pv must be non-negative")
})

# ============================================================================
# CV Error Handling Tests
# ============================================================================
test_that("cv validates NULL inputs", {
  expect_error(cv(NULL, 36000), "ev and ac must not be NULL")
  expect_error(cv(35000, NULL), "ev and ac must not be NULL")
})

test_that("cv validates numeric inputs", {
  expect_error(cv("35000", 36000), "ev and ac must be numeric")
  expect_error(cv(35000, "36000"), "ev and ac must be numeric")
})

test_that("cv validates single value inputs", {
  expect_error(cv(c(35000, 36000), 36000), "ev and ac must be single numeric values")
  expect_error(cv(35000, c(36000, 37000)), "ev and ac must be single numeric values")
})

test_that("cv rejects NaN inputs", {
  expect_error(cv(NaN, 36000), "ev and ac must not be NaN")
  expect_error(cv(35000, NaN), "ev and ac must not be NaN")
})

test_that("cv rejects NA inputs", {
  expect_error(cv(NA_real_, 36000), "ev and ac must not be NA")
  expect_error(cv(35000, NA_real_), "ev and ac must not be NA")
})

test_that("cv rejects Inf inputs", {
  expect_error(cv(Inf, 36000), "ev and ac must not be infinite")
  expect_error(cv(35000, Inf), "ev and ac must not be infinite")
})

test_that("cv validates non-negative inputs", {
  expect_error(cv(-35000, 36000), "ev must be non-negative")
  expect_error(cv(35000, -36000), "ac must be non-negative")
})

# ============================================================================
# SPI Error Handling Tests
# ============================================================================
test_that("spi validates NULL inputs", {
  expect_error(spi(NULL, 40000), "ev and pv must not be NULL")
  expect_error(spi(35000, NULL), "ev and pv must not be NULL")
})

test_that("spi validates numeric inputs", {
  expect_error(spi("35000", 40000), "ev and pv must be numeric")
  expect_error(spi(35000, "40000"), "ev and pv must be numeric")
})

test_that("spi validates single value inputs", {
  expect_error(spi(c(35000, 36000), 40000), "ev and pv must be single numeric values")
  expect_error(spi(35000, c(40000, 41000)), "ev and pv must be single numeric values")
})

test_that("spi rejects NaN inputs", {
  expect_error(spi(NaN, 40000), "ev and pv must not be NaN")
  expect_error(spi(35000, NaN), "ev and pv must not be NaN")
})

test_that("spi rejects NA inputs", {
  expect_error(spi(NA_real_, 40000), "ev and pv must not be NA")
  expect_error(spi(35000, NA_real_), "ev and pv must not be NA")
})

test_that("spi rejects Inf inputs", {
  expect_error(spi(Inf, 40000), "ev and pv must not be infinite")
  expect_error(spi(35000, Inf), "ev and pv must not be infinite")
})

test_that("spi validates ev non-negative", {
  expect_error(spi(-35000, 40000), "ev must be non-negative")
})

test_that("spi validates pv greater than zero", {
  expect_error(spi(35000, 0), "pv must be greater than zero")
  expect_error(spi(35000, -40000), "pv must be greater than zero")
})

# ============================================================================
# CPI Error Handling Tests
# ============================================================================
test_that("cpi validates NULL inputs", {
  expect_error(cpi(NULL, 36000), "ev and ac must not be NULL")
  expect_error(cpi(35000, NULL), "ev and ac must not be NULL")
})

test_that("cpi validates numeric inputs", {
  expect_error(cpi("35000", 36000), "ev and ac must be numeric")
  expect_error(cpi(35000, "36000"), "ev and ac must be numeric")
})

test_that("cpi validates single value inputs", {
  expect_error(cpi(c(35000, 36000), 36000), "ev and ac must be single numeric values")
  expect_error(cpi(35000, c(36000, 37000)), "ev and ac must be single numeric values")
})

test_that("cpi rejects NaN inputs", {
  expect_error(cpi(NaN, 36000), "ev and ac must not be NaN")
  expect_error(cpi(35000, NaN), "ev and ac must not be NaN")
})

test_that("cpi rejects NA inputs", {
  expect_error(cpi(NA_real_, 36000), "ev and ac must not be NA")
  expect_error(cpi(35000, NA_real_), "ev and ac must not be NA")
})

test_that("cpi rejects Inf inputs", {
  expect_error(cpi(Inf, 36000), "ev and ac must not be infinite")
  expect_error(cpi(35000, Inf), "ev and ac must not be infinite")
})

test_that("cpi validates ev non-negative", {
  expect_error(cpi(-35000, 36000), "ev must be non-negative")
})

test_that("cpi validates ac greater than zero", {
  expect_error(cpi(35000, 0), "ac must be greater than zero")
  expect_error(cpi(35000, -36000), "ac must be greater than zero")
})

# ============================================================================
# EAC Tests
# ============================================================================
test_that("eac typical method calculation is correct", {
  bac <- 100000
  cpi_val <- 0.8
  expected <- bac / cpi_val
  expect_equal(eac(bac, cpi = cpi_val), expected)
})

test_that("eac atypical method calculation is correct", {
  bac <- 100000
  ac_val <- 63000
  ev_val <- 35000
  expected <- ac_val + (bac - ev_val)
  expect_equal(eac(bac, method = "atypical", ac = ac_val, ev = ev_val), expected)
})

test_that("eac combined method calculation is correct", {
  bac <- 100000
  cpi_val <- 0.8
  spi_val <- 0.9
  ac_val <- 63000
  ev_val <- 35000
  expected <- ac_val + (bac - ev_val) / (cpi_val * spi_val)
  expect_equal(eac(bac, method = "combined", cpi = cpi_val, ac = ac_val, ev = ev_val, spi = spi_val), expected)
})

test_that("eac validates method parameter", {
  expect_error(eac(100000, method = "invalid"), "method must be one of: typical, atypical, combined")
})

test_that("eac typical method validates required parameters", {
  expect_error(eac(100000, method = "typical"), "cpi is required for the 'typical' method")
  expect_error(eac(100000, method = "typical", cpi = 0), "cpi must be greater than zero")
})

test_that("eac atypical method validates required parameters", {
  expect_error(eac(100000, method = "atypical", ac = 63000), "ac and ev are required for the 'atypical' method")
  expect_error(eac(100000, method = "atypical", ev = 35000), "ac and ev are required for the 'atypical' method")
})

test_that("eac combined method validates required parameters", {
  expect_error(
    eac(100000, method = "combined", cpi = 0.8, ac = 63000, ev = 35000),
    "cpi, ac, ev, and spi are required for the 'combined' method"
  )
  expect_error(
    eac(100000, method = "combined", cpi = 0, ac = 63000, ev = 35000, spi = 0.9),
    "cpi and spi must be greater than zero"
  )
})

test_that("eac validates bac", {
  expect_error(eac(NULL, cpi = 0.8), "bac must not be NULL")
  expect_error(eac(-100000, cpi = 0.8), "bac must be non-negative")
})

test_that("eac rejects NaN bac", {
  expect_error(eac(NaN, cpi = 0.8), "bac must be a single non-NaN numeric value")
})

test_that("eac rejects NA bac", {
  expect_error(eac(NA_real_, cpi = 0.8), "bac must be a single numeric value")
})

test_that("eac rejects Inf bac", {
  expect_error(eac(Inf, cpi = 0.8), "bac must not be infinite")
})

# Typical method NaN/NA/Inf for cpi
test_that("eac typical rejects NaN cpi", {
  expect_error(eac(100000, method = "typical", cpi = NaN), "cpi must be a single non-NaN numeric value")
})

test_that("eac typical rejects NA cpi", {
  expect_error(eac(100000, method = "typical", cpi = NA_real_), "cpi must be a single numeric value")
})

test_that("eac typical rejects Inf cpi", {
  expect_error(eac(100000, method = "typical", cpi = Inf), "cpi must not be infinite")
})

# Atypical method NaN/NA/Inf for ac, ev
test_that("eac atypical rejects NaN ac", {
  expect_error(eac(100000, method = "atypical", ac = NaN, ev = 35000), "ac must be a single non-NaN numeric value")
})

test_that("eac atypical rejects NA ac", {
  expect_error(eac(100000, method = "atypical", ac = NA_real_, ev = 35000), "ac must be a single numeric value")
})

test_that("eac atypical rejects Inf ac", {
  expect_error(eac(100000, method = "atypical", ac = Inf, ev = 35000), "ac must not be infinite")
})

test_that("eac atypical rejects NaN ev", {
  expect_error(eac(100000, method = "atypical", ac = 63000, ev = NaN), "ev must be a single non-NaN numeric value")
})

test_that("eac atypical rejects NA ev", {
  expect_error(eac(100000, method = "atypical", ac = 63000, ev = NA_real_), "ev must be a single numeric value")
})

test_that("eac atypical rejects Inf ev", {
  expect_error(eac(100000, method = "atypical", ac = 63000, ev = Inf), "ev must not be infinite")
})

test_that("eac atypical rejects negative ac/ev", {
  expect_error(eac(100000, method = "atypical", ac = -1, ev = 35000), "ac and ev must be non-negative")
  expect_error(eac(100000, method = "atypical", ac = 63000, ev = -1), "ac and ev must be non-negative")
})

# Combined method NaN/NA/Inf for cpi, spi, ac, ev
test_that("eac combined rejects NaN cpi", {
  expect_error(
    eac(100000, method = "combined", cpi = NaN, ac = 63000, ev = 35000, spi = 0.9),
    "cpi must be a single non-NaN numeric value for 'combined' method"
  )
})

test_that("eac combined rejects NA cpi", {
  expect_error(
    eac(100000, method = "combined", cpi = NA_real_, ac = 63000, ev = 35000, spi = 0.9),
    "cpi must be a single numeric value"
  )
})

test_that("eac combined rejects Inf cpi", {
  expect_error(
    eac(100000, method = "combined", cpi = Inf, ac = 63000, ev = 35000, spi = 0.9),
    "cpi must not be infinite for 'combined' method"
  )
})

test_that("eac combined rejects NaN spi", {
  expect_error(
    eac(100000, method = "combined", cpi = 0.8, ac = 63000, ev = 35000, spi = NaN),
    "spi must be a single non-NaN numeric value"
  )
})

test_that("eac combined rejects NA spi", {
  expect_error(
    eac(100000, method = "combined", cpi = 0.8, ac = 63000, ev = 35000, spi = NA_real_),
    "spi must be a single numeric value"
  )
})

test_that("eac combined rejects Inf spi", {
  expect_error(
    eac(100000, method = "combined", cpi = 0.8, ac = 63000, ev = 35000, spi = Inf),
    "spi must not be infinite"
  )
})

test_that("eac combined rejects NaN ac", {
  expect_error(
    eac(100000, method = "combined", cpi = 0.8, ac = NaN, ev = 35000, spi = 0.9),
    "ac must be a single non-NaN numeric value for 'combined' method"
  )
})

test_that("eac combined rejects NA ac", {
  expect_error(
    eac(100000, method = "combined", cpi = 0.8, ac = NA_real_, ev = 35000, spi = 0.9),
    "ac must be a single numeric value"
  )
})

test_that("eac combined rejects Inf ac", {
  expect_error(
    eac(100000, method = "combined", cpi = 0.8, ac = Inf, ev = 35000, spi = 0.9),
    "ac must not be infinite for 'combined' method"
  )
})

test_that("eac combined rejects NaN ev", {
  expect_error(
    eac(100000, method = "combined", cpi = 0.8, ac = 63000, ev = NaN, spi = 0.9),
    "ev must be a single non-NaN numeric value for 'combined' method"
  )
})

test_that("eac combined rejects NA ev", {
  expect_error(
    eac(100000, method = "combined", cpi = 0.8, ac = 63000, ev = NA_real_, spi = 0.9),
    "ev must be a single numeric value"
  )
})

test_that("eac combined rejects Inf ev", {
  expect_error(
    eac(100000, method = "combined", cpi = 0.8, ac = 63000, ev = Inf, spi = 0.9),
    "ev must not be infinite for 'combined' method"
  )
})

test_that("eac combined rejects negative ac/ev", {
  expect_error(
    eac(100000, method = "combined", cpi = 0.8, ac = -1, ev = 35000, spi = 0.9),
    "ac and ev must be non-negative"
  )
  expect_error(
    eac(100000, method = "combined", cpi = 0.8, ac = 63000, ev = -1, spi = 0.9),
    "ac and ev must be non-negative"
  )
})

# ============================================================================
# ETC Tests
# ============================================================================
test_that("etc calculation without cpi is correct", {
  bac <- 100000
  ev_val <- 35000
  expected <- bac - ev_val
  expect_equal(etc(bac, ev_val), expected)
})

test_that("etc calculation with cpi is correct", {
  bac <- 100000
  ev_val <- 35000
  cpi_val <- 0.8
  expected <- (bac - ev_val) / cpi_val
  expect_equal(etc(bac, ev_val, cpi_val), expected)
})

test_that("etc validates NULL inputs", {
  expect_error(etc(NULL, 35000), "bac and ev must not be NULL")
  expect_error(etc(100000, NULL), "bac and ev must not be NULL")
})

test_that("etc validates cpi greater than zero", {
  expect_error(etc(100000, 35000, 0), "cpi must be greater than zero")
  expect_error(etc(100000, 35000, -0.5), "cpi must be greater than zero")
})

test_that("etc validates numeric inputs", {
  expect_error(etc("100000", 35000), "bac and ev must be numeric")
  expect_error(etc(100000, "35000"), "bac and ev must be numeric")
})

test_that("etc validates single value inputs", {
  expect_error(etc(c(100000, 200000), 35000), "bac and ev must be single numeric values")
  expect_error(etc(100000, c(35000, 36000)), "bac and ev must be single numeric values")
})

test_that("etc rejects NaN inputs", {
  expect_error(etc(NaN, 35000), "bac and ev must not be NaN")
  expect_error(etc(100000, NaN), "bac and ev must not be NaN")
})

test_that("etc rejects NA inputs", {
  expect_error(etc(NA_real_, 35000), "bac and ev must not be NA")
  expect_error(etc(100000, NA_real_), "bac and ev must not be NA")
})

test_that("etc rejects Inf inputs", {
  expect_error(etc(Inf, 35000), "bac and ev must not be infinite")
  expect_error(etc(100000, Inf), "bac and ev must not be infinite")
})

test_that("etc rejects NaN cpi", {
  expect_error(etc(100000, 35000, NaN), "cpi must be a single non-NaN numeric value")
})

test_that("etc rejects NA cpi", {
  expect_error(etc(100000, 35000, NA_real_), "cpi must be a single numeric value")
})

test_that("etc rejects Inf cpi", {
  expect_error(etc(100000, 35000, Inf), "cpi must not be infinite")
})

test_that("etc validates non-negative inputs", {
  expect_error(etc(-100000, 35000), "bac and ev must be non-negative")
  expect_error(etc(100000, -35000), "bac and ev must be non-negative")
})

# ============================================================================
# VAC Tests
# ============================================================================
test_that("vac calculation is correct", {
  bac <- 100000
  eac_val <- 120000
  expected <- bac - eac_val
  expect_equal(vac(bac, eac_val), expected)
})

test_that("vac validates NULL inputs", {
  expect_error(vac(NULL, 120000), "bac and eac must not be NULL")
  expect_error(vac(100000, NULL), "bac and eac must not be NULL")
})

test_that("vac validates numeric inputs", {
  expect_error(vac("100000", 120000), "bac and eac must be numeric")
  expect_error(vac(100000, "120000"), "bac and eac must be numeric")
})

test_that("vac validates single value inputs", {
  expect_error(vac(c(100000, 200000), 120000), "bac and eac must be single numeric values")
  expect_error(vac(100000, c(120000, 130000)), "bac and eac must be single numeric values")
})

test_that("vac rejects NaN inputs", {
  expect_error(vac(NaN, 120000), "bac and eac must not be NaN")
  expect_error(vac(100000, NaN), "bac and eac must not be NaN")
})

test_that("vac rejects NA inputs", {
  expect_error(vac(NA_real_, 120000), "bac and eac must not be NA")
  expect_error(vac(100000, NA_real_), "bac and eac must not be NA")
})

test_that("vac rejects Inf inputs", {
  expect_error(vac(Inf, 120000), "bac and eac must not be infinite")
  expect_error(vac(100000, Inf), "bac and eac must not be infinite")
})

test_that("vac validates non-negative inputs", {
  expect_error(vac(-100000, 120000), "bac and eac must be non-negative")
  expect_error(vac(100000, -120000), "bac and eac must be non-negative")
})

# ============================================================================
# TCPI Tests
# ============================================================================
test_that("tcpi to BAC calculation is correct", {
  bac <- 100000
  ev_val <- 35000
  ac_val <- 40000
  expected <- (bac - ev_val) / (bac - ac_val)
  expect_equal(tcpi(bac, ev_val, ac_val), expected)
})

test_that("tcpi to EAC calculation is correct", {
  bac <- 100000
  ev_val <- 35000
  ac_val <- 40000
  eac_val <- 120000
  expected <- (bac - ev_val) / (eac_val - ac_val)
  expect_equal(tcpi(bac, ev_val, ac_val, target = "eac", eac = eac_val), expected)
})

test_that("tcpi validates NULL inputs", {
  expect_error(tcpi(NULL, 35000, 40000), "bac, ev, and ac must not be NULL")
  expect_error(tcpi(100000, NULL, 40000), "bac, ev, and ac must not be NULL")
  expect_error(tcpi(100000, 35000, NULL), "bac, ev, and ac must not be NULL")
})

test_that("tcpi validates target parameter", {
  expect_error(tcpi(100000, 35000, 40000, target = "invalid"), "target must be either 'bac' or 'eac'")
})

test_that("tcpi validates eac required for eac target", {
  expect_error(tcpi(100000, 35000, 40000, target = "eac"), "eac is required when target = 'eac'")
})

test_that("tcpi validates denominator not zero for BAC target", {
  expect_error(tcpi(100000, 35000, 100000), "Cannot calculate TCPI: actual cost already meets or exceeds budget")
})

test_that("tcpi validates numeric inputs", {
  expect_error(tcpi("100000", 35000, 40000), "bac, ev, and ac must be numeric")
  expect_error(tcpi(100000, "35000", 40000), "bac, ev, and ac must be numeric")
  expect_error(tcpi(100000, 35000, "40000"), "bac, ev, and ac must be numeric")
})

test_that("tcpi validates single value inputs", {
  expect_error(tcpi(c(100000, 200000), 35000, 40000), "bac, ev, and ac must be single numeric values")
  expect_error(tcpi(100000, c(35000, 36000), 40000), "bac, ev, and ac must be single numeric values")
  expect_error(tcpi(100000, 35000, c(40000, 41000)), "bac, ev, and ac must be single numeric values")
})

test_that("tcpi rejects NaN inputs", {
  expect_error(tcpi(NaN, 35000, 40000), "bac, ev, and ac must not be NaN")
  expect_error(tcpi(100000, NaN, 40000), "bac, ev, and ac must not be NaN")
  expect_error(tcpi(100000, 35000, NaN), "bac, ev, and ac must not be NaN")
})

test_that("tcpi rejects NA inputs", {
  expect_error(tcpi(NA_real_, 35000, 40000), "bac, ev, and ac must not be NA")
  expect_error(tcpi(100000, NA_real_, 40000), "bac, ev, and ac must not be NA")
  expect_error(tcpi(100000, 35000, NA_real_), "bac, ev, and ac must not be NA")
})

test_that("tcpi rejects Inf inputs", {
  expect_error(tcpi(Inf, 35000, 40000), "bac, ev, and ac must not be infinite")
  expect_error(tcpi(100000, Inf, 40000), "bac, ev, and ac must not be infinite")
  expect_error(tcpi(100000, 35000, Inf), "bac, ev, and ac must not be infinite")
})

test_that("tcpi validates non-negative inputs", {
  expect_error(tcpi(-100000, 35000, 40000), "bac, ev, and ac must be non-negative")
  expect_error(tcpi(100000, -35000, 40000), "bac, ev, and ac must be non-negative")
  expect_error(tcpi(100000, 35000, -40000), "bac, ev, and ac must be non-negative")
})

test_that("tcpi eac target rejects NaN eac", {
  expect_error(
    tcpi(100000, 35000, 40000, target = "eac", eac = NaN),
    "eac must be a single non-NaN numeric value"
  )
})

test_that("tcpi eac target rejects NA eac", {
  expect_error(
    tcpi(100000, 35000, 40000, target = "eac", eac = NA_real_),
    "eac must be a single numeric value"
  )
})

test_that("tcpi eac target rejects Inf eac", {
  expect_error(
    tcpi(100000, 35000, 40000, target = "eac", eac = Inf),
    "eac must not be infinite"
  )
})

test_that("tcpi eac target rejects negative eac", {
  expect_error(
    tcpi(100000, 35000, 40000, target = "eac", eac = -1),
    "eac must be non-negative"
  )
})

test_that("tcpi validates denominator not zero for EAC target", {
  expect_error(
    tcpi(100000, 35000, 120000, target = "eac", eac = 120000),
    "Cannot calculate TCPI: actual cost already meets or exceeds EAC"
  )
})

# ============================================================================
# G5.3: EVM Return Value Tests
# ============================================================================
test_that("pv result contains no NA, NaN, or Inf", {
  result <- pv(100000, c(0.1, 0.2, 0.4, 0.7, 1.0), 3)
  expect_false(is.na(result))
  expect_false(is.nan(result))
  expect_false(is.infinite(result))
})

test_that("ev result contains no NA, NaN, or Inf", {
  result <- ev(100000, 0.35)
  expect_false(is.na(result))
  expect_false(is.nan(result))
  expect_false(is.infinite(result))
})

test_that("ac result contains no NA, NaN, or Inf", {
  result <- ac(c(9000, 18000, 36000), 2)
  expect_false(is.na(result))
  expect_false(is.nan(result))
  expect_false(is.infinite(result))
})

test_that("sv result contains no NA, NaN, or Inf", {
  result <- sv(35000, 40000)
  expect_false(is.na(result))
  expect_false(is.nan(result))
  expect_false(is.infinite(result))
})

test_that("cv result contains no NA, NaN, or Inf", {
  result <- cv(35000, 36000)
  expect_false(is.na(result))
  expect_false(is.nan(result))
  expect_false(is.infinite(result))
})

test_that("spi result contains no NA, NaN, or Inf", {
  result <- spi(35000, 40000)
  expect_false(is.na(result))
  expect_false(is.nan(result))
  expect_false(is.infinite(result))
})

test_that("cpi result contains no NA, NaN, or Inf", {
  result <- cpi(35000, 36000)
  expect_false(is.na(result))
  expect_false(is.nan(result))
  expect_false(is.infinite(result))
})

test_that("eac result contains no NA, NaN, or Inf", {
  result <- eac(100000, cpi = 0.8)
  expect_false(is.na(result))
  expect_false(is.nan(result))
  expect_false(is.infinite(result))
})

test_that("etc result contains no NA, NaN, or Inf", {
  result <- etc(100000, 35000)
  expect_false(is.na(result))
  expect_false(is.nan(result))
  expect_false(is.infinite(result))
})

test_that("vac result contains no NA, NaN, or Inf", {
  result <- vac(100000, 120000)
  expect_false(is.na(result))
  expect_false(is.nan(result))
  expect_false(is.infinite(result))
})

test_that("tcpi result contains no NA, NaN, or Inf", {
  result <- tcpi(100000, 35000, 40000)
  expect_false(is.na(result))
  expect_false(is.nan(result))
  expect_false(is.infinite(result))
})

# ============================================================================
# Parameter Recovery Tests (G5.6, G5.6a)
# ============================================================================

test_that("pv recovers known planned value from schedule", {
  bac <- 100000
  schedule <- c(0.1, 0.2, 0.4, 0.7, 1.0)
  time_period <- 3

  result <- pv(bac, schedule, time_period)

  # Expected: BAC * schedule[time_period] = 100000 * 0.4 = 40000
  expected <- 100000 * 0.4

  expect_equal(result, expected, tolerance = 1e-6)
})

test_that("ev recovers known earned value from completion", {
  bac <- 100000
  actual_per_complete <- 0.35

  result <- ev(bac, actual_per_complete)

  # Expected: BAC * actual_per_complete = 100000 * 0.35 = 35000
  expected <- 100000 * 0.35

  expect_equal(result, expected, tolerance = 1e-6)
})

test_that("sv recovers known schedule variance", {
  ev_val <- 35000
  pv_val <- 40000

  result <- sv(ev_val, pv_val)

  # Expected: EV - PV = 35000 - 40000 = -5000
  expected <- 35000 - 40000

  expect_equal(result, expected, tolerance = 1e-6)
})

test_that("cv recovers known cost variance", {
  ev_val <- 35000
  ac_val <- 36000

  result <- cv(ev_val, ac_val)

  # Expected: EV - AC = 35000 - 36000 = -1000
  expected <- 35000 - 36000

  expect_equal(result, expected, tolerance = 1e-6)
})

test_that("spi recovers known schedule performance index", {
  ev_val <- 35000
  pv_val <- 40000

  result <- spi(ev_val, pv_val)

  # Expected: EV / PV = 35000 / 40000 = 0.875
  expected <- 35000 / 40000

  expect_equal(result, expected, tolerance = 1e-6)
})

test_that("cpi recovers known cost performance index", {
  ev_val <- 35000
  ac_val <- 36000

  result <- cpi(ev_val, ac_val)

  # Expected: EV / AC = 35000 / 36000 = 0.9722...
  expected <- 35000 / 36000

  expect_equal(result, expected, tolerance = 1e-6)
})

# ============================================================================
# Edge Condition Tests (G5.8a) - Zero-Length Data
# ============================================================================

test_that("pv rejects zero-length schedule", {
  expect_error(pv(100000, numeric(0), 1), "schedule must not be empty")
})

test_that("ac rejects zero-length actual_costs", {
  expect_error(ac(numeric(0), 1), "actual_costs must not be empty")
})

# ============================================================================
# Edge Condition Tests (G5.8d) - Out-of-Scope Data
# ============================================================================

test_that("pv rejects time_period beyond schedule length", {
  bac <- 100000
  schedule <- c(0.1, 0.2, 0.4, 0.7, 1.0)

  expect_error(pv(bac, schedule, 10), "time_period must be within the range of the schedule vector")
})

test_that("ac rejects time_period beyond actual_costs length", {
  actual_costs <- c(10000, 20000, 30000)

  expect_error(ac(actual_costs, 10), "time_period must be within the range of the actual_costs vector")
})

# ============================================================================
# Noise Susceptibility Tests (G5.9a) - Trivial Noise
# ============================================================================

test_that("pv is stable to trivial noise in inputs", {
  bac <- 100000
  schedule <- c(0.1, 0.2, 0.4, 0.7, 1.0)
  time_period <- 3

  result_clean <- pv(bac, schedule, time_period)

  # Add trivial noise
  bac_noisy <- bac + runif(1, -.Machine$double.eps, .Machine$double.eps)
  result_noisy <- pv(bac_noisy, schedule, time_period)

  expect_equal(result_clean, result_noisy, tolerance = 10 * .Machine$double.eps)
})

# Default embedding model used across RAG functions
.pra_default_embed_model <- "nomic-embed-text"

#' Create a PRA Risk Analysis Chat Agent
#'
#' Creates an ellmer chat object configured as a project risk analysis expert,
#' with all PRA functions registered as tools and optional RAG context retrieval
#' from the bundled knowledge base.
#'
#' By default, uses a local Ollama model for fully offline, private operation.
#' Alternatively, supply a pre-configured ellmer chat object (e.g.,
#' `ellmer::chat_openai()`) via the `chat` parameter for cloud-hosted models.
#'
#' @param chat An optional pre-configured ellmer chat object. If provided,
#'   `model` is ignored and tools are registered on this object instead.
#' @param model Character. Ollama model name (default `"llama3.2"`). Must support
#'   tool calling. Other options: `"qwen2.5"`, `"llama3.1:70b"`.
#' @param rag Logical. Whether to use RAG context from the PRA knowledge base
#'   (default `TRUE`). Requires Ollama embedding model to be available.
#' @param embed_model Character. Ollama embedding model for RAG (default
#'   `"nomic-embed-text"`). Only used when `rag = TRUE`.
#'
#' @return A configured ellmer chat object with PRA tools registered. Use
#'   `chat$chat("your question")` to interact.
#'
#' @examples
#' \dontrun{
#' # Default: local Ollama model
#' chat <- pra_chat()
#' chat$chat("Run a Monte Carlo simulation for a 3-task project with
#'   Task A: normal(10, 2), Task B: triangular(5, 10, 15), Task C: uniform(8, 12)")
#'
#' # Use a cloud model for better accuracy
#' chat <- pra_chat(chat = ellmer::chat_openai(model = "gpt-4o"))
#'
#' # Follow-up questions use conversation context
#' chat$chat("What is the contingency reserve at 95% confidence?")
#' }
#'
#' @export
pra_chat <- function(chat = NULL, model = "llama3.2", rag = TRUE,
                     embed_model = .pra_default_embed_model) {
  if (!requireNamespace("ellmer", quietly = TRUE)) {
    stop("Package 'ellmer' is required. Install with: install.packages('ellmer')")
  }

  # Build system prompt
  system_prompt <- pra_system_prompt()

  # Create or configure chat object

  if (is.null(chat)) {
    chat <- ellmer::chat_ollama(
      model = model,
      system_prompt = system_prompt,
      api_args = list(
        options = list(num_ctx = 16384L)
      )
    )
  } else {
    # User supplied a pre-configured chat; set the system prompt
    chat$set_system_prompt(system_prompt)
  }

  # Register all PRA tools
  tools <- pra_tools()
  for (tool in tools) {
    chat$register_tool(tool)
  }

  # Optionally set up RAG
  if (rag) {
    tryCatch(
      {
        if (!requireNamespace("ragnar", quietly = TRUE)) {
          message("Package 'ragnar' not installed. Chat will proceed without RAG context.")
        } else {
          store <- build_knowledge_base(embed_model = embed_model)
          .pra_agent_env$rag_store <- store
          message("RAG knowledge base loaded. Context retrieval enabled.")
        }
      },
      error = function(e) {
        warning(
          "Could not build RAG knowledge base: ", e$message,
          "\nChat will proceed without RAG context."
        )
      }
    )
  }

  chat
}

#' Generate the PRA System Prompt
#'
#' Creates the system prompt that defines the agent's persona, methodology
#' guidance, and tool selection heuristics. Kept concise to work well with
#' smaller local models.
#'
#' @return A character string containing the system prompt.
#' @keywords internal
pra_system_prompt <- function() {
  paste0(
    "You are a Project Risk Analysis expert. ",
    "You answer questions about risk analysis methods, interpret results, ",
    "and run computations when the user provides specific numerical data.\n\n",
    "## When to use tools vs. RAG context\n",
    "- If the user provides SPECIFIC NUMERICAL DATA for computation ",
    "(distributions, costs, schedules, probabilities), call the appropriate tool.\n",
    "- If the user asks a CONCEPTUAL or EXPLANATORY question ",
    "(\"what is EVM?\", \"how does Monte Carlo work?\", \"explain CPI\"), ",
    "answer using the RAG context provided. Do NOT call tools for conceptual questions.\n",
    "- If RAG context is present, use it to inform your answer and cite the source(s) ",
    "at the end (e.g., 'Sources: mcs_methods.md, evm_standards.md').\n",
    "- Never fabricate numerical results. If you need to compute something, call a tool.\n",
    "- After tool results, interpret them briefly. Be concise.\n\n",
    "## IMPORTANT: Do not use /commands\n",
    "Slash commands like /mcs, /evm, /risk are for USERS to type directly. ",
    "You must NEVER output /commands in your responses. ",
    "When you need to compute, call the registered tool functions instead.\n\n",
    "## Tool selection (only when user provides numerical data)\n",
    "- Distributions (normal/triangular/uniform) -> mcs_tool -> contingency_tool + sensitivity_tool\n",
    "- Means + variances only -> smm_tool\n",
    "- BAC + schedule + actuals -> evm_analysis_tool\n",
    "- Root cause probabilities -> risk_prob_tool or risk_post_prob_tool\n",
    "- Cost risk simulation -> cost_pdf_tool or cost_post_pdf_tool\n",
    "- Historical x/y data -> fit_and_predict_sigmoidal_tool\n",
    "- Dependency matrix -> parent_dsm_tool or grandparent_dsm_tool\n"
  )
}

#' Posterior Risk Probability.
#'
#' This function calculates the posterior probability of a risk event 'R' occurring
#' based on observations of multiple root causes and their associated conditional probabilities.
#'
#' @srrstats {G1.0} *Software lists primary reference from published academic literature.*
#' @srrstats {G1.1} *Software is the first implementation within **R** of the algorithm which has previously been implemented in other languages or contexts.*
#' @srrstats {G1.4} *Software uses [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {G2.0} *Implements assertions on lengths of inputs - all vectors must have same length.*
#' @srrstats {G2.0a} *Parameter documentation explicitly states expected input structure.*
#' @srrstats {G2.1} *Implements assertions on types of inputs - probability values validated between 0 and 1.*
#' @srrstats {G2.1a} *Parameter documentation explicitly states data types expected.*
#' @srrstats {G2.13} *Implements checks for NA values in observed_causes - NA is permitted to indicate unobserved causes.*
#' @srrstats {G2.14b} *Ignores NA values in observed_causes with appropriate handling for unobserved causes.*
#' @srrstats {G2.15} *Implements checks for NaN values via is.nan() prior to processing.*
#' @srrstats {G2.16} *Implements checks for Inf/-Inf values via is.infinite() prior to processing.*
#' @srrstats {G5.2a} *Each error message produced by stop() is unique.*
#'
#' @param cause_probs A vector of prior probabilities for each root cause 'C_i'.
#' @param risks_given_causes A vector of conditional probabilities of the risk event 'R' given each cause 'C_i'.
#' @param risks_given_not_causes A vector of conditional probabilities of the risk event 'R' given not each cause 'C_i'.
#' @param observed_causes A vector of observed values for each cause 'C_i' (1 if observed, 0 if not observed, NA if unobserved).
#' @return A numeric value for the posterior probability of the risk event given the observed causes.
#' @references
#' Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
#' construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
#' @examples
#' cause_probs <- c(0.3, 0.2)
#' risks_given_causes <- c(0.8, 0.6)
#' risks_given_not_causes <- c(0.2, 0.4)
#' observed_causes <- c(1, NA)
#' risk_post_prob <- risk_post_prob(
#'   cause_probs, risks_given_causes,
#'   risks_given_not_causes, observed_causes
#' )
#' print(risk_post_prob)
#' @export
risk_post_prob <- function(cause_probs, risks_given_causes, risks_given_not_causes, observed_causes) {
  # Validate inputs
  if (any(is.nan(cause_probs)) || any(is.nan(risks_given_causes)) || any(is.nan(risks_given_not_causes))) {
    stop("cause_probs, risks_given_causes, and risks_given_not_causes must not contain NaN values.")
  }
  if (anyNA(cause_probs) || anyNA(risks_given_causes) || anyNA(risks_given_not_causes)) {
    stop("cause_probs, risks_given_causes, and risks_given_not_causes must not contain NA values.")
  }
  if (any(is.infinite(cause_probs)) || any(is.infinite(risks_given_causes)) || any(is.infinite(risks_given_not_causes))) {
    stop("cause_probs, risks_given_causes, and risks_given_not_causes must not contain infinite values.")
  }
  if (any(is.infinite(observed_causes[!is.na(observed_causes)]))) {
    stop("observed_causes must not contain infinite values.")
  }
  if (length(cause_probs) != length(risks_given_causes) ||
    length(cause_probs) != length(risks_given_not_causes) ||
    length(cause_probs) != length(observed_causes)) {
    stop("All input vectors must have the same length.")
  }
  if (any(cause_probs < 0 | cause_probs > 1)) {
    stop("All values in cause_probs must be between 0 and 1.")
  }
  if (any(risks_given_causes < 0 | risks_given_causes > 1)) {
    stop("All values in risks_given_causes must be between 0 and 1.")
  }
  if (any(risks_given_not_causes < 0 | risks_given_not_causes > 1)) {
    stop("All values in risks_given_not_causes must be between 0 and 1.")
  }
  if (!all(is.na(observed_causes) | observed_causes %in% c(0, 1))) {
    stop("All values in observed_causes must be 0, 1, or NA.")
  }

  # Initialize posterior probability of the risk event
  numerator <- 1
  denominator <- 1

  for (i in seq_along(cause_probs)) {
    if (!is.na(observed_causes[i])) {
      if (observed_causes[i] == 1) {
        numerator <- numerator * risks_given_causes[i] * cause_probs[i]
        denominator <- denominator * (risks_given_causes[i] * cause_probs[i] +
          risks_given_not_causes[i] * (1 - cause_probs[i]))
      } else {
        numerator <- numerator * risks_given_not_causes[i] * (1 - cause_probs[i])
        denominator <- denominator * (risks_given_causes[i] * cause_probs[i] +
          risks_given_not_causes[i] * (1 - cause_probs[i]))
      }
    }
  }

  # Return the normalized posterior probability
  return(numerator / denominator)
}

#' Posterior Cost Probability Density.
#'
#' This function generates random samples from the posterior distribution of the cost 'A'
#' given observations of multiple risk events 'R_i'. Each risk event has its own mean
#' and standard deviation for the cost distribution. The function also accounts for a baseline cost
#' when no risk event occurs.
#'
#' @srrstats {G1.0} *Software lists primary reference from published academic literature.*
#' @srrstats {G1.1} *Software is the first implementation within **R** of the algorithm which has previously been implemented in other languages or contexts.*
#' @srrstats {G1.4} *Software uses [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {G2.0} *Implements assertions on lengths of inputs - all parameter vectors must have same length.*
#' @srrstats {G2.0a} *Parameter documentation explicitly states expected input structure.*
#' @srrstats {G2.1} *Implements assertions on types of inputs via is.numeric() checks.*
#' @srrstats {G2.1a} *Parameter documentation explicitly states data types expected.*
#' @srrstats {G2.2} *Prohibits multivariate input for num_sims which must be a positive integer.*
#' @srrstats {G2.13} *Implements checks for NA values in observed_risks - NA permitted to indicate unobserved risks.*
#' @srrstats {G2.14b} *Ignores NA values in observed_risks with appropriate handling for unobserved risks.*
#' @srrstats {G2.15} *Implements checks for NaN values via is.nan() prior to processing.*
#' @srrstats {G2.16} *Implements checks for Inf/-Inf values via is.infinite() prior to processing.*
#' @srrstats {G5.2a} *Each error message produced by stop() is unique.*
#'
#' @param num_sims Number of random samples to draw from the posterior distribution.
#' @param observed_risks A vector of observed values for each risk event 'R_i' (1 if observed, 0 if not observed, NA if unobserved).
#' @param means_given_risks A vector of means of the normal distribution for cost 'A' given each risk event 'R_i'.
#' @param sds_given_risks A vector of standard deviations of the normal distribution for cost 'A' given each risk event 'R_i'.
#' @param base_cost The baseline cost given no risk event occurs.
#' @return A numeric vector of random samples from the posterior distribution of costs.
#' @references
#' Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
#' construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
#' @examples
#' # Example with three risk events
#' num_sims <- 1000
#' observed_risks <- c(1, NA, 1)
#' means_given_risks <- c(10000, 15000, 5000)
#' sds_given_risks <- c(2000, 1000, 1000)
#' base_cost <- 2000
#' posterior_samples <- cost_post_pdf(
#'   num_sims = num_sims,
#'   observed_risks = observed_risks,
#'   means_given_risks = means_given_risks,
#'   sds_given_risks = sds_given_risks,
#'   base_cost = base_cost
#' )
#' hist(posterior_samples, breaks = 30, col = "skyblue", main = "Posterior Cost PDF", xlab = "Cost")
#' @importFrom stats rnorm
#' @export
cost_post_pdf <- function(num_sims, observed_risks, means_given_risks, sds_given_risks, base_cost = 0) {
  # Validate inputs
  if (num_sims <= 0 || !is.numeric(num_sims)) stop("num_sims must be a positive integer.")
  if (any(is.nan(means_given_risks)) || any(is.nan(sds_given_risks))) {
    stop("means_given_risks and sds_given_risks must not contain NaN values.")
  }
  if (anyNA(means_given_risks) || anyNA(sds_given_risks)) {
    stop("means_given_risks and sds_given_risks must not contain NA values.")
  }
  if (any(is.infinite(means_given_risks)) || any(is.infinite(sds_given_risks))) {
    stop("means_given_risks and sds_given_risks must not contain infinite values.")
  }
  if (any(is.infinite(observed_risks[!is.na(observed_risks)]))) {
    stop("observed_risks must not contain infinite values.")
  }
  if (is.nan(base_cost)) stop("base_cost must not be NaN.")
  if (is.na(base_cost)) stop("base_cost must not be NA.")
  if (is.infinite(base_cost)) stop("base_cost must not be infinite.")
  if (!all(is.na(observed_risks) | observed_risks %in% c(0, 1))) stop("All values in observed_risks must be 0, 1, or NA.")
  if (length(observed_risks) != length(means_given_risks) || length(observed_risks) != length(sds_given_risks)) {
    stop("observed_risks, means_given_risks, and sds_given_risks must have the same length.")
  }
  if (any(sds_given_risks < 0)) stop("Standard deviations must be non-negative.")

  # Number of risk events
  num_risks <- length(observed_risks)

  # Initialize cost samples with base cost
  samples <- rep(base_cost, num_sims)

  # Iterate over each risk event
  for (i in seq_len(num_risks)) {
    if (!is.na(observed_risks[i]) && observed_risks[i] == 1) {
      # Add cost samples for the observed risk event
      samples <- samples + rnorm(num_sims, mean = means_given_risks[i], sd = sds_given_risks[i])
    }
  }

  return(samples)
}

#' @srrstats {G5.2} *Error and warning behaviour is explicitly demonstrated through tests.*
#' @srrstats {G5.2a} *Every error message is unique and tested.*
#' @srrstats {G5.2b} *Tests trigger every error message and compare with expected values.*
#' @srrstats {G5.3} *Return objects tested for absence of NA, NaN, Inf.*
#' @srrstats {G5.6} *Parameter recovery tests verify implementations produce expected results given data with known properties.*
#' @srrstats {G5.6a} *Parameter recovery tests succeed within defined tolerance rather than exact values.*
#' @srrstats {G5.6b} *Parameter recovery tests run with multiple random seeds when randomness is involved.*
#' @srrstats {G5.7} *Algorithm performance tests verify implementations perform correctly as data properties change.*
#' @srrstats {G5.8} *Edge condition tests verify appropriate behavior with extreme data properties.*
#' @srrstats {G5.8a} *Zero-length data tests trigger clear errors.*
#' @srrstats {G5.8b} *Unsupported data type tests trigger clear errors.*
#' @srrstats {G5.8c} *All-NA and all-identical data tests trigger clear errors or warnings.*
#' @srrstats {G5.8d} *Out-of-scope data tests verify appropriate behavior.*
#' @srrstats {G5.9} *Noise susceptibility tests verify stochastic behavior stability.*
#' @srrstats {G5.9a} *Trivial noise tests show results are stable at machine epsilon scale.*
#' @srrstats {G5.9b} *Random seed stability tests show consistent behavior across different seeds.*

test_that("risk_post_prob handles fully observed causes", {
  cause_probs <- c(0.3, 0.2)
  risks_given_causes <- c(0.8, 0.6)
  risks_given_not_causes <- c(0.2, 0.4)
  observed_causes <- c(1, 0)

  result <- risk_post_prob(cause_probs, risks_given_causes, risks_given_not_causes, observed_causes)
  expect_true(is.numeric(result))
  expect_true(result >= 0 && result <= 1)
})

test_that("risk_post_prob handles partially observed causes", {
  cause_probs <- c(0.3, 0.2, 0.5)
  risks_given_causes <- c(0.8, 0.6, 0.7)
  risks_given_not_causes <- c(0.2, 0.4, 0.3)
  observed_causes <- c(1, NA, 0)

  result <- risk_post_prob(cause_probs, risks_given_causes, risks_given_not_causes, observed_causes)
  expect_true(is.numeric(result))
  expect_true(result >= 0 && result <= 1)
})

test_that("cost_post_pdf generates valid samples", {
  num_sims <- 1000
  observed_risks <- c(1, NA, 1)
  means_given_risks <- c(10000, 15000, 5000)
  sds_given_risks <- c(2000, 1000, 1000)
  base_cost <- 2000

  samples <- cost_post_pdf(num_sims, observed_risks, means_given_risks, sds_given_risks, base_cost)
  expect_equal(length(samples), num_sims)
  expect_true(all(is.numeric(samples)))
})

test_that("cost_post_pdf handles no observed risks", {
  num_sims <- 1000
  observed_risks <- c(NA, NA, NA)
  means_given_risks <- c(10000, 15000, 5000)
  sds_given_risks <- c(2000, 1000, 1000)
  base_cost <- 2000

  samples <- cost_post_pdf(num_sims, observed_risks, means_given_risks, sds_given_risks, base_cost)
  expect_equal(length(samples), num_sims)
  expect_true(all(samples == base_cost))
})

# Error handling tests for risk_post_prob
test_that("risk_post_prob validates input vector lengths", {
  expect_error(
    risk_post_prob(c(0.3), c(0.8, 0.6), c(0.2, 0.4), c(1, 0)),
    "All input vectors must have the same length."
  )
})

test_that("risk_post_prob validates probability ranges", {
  expect_error(
    risk_post_prob(c(0.3, 1.5), c(0.8, 0.6), c(0.2, 0.4), c(1, 0)),
    "All values in cause_probs must be between 0 and 1."
  )
  expect_error(
    risk_post_prob(c(0.3, 0.2), c(0.8, 1.6), c(0.2, 0.4), c(1, 0)),
    "All values in risks_given_causes must be between 0 and 1."
  )
  expect_error(
    risk_post_prob(c(0.3, 0.2), c(0.8, 0.6), c(0.2, -0.4), c(1, 0)),
    "All values in risks_given_not_causes must be between 0 and 1."
  )
})

test_that("risk_post_prob validates observed_causes values", {
  expect_error(
    risk_post_prob(c(0.3, 0.2), c(0.8, 0.6), c(0.2, 0.4), c(1, 2)),
    "All values in observed_causes must be 0, 1, or NA."
  )
})

# Error handling tests for cost_post_pdf
test_that("cost_post_pdf validates num_sims", {
  expect_error(
    cost_post_pdf(-100, c(1, 0), c(10000, 15000), c(2000, 1000), 2000),
    "num_sims must be a positive integer."
  )
  expect_error(
    cost_post_pdf("not a number", c(1, 0), c(10000, 15000), c(2000, 1000), 2000),
    "num_sims must be a positive integer."
  )
})

test_that("cost_post_pdf validates observed_risks values", {
  expect_error(
    cost_post_pdf(1000, c(1, 2), c(10000, 15000), c(2000, 1000), 2000),
    "All values in observed_risks must be 0, 1, or NA."
  )
})

test_that("cost_post_pdf validates vector lengths", {
  expect_error(
    cost_post_pdf(1000, c(1, 0, 1), c(10000, 15000), c(2000, 1000), 2000),
    "observed_risks, means_given_risks, and sds_given_risks must have the same length."
  )
})

test_that("cost_post_pdf validates non-negative standard deviations", {
  expect_error(
    cost_post_pdf(1000, c(1, 0), c(10000, 15000), c(2000, -1000), 2000),
    "Standard deviations must be non-negative."
  )
})

test_that("risk_post_prob handles all observed causes correctly", {
  cause_probs <- c(0.3, 0.2)
  risks_given_causes <- c(0.8, 0.6)
  risks_given_not_causes <- c(0.2, 0.4)
  observed_causes <- c(1, 1)

  result <- risk_post_prob(cause_probs, risks_given_causes, risks_given_not_causes, observed_causes)
  expect_true(is.numeric(result))
  expect_true(result >= 0 && result <= 1)
})

test_that("cost_post_pdf handles zero observed risks correctly", {
  num_sims <- 1000
  observed_risks <- c(0, 0, 0)
  means_given_risks <- c(10000, 15000, 5000)
  sds_given_risks <- c(2000, 1000, 1000)
  base_cost <- 2000

  samples <- cost_post_pdf(num_sims, observed_risks, means_given_risks, sds_given_risks, base_cost)
  expect_equal(length(samples), num_sims)
  expect_true(all(samples == base_cost))
})

# ============================================================================
# NaN/NA/Inf Error Tests (G5.2, G5.2b)
# ============================================================================
test_that("risk_post_prob rejects NaN in cause params", {
  expect_error(
    risk_post_prob(c(NaN, 0.2), c(0.8, 0.6), c(0.2, 0.4), c(1, 0)),
    "cause_probs, risks_given_causes, and risks_given_not_causes must not contain NaN values."
  )
  expect_error(
    risk_post_prob(c(0.3, 0.2), c(NaN, 0.6), c(0.2, 0.4), c(1, 0)),
    "cause_probs, risks_given_causes, and risks_given_not_causes must not contain NaN values."
  )
  expect_error(
    risk_post_prob(c(0.3, 0.2), c(0.8, 0.6), c(NaN, 0.4), c(1, 0)),
    "cause_probs, risks_given_causes, and risks_given_not_causes must not contain NaN values."
  )
})

test_that("risk_post_prob rejects NA in cause params", {
  expect_error(
    risk_post_prob(c(NA_real_, 0.2), c(0.8, 0.6), c(0.2, 0.4), c(1, 0)),
    "cause_probs, risks_given_causes, and risks_given_not_causes must not contain NA values."
  )
})

test_that("risk_post_prob rejects Inf in cause params", {
  expect_error(
    risk_post_prob(c(Inf, 0.2), c(0.8, 0.6), c(0.2, 0.4), c(1, 0)),
    "cause_probs, risks_given_causes, and risks_given_not_causes must not contain infinite values."
  )
})

test_that("risk_post_prob rejects Inf in observed_causes", {
  expect_error(
    risk_post_prob(c(0.3, 0.2), c(0.8, 0.6), c(0.2, 0.4), c(Inf, 0)),
    "observed_causes must not contain infinite values."
  )
})

test_that("cost_post_pdf rejects NaN in means/sds", {
  expect_error(
    cost_post_pdf(1000, c(1, 0), c(NaN, 15000), c(2000, 1000), 2000),
    "means_given_risks and sds_given_risks must not contain NaN values."
  )
  expect_error(
    cost_post_pdf(1000, c(1, 0), c(10000, 15000), c(NaN, 1000), 2000),
    "means_given_risks and sds_given_risks must not contain NaN values."
  )
})

test_that("cost_post_pdf rejects NA in means/sds", {
  expect_error(
    cost_post_pdf(1000, c(1, 0), c(NA_real_, 15000), c(2000, 1000), 2000),
    "means_given_risks and sds_given_risks must not contain NA values."
  )
})

test_that("cost_post_pdf rejects Inf in means/sds", {
  expect_error(
    cost_post_pdf(1000, c(1, 0), c(Inf, 15000), c(2000, 1000), 2000),
    "means_given_risks and sds_given_risks must not contain infinite values."
  )
})

test_that("cost_post_pdf rejects Inf in observed_risks", {
  expect_error(
    cost_post_pdf(1000, c(Inf, 0), c(10000, 15000), c(2000, 1000), 2000),
    "observed_risks must not contain infinite values."
  )
})

test_that("cost_post_pdf rejects NaN base_cost", {
  expect_error(
    cost_post_pdf(1000, c(1, 0), c(10000, 15000), c(2000, 1000), NaN),
    "base_cost must not be NaN."
  )
})

test_that("cost_post_pdf rejects NA base_cost", {
  expect_error(
    cost_post_pdf(1000, c(1, 0), c(10000, 15000), c(2000, 1000), NA_real_),
    "base_cost must not be NA."
  )
})

test_that("cost_post_pdf rejects Inf base_cost", {
  expect_error(
    cost_post_pdf(1000, c(1, 0), c(10000, 15000), c(2000, 1000), Inf),
    "base_cost must not be infinite."
  )
})

# ============================================================================
# G5.3: Return value tests
# ============================================================================
test_that("risk_post_prob result contains no NA, NaN, or Inf", {
  result <- risk_post_prob(c(0.3, 0.2), c(0.8, 0.6), c(0.2, 0.4), c(1, 0))
  expect_false(is.na(result))
  expect_false(is.nan(result))
  expect_false(is.infinite(result))
})

test_that("cost_post_pdf result contains no NA, NaN, or Inf", {
  set.seed(42)
  samples <- cost_post_pdf(100, c(1, 0), c(10000, 15000), c(2000, 1000), 2000)
  expect_false(anyNA(samples))
  expect_false(any(is.nan(samples)))
  expect_false(any(is.infinite(samples)))
})

# ============================================================================
# Parameter Recovery Tests (G5.6, G5.6a)
# ============================================================================

test_that("risk_post_prob handles all NA observations correctly", {
  cause_probs <- c(0.3, 0.2)
  risks_given_causes <- c(0.8, 0.6)
  risks_given_not_causes <- c(0.2, 0.4)
  observed_causes <- c(NA_real_, NA_real_) # No observations

  result <- risk_post_prob(
    cause_probs, risks_given_causes,
    risks_given_not_causes, observed_causes
  )

  # Result should be a valid probability
  expect_true(is.numeric(result))
  expect_true(result >= 0 && result <= 1)
})

#' Fit a Sigmoidal Model to Data.
#'
#' This function fits a sigmoidal model (Pearl, Gompertz, or Logistic) to the provided data.
#'
#' @srrstats {G1.0} *Software lists primary reference from published academic literature.*
#' @srrstats {G1.1} *Software is the first implementation within **R** of these sigmoidal models for project risk management.*
#' @srrstats {G1.4} *Software uses [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {G2.0} *Implements assertions on input structure - validates column existence in data frame.*
#' @srrstats {G2.0a} *Parameter documentation explicitly states expected input structure (data frame with named columns).*
#' @srrstats {G2.1} *Implements assertions on column existence in data frame.*
#' @srrstats {G2.1a} *Parameter documentation explicitly states data types expected.*
#' @srrstats {G2.3a} *Uses explicit validation of model_type against valid options.*
#' @srrstats {G2.3b} *Uses tolower() to ensure model_type parameter is not case sensitive.*
#' @srrstats {G2.7} *Accepts data.frame as standard tabular input.*
#' @srrstats {G2.15} *Implements checks for NaN values via is.nan() prior to processing.*
#' @srrstats {G2.16} *Implements checks for Inf/-Inf values via is.infinite() prior to processing.*
#' @srrstats {G5.2a} *Each error message produced by stop() is unique.*
#'
#' @param data A data frame containing the time (x_col) and completion (y_col) vectors.
#' @param x_col The name of the time vector.
#' @param y_col The name of the completion vector.
#' @param model_type The name of the sigmoidal model (Pearl, Gompertz, or Logistic).
#' @return The function returns a list of results for the sigmoidal model.
#' @references
#' Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
#' construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
#' @examples
#' # Set up a data frame of time and completion percentage data
#' data <- data.frame(time = 1:10, completion = c(5, 15, 40, 60, 70, 75, 80, 85, 90, 95))
#'
#' # Fit a logistic model to the data.
#' fit <- fit_sigmoidal(data, "time", "completion", "logistic")
#'
#' # Use the model to predict future completion times.
#' predictions <- predict_sigmoidal(fit, seq(min(data$time), max(data$time),
#'   length.out = 100
#' ), "logistic")
#'
#' # Predict with 95% confidence bounds
#' predictions_ci <- predict_sigmoidal(fit, seq(min(data$time), max(data$time),
#'   length.out = 100
#' ), "logistic", conf_level = 0.95)
#'
#' @importFrom minpack.lm nlsLM
#' @importFrom stats median
#' @export

# Fit a Sigmoidal Model
fit_sigmoidal <- function(data, x_col, y_col, model_type) {
  # Convert model_type to lowercase for case-insensitive matching
  model_type <- tolower(model_type)

  # Error handling
  if (!x_col %in% names(data)) {
    stop(paste("Column", x_col, "not found in data frame."))
  }
  if (!y_col %in% names(data)) {
    stop(paste("Column", y_col, "not found in data frame."))
  }

  x <- data[[x_col]]
  y <- data[[y_col]]

  if (any(is.nan(x)) || any(is.nan(y))) {
    stop("Data columns must not contain NaN values.")
  }
  if (anyNA(x) || anyNA(y)) {
    stop("Data columns must not contain NA values.")
  }
  if (any(is.infinite(x)) || any(is.infinite(y))) {
    stop("Data columns must not contain infinite values.")
  }

  # Pearl Sigmoidal Model
  pearl <- function(x, K, r, t0) {
    K / (1 + exp(-r * (x - t0)))
  }

  # Gompertz Sigmoidal Model
  gompertz <- function(x, A, b, c) {
    A * exp(-b * exp(-c * x))
  }

  # Logistic Sigmoidal Model
  logistic <- function(x, K, r, t0) {
    K / (1 + exp(-r * (x - t0)))
  }

  if (model_type == "pearl") {
    fit <- minpack.lm::nlsLM(y ~ pearl(x, K, r, t0), start = list(K = max(y), r = 0.1, t0 = stats::median(x)))
  } else if (model_type == "gompertz") {
    fit <- minpack.lm::nlsLM(y ~ gompertz(x, A, b, c), start = list(A = max(y), b = 1, c = 0.1))
  } else if (model_type == "logistic") {
    fit <- minpack.lm::nlsLM(y ~ logistic(x, K, r, t0), start = list(K = max(y), r = 0.1, t0 = stats::median(x)))
  } else {
    stop("Invalid model type. Choose 'pearl', 'gompertz', or 'logistic'.")
  }

  class(fit) <- c("pra_sigmoidal_fit", class(fit))
  return(fit)
}

#' Predict a Sigmoidal Function Using Fitted Model.
#'
#' This function predicts values using a fitted sigmoidal model (Pearl, Gompertz,
#' or Logistic) over a specified range of time values.
#'
#' @srrstats {G1.0} *Software lists primary reference from published academic literature.*
#' @srrstats {G1.1} *Software is the first implementation within **R** of these sigmoidal prediction methods for project risk management.*
#' @srrstats {G1.4} *Software uses [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {G2.0} *Implements assertions on input structure - validates fitted model is not NULL.*
#' @srrstats {G2.0a} *Parameter documentation explicitly states expected input structure.*
#' @srrstats {G2.1} *Implements assertions on types of inputs via is.numeric() checks.*
#' @srrstats {G2.1a} *Parameter documentation explicitly states data types expected.*
#' @srrstats {G2.3a} *Uses explicit validation of model_type against valid options.*
#' @srrstats {G2.3b} *Uses tolower() to ensure model_type parameter is not case sensitive.*
#' @srrstats {G2.15} *Implements checks for NaN values via is.nan() prior to processing.*
#' @srrstats {G2.16} *Implements checks for Inf/-Inf values via is.infinite() prior to processing.*
#' @srrstats {G5.2a} *Each error message produced by stop() is unique.*
#'
#' @param fit A list containing the results of a sigmoidal model.
#' @param x_range A vector of time values for the prediction.
#' @param model_type The type of model (Pearl, Gompertz, or Logistic) for the prediction.
#' @param conf_level Optional confidence level for confidence bounds (e.g., 0.95 for 95%).
#'   If NULL (default), no confidence bounds are computed.
#' @return The function returns a data frame containing the time (x), predicted values (pred),
#'   and optionally lower (lwr) and upper (upr) confidence bounds.
#' @references
#' Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
#' construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
#' @examples
#' # Set up a data frame of time and completion percentage data
#' data <- data.frame(time = 1:10, completion = c(5, 15, 40, 60, 70, 75, 80, 85, 90, 95))
#'
#' # Fit a logistic model to the data.
#' fit <- fit_sigmoidal(data, "time", "completion", "logistic")
#'
#' # Use the model to predict future completion times.
#' predictions <- predict_sigmoidal(fit, seq(min(data$time), max(data$time),
#'   length.out = 100
#' ), "logistic")
#'
#' # Predict with 95% confidence bounds
#' predictions_ci <- predict_sigmoidal(fit, seq(min(data$time), max(data$time),
#'   length.out = 100
#' ), "logistic", conf_level = 0.95)
#'
#' @importFrom stats predict coef vcov df.residual qt
#' @export
# Predict a Sigmoidal Function
predict_sigmoidal <- function(fit, x_range, model_type, conf_level = NULL) {
  # Convert model_type to lowercase for case-insensitive matching
  model_type <- tolower(model_type)

  # Error handling
  if (is.null(fit)) {
    stop("Fitted model is NULL.")
  }
  if (!is.numeric(x_range)) {
    stop("x_range must be a numeric vector.")
  }
  if (any(is.nan(x_range))) {
    stop("x_range must not contain NaN values.")
  }
  if (anyNA(x_range)) {
    stop("x_range must not contain NA values.")
  }
  if (any(is.infinite(x_range))) {
    stop("x_range must not contain infinite values.")
  }

  new_data <- data.frame(x = x_range)

  # Define model functions for gradient computation
  pearl <- function(x, K, r, t0) {
    K / (1 + exp(-r * (x - t0)))
  }

  gompertz <- function(x, A, b, c) {
    A * exp(-b * exp(-c * x))
  }

  logistic <- function(x, K, r, t0) {
    K / (1 + exp(-r * (x - t0)))
  }

  if (!model_type %in% c("pearl", "gompertz", "logistic")) {
    stop("Invalid model type. Choose 'pearl', 'gompertz', or 'logistic'.")
  }

  new_data$pred <- stats::predict(fit, newdata = new_data)

  # Compute confidence bounds if requested
  if (!is.null(conf_level)) {
    if (conf_level <= 0 || conf_level >= 1) {
      stop("conf_level must be between 0 and 1 (e.g., 0.95 for 95% CI).")
    }

    # Get model parameters and variance-covariance matrix
    params <- stats::coef(fit)
    vcov_mat <- stats::vcov(fit)
    df_resid <- stats::df.residual(fit)

    # Compute gradient for each prediction point using numerical differentiation
    eps <- 1e-6
    n_params <- length(params)
    n_points <- length(x_range)
    grad_matrix <- matrix(0, nrow = n_points, ncol = n_params)

    for (j in seq_len(n_params)) {
      params_plus <- params
      params_plus[j] <- params_plus[j] + eps

      if (model_type == "pearl" || model_type == "logistic") {
        pred_plus <- pearl(x_range, params_plus["K"], params_plus["r"], params_plus["t0"])
        pred_base <- pearl(x_range, params["K"], params["r"], params["t0"])
      } else {
        pred_plus <- gompertz(x_range, params_plus["A"], params_plus["b"], params_plus["c"])
        pred_base <- gompertz(x_range, params["A"], params["b"], params["c"])
      }

      grad_matrix[, j] <- (pred_plus - pred_base) / eps
    }

    # Compute standard errors using the delta method
    se <- sqrt(rowSums((grad_matrix %*% vcov_mat) * grad_matrix))

    # Compute confidence bounds
    t_crit <- stats::qt((1 + conf_level) / 2, df_resid)
    new_data$lwr <- new_data$pred - t_crit * se
    new_data$upr <- new_data$pred + t_crit * se
  }

  return(new_data)
}

#' Plot a Fitted Sigmoidal Model.
#'
#' This function creates a base R plot of a fitted sigmoidal model with the
#' original data points, fitted curve, and optional confidence bounds.
#'
#' @srrstats {G1.0} *Software lists primary reference from published academic literature.*
#' @srrstats {G1.1} *Software is the first implementation within **R** of this sigmoidal plotting method for project risk management.*
#' @srrstats {G1.4} *Software uses [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {G2.0} *Implements assertions on input structure - validates fitted model and column existence.*
#' @srrstats {G2.0a} *Parameter documentation explicitly states expected input structure.*
#' @srrstats {G2.1} *Implements assertions on column existence in data frame.*
#' @srrstats {G2.1a} *Parameter documentation explicitly states data types expected.*
#' @srrstats {G2.3b} *Uses tolower() to ensure model_type parameter is not case sensitive.*
#' @srrstats {G2.7} *Accepts data.frame as standard tabular input.*
#' @srrstats {G2.15} *Implements checks for NaN values via is.nan() prior to processing.*
#' @srrstats {G2.16} *Implements checks for Inf/-Inf values via is.infinite() prior to processing.*
#' @srrstats {G5.2a} *Each error message produced by stop() is unique.*
#'
#' @param fit A fitted sigmoidal model object from fit_sigmoidal.
#' @param data The original data frame used to fit the model.
#' @param x_col The name of the x (time) column in the data.
#' @param y_col The name of the y (completion) column in the data.
#' @param model_type The type of model (pearl, gompertz, or logistic).
#' @param conf_level Optional confidence level for confidence bounds (e.g., 0.95 for 95%).
#'   If NULL (default), no confidence bounds are plotted.
#' @param n_points Number of points to use for the fitted curve (default 100).
#' @param main Plot title. If NULL, a default title is generated.
#' @param xlab X-axis label. If NULL, uses x_col.
#' @param ylab Y-axis label. If NULL, uses y_col.
#' @param line_col Color for the fitted curve (default "red").
#' @param ci_col Color for the confidence band (default "lightblue").
#' @param pch Point character for data points (default 16).
#' @param ... Additional arguments passed to plot().
#' @return Invisibly returns the predictions data frame.
#' @references
#' Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
#' construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
#'
#' @examples
#' # Set up a data frame of time and completion percentage data
#' data <- data.frame(time = 1:10, completion = c(5, 15, 40, 60, 70, 75, 80, 85, 90, 95))
#'
#' # Fit a logistic model to the data.
#' fit <- fit_sigmoidal(data, "time", "completion", "logistic")
#'
#' # Plot the fitted model
#' plot_sigmoidal(fit, data, "time", "completion", "logistic")
#'
#' # Plot with 95% confidence bounds
#' plot_sigmoidal(fit, data, "time", "completion", "logistic", conf_level = 0.95)
#'
#' # Customize the plot
#' plot_sigmoidal(fit, data, "time", "completion", "logistic",
#'   conf_level = 0.95,
#'   main = "Project Completion Forecast",
#'   xlab = "Time (weeks)",
#'   ylab = "Completion (%)",
#'   line_col = "blue",
#'   ci_col = "lightgray"
#' )
#'
#' @importFrom graphics plot lines polygon points
#' @export
plot_sigmoidal <- function(fit, data, x_col, y_col, model_type,
                           conf_level = NULL, n_points = 100,
                           main = NULL, xlab = NULL, ylab = NULL,
                           line_col = "red", ci_col = "lightblue",
                           pch = 16, ...) {
  # Convert model_type to lowercase for case-insensitive matching
  model_type <- tolower(model_type)

  # Error handling
  if (is.null(fit)) {
    stop("Fitted model is NULL.")
  }
  if (!x_col %in% names(data)) {
    stop(paste("Column", x_col, "not found in data frame."))
  }
  if (!y_col %in% names(data)) {
    stop(paste("Column", y_col, "not found in data frame."))
  }

  # Extract data
  x <- data[[x_col]]
  y <- data[[y_col]]

  if (any(is.nan(x)) || any(is.nan(y))) {
    stop("Data columns must not contain NaN values for plotting.")
  }
  if (anyNA(x) || anyNA(y)) {
    stop("Data columns must not contain NA values for plotting.")
  }
  if (any(is.infinite(x)) || any(is.infinite(y))) {
    stop("Data columns must not contain infinite values for plotting.")
  }

  # Generate prediction range
  x_range <- seq(min(x), max(x), length.out = n_points)

  # Get predictions with optional confidence bounds
  predictions <- predict_sigmoidal(fit, x_range, model_type, conf_level)

  # Set default labels
  if (is.null(main)) {
    main <- paste("Fitted", tools::toTitleCase(model_type), "Model")
  }
  if (is.null(xlab)) {
    xlab <- x_col
  }
  if (is.null(ylab)) {
    ylab <- y_col
  }

  # Determine y-axis limits
  if (!is.null(conf_level) && "lwr" %in% names(predictions)) {
    ylim <- range(c(y, predictions$lwr, predictions$upr), na.rm = TRUE)
  } else {
    ylim <- range(c(y, predictions$pred), na.rm = TRUE)
  }

  # Create plot
  graphics::plot(x, y,
    pch = pch, main = main, xlab = xlab, ylab = ylab,
    ylim = ylim, ...
  )

  # Add confidence band if requested
  if (!is.null(conf_level) && "lwr" %in% names(predictions)) {
    graphics::polygon(
      x = c(predictions$x, rev(predictions$x)),
      y = c(predictions$lwr, rev(predictions$upr)),
      col = ci_col, border = NA
    )
    # Re-plot points on top of the confidence band
    graphics::points(x, y, pch = pch)
  }

  # Add fitted curve
  graphics::lines(predictions$x, predictions$pred, col = line_col, lwd = 2)

  invisible(predictions)
}

#' Print method for Sigmoidal Model
#'
#' Displays the summary of the fitted sigmoidal model in a readable format.
#' @param x An object of class \code{"pra_sigmoidal_fit"} as returned by
#'   \code{\link{fit_sigmoidal}}.
#' @param ... Additional arguments (not used).
#' @return No return value, called for side effects.
#' @examples
#' # Set up a data frame of time and completion percentage data
#' data <- data.frame(time = 1:10, completion = c(
#'   5, 15, 40, 60, 70, 75, 80, 85,
#'   90, 95
#' ))
#'
#' # Fit a logistic model to the data.
#' fit <- fit_sigmoidal(data, "time", "completion", "logistic")
#' # Print the model summary
#' print(fit)
#' @export
#' @method print pra_sigmoidal_fit
print.pra_sigmoidal_fit <- function(x, ...) {
  cat("Sigmoidal Model Fit Summary:\n")
  # Temporarily strip custom class so stats:::print.nls handles the nls part
  class(x) <- setdiff(class(x), "pra_sigmoidal_fit")
  print(summary(x))
}

#' @srrstats {G5.2} *Error and warning behaviour is explicitly demonstrated through tests.*
#' @srrstats {G5.2a} *Every error message is unique and tested.*
#' @srrstats {G5.2b} *Tests trigger every error message and compare with expected values.*
#' @srrstats {G5.3} *Return objects tested for absence of NA, NaN, Inf.*
#' @srrstats {G5.6} *Parameter recovery tests verify implementations produce expected results given data with known properties.*
#' @srrstats {G5.6a} *Parameter recovery tests succeed within defined tolerance rather than exact values.*
#' @srrstats {G5.6b} *Parameter recovery tests run with multiple random seeds when randomness is involved.*
#' @srrstats {G5.7} *Algorithm performance tests verify implementations perform correctly as data properties change.*
#' @srrstats {G5.8} *Edge condition tests verify appropriate behavior with extreme data properties.*
#' @srrstats {G5.8a} *Zero-length data tests trigger clear errors.*
#' @srrstats {G5.8b} *Unsupported data type tests trigger clear errors.*
#' @srrstats {G5.8c} *All-NA and all-identical data tests trigger clear errors or warnings.*
#' @srrstats {G5.8d} *Out-of-scope data tests verify appropriate behavior.*
#' @srrstats {G5.9} *Noise susceptibility tests verify stochastic behavior stability.*
#' @srrstats {G5.9a} *Trivial noise tests show results are stable at machine epsilon scale.*
#' @srrstats {G5.9b} *Random seed stability tests show consistent behavior across different seeds.*

# Sample data for testing
data <- data.frame(time = 1:10, completion = c(5, 15, 40, 60, 70, 75, 80, 85, 90, 95))

test_that("fit_sigmoidal returns a valid model for logistic type", {
  fit <- fit_sigmoidal(data, "time", "completion", "logistic")
  expect_s3_class(fit, "nls")
})

test_that("fit_sigmoidal returns a valid model for pearl type", {
  fit <- fit_sigmoidal(data, "time", "completion", "pearl")
  expect_s3_class(fit, "nls")
})

test_that("fit_sigmoidal returns a valid model for gompertz type", {
  fit <- fit_sigmoidal(data, "time", "completion", "gompertz")
  expect_s3_class(fit, "nls")
})

test_that("fit_sigmoidal stops with an invalid model type", {
  expect_error(fit_sigmoidal(data, "time", "completion", "invalid_type"), "Invalid model type. Choose 'pearl', 'gompertz', or 'logistic'.")
})

test_that("predict_sigmoidal returns a data frame with predictions for logistic type", {
  fit <- fit_sigmoidal(data, "time", "completion", "logistic")
  predictions <- predict_sigmoidal(fit, seq(min(data$time), max(data$time), length.out = 100), "logistic")
  expect_s3_class(predictions, "data.frame")
  expect_true(all(c("x", "pred") %in% names(predictions)))
})

test_that("predict_sigmoidal returns a data frame with predictions for pearl type", {
  fit <- fit_sigmoidal(data, "time", "completion", "pearl")
  predictions <- predict_sigmoidal(fit, seq(min(data$time), max(data$time), length.out = 100), "pearl")
  expect_s3_class(predictions, "data.frame")
  expect_true(all(c("x", "pred") %in% names(predictions)))
})

test_that("predict_sigmoidal returns a data frame with predictions for gompertz type", {
  fit <- fit_sigmoidal(data, "time", "completion", "gompertz")
  predictions <- predict_sigmoidal(fit, seq(min(data$time), max(data$time), length.out = 100), "gompertz")
  expect_s3_class(predictions, "data.frame")
  expect_true(all(c("x", "pred") %in% names(predictions)))
})

test_that("predict_sigmoidal stops with an invalid model type", {
  fit <- fit_sigmoidal(data, "time", "completion", "logistic")
  expect_error(predict_sigmoidal(fit, seq(min(data$time), max(data$time), length.out = 100), "invalid_type"), "Invalid model type. Choose 'pearl', 'gompertz', or 'logistic'.")
})

# ============================================================================
# fit_sigmoidal Error Handling Tests
# ============================================================================
test_that("fit_sigmoidal validates column existence", {
  expect_error(
    fit_sigmoidal(data, "nonexistent", "completion", "logistic"),
    "Column nonexistent not found in data frame."
  )
  expect_error(
    fit_sigmoidal(data, "time", "nonexistent", "logistic"),
    "Column nonexistent not found in data frame."
  )
})

test_that("fit_sigmoidal returns correct coefficients for logistic", {
  fit <- fit_sigmoidal(data, "time", "completion", "logistic")
  coefs <- coef(fit)
  expect_true("K" %in% names(coefs))
  expect_true("r" %in% names(coefs))
  expect_true("t0" %in% names(coefs))
})

test_that("fit_sigmoidal returns correct coefficients for gompertz", {
  fit <- fit_sigmoidal(data, "time", "completion", "gompertz")
  coefs <- coef(fit)
  expect_true("A" %in% names(coefs))
  expect_true("b" %in% names(coefs))
  expect_true("c" %in% names(coefs))
})

test_that("fit_sigmoidal returns correct coefficients for pearl", {
  fit <- fit_sigmoidal(data, "time", "completion", "pearl")
  coefs <- coef(fit)
  expect_true("K" %in% names(coefs))
  expect_true("r" %in% names(coefs))
  expect_true("t0" %in% names(coefs))
})

# ============================================================================
# predict_sigmoidal Error Handling Tests
# ============================================================================
test_that("predict_sigmoidal validates NULL fit", {
  expect_error(predict_sigmoidal(NULL, 1:10, "logistic"), "Fitted model is NULL.")
})

test_that("predict_sigmoidal validates numeric x_range", {
  fit <- fit_sigmoidal(data, "time", "completion", "logistic")
  expect_error(predict_sigmoidal(fit, "not numeric", "logistic"), "x_range must be a numeric vector.")
})

test_that("predict_sigmoidal returns predictions with correct dimensions", {
  fit <- fit_sigmoidal(data, "time", "completion", "logistic")
  x_range <- seq(1, 10, length.out = 50)
  predictions <- predict_sigmoidal(fit, x_range, "logistic")

  expect_equal(nrow(predictions), 50)
  expect_true("x" %in% names(predictions))
  expect_true("pred" %in% names(predictions))
})

# ============================================================================
# predict_sigmoidal Confidence Bounds Tests
# ============================================================================
test_that("predict_sigmoidal with confidence bounds returns lwr and upr columns", {
  fit <- fit_sigmoidal(data, "time", "completion", "logistic")
  x_range <- seq(1, 10, length.out = 50)
  predictions <- predict_sigmoidal(fit, x_range, "logistic", conf_level = 0.95)

  expect_true("lwr" %in% names(predictions))
  expect_true("upr" %in% names(predictions))
  expect_true(all(predictions$lwr <= predictions$pred))
  expect_true(all(predictions$upr >= predictions$pred))
})

test_that("predict_sigmoidal validates conf_level range", {
  fit <- fit_sigmoidal(data, "time", "completion", "logistic")
  x_range <- seq(1, 10, length.out = 50)

  expect_error(
    predict_sigmoidal(fit, x_range, "logistic", conf_level = 0),
    "conf_level must be between 0 and 1"
  )
  expect_error(
    predict_sigmoidal(fit, x_range, "logistic", conf_level = 1),
    "conf_level must be between 0 and 1"
  )
  expect_error(
    predict_sigmoidal(fit, x_range, "logistic", conf_level = 1.5),
    "conf_level must be between 0 and 1"
  )
  expect_error(
    predict_sigmoidal(fit, x_range, "logistic", conf_level = -0.5),
    "conf_level must be between 0 and 1"
  )
})

test_that("predict_sigmoidal confidence bounds work for pearl model", {
  fit <- fit_sigmoidal(data, "time", "completion", "pearl")
  x_range <- seq(1, 10, length.out = 50)
  predictions <- predict_sigmoidal(fit, x_range, "pearl", conf_level = 0.95)

  expect_true("lwr" %in% names(predictions))
  expect_true("upr" %in% names(predictions))
})

test_that("predict_sigmoidal confidence bounds work for gompertz model", {
  fit <- fit_sigmoidal(data, "time", "completion", "gompertz")
  x_range <- seq(1, 10, length.out = 50)
  predictions <- predict_sigmoidal(fit, x_range, "gompertz", conf_level = 0.95)

  expect_true("lwr" %in% names(predictions))
  expect_true("upr" %in% names(predictions))
})

test_that("predict_sigmoidal different confidence levels produce different bounds", {
  fit <- fit_sigmoidal(data, "time", "completion", "logistic")
  x_range <- seq(1, 10, length.out = 50)

  pred_90 <- predict_sigmoidal(fit, x_range, "logistic", conf_level = 0.90)
  pred_95 <- predict_sigmoidal(fit, x_range, "logistic", conf_level = 0.95)
  pred_99 <- predict_sigmoidal(fit, x_range, "logistic", conf_level = 0.99)

  # Higher confidence level should produce wider bounds
  expect_true(all(pred_95$upr - pred_95$lwr >= pred_90$upr - pred_90$lwr - 1e-10))
  expect_true(all(pred_99$upr - pred_99$lwr >= pred_95$upr - pred_95$lwr - 1e-10))
})

# ============================================================================
# plot_sigmoidal Tests
# ============================================================================
test_that("plot_sigmoidal validates NULL fit", {
  expect_error(
    plot_sigmoidal(NULL, data, "time", "completion", "logistic"),
    "Fitted model is NULL."
  )
})

test_that("plot_sigmoidal validates column existence", {
  fit <- fit_sigmoidal(data, "time", "completion", "logistic")
  expect_error(
    plot_sigmoidal(fit, data, "nonexistent", "completion", "logistic"),
    "Column nonexistent not found in data frame."
  )
  expect_error(
    plot_sigmoidal(fit, data, "time", "nonexistent", "logistic"),
    "Column nonexistent not found in data frame."
  )
})

test_that("plot_sigmoidal returns predictions invisibly", {
  fit <- fit_sigmoidal(data, "time", "completion", "logistic")
  result <- plot_sigmoidal(fit, data, "time", "completion", "logistic")

  expect_s3_class(result, "data.frame")
  expect_true("x" %in% names(result))
  expect_true("pred" %in% names(result))
})

test_that("plot_sigmoidal returns confidence bounds when requested", {
  fit <- fit_sigmoidal(data, "time", "completion", "logistic")
  result <- plot_sigmoidal(fit, data, "time", "completion", "logistic", conf_level = 0.95)

  expect_true("lwr" %in% names(result))
  expect_true("upr" %in% names(result))
})

test_that("plot_sigmoidal works with custom parameters", {
  fit <- fit_sigmoidal(data, "time", "completion", "logistic")

  # Should not error with custom parameters
  result <- plot_sigmoidal(fit, data, "time", "completion", "logistic",
    n_points = 50,
    main = "Test Plot",
    xlab = "X Label",
    ylab = "Y Label",
    line_col = "blue",
    ci_col = "gray"
  )

  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 50)
})

test_that("plot_sigmoidal works with all model types", {
  # Logistic
  fit_log <- fit_sigmoidal(data, "time", "completion", "logistic")
  result_log <- plot_sigmoidal(fit_log, data, "time", "completion", "logistic")
  expect_s3_class(result_log, "data.frame")

  # Pearl
  fit_pearl <- fit_sigmoidal(data, "time", "completion", "pearl")
  result_pearl <- plot_sigmoidal(fit_pearl, data, "time", "completion", "pearl")
  expect_s3_class(result_pearl, "data.frame")

  # Gompertz
  fit_gomp <- fit_sigmoidal(data, "time", "completion", "gompertz")
  result_gomp <- plot_sigmoidal(fit_gomp, data, "time", "completion", "gompertz")
  expect_s3_class(result_gomp, "data.frame")
})

# ============================================================================
# Edge Cases and Integration Tests
# ============================================================================
test_that("sigmoidal functions work with minimal data", {
  minimal_data <- data.frame(time = 1:5, completion = c(10, 30, 50, 70, 90))
  fit <- fit_sigmoidal(minimal_data, "time", "completion", "logistic")
  expect_s3_class(fit, "nls")

  predictions <- predict_sigmoidal(fit, 1:5, "logistic")
  expect_equal(nrow(predictions), 5)
})

test_that("sigmoidal predictions are monotonically reasonable", {
  fit <- fit_sigmoidal(data, "time", "completion", "logistic")
  x_range <- seq(1, 10, length.out = 100)
  predictions <- predict_sigmoidal(fit, x_range, "logistic")

  # For logistic/sigmoidal curves, predictions should generally increase
  # (allowing for some numerical tolerance)
  diffs <- diff(predictions$pred)
  expect_true(sum(diffs >= -0.1) > length(diffs) * 0.9)
})

test_that("fitted values are close to original data", {
  fit <- fit_sigmoidal(data, "time", "completion", "logistic")
  predictions <- predict_sigmoidal(fit, data$time, "logistic")

  # Predictions should be reasonably close to actual values
  residuals <- abs(predictions$pred - data$completion)
  expect_true(mean(residuals) < 10) # Average error < 10
})

# ============================================================================
# NaN/NA/Inf Error Tests (G5.2, G5.2b)
# ============================================================================
test_that("fit_sigmoidal rejects NaN in data columns", {
  bad_data <- data.frame(time = c(1, NaN, 3), completion = c(5, 15, 40))
  expect_error(
    fit_sigmoidal(bad_data, "time", "completion", "logistic"),
    "Data columns must not contain NaN values."
  )
})

test_that("fit_sigmoidal rejects NA in data columns", {
  bad_data <- data.frame(time = c(1, NA, 3), completion = c(5, 15, 40))
  expect_error(
    fit_sigmoidal(bad_data, "time", "completion", "logistic"),
    "Data columns must not contain NA values."
  )
})

test_that("fit_sigmoidal rejects Inf in data columns", {
  bad_data <- data.frame(time = c(1, Inf, 3), completion = c(5, 15, 40))
  expect_error(
    fit_sigmoidal(bad_data, "time", "completion", "logistic"),
    "Data columns must not contain infinite values."
  )
})

test_that("predict_sigmoidal rejects NaN in x_range", {
  fit <- fit_sigmoidal(data, "time", "completion", "logistic")
  expect_error(
    predict_sigmoidal(fit, c(1, NaN, 3), "logistic"),
    "x_range must not contain NaN values."
  )
})

test_that("predict_sigmoidal rejects NA in x_range", {
  fit <- fit_sigmoidal(data, "time", "completion", "logistic")
  expect_error(
    predict_sigmoidal(fit, c(1, NA, 3), "logistic"),
    "x_range must not contain NA values."
  )
})

test_that("predict_sigmoidal rejects Inf in x_range", {
  fit <- fit_sigmoidal(data, "time", "completion", "logistic")
  expect_error(
    predict_sigmoidal(fit, c(1, Inf, 3), "logistic"),
    "x_range must not contain infinite values."
  )
})

test_that("plot_sigmoidal rejects NaN in data columns for plotting", {
  fit <- fit_sigmoidal(data, "time", "completion", "logistic")
  bad_data <- data.frame(time = c(1, NaN, 3), completion = c(5, 15, 40))
  expect_error(
    plot_sigmoidal(fit, bad_data, "time", "completion", "logistic"),
    "Data columns must not contain NaN values for plotting."
  )
})

test_that("plot_sigmoidal rejects NA in data columns for plotting", {
  fit <- fit_sigmoidal(data, "time", "completion", "logistic")
  bad_data <- data.frame(time = c(1, NA, 3), completion = c(5, 15, 40))
  expect_error(
    plot_sigmoidal(fit, bad_data, "time", "completion", "logistic"),
    "Data columns must not contain NA values for plotting."
  )
})

test_that("plot_sigmoidal rejects Inf in data columns for plotting", {
  fit <- fit_sigmoidal(data, "time", "completion", "logistic")
  bad_data <- data.frame(time = c(1, Inf, 3), completion = c(5, 15, 40))
  expect_error(
    plot_sigmoidal(fit, bad_data, "time", "completion", "logistic"),
    "Data columns must not contain infinite values for plotting."
  )
})

# ============================================================================
# G5.3: Return value tests
# ============================================================================
test_that("predict_sigmoidal result contains no NA, NaN, or Inf", {
  fit <- fit_sigmoidal(data, "time", "completion", "logistic")
  predictions <- predict_sigmoidal(fit, seq(1, 10, length.out = 50), "logistic")
  expect_false(anyNA(predictions$x))
  expect_false(any(is.nan(predictions$x)))
  expect_false(any(is.infinite(predictions$x)))
  expect_false(anyNA(predictions$pred))
  expect_false(any(is.nan(predictions$pred)))
  expect_false(any(is.infinite(predictions$pred)))
})

# ============================================================================
# Parameter Recovery Tests (G5.6, G5.6a, G5.6b)
# ============================================================================

test_that("fit_sigmoidal recovers logistic parameters from synthetic data (seed 123)", {
  set.seed(123)

  # Generate synthetic data from known logistic curve
  K_true <- 100
  r_true <- 0.5
  t0_true <- 10

  x <- seq(1, 20, length.out = 50)
  y_true <- K_true / (1 + exp(-r_true * (x - t0_true)))
  # Add small noise
  y <- y_true + rnorm(50, mean = 0, sd = 1)

  synthetic_data <- data.frame(time = x, completion = y)
  fit <- fit_sigmoidal(synthetic_data, "time", "completion", "logistic")
  coefs <- coef(fit)

  # Recovered parameters should be close to true values
  expect_equal(as.numeric(coefs["K"]), K_true, tolerance = 5)
  expect_equal(as.numeric(coefs["r"]), r_true, tolerance = 0.1)
  expect_equal(as.numeric(coefs["t0"]), t0_true, tolerance = 2)
})

test_that("fit_sigmoidal recovers logistic parameters from synthetic data (seed 42)", {
  set.seed(42)

  # Same test with different seed to verify robustness (G5.6b)
  K_true <- 100
  r_true <- 0.5
  t0_true <- 10

  x <- seq(1, 20, length.out = 50)
  y_true <- K_true / (1 + exp(-r_true * (x - t0_true)))
  y <- y_true + rnorm(50, mean = 0, sd = 1)

  synthetic_data <- data.frame(time = x, completion = y)
  fit <- fit_sigmoidal(synthetic_data, "time", "completion", "logistic")
  coefs <- coef(fit)

  expect_equal(as.numeric(coefs["K"]), K_true, tolerance = 5)
  expect_equal(as.numeric(coefs["r"]), r_true, tolerance = 0.1)
  expect_equal(as.numeric(coefs["t0"]), t0_true, tolerance = 2)
})

test_that("fit_sigmoidal recovers gompertz parameters from synthetic data (seed 123)", {
  set.seed(123)

  # Generate synthetic Gompertz data
  A_true <- 100
  b_true <- 2
  c_true <- 0.3

  x <- seq(1, 20, length.out = 50)
  y_true <- A_true * exp(-b_true * exp(-c_true * x))
  y <- y_true + rnorm(50, mean = 0, sd = 1)

  synthetic_data <- data.frame(time = x, completion = y)
  fit <- fit_sigmoidal(synthetic_data, "time", "completion", "gompertz")
  coefs <- coef(fit)

  expect_equal(as.numeric(coefs["A"]), A_true, tolerance = 5)
  expect_equal(as.numeric(coefs["b"]), b_true, tolerance = 0.5)
  expect_equal(as.numeric(coefs["c"]), c_true, tolerance = 0.1)
})

test_that("fit_sigmoidal recovers gompertz parameters from synthetic data (seed 42)", {
  set.seed(42)

  # Same test with different seed (G5.6b)
  A_true <- 100
  b_true <- 2
  c_true <- 0.3

  x <- seq(1, 20, length.out = 50)
  y_true <- A_true * exp(-b_true * exp(-c_true * x))
  y <- y_true + rnorm(50, mean = 0, sd = 1)

  synthetic_data <- data.frame(time = x, completion = y)
  fit <- fit_sigmoidal(synthetic_data, "time", "completion", "gompertz")
  coefs <- coef(fit)

  expect_equal(as.numeric(coefs["A"]), A_true, tolerance = 5)
  expect_equal(as.numeric(coefs["b"]), b_true, tolerance = 0.5)
  expect_equal(as.numeric(coefs["c"]), c_true, tolerance = 0.1)
})

# ============================================================================
# Algorithm Performance Tests (G5.7)
# ============================================================================

test_that("fit_sigmoidal improves with more data points", {
  set.seed(123)

  K_true <- 100
  r_true <- 0.5
  t0_true <- 10

  # Test with different numbers of data points
  for (n_points in c(10, 30, 50)) {
    x <- seq(1, 20, length.out = n_points)
    y_true <- K_true / (1 + exp(-r_true * (x - t0_true)))
    y <- y_true + rnorm(n_points, mean = 0, sd = 1)

    data_test <- data.frame(time = x, completion = y)
    fit <- fit_sigmoidal(data_test, "time", "completion", "logistic")
    coefs <- coef(fit)

    # Store error for comparison
    error_K <- abs(as.numeric(coefs["K"]) - K_true)

    # More data should generally give better fits (with some randomness)
    expect_true(is.numeric(error_K))
    expect_true(error_K < 20) # Should be reasonably close
  }
})

test_that("fit_sigmoidal convergence improves with less noisy data", {
  set.seed(42)

  K_true <- 100
  r_true <- 0.5
  t0_true <- 10

  x <- seq(1, 20, length.out = 50)
  y_true <- K_true / (1 + exp(-r_true * (x - t0_true)))

  # Test with different noise levels
  noise_levels <- c(5, 2, 0.5)
  errors_K <- numeric(length(noise_levels))

  for (i in seq_along(noise_levels)) {
    y <- y_true + rnorm(50, mean = 0, sd = noise_levels[i])
    data_test <- data.frame(time = x, completion = y)
    fit <- fit_sigmoidal(data_test, "time", "completion", "logistic")
    coefs <- coef(fit)
    errors_K[i] <- abs(as.numeric(coefs["K"]) - K_true)
  }

  # Lower noise should give better parameter estimates
  expect_true(errors_K[3] < errors_K[1])
})

# ============================================================================
# Edge Condition Tests (G5.8c) - All-Identical Data
# ============================================================================

test_that("fit_sigmoidal fails gracefully with all identical y values", {
  data_constant <- data.frame(time = 1:10, completion = rep(50, 10))

  # Should error or fail to converge since there's no curve to fit
  # Platform-dependent: nlsLM may error or succeed with degenerate fit
  result <- tryCatch(
    {
      fit <- fit_sigmoidal(data_constant, "time", "completion", "logistic")
      # If it doesn't error, check that it's a degenerate fit
      # (constant data means fit should have issues)
      list(success = TRUE, fit = fit)
    },
    error = function(e) {
      list(success = FALSE, error = e)
    }
  )

  # Test passes if either:
  # 1. An error was thrown (success = FALSE), OR
  # 2. The fit succeeded but is degenerate (all predictions ≈ constant)
  if (result$success) {
    # Check that the fit is degenerate (predictions are essentially constant)
    preds <- predict(result$fit, newdata = data_constant)
    # All predictions should be very close to each other (constant)
    expect_true(sd(preds) < 1e-6)
  } else {
    # An error was thrown - this is expected behavior
    expect_true(!result$success)
  }
})

test_that("fit_sigmoidal handles all NA in completion column", {
  data_na <- data.frame(time = 1:10, completion = rep(NA_real_, 10))

  expect_error(
    fit_sigmoidal(data_na, "time", "completion", "logistic"),
    "Data columns must not contain NA values"
  )
})

# ============================================================================
# Noise Susceptibility Tests (G5.9b) - Random Seed Stability
# ============================================================================

test_that("fit_sigmoidal produces consistent results across seeds", {
  K_true <- 100
  r_true <- 0.5
  t0_true <- 10

  x <- seq(1, 20, length.out = 50)
  y_true <- K_true / (1 + exp(-r_true * (x - t0_true)))

  # Fit with different noise realizations
  set.seed(111)
  y1 <- y_true + rnorm(50, 0, 1)
  data1 <- data.frame(time = x, completion = y1)
  fit1 <- fit_sigmoidal(data1, "time", "completion", "logistic")

  set.seed(222)
  y2 <- y_true + rnorm(50, 0, 1)
  data2 <- data.frame(time = x, completion = y2)
  fit2 <- fit_sigmoidal(data2, "time", "completion", "logistic")

  set.seed(333)
  y3 <- y_true + rnorm(50, 0, 1)
  data3 <- data.frame(time = x, completion = y3)
  fit3 <- fit_sigmoidal(data3, "time", "completion", "logistic")

  # Parameter estimates should be reasonably consistent
  coef1 <- coef(fit1)
  coef2 <- coef(fit2)
  coef3 <- coef(fit3)

  # All K estimates should be within reasonable range
  expect_true(abs(as.numeric(coef1["K"]) - as.numeric(coef2["K"])) < 10)
  expect_true(abs(as.numeric(coef2["K"]) - as.numeric(coef3["K"])) < 10)
})

# ============================================================================
# print.pra_sigmoidal_fit
# ============================================================================

test_that("print.pra_sigmoidal_fit produces output and returns invisibly", {
  fit <- fit_sigmoidal(data, "time", "completion", "logistic")
  expect_s3_class(fit, "pra_sigmoidal_fit")
  out <- capture.output(result <- print(fit))
  expect_true(any(grepl("Sigmoidal Model Fit Summary", out)))
  # Should include coefficient information from summary
  expect_true(any(grepl("K|r|t0", out)))
})

test_that("fit_sigmoidal assigns pra_sigmoidal_fit class", {
  fit <- fit_sigmoidal(data, "time", "completion", "pearl")
  expect_s3_class(fit, "pra_sigmoidal_fit")
  expect_s3_class(fit, "nls")
  # Class should have pra_sigmoidal_fit as first element
  expect_equal(class(fit)[1], "pra_sigmoidal_fit")
})

#' srr_stats
#'
#' @srrstatsVerbose TRUE
#'
#' @srrstats {G1.0} *Statistical Software lists primary references from published academic literature - documented in each function file.*
#' @srrstats {G1.1} *Statistical Software documents algorithm implementation status - documented in each function file.*
#' @srrstats {G1.2} *Life Cycle Statement is included in README and package documentation.*
#' @srrstats {G1.3} *All statistical terminology is clarified in function documentation.*
#' @srrstats {G1.4} *Software uses [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions - documented in each function file.*
#' @srrstats {G1.4a} *All internal functions are documented with roxygen2 format.*
#' @srrstats {G2.0} *Implements assertions on lengths of inputs - documented in applicable function files.*
#' @srrstats {G2.0a} *Provides explicit secondary documentation of expectations on lengths of inputs - documented in applicable function files.*
#' @srrstats {G2.1} *Implements assertions on types of inputs - documented in applicable function files.*
#' @srrstats {G2.1a} *Provides explicit secondary documentation of expectations on data types - documented in applicable function files.*
#' @srrstats {G2.2} *Appropriately prohibits multivariate input to univariate parameters - documented in applicable function files.*
#' @srrstats {G2.3a} *Uses explicit validation for character parameters where applicable - documented in eac(), tcpi(), fit_sigmoidal(), predict_sigmoidal().*
#' @srrstats {G2.3b} *Uses tolower() to convert character inputs to lowercase for case-insensitive matching - documented in eac(), tcpi(), fit_sigmoidal(), predict_sigmoidal(), plot_sigmoidal().*
#' @srrstats {G2.4a} *Uses as.integer() for integer comparisons where applicable - documented in cor_matrix() and mcs().*
#' @srrstats {G2.7} *Accepts standard tabular forms (matrix, data.frame) where applicable - documented in dsm.R and sigmoidal.R.*
#' @srrstats {G2.13} *Implements checks for missing data via anyNA() and is.na() - documented in applicable function files.*
#' @srrstats {G2.14} *All functions provide options for users to specify how to handle missing (NA) data - documented in each function file.*
#' @srrstats {G2.14a} *Errors on missing data with informative messages - documented in applicable function files.*
#' @srrstats {G2.14b} *Ignores missing data where appropriate with proper handling - documented in learning.R for observed causes/risks.*
#' @srrstats {G3.1} *Covariance calculations allow user-specified correlation matrices - documented in mcs.R, sensitivity.R, and smm.R.*
#' @srrstats {G3.1a} *Documentation describes usage of correlation matrices in examples.*
#' @srrstats {G2.15} *All functions implement explicit checks for NaN values via is.nan() with informative error messages - documented in each function file.*
#' @srrstats {G2.16} *All functions implement explicit checks for undefined (Inf, -Inf) values via is.infinite() with informative error messages - documented in each function file.*
#' @srrstats {G5.2} *Error and warning behaviour is explicitly demonstrated through tests using expect_error() for every stop() message in each function.*
#' @srrstats {G5.2a} *Every error message produced by stop() is unique - documented in applicable function files.*
#' @srrstats {G5.2b} *Tests trigger every error message produced by stop() and compare against expected values using expect_error().*
#' @srrstats {G5.3} *Return objects from all functions are tested for absence of NA, NaN, and Inf values.*
#' @srrstats {G5.6} *Parameter recovery tests verify implementations produce expected results given data with known properties - documented in all test files.*
#' @srrstats {G5.6a} *Parameter recovery tests succeed within defined tolerance rather than exact values - documented in all test files.*
#' @srrstats {G5.6b} *Parameter recovery tests run with multiple random seeds when randomness is involved - documented in test-mcs.R, test-sigmoidal.R, test-inference.R.*
#' @srrstats {G5.7} *Algorithm performance tests verify implementations perform correctly as data properties change - documented in test-mcs.R, test-sigmoidal.R, test-inference.R, test-cormat.R.*
#' @srrstats {G5.8} *Edge condition tests verify appropriate behavior with extreme data properties - documented in all test files.*
#' @srrstats {G5.8a} *Zero-length data tests trigger clear errors - documented in all test files.*
#' @srrstats {G5.8b} *Unsupported data type tests trigger clear errors - documented in all test files.*
#' @srrstats {G5.8c} *All-NA and all-identical data tests trigger clear errors or warnings - documented in all test files.*
#' @srrstats {G5.8d} *Out-of-scope data tests verify appropriate behavior - documented in test-dsm.R, test-evm.R, test-sigmoidal.R.*
#' @srrstats {G5.9} *Noise susceptibility tests verify stochastic behavior stability - documented in test-mcs.R, test-sigmoidal.R, test-inference.R, test-cormat.R, test-smm.R.*
#' @srrstats {G5.9a} *Trivial noise tests show results are stable at machine epsilon scale - documented in applicable test files.*
#' @srrstats {G5.9b} *Random seed stability tests show consistent behavior across different seeds - documented in test-mcs.R, test-sigmoidal.R, test-inference.R, test-cormat.R.*
#' @noRd
NULL

#' NA_standards
#'
#' The following standards have been deemed not applicable to this package.
#' Explanations are provided for each standard.
#'
#' @srrstatsNA {G1.5} *Not applicable - package does not make performance claims requiring reproducible results from publications.*
#' @srrstatsNA {G1.6} *Not applicable - this is the first R implementation; no alternative R packages exist for comparison.*
#' @srrstatsNA {G2.4} *Not applicable - package works with numeric inputs directly; conversion between types not required.*
#' @srrstatsNA {G2.4b} *Not applicable - inputs are already numeric; no conversion to continuous needed.*
#' @srrstatsNA {G2.4c} *Not applicable - package does not convert to character type.*
#' @srrstatsNA {G2.4d} *Not applicable - package does not use factor inputs.*
#' @srrstatsNA {G2.4e} *Not applicable - package does not convert from factor type.*
#' @srrstatsNA {G2.5} *Not applicable - package does not use factor inputs.*
#' @srrstatsNA {G2.6} *Not applicable - one-dimensional inputs are appropriately handled as vectors.*
#' @srrstatsNA {G2.8} *Not applicable - functions receive inputs of defined types directly.*
#' @srrstatsNA {G2.9} *Not applicable - package does not perform type conversions that would lose information.*
#' @srrstatsNA {G2.10} *Not applicable - package does not extract columns from tabular inputs in ways requiring special handling.*
#' @srrstatsNA {G2.11} *Not applicable - package does not process data.frame columns with non-standard class attributes.*
#' @srrstatsNA {G2.12} *Not applicable - package does not process data.frames with list columns.*
#' @srrstatsNA {G2.14c} *Not applicable - imputation is not appropriate for this package's statistical methods.*
#' @srrstatsNA {G3.0} *Not applicable - package does not compare floating point numbers for equality.*
#' @srrstatsNA {G4.0} *Not applicable - package does not write outputs to local files.*
#' @srrstatsNA {G5.0} *Consider updating* *Not applicable - package algorithms are validated through documented examples rather than standard reference datasets.*
#' @srrstatsNA {G5.1} *Consider updating* *Not applicable - package uses inline examples rather than separate test datasets.*
#' @srrstatsNA {G5.4} *Consider updating* *Correctness is validated through documented examples with known inputs and expected outputs.*
#' @srrstatsNA {G5.4a} *Not applicable - methods are implementations of established algorithms.*
#' @srrstatsNA {G5.4b} *Consider updating* *Not applicable - this is the first R implementation of these methods.*
#' @srrstatsNA {G5.4c} *Consider updating* *Correctness is validated against formulas from referenced academic publications.*
#' @srrstatsNA {G5.5} *Consider updating* *Examples use deterministic inputs; stochastic functions use consistent parameters.*
#' @srrstatsNA {G5.10} *Not applicable - no extended tests requiring special environment variables.*
#' @srrstatsNA {G5.11} *Not applicable - no large datasets required for testing.*
#' @srrstatsNA {G5.11a} *Not applicable - no external downloads required for tests.*
#' @srrstatsNA {G5.12} *Not applicable - no special conditions required for extended tests.*
#' @noRd
NULL

---
title: "Agentic Risk Analysis"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Agentic Risk Analysis}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)
```

## Overview

PRA includes an AI agent framework with three routing modes for user
input:

| Input type | Route | Example |
|------------|-------|---------|
| `/command` | **Deterministic** — executes the tool directly, no LLM | `/mcs tasks=[...]` |
| Numerical data for computation | **LLM tool call** — the model selects and calls the right tool | "Simulate 3 tasks: Normal(10,2)..." |
| Conceptual / explanatory question | **RAG** — answered from the knowledge base, no tool call | "What is earned value?" |

Three interfaces are available:

1. **Slash commands** (`/mcs`, `/evm`, `/risk`, ...) - deterministic tool
   calls that bypass the LLM for instant, reliable results
2. **Chat interface** (`pra_chat()`) - programmatic R chat object powered
   by ellmer where the LLM selects tools or answers from RAG
3. **Shiny app** (`pra_app()`) - browser-based experience combining all
   three modes, powered by shinychat

## Prerequisites

### Install Ollama

Download from <https://ollama.com>, then pull a model:

```bash
ollama serve
ollama pull llama3.2
ollama pull nomic-embed-text  # for RAG embeddings
```

### Install R dependencies

```{r eval = FALSE}
install.packages(c("ellmer", "ragnar", "shiny", "bslib", "shinychat", "jsonlite"))
```

## Slash Commands

Slash commands provide deterministic tool execution, no LLM required.
Type `/help` to see all available commands, or `/help <command>` for
detailed usage with argument descriptions and examples.

### Available commands

```{r}
library(PRA)
cat(PRA:::format_help_overview())
```

### Getting help for a command

Each command includes argument specifications, defaults, and examples:

```{r}
cat(PRA:::format_command_help("mcs", PRA:::pra_command_registry()$mcs))
```

### Example: Monte Carlo simulation

Run a simulation for a 3-task project directly:

```{r, eval = TRUE}
set.seed(42)
r <- PRA:::execute_command(
  '/mcs n=10000 tasks=[{"type":"normal","mean":10,"sd":2},{"type":"triangular","a":5,"b":10,"c":15},{"type":"uniform","min":8,"max":12}]'
)
cat(r$result)
```

```{r, eval = TRUE, fig.width = 7, fig.height = 4.5, out.width = "100%"}
# The /mcs command stores results for chaining — visualize them:
result <- PRA:::.pra_agent_env$last_mcs
hist(result$total_distribution,
  freq = FALSE, breaks = 50,
  main = "Monte Carlo Simulation Results",
  xlab = "Total Project Duration/Cost",
  col = "#18bc9c80", border = "white"
)
curve(dnorm(x, mean = result$total_mean, sd = result$total_sd),
  add = TRUE, col = "#2c3e50", lwd = 2
)
abline(
  v = quantile(result$total_distribution, c(0.50, 0.95)),
  col = c("#3498db", "#e74c3c"), lty = 2, lwd = 1.5
)
legend("topright",
  legend = c("Normal fit", "P50", "P95"),
  col = c("#2c3e50", "#3498db", "#e74c3c"),
  lty = c(1, 2, 2), lwd = c(2, 1.5, 1.5),
  cex = 0.8, bg = "white"
)
```

### Example: Chaining MCS to contingency

After running `/mcs`, chain to `/contingency` for the reserve estimate:

```{r, eval = TRUE}
r <- PRA:::execute_command("/contingency phigh=0.95 pbase=0.50")
cat(r$result)
```

### Example: Sensitivity analysis

Identify which tasks drive the most variance:

```{r, eval = TRUE}
r <- PRA:::execute_command(
  '/sensitivity tasks=[{"type":"normal","mean":10,"sd":2},{"type":"triangular","a":5,"b":10,"c":15},{"type":"uniform","min":8,"max":12}]'
)
cat(r$result)
```

### Example: Earned Value Management

Full EVM analysis with a single command:

```{r, eval = TRUE}
r <- PRA:::execute_command(
  "/evm bac=500000 schedule=[0.2,0.4,0.6,0.8,1.0] period=3 complete=0.35 costs=[90000,195000,310000]"
)
cat(r$result)
```

### Example: Bayesian risk probability

Calculate prior risk from two root causes:

```{r, eval = TRUE}
r <- PRA:::execute_command(
  "/risk causes=[0.3,0.2] given=[0.8,0.6] not_given=[0.2,0.4]"
)
cat(r$result)
```

Then update with observations — Cause 1 occurred, Cause 2 unknown:

```{r, eval = TRUE}
r <- PRA:::execute_command(
  "/risk_post causes=[0.3,0.2] given=[0.8,0.6] not_given=[0.2,0.4] observed=[1,null]"
)
cat(r$result)
```

### Example: Second Moment Method

Quick analytical estimate without simulation:

```{r, eval = TRUE}
r <- PRA:::execute_command("/smm means=[10,12,8] vars=[4,9,2]")
cat(r$result)
```

### Input validation and guidance

Missing or invalid arguments produce helpful error messages:

```{r, eval = TRUE}
# Missing required arguments
r <- PRA:::execute_command("/risk causes=[0.3]")
cat(r$result)
```

```{r, eval = TRUE}
# Unknown command
r <- PRA:::execute_command("/simulate")
cat(r$result)
```

## Chat Interface

The chat interface routes queries through the LLM, which decides whether
to call a tool or answer from RAG context:

- **Numerical data** (distributions, costs, schedules) → LLM calls the
  appropriate tool and interprets the results
- **Conceptual questions** ("what is CPI?", "how does MCS work?") → LLM
  answers from RAG knowledge base with source citations
- **Ambiguous** → LLM uses RAG context and only calls a tool if data is
  present

```{r}
library(PRA)
chat <- pra_chat(model = "llama3.2")

# Tool call: user provides numerical data
chat$chat("Run a Monte Carlo simulation for a 3-task project with
  Task A ~ Normal(10, 2), Task B ~ Triangular(5, 10, 15),
  Task C ~ Uniform(8, 12). Use 10,000 simulations.")

# RAG: conceptual question, no computation needed
chat$chat("What is the difference between SPI and CPI?")
```

For guaranteed reliability with computations, use `/commands` instead.
The chat interface is best suited for exploratory questions and
interpretation.

### Using cloud models

For better accuracy with complex queries, supply a pre-configured ellmer
chat object:

```{r}
# OpenAI
chat <- pra_chat(chat = ellmer::chat_openai(model = "gpt-4o"))

# Anthropic
chat <- pra_chat(chat = ellmer::chat_anthropic(model = "claude-sonnet-4-20250514"))
```

### Notes on chat reliability

- **Conceptual questions** work well even with small models since they
  only require reading RAG context, not tool calling.
- **Tool calling** quality depends on model size. `llama3.2` (3B)
  handles simple single-tool queries; larger models (8B+) are more
  reliable for multi-step chains.
- If the model attempts to call a tool for a conceptual question, or
  describes what it would do instead of calling tools, use `/commands`
  for deterministic results.

## Interactive Shiny App

For a browser-based experience with streaming responses and inline
visualizations:

```{r}
pra_app()
```

The app supports all three input modes in the same chat panel:

- Type `/mcs tasks=[...]` for **instant deterministic** results
- Type "Simulate 3 tasks..." for **LLM tool calling**
- Type "What is earned value?" for **RAG-powered answers**

```{r, echo = FALSE, eval = TRUE, out.width = "100%"}
knitr::include_graphics("../man/figures/pra-app-welcome.png")
```

Clicking an example prompt executes the `/command` instantly and displays
rich results with tables and plots:

```{r, echo = FALSE, eval = TRUE, out.width = "100%"}
knitr::include_graphics("../man/figures/pra-app-tool-call.png")
```

### Features

- **Three input modes** — `/commands` (deterministic), natural language
  (LLM tool calls), and conceptual questions (RAG)
- **Clickable example prompts** that execute `/commands` on click
- **Streaming chat** powered by shinychat with token-by-token responses
- **Inline tool results** displayed as rich HTML tables and plots
- **RAG source citations** — the agent cites knowledge base files for
  conceptual answers
- **Connection status** badge showing model connectivity
- **Collapsible sidebar** with model selection, RAG toggle, and CSV upload
- **Export** — download the conversation as a markdown file

### Configuration options

```{r}
# Custom model and port
pra_app(model = "qwen2.5", port = 3838)

# Disable RAG for faster responses
pra_app(rag = FALSE)
```

## RAG Knowledge Base

The agent is enhanced with domain knowledge through retrieval-augmented
generation (RAG). When RAG context is retrieved, the agent cites the
source files in its response.

### Built-in knowledge files

| File | Topics |
|------|--------|
| `mcs_methods.md` | Distribution selection, correlation, interpreting percentiles |
| `evm_standards.md` | EVM metrics, performance indices, forecasting methods |
| `bayesian_risk.md` | Prior/posterior risk, Bayes' theorem for root cause analysis |
| `learning_curves.md` | Sigmoidal models (logistic, Gompertz, Pearl), curve fitting |
| `sensitivity_contingency.md` | Variance decomposition, contingency reserves |
| `pra_functions.md` | PRA package function reference |

### How RAG context flows

1. User asks a question
2. The question is embedded and matched against the knowledge base using
   hybrid search (vector similarity + BM25)
3. The top 3 matching chunks are prepended to the query with
   `[Source: filename]` tags
4. The agent is instructed to cite these sources in its response
5. The agent uses the context to inform its answer while still calling
   tools for computation

### Adding your own documents

```{r}
store <- build_knowledge_base()

# Add a single file
add_documents(store, "path/to/my_risk_register.md")

# Add all .md and .txt files in a directory
add_documents(store, "path/to/project_docs/")
```

### Disabling RAG

```{r}
# Chat without RAG
chat <- pra_chat(model = "llama3.2", rag = FALSE)

# App without RAG
pra_app(rag = FALSE)
```

## Available Commands and Tools

### Slash commands (deterministic)

| Command | Description |
|---------|-------------|
| `/mcs` | Monte Carlo simulation with task distributions |
| `/smm` | Second Moment Method (analytical estimate) |
| `/contingency` | Contingency reserve from last MCS |
| `/sensitivity` | Variance contribution per task |
| `/evm` | Full Earned Value Management analysis |
| `/risk` | Bayesian prior risk probability |
| `/risk_post` | Bayesian posterior risk after observations |
| `/learning` | Sigmoidal learning curve fit and prediction |
| `/dsm` | Design Structure Matrix |
| `/help` | List all commands or get help for one |

### LLM tools (via chat)

| Module | Tool | Use case |
|--------|------|----------|
| Simulation | `mcs_tool` | Full Monte Carlo with distributions |
| Analytical | `smm_tool` | Quick mean/variance estimate |
| Post-MCS | `contingency_tool` | Reserve at confidence level |
| Post-MCS | `sensitivity_tool` | Variance contribution per task |
| EVM | `evm_analysis_tool` | All 12 EVM metrics in one call |
| Bayesian | `risk_prob_tool` | Prior risk from root causes |
| Bayesian | `risk_post_prob_tool` | Posterior risk after observations |
| Bayesian | `cost_pdf_tool` | Prior cost distribution |
| Bayesian | `cost_post_pdf_tool` | Posterior cost distribution |
| Learning | `fit_and_predict_sigmoidal_tool` | Pearl/Gompertz/Logistic |
| DSM | `parent_dsm_tool` | Resource-task dependencies |
| DSM | `grandparent_dsm_tool` | Risk-resource-task dependencies |

## Evaluation with vitals

PRA includes an evaluation framework for measuring LLM tool-calling
accuracy using the [vitals](https://vitals.tidyverse.org) package. The
evaluation suite in `inst/eval/pra_eval.R` tests 15 scenarios across three
tiers:

| Tier | Description | Example |
|------|-------------|---------|
| Single-tool | One tool call | "Simulate 3 tasks with distributions..." |
| Multi-tool chain | Sequential tool calls | "Run MCS then calculate contingency at 95%" |
| Open-ended | Requires interpretation | "My project is behind schedule, here's EVM data..." |

```{r}
# Run evaluation
source(system.file("eval/pra_eval.R", package = "PRA"))
results <- run_pra_eval(model = "llama3.2")

# Compare models
comparison <- run_pra_comparison(
  models = c("llama3.2", "qwen2.5"),
  rag_options = c(TRUE, FALSE)
)
```

## Troubleshooting

### Tool calling not working

Small models sometimes describe what they would do rather than actually
calling tools. Use `/commands` for reliable deterministic execution:

```{r}
# Instead of asking the LLM:
chat$chat("Run a Monte Carlo simulation...")

# Use the /command directly in the app:
# /mcs tasks=[{"type":"normal","mean":10,"sd":2}]
```

Other workarounds for LLM chat:

- Use `llama3.1` (8B) or larger for better tool calling than 3B models
- Be explicit: "Call the mcs_tool with these parameters..."
- Use a cloud model: `pra_chat(chat = ellmer::chat_openai(model = "gpt-4o"))`

### Slow responses

- Use a smaller model (`llama3.2` is 3B, faster than `llama3.1` 8B)
- Ensure GPU acceleration is active (`ollama ps`)
- Disable RAG if not needed: `pra_chat(rag = FALSE)`
- Use `/commands` — they execute instantly without LLM overhead

### RAG build fails

```{r}
install.packages("ragnar")
```

```bash
ollama pull nomic-embed-text
```

### Source citations missing

If the agent does not cite sources in RAG-enabled responses, try:

- Using a larger model (8B+ models follow citation instructions more reliably)
- Being explicit: "Cite your sources in the response"

---
name: pra-sensitivity-contingency
description: >
  Size a project contingency reserve and rank the tasks that drive total
  uncertainty, using the PRA package. Use after (or alongside) a Monte Carlo
  simulation when a user asks "how much reserve do I need?" or "which tasks
  should I focus mitigation on?"
---

# Sensitivity analysis

`sensitivity_tool(task_dists_json, cor_mat_json)` measures each task's
contribution to total project variance. It takes the **same** input format as
`mcs_tool` (a JSON array of task distributions plus an optional correlation
matrix). Returns one value per task; higher values are the primary drivers of
overall uncertainty. Results are typically shown as a tornado chart. Direct
mitigation resources at the highest-sensitivity tasks first.

# Contingency reserve

`contingency_tool(phigh, pbase)` returns the buffer between a high-confidence
percentile and the base estimate:

```
contingency = P_high - P_base
```

- `phigh` (default `0.95`): high-confidence percentile — there is a `phigh`
  probability the actual value will not exceed it.
- `pbase` (default `0.50`): the median ("most likely") total.

**Important:** call `mcs_tool` first — `contingency_tool` operates on the most
recent Monte Carlo results.

# Common confidence levels

- **P80 contingency**: moderate confidence, common in commercial projects.
- **P95 contingency**: high confidence, common in government/defense projects.
- **Management reserve**: an additional buffer for unknown-unknowns, typically
  5–10% of BAC, held above contingency.

# Workflow

1. `mcs_tool(num_sims, task_dists_json, cor_mat_json)`
2. `contingency_tool(phigh = 0.95, pbase = 0.50)`
3. `sensitivity_tool(task_dists_json, cor_mat_json)`
4. Reduce uncertainty in high-sensitivity tasks to shrink the needed reserve.

# References

- GAO Cost Estimating and Assessment Guide (GAO-20-195G).
- GAO Schedule Assessment Guide (GAO-16-89G).

# Earned Value Management (EVM)

## Overview

Earned Value Management is a project performance measurement methodology that integrates scope, schedule, and cost data. It provides early warning indicators of project health and enables forecasting of final cost and schedule outcomes.

## Core Metrics

### Baseline Metrics
- **BAC (Budget at Completion)**: Total planned budget for the project.
- **PV (Planned Value)**: The authorized budget for work scheduled to be completed by a given time. PV = BAC * planned_percent_complete.
- **EV (Earned Value)**: The authorized budget for work actually completed. EV = BAC * actual_percent_complete.
- **AC (Actual Cost)**: The realized cost for work performed in a given period.

### Variance Indicators
- **SV (Schedule Variance)**: SV = EV - PV. Positive means ahead of schedule.
- **CV (Cost Variance)**: CV = EV - AC. Positive means under budget.

### Performance Indices
- **SPI (Schedule Performance Index)**: SPI = EV / PV. Values > 1.0 indicate ahead of schedule.
- **CPI (Cost Performance Index)**: CPI = EV / AC. Values > 1.0 indicate under budget.

## Forecasting

### EAC (Estimate at Completion)
Three methods depending on assumptions about future performance:

- **Typical (CPI-based)**: EAC = BAC / CPI. Assumes future cost performance matches current CPI. Use when current trends are expected to continue.
- **Atypical**: EAC = AC + (BAC - EV). Assumes future work proceeds at the original planned rate. Use when current variances are one-time anomalies.
- **Combined (CPI * SPI)**: EAC = AC + (BAC - EV) / (CPI * SPI). Accounts for both cost and schedule performance. Use when schedule pressure affects costs.

### Other Forecasts
- **ETC (Estimate to Complete)**: ETC = EAC - AC, or ETC = (BAC - EV) / CPI. Remaining cost to finish.
- **VAC (Variance at Completion)**: VAC = BAC - EAC. Expected overrun (negative) or underrun (positive).
- **TCPI (To-Complete Performance Index)**: TCPI = (BAC - EV) / (BAC - AC) or (BAC - EV) / (EAC - AC). The CPI needed on remaining work to meet a target.

## Warning Thresholds

- CPI or SPI below 0.9 typically signals a project in trouble.
- Research shows that CPI rarely improves by more than 10% after a project is 20% complete.
- TCPI > 1.0 means the project must improve performance to meet its target.

## References

- DOE Earned Value Management Tutorial Series.
- DOE EVMS Interpretation Handbook (EVMSIH v2.0).
- ANSI/EIA-748 Earned Value Management Systems standard.

# Learning Curves for Project Risk Analysis

## Overview

Learning curves model how performance improves with experience. In project risk analysis, they are used to forecast production costs, construction durations, and other metrics that improve as workers and processes gain experience.

## Sigmoidal Models

PRA implements three sigmoidal (S-curve) models that capture the typical pattern of slow initial progress, rapid acceleration, and eventual plateau:

### Pearl Model
y = K / (1 + exp(-r * (x - x0)))

- K: Upper asymptote (maximum capacity/completion)
- r: Growth rate
- x0: Midpoint (inflection point)
- Best for: Symmetric S-curves where growth and saturation are balanced

### Gompertz Model
y = K * exp(-exp(-r * (x - x0)))

- Same parameters as Pearl
- Best for: Asymmetric curves with slow initial growth and rapid mid-phase acceleration
- Common in biological and production learning contexts

### Logistic Model
y = K / (1 + ((K - y0) / y0) * exp(-r * x))

- K: Upper asymptote
- r: Growth rate
- y0: Initial value
- Best for: Cases where the initial value is known and growth is logistic

## Fitting and Prediction

1. **Fit**: Provide a data.frame with x (time/unit) and y (cost/completion) columns, plus the model type. Uses nonlinear least squares (Levenberg-Marquardt algorithm).
2. **Predict**: Use the fitted model to forecast y values for new x values. Optionally compute confidence intervals at a specified confidence level (e.g., 0.95 for 95% CI).
3. **Plot**: Visualize the fitted curve against the original data, with optional confidence bands.

## Choosing a Model

- If the data shows symmetric growth: try Pearl or Logistic
- If early growth is slow relative to mid-phase acceleration: try Gompertz
- Fit all three and compare residuals to select the best fit
- Use confidence intervals to assess prediction uncertainty

## References

- ICEAA Learning Curve Analysis Guide.
- DAU Learning Curve Workshop materials.

---
name: pra-earned-value
description: >
  Measure project cost/schedule performance and forecast final outcomes with
  Earned Value Management using the PRA package. Use when a user provides a
  budget (BAC), a planned schedule curve, and actual costs/progress and wants
  SPI, CPI, variances, or an Estimate at Completion (EAC).
---

# When to use

The user has a Budget at Completion, a cumulative planned-value curve, current
progress, and actual costs, and wants performance indices or forecasts.

# MCP tool

`evm_analysis_tool(bac, schedule_json, time_period, actual_per_complete,
actual_costs_json, cumulative)` runs the complete ANSI/EIA-748 suite in one
call — do not chain the individual metrics manually.

- `bac`: Budget at Completion (total planned budget).
- `schedule_json`: JSON array of cumulative planned completion fractions, e.g.
  `[0.1, 0.2, 0.4, 0.7, 1.0]`.
- `time_period`: current period (1-based index into the schedule).
- `actual_per_complete`: actual percent complete as a decimal (e.g. `0.35`).
- `actual_costs_json`: JSON array of actual costs per period, e.g.
  `[10000, 22000, 37000]`.
- `cumulative`: `"true"` if the costs are already cumulative, else `"false"`.

# What it returns

- **Baseline**: PV = BAC × planned%, EV = BAC × actual%, AC.
- **Variances**: SV = EV − PV (schedule), CV = EV − AC (cost).
- **Indices**: SPI = EV / PV, CPI = EV / AC (>1 is favorable).
- **Forecasts (EAC)**:
  - Typical (CPI-based): `BAC / CPI` — current trend continues.
  - Atypical: `AC + (BAC − EV)` — current variance is a one-off.
  - Combined: `AC + (BAC − EV) / (CPI × SPI)` — schedule pressure affects cost.
- **ETC** = EAC − AC, **VAC** = BAC − EAC, **TCPI** = efficiency required on
  remaining work to hit the target.

# Warning thresholds

- CPI or SPI below ~0.9 signals a project in trouble.
- CPI rarely improves by more than ~10% after a project is 20% complete.
- TCPI substantially above 1.0 is an unrealistic recovery target.

# References

- ANSI/EIA-748 Earned Value Management Systems standard.
- DOE EVMS Interpretation Handbook (EVMSIH v2.0).

---
name: pra-monte-carlo
description: >
  Run Monte Carlo (or fast Second Moment) schedule/cost risk simulations with
  the PRA package. Use when a user provides task-level uncertainty
  distributions and needs the range of possible project outcomes — mean,
  standard deviation, and P5/P50/P95 percentiles — optionally accounting for
  task correlation.
---

# When to use

- Task-level uncertainty can be described with probability distributions.
- The user needs percentile-based estimates (P50, P80, P95) for budgeting or
  scheduling, or a confidence level for a cost/schedule target.
- Tasks may be correlated and you need to capture joint uncertainty.

If only first and second moments (means and variances) are available, or a
rapid analytical estimate is enough, use the Second Moment Method instead.

# MCP tools

- `mcs_tool(num_sims, task_dists_json, cor_mat_json)` — full Monte Carlo
  simulation. `num_sims` is typically `10000`.
- `smm_tool(mean_json, var_json, cor_mat_json)` — fast analytical estimate of
  total mean and variance from task means and variances.

`task_dists_json` is a JSON array of task distribution objects:

```json
[{"type":"normal","mean":10,"sd":2},
 {"type":"triangular","a":5,"b":10,"c":15},
 {"type":"uniform","min":8,"max":12}]
```

- **normal**: `mean`, `sd`
- **triangular**: `a` (min), `b` (max), `c` (mode) — most common in practice
- **uniform**: `min`, `max`

`cor_mat_json` is an optional JSON 2-D array; use `"null"` (or omit) for
independent tasks. Ignoring positive correlation underestimates total variance.

# Interpreting results

- **P50 (median)**: value with a 50% chance of being exceeded.
- **P80**: common contingency-budgeting threshold.
- **P95**: high-confidence estimate for management reserve / worst case.
- **Mean** may differ from P50 for skewed distributions.

Use at least 10,000 simulations for stable tail percentiles.

# Follow-ups

- `contingency_tool` for a reserve buffer (skill: `pra-sensitivity-contingency`).
- `sensitivity_tool` to find the dominant risk drivers.

# References

- GAO Cost Estimating and Assessment Guide (GAO-20-195G), Ch. 12.
- RAND, Quantitative Risk Analysis for Project Management (WR-112).

---
title: "Earned Value Management"
output:
  html_vignette:
    fig_width: 7
    fig_height: 5
vignette: >
  %\VignetteIndexEntry{Earned Value Management}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

Earned Value Management (EVM) is a project management technique that integrates scope, schedule, and cost to measure project performance objectively. By comparing planned work against actual work accomplished and actual cost incurred, EVM provides early warning of cost overruns and schedule delays.

## Key Metrics

| Metric | Formula | Interpretation |
|--------|---------|----------------|
| **PV** – Planned Value | BAC × Planned% | Budget authorized for scheduled work |
| **EV** – Earned Value | BAC × Actual% | Budget for work actually completed |
| **AC** – Actual Cost | Σ period costs | Total cost incurred to date |
| **CV** – Cost Variance | EV − AC | > 0: under budget; < 0: over budget |
| **SV** – Schedule Variance | EV − PV | > 0: ahead of schedule; < 0: behind schedule |
| **CPI** – Cost Performance Index | EV / AC | > 1: efficient; = 1: on target; < 1: inefficient |
| **SPI** – Schedule Performance Index | EV / PV | > 1: ahead; = 1: on target; < 1: behind |
| **EAC** – Estimate at Completion | See below | Forecast of total project cost |
| **ETC** – Estimate to Complete | EAC − AC | Remaining cost to finish the project |
| **VAC** – Variance at Completion | BAC − EAC | Expected budget surplus (positive) or overrun (negative) |
| **TCPI** – To-Complete Performance Index | (BAC−EV) / (BAC−AC) | Efficiency required on remaining work |

## Example Setup

```{r setup}
library(PRA)
```

We will track a project with a total budget of $500,000 over a 5-period schedule.

```{r}
bac <- 500000
schedule <- c(0.10, 0.25, 0.50, 0.75, 1.00)
time_period <- 3
```

### Planned Value (PV)

PV is the authorized budget for work planned through the current period.

```{r results='asis'}
pv_val <- pv(bac, schedule, time_period)
cat("Planned Value (PV): $", format(pv_val, big.mark = ","), "\n")
```

The project was planned to be 50% complete by period 3, so PV = $250,000.

### Earned Value (EV)

EV reflects the budget value of work actually completed.

```{r results='asis'}
actual_per_complete <- 0.40
ev_val <- ev(bac, actual_per_complete)
cat("Earned Value (EV): $", format(ev_val, big.mark = ","), "\n")
```

Only 40% of work is done despite 50% being planned — we are behind schedule.

### Actual Cost (AC)

AC is the cumulative cost incurred. Use `cumulative = FALSE` when passing individual period costs.

```{r results='asis'}
period_costs <- c(45000, 110000, 135000)
ac_val <- ac(period_costs, time_period, cumulative = FALSE)
cat("Actual Cost (AC): $", format(ac_val, big.mark = ","), "\n")
```

### Performance Indicators

```{r results='asis'}
sv_val <- sv(ev_val, pv_val)
cv_val <- cv(ev_val, ac_val)
spi_val <- spi(ev_val, pv_val)
cpi_val <- cpi(ev_val, ac_val)

cat("Schedule Variance (SV):          $", format(sv_val, big.mark = ","), "\n")
cat("Cost Variance (CV):              $", format(cv_val, big.mark = ","), "\n")
cat("Schedule Performance Index (SPI):", round(spi_val, 3), "\n")
cat("Cost Performance Index (CPI):    ", round(cpi_val, 3), "\n")
```

**Interpretation:** SPI < 1 means we are behind schedule (only earning 80 cents of planned value per dollar of schedule). CPI < 1 means we are over budget (earning only 69 cents of value per dollar spent).

## Forecasting: Estimate at Completion (EAC)

EAC forecasts the total cost at project completion. Three methods are available, each making a different assumption about future performance:

| Method | Formula | When to use |
|--------|---------|-------------|
| **Typical** | BAC / CPI | Current cost inefficiency is expected to continue |
| **Atypical** | AC + (BAC − EV) | Cost overrun was a one-time event; future work will proceed at planned rate |
| **Combined** | AC + (BAC − EV) / (CPI × SPI) | Both cost and schedule performance will influence future costs |

```{r results='asis'}
eac_typical <- eac(bac, method = "typical", cpi = cpi_val)
eac_atypical <- eac(bac, method = "atypical", ac = ac_val, ev = ev_val)
eac_combined <- eac(bac,
  method = "combined", cpi = cpi_val, ac = ac_val,
  ev = ev_val, spi = spi_val
)

cat("EAC (typical):  $", format(round(eac_typical), big.mark = ","), "\n")
cat("EAC (atypical): $", format(round(eac_atypical), big.mark = ","), "\n")
cat("EAC (combined): $", format(round(eac_combined), big.mark = ","), "\n")
```

The typical method gives the most conservative (highest cost) estimate because it assumes the current CPI persists. The atypical method is the most optimistic, assuming past overruns won't recur.

### EAC Comparison Table

```{r}
eac_table <- data.frame(
  Method = c("Typical", "Atypical", "Combined"),
  EAC = c(round(eac_typical), round(eac_atypical), round(eac_combined)),
  Overrun = c(
    round(eac_typical - bac),
    round(eac_atypical - bac),
    round(eac_combined - bac)
  ),
  Assumption = c(
    "Current CPI continues",
    "Future work at planned rate",
    "CPI and SPI both factor in"
  )
)
knitr::kable(eac_table,
  format.args = list(big.mark = ","),
  caption = "EAC Comparison by Method"
)
```

## Additional Metrics

```{r results='asis'}
etc_val <- etc(bac, ev_val, cpi_val)
vac_val <- vac(bac, eac_typical)

# TCPI to meet original BAC
tcpi_bac <- tcpi(bac, ev_val, ac_val, target = "bac")

# TCPI to meet revised EAC (typical)
tcpi_eac <- tcpi(bac, ev_val, ac_val, target = "eac", eac = eac_typical)

cat("Estimate to Complete (ETC):         $", format(round(etc_val), big.mark = ","), "\n")
cat("Variance at Completion (VAC):       $", format(round(vac_val), big.mark = ","), "\n")
cat("TCPI (to meet BAC):                ", round(tcpi_bac, 3), "\n")
cat("TCPI (to meet EAC):                ", round(tcpi_eac, 3), "\n")
```

**Interpretation:** TCPI > 1 means the team must work more efficiently than they have been to meet the target. A TCPI of `r round(tcpi_bac, 2)` to meet BAC means the remaining work must be done at `r round(tcpi_bac * 100, 0)`% efficiency — significantly better than the current CPI of `r round(cpi_val, 2)`. Using the EAC target is more realistic and shows whether the revised budget is achievable.

## Performance Trend Chart

The chart below shows cumulative PV, AC, and EV over time, with horizontal reference lines for BAC and EAC.

```{r warning=FALSE}
time_periods <- c(1, 2, 3)
actual_pct <- c(0.08, 0.22, 0.40)
p_costs <- c(45000, 110000, 135000)

pv_vals <- sapply(time_periods, function(t) pv(bac, schedule, t))
ac_vals <- cumsum(p_costs)
ev_vals <- sapply(actual_pct, function(a) ev(bac, a))

trend_data <- data.frame(
  Period = time_periods,
  PV     = pv_vals,
  AC     = ac_vals,
  EV     = ev_vals
)

p <- ggplot2::ggplot(trend_data, ggplot2::aes(x = Period)) +
  ggplot2::geom_line(ggplot2::aes(y = PV, color = "Planned Value (PV)"), linewidth = 1.2) +
  ggplot2::geom_line(ggplot2::aes(y = AC, color = "Actual Cost (AC)"), linewidth = 1.2) +
  ggplot2::geom_line(ggplot2::aes(y = EV, color = "Earned Value (EV)"), linewidth = 1.2) +
  ggplot2::geom_point(ggplot2::aes(y = PV, color = "Planned Value (PV)"), size = 3) +
  ggplot2::geom_point(ggplot2::aes(y = AC, color = "Actual Cost (AC)"), size = 3) +
  ggplot2::geom_point(ggplot2::aes(y = EV, color = "Earned Value (EV)"), size = 3) +
  ggplot2::geom_hline(yintercept = bac, linetype = "dashed", color = "black", linewidth = 0.8) +
  ggplot2::geom_hline(yintercept = eac_typical, linetype = "dotted", color = "darkred", linewidth = 0.8) +
  ggplot2::annotate("text", x = 2.8, y = bac + 12000, label = "BAC = $500K", size = 3.5) +
  ggplot2::annotate("text", x = 2.8, y = eac_typical + 12000, label = "EAC = $725K", size = 3.5) +
  ggplot2::scale_color_manual(values = c(
    "Planned Value (PV)" = "steelblue",
    "Actual Cost (AC)"   = "tomato",
    "Earned Value (EV)"  = "forestgreen"
  )) +
  ggplot2::scale_y_continuous(labels = scales::label_dollar(scale = 1e-3, suffix = "K")) +
  ggplot2::labs(
    title  = "Earned Value Management - Performance Trend",
    x      = "Time Period",
    y      = "Value",
    color  = NULL
  ) +
  ggplot2::theme_minimal() +
  ggplot2::theme(legend.position = "bottom")

print(p)
```

**Reading the chart:** The gap between PV and EV (both measured on the same vertical axis) shows the schedule gap, EV is below PV, confirming we are behind. The gap between AC and EV shows the cost overrun, we have spent more than the value we have earned.

---
name: pra-learning-curves
description: >
  Fit sigmoidal (S-curve) learning curves and forecast future values with the
  PRA package. Use when a user has historical time/completion (or unit/cost)
  data showing slow start, rapid acceleration, and plateau, and wants a fitted
  model plus predictions with confidence intervals.
---

# When to use

The user has historical `x` (time or unit number) and `y` (completion, cost,
etc.) data and wants to model the learning curve and project it forward.

# MCP tool

`fit_and_predict_sigmoidal_tool(x_json, y_json, model_type, predict_x_json,
conf_level)` fits the model with nonlinear least squares
(Levenberg–Marquardt) and returns fitted coefficients plus predictions.

- `x_json`: JSON array of x values, e.g. `[1,2,3,4,5,6,7,8,9,10]`.
- `y_json`: JSON array of y values, e.g. `[5,15,40,60,70,75,80,85,90,95]`.
- `model_type`: `"pearl"`, `"gompertz"`, or `"logistic"`.
- `predict_x_json`: x values to forecast at; omit to predict at the original x.
- `conf_level`: e.g. `0.95` for 95% prediction intervals; omit for none.

# Choosing a model

- **Pearl**: symmetric S-curve; growth and saturation balanced.
- **Gompertz**: asymmetric — slow initial growth, rapid mid-phase acceleration.
- **Logistic**: growth from a known initial value.

Fit all three and compare residuals to select the best fit; use confidence
intervals to gauge prediction uncertainty (bands widen as extrapolation extends
beyond the observed range).

# References

- ICEAA Learning Curve Analysis Guide.
- DAU Learning Curve Workshop materials.

# Sensitivity Analysis and Contingency Reserves

## Sensitivity Analysis

### Purpose
Sensitivity analysis identifies which tasks or risk factors contribute most to total project uncertainty. This guides risk mitigation by focusing attention on the tasks where reducing uncertainty has the greatest impact on the project outcome.

### Method
PRA computes sensitivity by measuring the variance contribution of each task to the total project variance. Tasks with higher sensitivity values are the primary drivers of overall uncertainty.

### Interpreting Results
- Results are returned as a numeric vector with one value per task
- Higher values indicate tasks that contribute more to total variance
- Use these to prioritize risk mitigation efforts: reducing uncertainty in high-sensitivity tasks yields the greatest reduction in overall project risk
- Typically presented as a tornado chart (horizontal bar chart sorted by sensitivity)

### Input Format
Sensitivity analysis uses the same input format as Monte Carlo simulation: a list of task distributions and an optional correlation matrix. This makes it easy to run both analyses on the same project data.

## Contingency Reserves

### Purpose
Contingency reserves provide a financial buffer to account for project uncertainty. The reserve amount is the difference between a high-confidence estimate and the baseline estimate.

### Calculation
Contingency = P_high - P_base

Where:
- **P_high** (default 0.95): The 95th percentile of the total cost/duration distribution. There is a 95% probability the actual value will not exceed this.
- **P_base** (default 0.50): The median (50th percentile) of the distribution. This represents the "most likely" total.

The contingency function takes the Monte Carlo simulation results as input, so run `mcs()` first, then pass the result to `contingency()`.

### Common Confidence Levels
- **P80 contingency**: Moderate confidence. Common in commercial projects.
- **P95 contingency**: High confidence. Common in government and defense projects.
- **Management reserve**: Additional buffer beyond contingency for unknown unknowns. Typically 5-10% of BAC.

### Workflow
1. Define task distributions
2. Run Monte Carlo simulation: `mcs(num_sims, task_dists, cor_mat)`
3. Compute contingency: `contingency(mcs_result, phigh = 0.95, pbase = 0.50)`
4. Run sensitivity: `sensitivity(task_dists, cor_mat)`
5. Focus risk mitigation on high-sensitivity tasks to reduce the needed contingency

## References

- GAO Cost Estimating and Assessment Guide (GAO-20-195G).
- GAO Schedule Assessment Guide (GAO-16-89G).

---
name: pra-dsm
description: >
  Quantify structural coupling between project tasks through shared resources
  and risks using the PRA package's Design Structure Matrices. Use when a user
  provides a resource–task matrix (and optionally a risk–resource matrix) and
  wants to see which task pairs are coupled and therefore tend to fail together.
---

# When to use

The user has a binary matrix mapping resources to tasks (and optionally risks to
resources) and wants to identify coupled task pairs — the structural pathways
through which disruptions and risks propagate.

# MCP tools

- `parent_dsm_tool(matrix_json)` — the **Parent DSM**, derived from the
  resource–task matrix **S** (rows = resources, columns = tasks) as
  `P = Sᵀ S`. Off-diagonal `P[j,k]` counts resources shared between tasks j and
  k. `matrix_json` is a JSON 2-D array, e.g.
  `[[1,0,1,0],[0,1,0,1],[1,0,1,1]]`.
- `grandparent_dsm_tool(s_matrix_json, r_matrix_json)` — the **Grandparent
  DSM**, which adds a risk layer via the risk–resource matrix **R**. Off-diagonal
  `G[j,k]` counts risks shared between tasks j and k through the resource chain.
  - `s_matrix_json`: resource–task matrix S (resources × tasks).
  - `r_matrix_json`: risk–resource matrix R (risks × resources).

# Interpreting results

Task pairs with high off-diagonal values are strongly coupled: in the Parent DSM
through shared resources, in the Grandparent DSM through shared risk exposure.
These pairs are prime candidates for contingency buffering or resource
decoupling, and pair naturally with a Monte Carlo correlation structure
(`pra-monte-carlo`).

# Related (R-only)

For distribution-level propagation along the causal chain from risks to total
project cost, the package also provides probabilistic networks (`prob_net`,
`prob_net_sim`, `prob_net_learn`, `prob_net_update`). These are available from
an R session but are not exposed as MCP tools.

# References

- Steward (1981), *Systems Analysis and Management*.
- Browning (2001), Applying the DSM to System Decomposition and Integration.
- Govan and Damnjanovic (2016, 2020), resource-based / structural-network
  measures for project risk.

# Monte Carlo Simulation for Project Risk Analysis

## Overview

Monte Carlo simulation (MCS) is a quantitative risk analysis technique that uses random sampling from probability distributions to estimate the range of possible project outcomes (cost, duration). It is the most widely used method for schedule and cost risk analysis in project management.

## When to Use

- When task-level uncertainty can be described with probability distributions
- When you need percentile-based estimates (P50, P80, P95) for budgeting or scheduling
- When tasks may be correlated and you need to capture joint uncertainty
- When management requires a confidence level for cost or schedule targets

## Distribution Selection

Choose distributions based on available information:

- **Normal**: Use when historical data suggests symmetric uncertainty around a mean. Parameters: mean, sd.
- **Triangular**: Use when you can estimate optimistic (a), most likely (c), and pessimistic (b) values. Most common in practice. Parameters: a (min), b (max), c (mode).
- **Uniform**: Use when all values in a range are equally likely, or when only a range is known. Parameters: min, max.

## Correlation

Tasks are often correlated (e.g., labor cost overruns affect multiple tasks). Ignoring positive correlation underestimates total project variance. Provide a correlation matrix to capture these dependencies. If no correlation matrix is specified, tasks are assumed independent.

## Interpreting Results

- **P50 (median)**: The value with a 50% chance of being exceeded. Often used as the "most likely" total.
- **P80**: Common threshold for project contingency budgeting. 80% confidence the actual value will not exceed this.
- **P95**: High-confidence estimate, often used for management reserve or worst-case planning.
- **Mean**: Expected value of the total. May differ from P50 for skewed distributions.
- **Standard deviation**: Measures spread of the total distribution.

## Convergence

Use at least 10,000 simulations for stable percentile estimates. Fewer simulations (1,000) may suffice for rough estimates but percentiles in the tails (P5, P95) require more samples.

## Contingency and Sensitivity

After running MCS, follow up with:
- **Contingency analysis**: Calculate the reserve needed as the difference between P80/P95 and P50.
- **Sensitivity analysis**: Identify which tasks contribute most to total variance, guiding risk mitigation priorities.

## References

- GAO Cost Estimating and Assessment Guide (GAO-20-195G), Chapter 12: Quantifying Risk and Uncertainty.
- RAND Corporation, Quantitative Risk Analysis for Project Management (WR-112).

---
name: pra-overview
description: >
  Route a project risk analysis request to the right PRA tool. Use this as the
  entry point whenever a user asks about project cost/schedule uncertainty,
  earned value, risk probabilities, learning curves, or task dependencies and
  you are unsure which PRA capability applies. Points to the method-specific
  skills and the underlying MCP tools served by pra_mcp_server().
---

# What PRA offers

The PRA R package exposes its analytical methods as tools over the Model
Context Protocol via `pra_mcp_server()`. Once the server is registered, an
agent can call these tools directly. Each analytical area also has a dedicated
skill with fuller guidance.

# Choosing the right tool

| The user has... | Use MCP tool | Skill |
|---|---|---|
| Task distributions (normal/triangular/uniform), wants total cost/duration | `mcs_tool` | `pra-monte-carlo` |
| Only task means + variances, wants a fast estimate | `smm_tool` | `pra-monte-carlo` |
| Monte Carlo results, wants a reserve buffer | `contingency_tool` | `pra-sensitivity-contingency` |
| Task distributions, wants the biggest risk drivers | `sensitivity_tool` | `pra-sensitivity-contingency` |
| BAC + schedule + actual costs, wants performance/forecasts | `evm_analysis_tool` | `pra-earned-value` |
| Root-cause probabilities, wants risk probability | `risk_prob_tool` / `risk_post_prob_tool` | `pra-bayesian-risk` |
| Risk cost impacts, wants a cost distribution | `cost_pdf_tool` / `cost_post_pdf_tool` | `pra-bayesian-risk` |
| Historical time/completion data, wants a forecast curve | `fit_and_predict_sigmoidal_tool` | `pra-learning-curves` |
| A resource–task (and optional risk–resource) matrix | `parent_dsm_tool` / `grandparent_dsm_tool` | `pra-dsm` |

# Common multi-step workflows

1. **Schedule/cost risk**: `mcs_tool` → `contingency_tool` + `sensitivity_tool`
2. **EVM analysis**: `evm_analysis_tool` (computes PV, EV, AC, SV, CV, SPI, CPI,
   EAC, ETC, VAC, TCPI in one call)
3. **Bayesian prior → posterior**: `risk_prob_tool` → `cost_pdf_tool` →
   [observe causes] → `risk_post_prob_tool` → `cost_post_pdf_tool`
4. **Learning curve**: `fit_and_predict_sigmoidal_tool`

# Conventions

- All array/matrix arguments are passed as JSON strings (e.g.
  `task_dists_json`, `cor_mat_json`). Pass `"null"` (or omit) an optional
  correlation matrix to treat tasks as independent.
- Never fabricate numerical results — call the tool.
- The probabilistic-network functions (`prob_net`, `prob_net_sim`,
  `prob_net_learn`, `prob_net_update`) are available in the R package but are
  not exposed as MCP tools; use them from an R session.

---
name: pra-bayesian-risk
description: >
  Compute and update the probability of a risk event from root causes, and turn
  risk probabilities into project cost distributions, using the PRA package. Use
  when a user has a risk register with root-cause probabilities and wants a
  prior risk probability, a Bayesian update after observing causes, or a
  cost-impact distribution.
---

# When to use

Best when you have identifiable root causes with estimated probabilities and
want to update estimates as the project progresses (i.e. a risk register). For
task-level distributions and total project cost/schedule, prefer Monte Carlo
(`pra-monte-carlo`). The two combine: estimate risk probabilities here, then
feed cost impacts into a cost distribution.

# Prior and posterior risk probability

`risk_prob_tool(cause_probs_json, risks_given_causes_json,
risks_given_not_causes_json)` computes P(R) by the law of total probability
over independent root causes. All three arrays are the same length:

- `cause_probs_json`: P(cause present), e.g. `[0.3, 0.2]`
- `risks_given_causes_json`: P(Risk | Cause), e.g. `[0.8, 0.6]`
- `risks_given_not_causes_json`: P(Risk | not Cause), e.g. `[0.2, 0.4]`

`risk_post_prob_tool(..., observed_causes_json)` updates P(R) after observing
cause states. `observed_causes_json` uses `1` (occurred), `0` (did not occur),
`null` (still unknown), e.g. `[1, null]`.

# Cost distributions

- `cost_pdf_tool(num_sims, risk_probs_json, means_given_risks_json,
  sds_given_risks_json, base_cost)` — prior cost distribution. Each simulation
  draws whether each risk occurs, then samples its cost impact.
- `cost_post_pdf_tool(num_sims, observed_risks_json, means_given_risks_json,
  sds_given_risks_json, base_cost)` — posterior after observing which risks
  occurred (`observed_risks_json` uses `true`/`false`/`null`).

Parameters: `means_given_risks_json` / `sds_given_risks_json` give the mean and
SD of the *additional* cost if a risk occurs (one per risk); `base_cost` is the
baseline before any risk impacts.

# Workflow

`risk_prob_tool` → `cost_pdf_tool` → [observe causes/risks] →
`risk_post_prob_tool` → `cost_post_pdf_tool`.

# References

- NASA Risk-Informed Decision Making Handbook (SP-2010-576).
- NASA Bayesian Schedule and Cost Risk Analysis.

# Bayesian Inference for Risk Analysis

## Overview

Bayesian methods in project risk analysis allow you to compute the probability of risk events from root causes, and to update those probabilities as new information (observations) becomes available. This is particularly valuable in early project stages when data is sparse.

## Prior Risk Probability

Given a set of root causes C_1, C_2, ..., C_n, the overall probability of a risk event R is computed using the law of total probability:

P(R) = sum over i of: P(R|C_i) * P(C_i) + P(R|not C_i) * (1 - P(C_i))

Each root cause contributes independently to the risk. You need:
- **cause_probs**: P(C_i) for each root cause (probability the cause is present)
- **risks_given_causes**: P(R|C_i) (probability of risk if cause is present)
- **risks_given_not_causes**: P(R|not C_i) (probability of risk if cause is absent)

## Posterior Updating

When you observe whether a cause has actually occurred, you can update the risk probability using Bayesian inference. Observed causes are no longer uncertain - they become known (TRUE/FALSE), and the posterior probability accounts for this evidence.

Use `risk_post_prob()` with an `observed_causes` vector where:
- TRUE = cause was observed to have occurred
- FALSE = cause was observed to not have occurred
- NA = cause status is still unknown (remains at prior probability)

## Cost Distribution

Risk probabilities can be translated into cost distributions:
- **Prior cost distribution** (`cost_pdf`): Simulates project costs under prior risk probabilities. Each simulation draws whether each risk occurs (based on risk_probs), then samples from the corresponding cost distribution.
- **Posterior cost distribution** (`cost_post_pdf`): Same process but using updated (observed) risk states. Observed risks are known TRUE/FALSE; unobserved risks use their prior probabilities.

Parameters:
- **means_given_risks / sds_given_risks**: Mean and SD of additional cost if the risk occurs (one pair per risk).
- **base_cost**: Baseline project cost before any risk impacts.

## When to Use Bayesian vs. MCS

- **Bayesian**: Best when you have identifiable root causes with estimated probabilities, and you want to update estimates as the project progresses. Good for risk registers.
- **MCS**: Best when you have task-level distribution estimates and want total project cost/schedule distributions. Better for comprehensive uncertainty quantification.
- They can be combined: use Bayesian methods to estimate risk probabilities, then feed those into an MCS.

## References

- NASA Risk-Informed Decision Making Handbook (SP-2010-576).
- NASA Bayesian Schedule and Cost Risk Analysis.

---
title: "Learning Curves"
output:
  html_vignette:
    fig_width: 7
    fig_height: 5
vignette: >
  %\VignetteIndexEntry{Learning Curves}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

Learning curves model how efficiency or completion rate evolves over time as a team gains experience. In project management they are used to:

- **Forecast completion**: Predict when a task or deliverable will reach 100% based on past progress.
- **Identify acceleration**: Detect when a project is in its rapid-improvement phase versus plateauing.
- **Set realistic milestones**: Calibrate schedule targets against demonstrated learning rates.

## Sigmoidal Models

Sigmoidal (S-shaped) functions capture the three phases of learning: slow start, rapid improvement, and plateau. The PRA package provides three model types:

| Model | Formula | Parameters |
|-------|---------|------------|
| **Logistic** | K / (1 + exp(−r(t − t₀))) | K = ceiling; r = growth rate; t₀ = inflection time |
| **Pearl** | K / (1 + exp(−r(t − t₀))) | Same as logistic (identical functional form) |
| **Gompertz** | A · exp(−b · exp(−c · t)) | A = ceiling; c = growth rate; b = initial suppression |

**Parameter interpretation (Logistic / Pearl):**
- **K** — the maximum achievable value (e.g., 100% completion)
- **r** — how fast the S-curve rises; larger r = steeper transition
- **t₀** — the time at which the outcome is at its midpoint (inflection point)

## Example: Fitting a Logistic Model

```{r setup}
library(PRA)
```

We have weekly completion percentage data for a construction deliverable over 9 weeks.

```{r}
data <- data.frame(
  time       = 1:9,
  completion = c(5, 15, 40, 60, 70, 75, 80, 85, 90)
)
```

### Fit the Model

```{r}
fit <- fit_sigmoidal(data, "time", "completion", "logistic")
```

### Assess Fit Quality

Use `summary()` to examine the fitted coefficients, their standard errors, and the residual standard error — a measure of how closely the model matches the observed data.

```{r}
summary(fit)
```

A small residual standard error (relative to the response scale) indicates a good fit. Coefficient `t values` with |t| > 2 are statistically meaningful.

### Plot with Confidence Bands

`plot_sigmoidal()` plots the data, fitted curve, and optional confidence bounds in a single call.

```{r}
plot_sigmoidal(
  fit, data, "time", "completion", "logistic",
  conf_level = 0.95,
  main = "Logistic Learning Curve - Completion Forecast",
  xlab = "Week",
  ylab = "Completion (%)"
)
```

The shaded region is the 95% confidence band — the range within which the true fitted curve is likely to lie. Wider bands at the tails reflect greater uncertainty in extrapolated predictions.

### Predict Future Completion

Use `predict_sigmoidal()` to generate numeric forecasts, including confidence bounds for specific future time points.

```{r}
future_times <- seq(1, 12, length.out = 100)
predictions <- predict_sigmoidal(fit, future_times, "logistic", conf_level = 0.95)

knitr::kable(
  tail(round(predictions, 1), 5),
  caption = "Predicted completion (final 5 points)",
  row.names = FALSE
)
```

## Comparing All Three Model Types

Different model types can produce different forecasts, especially in the tails. It is good practice to fit multiple models and compare their goodness-of-fit.

```{r warning=FALSE}
fit_logistic <- fit_sigmoidal(data, "time", "completion", "logistic")
fit_pearl <- fit_sigmoidal(data, "time", "completion", "pearl")
fit_gompertz <- fit_sigmoidal(data, "time", "completion", "gompertz")
```

```{r}
# Residual standard errors for comparison
rse <- function(fit) summary(fit)$sigma

comparison <- data.frame(
  Model = c("Logistic", "Pearl", "Gompertz"),
  Residual_StdError = round(c(rse(fit_logistic), rse(fit_pearl), rse(fit_gompertz)), 3)
)
knitr::kable(comparison, caption = "Model Fit Comparison (lower RSE = better fit)")
```

Now plot all three fits side by side:

```{r}
x_seq <- seq(1, 12, length.out = 200)

pred_log <- predict_sigmoidal(fit_logistic, x_seq, "logistic")
pred_prl <- predict_sigmoidal(fit_pearl, x_seq, "pearl")
pred_gom <- predict_sigmoidal(fit_gompertz, x_seq, "gompertz")

# Base plot with observed data
plot(data$time, data$completion,
  pch = 16, xlim = c(1, 12), ylim = c(0, 105),
  main = "Learning Curve: Model Comparison",
  xlab = "Week", ylab = "Completion (%)"
)
lines(pred_log$x, pred_log$pred, col = "steelblue", lwd = 2)
lines(pred_prl$x, pred_prl$pred, col = "tomato", lwd = 2, lty = 2)
lines(pred_gom$x, pred_gom$pred, col = "darkgreen", lwd = 2, lty = 3)
legend("bottomright",
  legend = c("Logistic", "Pearl", "Gompertz"),
  col = c("steelblue", "tomato", "darkgreen"),
  lty = c(1, 2, 3), lwd = 2, bty = "n"
)
```

**Note:** Logistic and Pearl share the same functional form and will produce identical fits on the same data. Gompertz has a different shape, its inflection point occurs earlier and the curve is asymmetric, making it better suited for processes that accelerate quickly early on.

## Summary

The sigmoidal workflow in PRA:

1. `fit_sigmoidal()` — fit a model to observed time-completion data
2. `summary(fit)` — inspect coefficient estimates and goodness-of-fit
3. `predict_sigmoidal()` — generate numeric forecasts with optional confidence bounds
4. `plot_sigmoidal()` — visualize the fit and confidence band

Choose the model type based on the shape of your data and theoretical expectations about the learning process.

---
title: "Monte Carlo Simulation"
output:
  html_vignette:
    fig_width: 7
    fig_height: 5
vignette: >
  %\VignetteIndexEntry{Monte Carlo Simulation}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
set.seed(42)
```

Monte Carlo (MC) simulation is a quantitative risk analysis technique that models uncertainty by running thousands of simulated project outcomes. Instead of using single-point estimates for task durations or costs, each task is described by a probability distribution. The simulation draws random samples from these distributions, sums them to get a total outcome, and repeats this thousands of times to build a full picture of possible project results.

## Steps in MC Simulation

1. **Model Definition** — Define project tasks and the variables that drive uncertainty (durations, costs).
2. **Assign Distributions** — Choose a probability distribution for each uncertain variable (e.g., triangular for tasks with optimistic/likely/pessimistic estimates).
3. **Specify Correlations** — If tasks are related (e.g., both affected by a shared risk), set a correlation coefficient between them.
4. **Run Simulation** — Draw random samples and compute the total outcome for each iteration (typically 10,000+).
5. **Analyze Results** — Summarize the distribution of totals using percentiles, mean, and variance.

## Example

```{r setup}
library(PRA)
```

We model a 3-task project (in weeks). Task A follows a normal distribution, Task B has a triangular distribution (optimistic/most-likely/pessimistic), and Task C is uniformly distributed.

```{r}
num_simulations <- 10000
task_distributions <- list(
  list(type = "normal", mean = 10, sd = 2), # Task A
  list(type = "triangular", a = 5, b = 10, c = 15), # Task B
  list(type = "uniform", min = 8, max = 12) # Task C
)
```

### Correlation Matrix

Tasks often move together due to shared resources or external risks. The correlation matrix captures this. Values range from −1 (perfectly opposed) to +1 (perfectly aligned); 0 means independent. Here Tasks A and B have moderate positive correlation (0.5), meaning delays in one tend to coincide with delays in the other.

```{r}
correlation_matrix <- matrix(c(
  1.0, 0.5, 0.3,
  0.5, 1.0, 0.4,
  0.3, 0.4, 1.0
), nrow = 3, byrow = TRUE)
```

### Run the Simulation

```{r}
results <- mcs(num_simulations, task_distributions, correlation_matrix)
```

```{r results='asis'}
cat("Mean Total Duration:     ", round(results$total_mean, 2), "weeks\n")
cat("Variance of Duration:    ", round(results$total_variance, 2), "\n")
cat("Std Dev of Duration:     ", round(results$total_sd, 2), "weeks\n")
```

### Distribution of Outcomes

The histogram below shows all 10,000 simulated total durations. The overlaid density curve reveals the shape of the distribution.

```{r}
hist_data <- results$total_distribution

hist(hist_data,
  breaks = 50, freq = FALSE,
  main = "Monte Carlo Simulation - Total Project Duration",
  xlab = "Total Duration (weeks)", col = "steelblue", border = "white"
)
lines(density(hist_data), col = "tomato", lwd = 2)
abline(v = results$total_mean, col = "black", lty = 2, lwd = 1.5)
legend("topright",
  legend = c("Density", paste0("Mean = ", round(results$total_mean, 1), " wks")),
  col = c("tomato", "black"), lty = c(1, 2), lwd = 2, bty = "n"
)
```

## Interpreting Percentiles

The `mcs()` function returns key percentiles of the total distribution. These answer the question: *"What duration has X% probability of not being exceeded?"*

```{r}
knitr::kable(
  data.frame(
    Percentile = c("P5", "P50 (Median)", "P95"),
    Duration = round(results$percentiles, 1),
    Meaning = c(
      "5% chance of finishing this fast or faster",
      "Equal chance of finishing above or below this",
      "95% chance of finishing by this date"
    )
  ),
  caption = "Simulation Percentiles"
)
```

## Contingency Analysis

Contingency is the buffer added above the base estimate to cover uncertainty. A common approach is to use the difference between the P95 (or chosen confidence level) outcome and the P50 (base estimate).

```{r results='asis'}
contingency_val <- contingency(results, phigh = 0.95, pbase = 0.50)
cat("Schedule contingency (P95 − P50):", round(contingency_val, 2), "weeks\n")
cat(
  "There is a 95% chance the project will finish within",
  round(results$percentiles["95%"], 1), "weeks.\n"
)
```

**Interpretation:** Adding `r round(contingency_val, 1)` weeks of schedule contingency to the P50 estimate gives a 95% confidence of on-time delivery. Teams with low risk tolerance should use P95; those with higher tolerance might use P80.

## Sensitivity Analysis

Sensitivity analysis identifies which tasks drive the most variability in the total outcome — the tasks that deserve the most management attention.

```{r}
sensitivity_results <- sensitivity(task_distributions, correlation_matrix)

sens_data <- data.frame(
  Task        = c("Task A (Normal)", "Task B (Triangular)", "Task C (Uniform)"),
  Sensitivity = sensitivity_results
)

p <- ggplot2::ggplot(
  sens_data,
  ggplot2::aes(x = Sensitivity, y = reorder(Task, Sensitivity))
) +
  ggplot2::geom_col(fill = "steelblue") +
  ggplot2::geom_text(ggplot2::aes(label = round(Sensitivity, 3)),
    hjust = -0.1, size = 3.5
  ) +
  ggplot2::labs(
    title = "Tornado Chart - Task Sensitivity",
    x     = "Sensitivity Coefficient",
    y     = NULL
  ) +
  ggplot2::xlim(0, max(sensitivity_results) * 1.2) +
  ggplot2::theme_minimal()

print(p)
```

**Interpretation:** Tasks with larger bars contribute more variance to the total. Prioritize risk mitigation efforts on the highest-sensitivity task. Even a small reduction in its uncertainty can meaningfully reduce overall project risk.

# PRA Package Function Reference

## Monte Carlo Simulation

### mcs(num_sims, task_dists, cor_mat = NULL)
Run a Monte Carlo simulation for total project cost or duration.

**Parameters:**
- `num_sims`: Integer. Number of simulations (typically 10000).
- `task_dists`: List of lists. Each inner list has a `type` field ("normal", "triangular", or "uniform") and distribution parameters:
  - normal: `mean`, `sd`
  - triangular: `a` (min), `b` (max), `c` (mode)
  - uniform: `min`, `max`
- `cor_mat`: Optional numeric matrix. Correlation matrix (dimensions must match number of tasks).

**Returns:** List with `total_mean`, `total_variance`, `total_sd`, `percentiles` (5%, 50%, 95%), `total_distribution` (numeric vector).

**Example task_dists:**
```
list(
  list(type = "normal", mean = 10, sd = 2),
  list(type = "triangular", a = 5, b = 10, c = 15),
  list(type = "uniform", min = 8, max = 12)
)
```

### smm(mean, var, cor_mat = NULL)
Second Moment Method for quick analytical estimate of total project mean and variance.

**Parameters:**
- `mean`: Numeric vector of task means.
- `var`: Numeric vector of task variances.
- `cor_mat`: Optional correlation matrix.

**Returns:** List with `total_mean`, `total_var`, `total_std`.

## Earned Value Management

### pv(bac, schedule, time_period)
Planned Value. `bac` = total budget, `schedule` = vector of cumulative planned completion percentages, `time_period` = current period (integer). Returns numeric scalar.

### ev(bac, actual_per_complete)
Earned Value. `bac` = total budget, `actual_per_complete` = actual percent complete (0-1). Returns numeric scalar.

### ac(actual_costs, time_period, cumulative = TRUE)
Actual Cost. `actual_costs` = vector of costs per period, `time_period` = current period. Returns numeric scalar.

### sv(ev, pv)
Schedule Variance = EV - PV. Returns numeric scalar.

### cv(ev, ac)
Cost Variance = EV - AC. Returns numeric scalar.

### spi(ev, pv)
Schedule Performance Index = EV / PV. Returns numeric scalar.

### cpi(ev, ac)
Cost Performance Index = EV / AC. Returns numeric scalar.

### eac(bac, method = "typical", cpi = NULL, ac = NULL, ev = NULL, spi = NULL)
Estimate at Completion. Methods: "typical" (BAC/CPI), "atypical" (AC + BAC - EV), "combined" (AC + (BAC-EV)/(CPI*SPI)). Returns numeric scalar.

### etc(bac, ev, cpi = NULL)
Estimate to Complete. Returns numeric scalar.

### vac(bac, eac)
Variance at Completion = BAC - EAC. Returns numeric scalar.

### tcpi(bac, ev, ac, target = "bac", eac = NULL)
To-Complete Performance Index. `target` = "bac" or "eac". Returns numeric scalar.

## Post-Simulation Analysis

### contingency(sims, phigh = 0.95, pbase = 0.50)
Calculate contingency reserve. `sims` = mcs result object. Returns numeric scalar (P_high - P_base).

### sensitivity(task_dists, cor_mat = NULL)
Compute variance contribution of each task. Same input format as `mcs()`. Returns numeric vector (one value per task).

## Bayesian Risk Analysis

### risk_prob(cause_probs, risks_given_causes, risks_given_not_causes)
Prior probability of risk event from root causes. All vectors must be same length, values between 0 and 1. Returns numeric scalar.

### risk_post_prob(cause_probs, risks_given_causes, risks_given_not_causes, observed_causes)
Posterior risk probability after observing causes. `observed_causes` = logical vector (TRUE/FALSE/NA). Returns numeric scalar.

### cost_pdf(num_sims, risk_probs, means_given_risks, sds_given_risks, base_cost = 0)
Simulate prior cost distribution. Returns numeric vector of length num_sims.

### cost_post_pdf(num_sims, observed_risks, means_given_risks, sds_given_risks, base_cost = 0)
Simulate posterior cost distribution. `observed_risks` = logical vector (TRUE/FALSE/NA). Returns numeric vector.

## Learning Curves

### fit_sigmoidal(data, x_col, y_col, model_type)
Fit a sigmoidal model. `data` = data.frame, `x_col`/`y_col` = column name strings, `model_type` = "pearl", "gompertz", or "logistic". Returns nls model object.

### predict_sigmoidal(fit, x_range, model_type, conf_level = NULL)
Predict from fitted model. `fit` = nls object from fit_sigmoidal, `x_range` = numeric vector of x values. Returns data.frame with columns: x, pred, and optionally lwr, upr.

### plot_sigmoidal(fit, data, x_col, y_col, model_type, conf_level = NULL)
Plot fitted curve with data. Creates a plot as a side effect. Returns data.frame invisibly.

## Dependency Structure Matrices

### parent_dsm(S)
Resource-task dependency matrix. `S` = numeric matrix (resources x tasks). Returns S * t(S).

### grandparent_dsm(S, R)
Risk-based task dependency matrix. `S` = resource-task matrix, `R` = risk-resource matrix. Returns (S * t(R)) * t(S * t(R)).

## Correlation

### cor_matrix(num_samples = 100, num_vars = 5, dists)
Generate a correlation matrix from samples. `dists` = list of distribution functions. Returns numeric correlation matrix.

## Tool Chaining Patterns

Common multi-step workflows:
1. **Schedule/Cost Risk**: mcs() -> contingency() + sensitivity()
2. **EVM Analysis**: pv() + ev() + ac() -> sv() + cv() -> spi() + cpi() -> eac() -> etc() + vac()
3. **Bayesian Prior->Posterior**: risk_prob() -> cost_pdf() -> [observe] -> risk_post_prob() -> cost_post_pdf()
4. **Learning Curve**: fit_sigmoidal() -> predict_sigmoidal() -> plot_sigmoidal()

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inference.R
\name{cost_pdf}
\alias{cost_pdf}
\title{Cost Probability Density.}
\usage{
cost_pdf(
  num_sims,
  risk_probs,
  means_given_risks,
  sds_given_risks,
  base_cost = 0
)
}
\arguments{
\item{num_sims}{Number of random samples to draw from the mixture model.}

\item{risk_probs}{A vector of probabilities for each risk event 'R_i'.}

\item{means_given_risks}{A vector of means of the normal distribution for cost 'A' given each risk event 'R_i'.}

\item{sds_given_risks}{A vector of standard deviations of the normal distribution for cost 'A' given each risk event 'R_i'.}

\item{base_cost}{The baseline cost given no risk event occurs.}
}
\value{
A numeric vector of random samples from the mixture model.
}
\description{
This function generates random samples from a mixture model representing the cost 'A'
associated with multiple risk events 'R_i'. Each risk event has its own probability,
mean, and standard deviation for the cost distribution. The function also accounts for a baseline cost
when no risk event occurs.
}
\examples{
# Example with three risk events
num_sims <- 1000
risk_probs <- c(0.3, 0.5, 0.2)
means_given_risks <- c(10000, 15000, 5000)
sds_given_risks <- c(2000, 1000, 1000)
base_cost <- 2000
samples <- cost_pdf(
  num_sims = num_sims,
  risk_probs = risk_probs,
  means_given_risks = means_given_risks,
  sds_given_risks = sds_given_risks,
  base_cost = base_cost
)
hist(samples, breaks = 30, col = "skyblue", main = "Histogram of Cost", xlab = "Cost")
}
\references{
Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/smm.R
\name{smm}
\alias{smm}
\title{Second Moment Method Analysis.}
\usage{
smm(mean, var, cor_mat = NULL)
}
\arguments{
\item{mean}{The mean vector.}

\item{var}{The variance vector.}

\item{cor_mat}{The correlation matrix (optional). If not provided, tasks are assumed to be independent.}
}
\value{
The function returns a list of the total mean, variance, and standard deviation for the project.
}
\description{
This function performs the Second Moment Method (SMM) analysis to estimate the
total mean, variance, and standard deviation of a project based on individual
task means, variances, and an optional correlation matrix.
}
\examples{

# Set the mean vector, variance vector, and correlation matrix for a toy project.
mean <- c(10, 15, 20)
var <- c(4, 9, 16)
cor_mat <- matrix(c(
  1, 0.5, 0.3,
  0.5, 1, 0.4,
  0.3, 0.4, 1
), nrow = 3, byrow = TRUE)

# Use the Second Moment Method to estimate the results for the project.
result <- smm(mean, var, cor_mat)
print(result)

# Without correlation matrix (independent tasks)
result <- smm(mean, var)
print(result)

# When certain tasks are discrete and others are continuous, the SMM can still
# be applied as long as the variance values accurately reflect the variability of each task.

discrete_mean <- c(5, 10)
discrete_var <- c(0, 0)
continuous_mean <- c(15, 20)
continuous_var <- c(4, 5)
mean <- c(discrete_mean, continuous_mean)
var <- c(discrete_var, continuous_var)
cor_mat <- matrix(c(
  1, 0, 0.2, 0.3,
  0, 1, 0.1, 0.2,
  0.2, 0.1, 1, 0.4,
  0.3, 0.2, 0.4,
  1
), nrow = 4, byrow = TRUE)
result <- smm(mean, var, cor_mat)
print(result)

}
\references{
Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{html_result_table}
\alias{html_result_table}
\title{Build an HTML table from name-value pairs}
\usage{
html_result_table(...)
}
\description{
Build an HTML table from name-value pairs
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{execute_command}
\alias{execute_command}
\title{Parse and execute a slash command}
\usage{
execute_command(input)

execute_command(input)
}
\arguments{
\item{input}{Character string of user input (e.g. "/mcs n=5000 tasks=\link{...}").}
}
\value{
A list with \code{ok} (logical) and \code{result} (character or ContentToolResult).

A list with \code{ok} and \code{result}.
}
\description{
Parses user input starting with \code{/}, validates arguments, and executes
the corresponding tool function. Returns a list with \code{ok} (logical) and
\code{result} (character string — either the tool output or a help/error message).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{tool_result}
\alias{tool_result}
\title{Build a ContentToolResult or fall back to plain text}
\usage{
tool_result(text, title = NULL, html = NULL)
}
\description{
Build a ContentToolResult or fall back to plain text
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/evm.R
\name{etc}
\alias{etc}
\title{Estimate to Complete (ETC).}
\usage{
etc(bac, ev, cpi = NULL)
}
\arguments{
\item{bac}{Budget at Completion (BAC) (total planned budget).}

\item{ev}{Earned Value.}

\item{cpi}{Cost Performance Index. If NULL, assumes remaining work will be completed
at planned cost (ETC = BAC - EV). If provided, adjusts for current performance
(ETC = (BAC - EV) / CPI).}
}
\value{
The function returns the Estimate to Complete (ETC).
}
\description{
Calculates the Estimate to Complete (ETC), which is the expected cost to finish
the remaining work.
}
\examples{
bac <- 100000
ev <- 35000
cpi <- 0.83

# ETC assuming remaining work at planned rate
etc <- etc(bac, ev)
cat("ETC (planned rate):", etc, "\n")

# ETC assuming remaining work at current CPI
etc <- etc(bac, ev, cpi)
cat("ETC (current CPI):", round(etc, 2), "\n")

}
\references{
Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
}
\seealso{
\code{\link{eac}}, \code{\link{vac}}, \code{\link{tcpi}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sigmoidal.R
\name{print.pra_sigmoidal_fit}
\alias{print.pra_sigmoidal_fit}
\title{Print method for Sigmoidal Model}
\usage{
\method{print}{pra_sigmoidal_fit}(x, ...)
}
\arguments{
\item{x}{An object of class \code{"pra_sigmoidal_fit"} as returned by
\code{\link{fit_sigmoidal}}.}

\item{...}{Additional arguments (not used).}
}
\value{
No return value, called for side effects.
}
\description{
Displays the summary of the fitted sigmoidal model in a readable format.
}
\examples{
# Set up a data frame of time and completion percentage data
data <- data.frame(time = 1:10, completion = c(
  5, 15, 40, 60, 70, 75, 80, 85,
  90, 95
))

# Fit a logistic model to the data.
fit <- fit_sigmoidal(data, "time", "completion", "logistic")
# Print the model summary
print(fit)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/evm.R
\name{sv}
\alias{sv}
\title{Schedule Variance (SV).}
\usage{
sv(ev, pv)
}
\arguments{
\item{ev}{Earned Value.}

\item{pv}{Planned Value.}
}
\value{
The function returns the Schedule Variance (SV) of work completed.
}
\description{
Calculates the Schedule Variance (SV) of work completed based on the Earned Value (EV)
and Planned Value (PV).
}
\examples{
# Set the BAC, schedule, and current time period for an example project.
bac <- 100000
schedule <- c(0.1, 0.2, 0.4, 0.7, 1.0)
time_period <- 3

# Calculate the PV.
pv <- pv(bac, schedule, time_period)

# Set the actual \% complete and calculate the EV.
actual_per_complete <- 0.35
ev <- ev(bac, actual_per_complete)

# Calculate the SV and print the results.
sv <- sv(ev, pv)
cat("Schedule Variance (SV):", sv, "\n")

}
\references{
Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
}
\seealso{
\code{\link{pv}}, \code{\link{ev}}, \code{\link{ac}}, \code{\link{cv}},
\code{\link{spi}}, \code{\link{cpi}}, \code{\link{eac}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chat.R
\name{pra_chat}
\alias{pra_chat}
\title{Create a PRA Risk Analysis Chat Agent}
\usage{
pra_chat(
  chat = NULL,
  model = "llama3.2",
  rag = TRUE,
  embed_model = .pra_default_embed_model
)
}
\arguments{
\item{chat}{An optional pre-configured ellmer chat object. If provided,
\code{model} is ignored and tools are registered on this object instead.}

\item{model}{Character. Ollama model name (default \code{"llama3.2"}). Must support
tool calling. Other options: \code{"qwen2.5"}, \code{"llama3.1:70b"}.}

\item{rag}{Logical. Whether to use RAG context from the PRA knowledge base
(default \code{TRUE}). Requires Ollama embedding model to be available.}

\item{embed_model}{Character. Ollama embedding model for RAG (default
\code{"nomic-embed-text"}). Only used when \code{rag = TRUE}.}
}
\value{
A configured ellmer chat object with PRA tools registered. Use
\code{chat$chat("your question")} to interact.
}
\description{
Creates an ellmer chat object configured as a project risk analysis expert,
with all PRA functions registered as tools and optional RAG context retrieval
from the bundled knowledge base.
}
\details{
By default, uses a local Ollama model for fully offline, private operation.
Alternatively, supply a pre-configured ellmer chat object (e.g.,
\code{ellmer::chat_openai()}) via the \code{chat} parameter for cloud-hosted models.
}
\examples{
\dontrun{
# Default: local Ollama model
chat <- pra_chat()
chat$chat("Run a Monte Carlo simulation for a 3-task project with
  Task A: normal(10, 2), Task B: triangular(5, 10, 15), Task C: uniform(8, 12)")

# Use a cloud model for better accuracy
chat <- pra_chat(chat = ellmer::chat_openai(model = "gpt-4o"))

# Follow-up questions use conversation context
chat$chat("What is the contingency reserve at 95\% confidence?")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sigmoidal.R
\name{plot_sigmoidal}
\alias{plot_sigmoidal}
\title{Plot a Fitted Sigmoidal Model.}
\usage{
plot_sigmoidal(
  fit,
  data,
  x_col,
  y_col,
  model_type,
  conf_level = NULL,
  n_points = 100,
  main = NULL,
  xlab = NULL,
  ylab = NULL,
  line_col = "red",
  ci_col = "lightblue",
  pch = 16,
  ...
)
}
\arguments{
\item{fit}{A fitted sigmoidal model object from fit_sigmoidal.}

\item{data}{The original data frame used to fit the model.}

\item{x_col}{The name of the x (time) column in the data.}

\item{y_col}{The name of the y (completion) column in the data.}

\item{model_type}{The type of model (pearl, gompertz, or logistic).}

\item{conf_level}{Optional confidence level for confidence bounds (e.g., 0.95 for 95\%).
If NULL (default), no confidence bounds are plotted.}

\item{n_points}{Number of points to use for the fitted curve (default 100).}

\item{main}{Plot title. If NULL, a default title is generated.}

\item{xlab}{X-axis label. If NULL, uses x_col.}

\item{ylab}{Y-axis label. If NULL, uses y_col.}

\item{line_col}{Color for the fitted curve (default "red").}

\item{ci_col}{Color for the confidence band (default "lightblue").}

\item{pch}{Point character for data points (default 16).}

\item{...}{Additional arguments passed to plot().}
}
\value{
Invisibly returns the predictions data frame.
}
\description{
This function creates a base R plot of a fitted sigmoidal model with the
original data points, fitted curve, and optional confidence bounds.
}
\examples{
# Set up a data frame of time and completion percentage data
data <- data.frame(time = 1:10, completion = c(5, 15, 40, 60, 70, 75, 80, 85, 90, 95))

# Fit a logistic model to the data.
fit <- fit_sigmoidal(data, "time", "completion", "logistic")

# Plot the fitted model
plot_sigmoidal(fit, data, "time", "completion", "logistic")

# Plot with 95\% confidence bounds
plot_sigmoidal(fit, data, "time", "completion", "logistic", conf_level = 0.95)

# Customize the plot
plot_sigmoidal(fit, data, "time", "completion", "logistic",
  conf_level = 0.95,
  main = "Project Completion Forecast",
  xlab = "Time (weeks)",
  ylab = "Completion (\%)",
  line_col = "blue",
  ci_col = "lightgray"
)

}
\references{
Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/smm.R
\name{print.smm}
\alias{print.smm}
\title{Print method for SMM results.}
\usage{
\method{print}{smm}(x, ...)
}
\arguments{
\item{x}{An object of class "smm" containing the SMM results.}

\item{...}{Additional arguments (not used).}
}
\value{
None. The function prints the SMM results to the console.
}
\description{
This function defines how to print the results of the Second Moment Method
(SMM) analysis. It formats the output to display the total mean, variance,
and standard deviation in a readable manner.
}
\examples{
mean <- c(10, 15, 20)
var <- c(4, 9, 16)
cor_mat <- matrix(c(
  1, 0.5, 0.3,
  0.5, 1, 0.4,
  0.3, 0.4, 1
), nrow = 3, byrow = TRUE)
result <- smm(mean, var, cor_mat)
print(result)

# Without correlation matrix (independent tasks)
result <- smm(mean, var)
print(result)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{format_help_overview}
\alias{format_help_overview}
\title{Format the /help overview}
\usage{
format_help_overview()

format_help_overview()
}
\value{
A markdown-formatted overview of all available /commands.
}
\description{
Format the /help overview

Format Help Overview of All Commands
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{save_pra_plot}
\alias{save_pra_plot}
\title{Save a plot to a temp file and return path}
\usage{
save_pra_plot(plot_fn, name, width = 700, height = 450)
}
\description{
Save a plot to a temp file and return path
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{pra_tools}
\alias{pra_tools}
\title{Create PRA Tool Definitions for LLM Agent}
\usage{
pra_tools()
}
\value{
A list of ellmer tool objects.
}
\description{
Creates a list of ellmer tool objects that wrap PRA's exported functions for
use with an LLM agent. Each tool includes a description that helps the LLM
select the appropriate analysis method and properly format parameters.
}
\details{
Tool wrappers handle serialization between the LLM (JSON strings) and R
(lists, matrices, data.frames). Complex inputs like task distribution lists
and correlation matrices are accepted as JSON strings and deserialized
internally.

Large output vectors (e.g., Monte Carlo simulation samples) are summarized
to mean, sd, and key percentiles rather than returning the full vector to
the LLM. The full results are stored in the package environment for use by
downstream tools (e.g., contingency analysis needs the full distribution).

When used with shinychat, tool results include rich HTML display with inline
plots via \code{ellmer::ContentToolResult}.
}
\examples{
\dontrun{
tools <- pra_tools()
chat <- ellmer::chat_ollama(model = "llama3.2")
for (tool in tools) chat$register_tool(tool)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inference.R
\name{risk_prob}
\alias{risk_prob}
\title{Bayesian Inference for Risk Probability.}
\usage{
risk_prob(cause_probs, risks_given_causes, risks_given_not_causes)
}
\arguments{
\item{cause_probs}{A vector of probabilities for each root cause 'C_i'.}

\item{risks_given_causes}{A vector of conditional probabilities of the risk event 'R' given each cause 'C_i'.}

\item{risks_given_not_causes}{A vector of conditional probabilities of the risk event 'R' given not each cause 'C_i'.}
}
\value{
The function returns a numeric value for the probability of risk event 'R'.
}
\description{
This function calculates the overall probability of a risk event 'R' occurring
based on the probabilities of multiple root causes and their associated conditional probabilities.
}
\examples{
cause_probs <- c(0.3, 0.2)
risks_given_causes <- c(0.8, 0.6)
risks_given_not_causes <- c(0.2, 0.4)
risk_prob_value <- risk_prob(cause_probs, risks_given_causes, risks_given_not_causes)
print(risk_prob_value)

}
\references{
Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/learning.R
\name{risk_post_prob}
\alias{risk_post_prob}
\title{Posterior Risk Probability.}
\usage{
risk_post_prob(
  cause_probs,
  risks_given_causes,
  risks_given_not_causes,
  observed_causes
)
}
\arguments{
\item{cause_probs}{A vector of prior probabilities for each root cause 'C_i'.}

\item{risks_given_causes}{A vector of conditional probabilities of the risk event 'R' given each cause 'C_i'.}

\item{risks_given_not_causes}{A vector of conditional probabilities of the risk event 'R' given not each cause 'C_i'.}

\item{observed_causes}{A vector of observed values for each cause 'C_i' (1 if observed, 0 if not observed, NA if unobserved).}
}
\value{
A numeric value for the posterior probability of the risk event given the observed causes.
}
\description{
This function calculates the posterior probability of a risk event 'R' occurring
based on observations of multiple root causes and their associated conditional probabilities.
}
\examples{
cause_probs <- c(0.3, 0.2)
risks_given_causes <- c(0.8, 0.6)
risks_given_not_causes <- c(0.2, 0.4)
observed_causes <- c(1, NA)
risk_post_prob <- risk_post_prob(
  cause_probs, risks_given_causes,
  risks_given_not_causes, observed_causes
)
print(risk_post_prob)
}
\references{
Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dsm.R
\name{grandparent_dsm}
\alias{grandparent_dsm}
\title{Risk-based 'Grandparent' Design Structure Matrix (DSM).}
\usage{
grandparent_dsm(S, R)
}
\arguments{
\item{S}{Resource-Task Matrix 'S' giving the links (arcs) between resources and tasks.
Rows represent resources and columns represent tasks.}

\item{R}{Risk-Resource Matrix 'R' giving the links (arcs) between risks and resources.
Rows represent risks and columns represent resources.}
}
\value{
An S3 object of class \code{"dsm"} with the following components:
\describe{
\item{matrix}{The Risk-based 'Grandparent' DSM giving the number of risks shared between each task.}
\item{type}{Character string \code{"grandparent"}.}
\item{n_tasks}{Number of tasks (columns in S).}
\item{n_resources}{Number of resources (rows in S).}
\item{n_risks}{Number of risks (rows in R).}
}
}
\description{
This function computes the Risk-based 'Grandparent' Design Structure Matrix (DSM)
from given Resource-Task Matrix 'S' and Risk-Resource Matrix 'R'.
The 'Grandparent' DSM indicates the number of risks shared between each pair of
tasks in a project.
}
\examples{
# Set the S and R matrices and print the results.
S <- matrix(c(1, 1, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1), nrow = 3, ncol = 4)
R <- matrix(c(1, 1, 0, 1, 0, 0), nrow = 2, ncol = 3)
cat("Resource-Task Matrix (3 resources x 4 tasks):\n")
print(S)
cat("\nRisk-Resource Matrix (2 risks x 3 resources):\n")
print(R)
# Calculate the Risk-based Grandparent Matrix and print the results.
risk_dsm <- grandparent_dsm(S, R)
print(risk_dsm)

}
\references{
Govan, Paul, and Ivan Damnjanovic. "The resource-based view on project risk management."
Journal of construction engineering and management 142.9 (2016): 04016034.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rag.R
\name{retrieve_context}
\alias{retrieve_context}
\title{Retrieve Relevant Context for a Query}
\usage{
retrieve_context(store, query, top_k = 3)
}
\arguments{
\item{store}{A ragnar store object from \code{\link[=build_knowledge_base]{build_knowledge_base()}}.}

\item{query}{Character string. The user's question or query.}

\item{top_k}{Integer. Number of chunks to retrieve (default 5).}
}
\value{
A character vector of relevant text chunks with source attribution,
suitable for injecting into an LLM prompt as additional context.
}
\description{
Searches the PRA knowledge base using combined vector similarity search (VSS)
and BM25 full-text search to find the most relevant chunks for a user query.
}
\examples{
\dontrun{
store <- build_knowledge_base()
chunks <- retrieve_context(store, "What is earned value management?")
cat(chunks, sep = "\n---\n")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/evm.R
\name{vac}
\alias{vac}
\title{Variance at Completion (VAC).}
\usage{
vac(bac, eac)
}
\arguments{
\item{bac}{Budget at Completion (BAC) (total planned budget).}

\item{eac}{Estimate at Completion.}
}
\value{
The function returns the Variance at Completion (VAC).
}
\description{
Calculates the Variance at Completion (VAC), which is the difference between
the budget and the expected final cost. Positive VAC indicates under budget,
negative indicates over budget.
}
\examples{
bac <- 100000
eac <- 120482 # From EAC calculation

vac <- vac(bac, eac)
cat("Variance at Completion (VAC):", round(vac, 2), "\n")
cat("Project is expected to be", abs(round(vac, 2)), ifelse(vac < 0, "over", "under"), "budget\n")

}
\references{
Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
}
\seealso{
\code{\link{eac}}, \code{\link{etc}}, \code{\link{tcpi}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{as_r_input}
\alias{as_r_input}
\title{Coerce an input to an R object}
\usage{
as_r_input(x)
}
\description{
Accepts either a JSON string (from LLM tool calls) or a native R object
(from direct R usage). If the input is a single character string, it is
parsed as JSON. Otherwise it is returned as-is.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sigmoidal.R
\name{fit_sigmoidal}
\alias{fit_sigmoidal}
\title{Fit a Sigmoidal Model to Data.}
\usage{
fit_sigmoidal(data, x_col, y_col, model_type)
}
\arguments{
\item{data}{A data frame containing the time (x_col) and completion (y_col) vectors.}

\item{x_col}{The name of the time vector.}

\item{y_col}{The name of the completion vector.}

\item{model_type}{The name of the sigmoidal model (Pearl, Gompertz, or Logistic).}
}
\value{
The function returns a list of results for the sigmoidal model.
}
\description{
This function fits a sigmoidal model (Pearl, Gompertz, or Logistic) to the provided data.
}
\examples{
# Set up a data frame of time and completion percentage data
data <- data.frame(time = 1:10, completion = c(5, 15, 40, 60, 70, 75, 80, 85, 90, 95))

# Fit a logistic model to the data.
fit <- fit_sigmoidal(data, "time", "completion", "logistic")

# Use the model to predict future completion times.
predictions <- predict_sigmoidal(fit, seq(min(data$time), max(data$time),
  length.out = 100
), "logistic")

# Predict with 95\% confidence bounds
predictions_ci <- predict_sigmoidal(fit, seq(min(data$time), max(data$time),
  length.out = 100
), "logistic", conf_level = 0.95)

}
\references{
Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{pra_command_registry}
\alias{pra_command_registry}
\title{Get the slash-command registry}
\usage{
pra_command_registry()

pra_command_registry()
}
\value{
Named list of command definitions.

A named list of command definitions.
}
\description{
Returns a named list of command definitions. Each entry has:
\describe{
\item{title}{Display name}
\item{description}{One-line summary}
\item{args}{Named list of argument specs (name, type, required, description, example)}
\item{fn}{Function to call with parsed arguments}
}

Returns a named list of command definitions. Each command has a title,
description, argument specs (name, type, required, help), usage examples,
and an executor function that calls the underlying PRA tool wrapper.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rag.R
\name{build_knowledge_base}
\alias{build_knowledge_base}
\title{Build the PRA Knowledge Base for RAG Retrieval}
\usage{
build_knowledge_base(
  store_path = NULL,
  embed_model = "nomic-embed-text",
  overwrite = FALSE
)
}
\arguments{
\item{store_path}{Path to store the DuckDB knowledge base. Defaults to a
cache directory under \code{\link[tools:userdir]{tools::R_user_dir()}}.}

\item{embed_model}{Ollama embedding model name (default \code{"nomic-embed-text"}).}

\item{overwrite}{Logical. If \code{TRUE}, rebuild the knowledge base even if a
cached version exists. Default \code{FALSE}.}
}
\value{
A ragnar store object that can be passed to \code{\link[=retrieve_context]{retrieve_context()}}.
}
\description{
Reads the curated risk analysis knowledge files bundled with the PRA package,
chunks them, generates embeddings via Ollama, and stores them in a DuckDB-backed
ragnar knowledge base for retrieval-augmented generation.
}
\details{
The knowledge base is built once and cached to disk. Subsequent calls with the
same \code{store_path} load the existing store.
}
\examples{
\dontrun{
store <- build_knowledge_base()
context <- retrieve_context(store, "How do I run a Monte Carlo simulation?")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sigmoidal.R
\name{predict_sigmoidal}
\alias{predict_sigmoidal}
\title{Predict a Sigmoidal Function Using Fitted Model.}
\usage{
predict_sigmoidal(fit, x_range, model_type, conf_level = NULL)
}
\arguments{
\item{fit}{A list containing the results of a sigmoidal model.}

\item{x_range}{A vector of time values for the prediction.}

\item{model_type}{The type of model (Pearl, Gompertz, or Logistic) for the prediction.}

\item{conf_level}{Optional confidence level for confidence bounds (e.g., 0.95 for 95\%).
If NULL (default), no confidence bounds are computed.}
}
\value{
The function returns a data frame containing the time (x), predicted values (pred),
and optionally lower (lwr) and upper (upr) confidence bounds.
}
\description{
This function predicts values using a fitted sigmoidal model (Pearl, Gompertz,
or Logistic) over a specified range of time values.
}
\examples{
# Set up a data frame of time and completion percentage data
data <- data.frame(time = 1:10, completion = c(5, 15, 40, 60, 70, 75, 80, 85, 90, 95))

# Fit a logistic model to the data.
fit <- fit_sigmoidal(data, "time", "completion", "logistic")

# Use the model to predict future completion times.
predictions <- predict_sigmoidal(fit, seq(min(data$time), max(data$time),
  length.out = 100
), "logistic")

# Predict with 95\% confidence bounds
predictions_ci <- predict_sigmoidal(fit, seq(min(data$time), max(data$time),
  length.out = 100
), "logistic", conf_level = 0.95)

}
\references{
Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mcs.R
\name{print.mcs}
\alias{print.mcs}
\title{Print method for Monte Carlo Simulation results.}
\usage{
\method{print}{mcs}(x, ...)
}
\arguments{
\item{x}{An object of class "mcs".}

\item{...}{Additional arguments (not used).}
}
\value{
None. Prints the results to the console.
}
\description{
Displays the total mean, variance, standard deviation, and percentiles of the
Monte Carlo Simulation results in a readable format.
}
\examples{
# Set the number of simulations and task distributions for a toy project.
num_sims <- 10000
task_dists <- list(
  list(type = "normal", mean = 10, sd = 2), # Task A: Normal distribution
  list(type = "triangular", a = 5, b = 10, c = 15), # Task B: Triangular distribution
  list(type = "uniform", min = 8, max = 12) # Task C: Uniform distribution
)

# Set the correlation matrix for the correlations between tasks.
cor_mat <- matrix(c(
  1, 0.5, 0.3,
  0.5, 1, 0.4,
  0.3, 0.4, 1
), nrow = 3, byrow = TRUE)

# Run the Monte Carlo sumulation and print the results.
results <- mcs(num_sims, task_dists, cor_mat)
# print(results)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/evm.R
\name{cpi}
\alias{cpi}
\title{Cost Performance Index (CPI).}
\usage{
cpi(ev, ac)
}
\arguments{
\item{ev}{Earned Value.}

\item{ac}{Actual Cost.}
}
\value{
The function returns the Cost Performance Index (CPI) of work completed.
}
\description{
Calculates the Cost Performance Index (CPI) of work completed based on the Earned Value (EV)
and Actual Cost (AC).
}
\examples{
# Set the BAC and actual \% complete for an example project.
bac <- 100000
actual_per_complete <- 0.35

# Calcualte the EV
ev <- ev(bac, actual_per_complete)

# Set the actual costs and current time period and calculate the AC.
actual_costs <- c(9000, 18000, 36000, 70000, 100000)
time_period <- 3
ac <- ac(actual_costs, time_period)

# Calculate the CPI and print the results.
cpi <- cpi(ev, ac)
cat("Cost Performance Index (CPI):", cpi, "\n")

}
\references{
Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
}
\seealso{
\code{\link{pv}}, \code{\link{ev}}, \code{\link{ac}}, \code{\link{sv}},
\code{\link{cv}}, \code{\link{spi}}, \code{\link{eac}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/learning.R
\name{cost_post_pdf}
\alias{cost_post_pdf}
\title{Posterior Cost Probability Density.}
\usage{
cost_post_pdf(
  num_sims,
  observed_risks,
  means_given_risks,
  sds_given_risks,
  base_cost = 0
)
}
\arguments{
\item{num_sims}{Number of random samples to draw from the posterior distribution.}

\item{observed_risks}{A vector of observed values for each risk event 'R_i' (1 if observed, 0 if not observed, NA if unobserved).}

\item{means_given_risks}{A vector of means of the normal distribution for cost 'A' given each risk event 'R_i'.}

\item{sds_given_risks}{A vector of standard deviations of the normal distribution for cost 'A' given each risk event 'R_i'.}

\item{base_cost}{The baseline cost given no risk event occurs.}
}
\value{
A numeric vector of random samples from the posterior distribution of costs.
}
\description{
This function generates random samples from the posterior distribution of the cost 'A'
given observations of multiple risk events 'R_i'. Each risk event has its own mean
and standard deviation for the cost distribution. The function also accounts for a baseline cost
when no risk event occurs.
}
\examples{
# Example with three risk events
num_sims <- 1000
observed_risks <- c(1, NA, 1)
means_given_risks <- c(10000, 15000, 5000)
sds_given_risks <- c(2000, 1000, 1000)
base_cost <- 2000
posterior_samples <- cost_post_pdf(
  num_sims = num_sims,
  observed_risks = observed_risks,
  means_given_risks = means_given_risks,
  sds_given_risks = sds_given_risks,
  base_cost = base_cost
)
hist(posterior_samples, breaks = 30, col = "skyblue", main = "Posterior Cost PDF", xlab = "Cost")
}
\references{
Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{format_command_help}
\alias{format_command_help}
\title{Format help text for a single command}
\usage{
format_command_help(cmd_name, cmd)

format_command_help(cmd_name, cmd)
}
\arguments{
\item{cmd_name}{Character command name (without /).}

\item{cmd}{Command definition from the registry.}
}
\value{
A markdown-formatted help string.
}
\description{
Format help text for a single command

Format Help for a Single Command
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/contingency.R
\name{contingency}
\alias{contingency}
\title{Contingency Calculation.}
\usage{
contingency(sims, phigh = 0.95, pbase = 0.5)
}
\arguments{
\item{sims}{List of results from a Monte Carlo simulation containing the total
project duration distribution.}

\item{phigh}{Percentile level for contingency calculation. Default is 0.95 (95th percentile).}

\item{pbase}{Base level for contingency calculation. Default is 0.5 (50th percentile).}
}
\value{
The function returns the value of calculated contingency based on the
specified percentiles.
}
\description{
This function calculates the contingency required for a project based on the results
of a Monte Carlo simulation. The contingency is determined by the difference between
the specified high percentile (phigh) and the base percentile (pbase) of the total
project duration distribution.
}
\examples{
# Set the number os simulations and the task distributions for a toy project.
num_sims <- 10000
task_dists <- list(
  list(type = "normal", mean = 10, sd = 2), # Task A: Normal distribution
  list(type = "triangular", a = 5, b = 10, c = 15), # Task B: Triangular distribution
  list(type = "uniform", min = 8, max = 12) # Task C: Uniform distribution
)

# Set the correlation matrix for the correlations between tasks.
cor_mat <- matrix(c(
  1, 0.5, 0.3,
  0.5, 1, 0.4,
  0.3, 0.4, 1
), nrow = 3, byrow = TRUE)

# Run the Monte Carlo simulation.
results <- mcs(num_sims, task_dists, cor_mat)

# Calculate the contingency and print the results.
contingency <- contingency(results, phigh = 0.95, pbase = 0.50)
cat("Contingency based on 95th percentile and 50th percentile:", contingency)

# Without correlation matrix
results_indep <- mcs(num_sims, task_dists)
contingency_indep <- contingency(results_indep,
  phigh = 0.95,
  pbase = 0.50
)
cat("Contingency based on 95th percentile and 50th percentile (
independent tasks):", contingency_indep)

# Build a barplot to visualize the contingency results.
contingency_data <- data.frame(
  Scenario = c("With Correlation", "Independent Tasks"),
  Contingency = c(contingency, contingency_indep)
)
barplot(
  height = contingency_data$Contingency,
  names = contingency_data$Scenario,
  col = c("orange", "purple"),
  horiz = TRUE,
  xlab = "Contingency",
  ylab = "Scenario"
)
title("Contingency Calculation for Project Scenarios")

}
\references{
Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{plot_to_html}
\alias{plot_to_html}
\title{Generate a base64-encoded HTML img tag from a plot function}
\usage{
plot_to_html(plot_fn, width = 700, height = 450)
}
\description{
Generate a base64-encoded HTML img tag from a plot function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dsm.R
\name{print.dsm}
\alias{print.dsm}
\title{Print a DSM object.}
\usage{
\method{print}{dsm}(x, ...)
}
\arguments{
\item{x}{A \code{dsm} object returned by \code{\link[=parent_dsm]{parent_dsm()}} or \code{\link[=grandparent_dsm]{grandparent_dsm()}}.}

\item{...}{Additional arguments passed to \code{\link[=print.default]{print.default()}}.}
}
\value{
Invisibly returns \code{x}.
}
\description{
Print a DSM object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rag.R
\name{add_documents}
\alias{add_documents}
\title{Add Custom Documents to the PRA Knowledge Base}
\usage{
add_documents(store, path, embed_model = "nomic-embed-text")
}
\arguments{
\item{store}{A ragnar store object from \code{\link[=build_knowledge_base]{build_knowledge_base()}}.}

\item{path}{Character. Path to a file or directory. If a directory, all
\code{.md} and \code{.txt} files are ingested recursively.}

\item{embed_model}{Character. Ollama embedding model (default
\code{"nomic-embed-text"}).}
}
\value{
The store object (invisibly), updated with the new documents.
}
\description{
Ingests additional markdown or text documents into an existing PRA knowledge
base. Use this to extend the agent's knowledge with your own project
documentation, standards, templates, or lessons learned.
}
\examples{
\dontrun{
store <- build_knowledge_base()

# Add a single file
add_documents(store, "path/to/my_risk_register.md")

# Add all .md and .txt files in a directory
add_documents(store, "path/to/project_docs/")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/evm.R
\name{ev}
\alias{ev}
\title{Earned Value (EV).}
\usage{
ev(bac, actual_per_complete)
}
\arguments{
\item{bac}{Budget at Completion (BAC) (total planned budget).}

\item{actual_per_complete}{Actual work completion percentage.}
}
\value{
The function returns the Earned Value (EV) of work completed.
}
\description{
Calculates the Earned Value (EV) of work completed based on the Budget at Completion
(BAC) and the actual work completion percentage.
}
\examples{
# Set the BAC and actual \% complete for a toy project.
bac <- 100000
actual_per_complete <- 0.35

# Calculate the EV and print the results.
ev <- ev(bac, actual_per_complete)
cat("Earned Value (EV):", ev, "\n")
}
\references{
Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
}
\seealso{
\code{\link{pv}}, \code{\link{ac}}, \code{\link{sv}}, \code{\link{cv}},
\code{\link{spi}}, \code{\link{cpi}}, \code{\link{eac}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sensitivity.R
\name{sensitivity}
\alias{sensitivity}
\title{Sensitivity Analysis.}
\usage{
sensitivity(task_dists, cor_mat = NULL)
}
\arguments{
\item{task_dists}{A list of lists describing each task distribution. Each inner
list should contain the type of distribution and its parameters. Supported distributions
are "normal", "triangular", and "uniform".}

\item{cor_mat}{The correlation matrix for the tasks (Optional). If provided, it
should be a square matrix with dimensions equal to the number of tasks. If not
provided, tasks are assumed to be independent.}
}
\value{
The function returns a vector of sensitivity results with respect to
each task. Each element in the vector corresponds to the sensitivity of the variance
in total project cost with respect to the variance in the respective task's cost.
}
\description{
This function performs sensitivity analysis on a project with multiple tasks,
each having its own cost distribution. It calculates the sensitivity of the
variance in total project cost with respect to the variance in each task's cost.
It can also account for correlations between task costs if a correlation matrix
is provided.
}
\examples{
# Set the task distributions for a toy project.
task_dists <- list(
  list(type = "normal", mean = 10, sd = 2), # Task A: Normal distribution
  list(type = "triangular", a = 5, b = 15, c = 10), # Task B: Triangular distribution
  list(type = "uniform", min = 8, max = 12) # Task C: Uniform distribution
)

# Set the correlation matrix between the tasks.
cor_mat <- matrix(c(
  1, 0.5, 0.3,
  0.5, 1, 0.4,
  0.3, 0.4, 1
), nrow = 3, byrow = TRUE)

# Calculate the sensitivity of each task and print the results
sensitivity_results <- sensitivity(task_dists, cor_mat)
print(sensitivity_results)

# Build a vertical barchart and display the results.
data <- data.frame(
  Tasks = c("A", "B", "C"),
  Sensitivity = sensitivity_results
)
barplot(
  height = data$Sensitivity, names = data$Tasks, col = "skyblue",
  horiz = TRUE, xlab = "Sensitivity", ylab = "Tasks"
)
title("Sensitivity Analysis of Project Tasks")

# Without correlation matrix
sensitivity_results_indep <- sensitivity(task_dists)
print(sensitivity_results_indep)

# Build a vertical barchart and display the results.
data_indep <- data.frame(
  Tasks = c("A", "B", "C"),
  Sensitivity = sensitivity_results_indep
)
barplot(
  height = data_indep$Sensitivity, names = data_indep$Tasks,
  col = "lightgreen",
  horiz = TRUE, xlab = "Sensitivity", ylab = "Tasks"
)
title("Sensitivity Analysis of Project Tasks (Independent)")

}
\references{
Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PRA-package.R
\docType{package}
\name{PRA-package}
\alias{PRA}
\alias{PRA-package}
\title{PRA: Project Risk Analysis}
\description{
Data analysis for Project Risk Management via the Second Moment Method,
Monte Carlo Simulation, Contingency Analysis, Sensitivity Analysis,
Earned Value Management, Learning Curves, Bayesian Methods, and more.
}
\section{Key modules}{

\describe{
\item{\strong{Monte Carlo Simulation} (\code{mcs()})}{
Propagates schedule or cost uncertainty through a triangular distribution
model using \code{mc2d}. Returns an S3 object with a \code{print} method.}
\item{\strong{Second Moment Method} (\code{smm()})}{
Analytical first- and second-moment propagation of uncertainty without
simulation. Returns an S3 object with a \code{print} method.}
\item{\strong{Earned Value Management} (\code{pv()}, \code{ev()}, \code{ac()}, \code{sv()}, \code{cv()},
\code{spi()}, \code{cpi()}, \code{eac()}, \code{etc()}, \code{tcpi()}, \code{vac()})}{
Full suite of EVM performance metrics and forecasting functions.}
\item{\strong{Contingency Analysis} (\code{contingency()})}{
Derives cost or schedule contingency from simulation output at a
user-specified confidence level.}
\item{\strong{Sensitivity Analysis} (\code{sensitivity()})}{
Ranks risk drivers by their contribution to overall project uncertainty.}
\item{\strong{Learning Curves} (\code{fit_sigmoidal()}, \code{predict_sigmoidal()},
\code{plot_sigmoidal()})}{
Fits Pearl, Gompertz, or Logistic sigmoidal growth models to
cumulative cost or progress data using nonlinear least squares
(\code{minpack.lm}).}
\item{\strong{Bayesian Risk Inference} (\code{risk_prob()}, \code{risk_post_prob()})}{
Beta-Binomial conjugate updating for risk event probabilities.}
\item{\strong{Correlation Matrices} (\code{cor_matrix()})}{
Constructs valid positive-definite correlation matrices for use in
multivariate simulations.}
\item{\strong{Design Structure Matrix} (\code{parent_dsm()}, \code{grandparent_dsm()})}{
Derives direct (parent) and indirect (grandparent) dependency structure
matrices from a binary adjacency matrix.}
}
}

\section{References}{

Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
}

\seealso{
Useful links:
\itemize{
  \item \url{https://paulgovan.github.io/PRA/}
  \item \url{https://github.com/paulgovan/PRA}
  \item Report bugs at \url{https://github.com/paulgovan/PRA/issues}
}

}
\author{
Paul Govan \email{paul.govan2@gmail.com}
(ORCID: \href{https://orcid.org/0000-0002-1821-8492}{0000-0002-1821-8492})
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dsm.R
\name{parent_dsm}
\alias{parent_dsm}
\title{Resource-based 'Parent' Design Structure Matrix (DSM).}
\usage{
parent_dsm(S)
}
\arguments{
\item{S}{Resource-Task Matrix 'S' giving the links (arcs) between resources and tasks.
Rows represent resources and columns represent tasks.}
}
\value{
An S3 object of class \code{"dsm"} with the following components:
\describe{
\item{matrix}{The Resource-based 'Parent' DSM giving the number of resources shared between each task.}
\item{type}{Character string \code{"parent"}.}
\item{n_tasks}{Number of tasks (columns in S).}
\item{n_resources}{Number of resources (rows in S).}
}
}
\description{
This function computes the Resource-based 'Parent' Design Structure Matrix (DSM)
from a given Resource-Task Matrix 'S'. The 'Parent' DSM indicates the number of resources
shared between each pair of tasks in a project.
}
\examples{
# Set the S matrix for a toy project (3 resources x 4 tasks).
s <- matrix(c(1, 1, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1), nrow = 3, ncol = 4)
cat("Resource-Task Matrix:\n")
print(s)

# Calculate the Resource-based Parent DSM and print the results.
resource_dsm <- parent_dsm(s)
print(resource_dsm)

}
\references{
Govan, Paul, and Ivan Damnjanovic. "The resource-based view on project risk management."
Journal of construction engineering and management 142.9 (2016): 04016034.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/evm.R
\name{pv}
\alias{pv}
\title{Planned Value (PV).}
\usage{
pv(bac, schedule, time_period)
}
\arguments{
\item{bac}{Budget at Completion (BAC) (total planned budget).}

\item{schedule}{Vector of planned work completion (in terms of percentage) at each
time period.}

\item{time_period}{Current time period.}
}
\value{
The function returns the Planned Value (PV) of work completed.
}
\description{
Calculates the Planned Value (PV) of work completed based on the Budget at Completion
(BAC) and the planned schedule.
}
\examples{
# Set the BAC, schedule, and current time period for a toy project.
bac <- 100000
schedule <- c(0.1, 0.2, 0.4, 0.7, 1.0)
time_period <- 3

# Calculate the PV and print the results.
pv <- pv(bac, schedule, time_period)
cat("Planned Value (PV):", pv, "\n")

}
\references{
Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
}
\seealso{
\code{\link{ev}}, \code{\link{ac}}, \code{\link{sv}}, \code{\link{cv}},
\code{\link{spi}}, \code{\link{cpi}}, \code{\link{eac}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/evm.R
\name{ac}
\alias{ac}
\title{Actual Cost (AC).}
\usage{
ac(actual_costs, time_period, cumulative = TRUE)
}
\arguments{
\item{actual_costs}{Vector of actual costs incurred at each time period. Can be either
period costs (cost per period) or cumulative costs depending on the cumulative parameter.}

\item{time_period}{Current time period.}

\item{cumulative}{Logical. If TRUE (default), actual_costs are already cumulative and
the value at time_period is returned directly. If FALSE, actual_costs are period costs
and will be summed up to time_period.}
}
\value{
The function returns the Actual Cost (AC) of work completed to date.
}
\description{
Calculates the Actual Cost (AC) of work completed based on the actual costs incurred
at each time period.
}
\examples{
# Using cumulative costs (default)
cumulative_costs <- c(9000, 27000, 63000, 133000, 233000)
time_period <- 3
ac <- ac(cumulative_costs, time_period)
cat("Actual Cost (AC):", ac, "\n")

# Using period costs
period_costs <- c(9000, 18000, 36000, 70000, 100000)
ac <- ac(period_costs, time_period, cumulative = FALSE)
cat("Actual Cost (AC):", ac, "\n")

}
\references{
Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
}
\seealso{
\code{\link{pv}}, \code{\link{ev}}, \code{\link{sv}}, \code{\link{cv}},
\code{\link{spi}}, \code{\link{cpi}}, \code{\link{eac}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{as_numeric_input}
\alias{as_numeric_input}
\title{Coerce input to a numeric vector}
\usage{
as_numeric_input(x)
}
\description{
Parses JSON input and ensures the result is a numeric vector. Handles
cases where LLMs send string representations of numbers.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/app.R
\name{pra_app}
\alias{pra_app}
\title{Launch the PRA Risk Analysis Agent App}
\usage{
pra_app(
  model = "llama3.2",
  rag = TRUE,
  embed_model = "nomic-embed-text",
  port = NULL,
  launch.browser = TRUE
)
}
\arguments{
\item{model}{Character. Ollama model name (default \code{"llama3.2"}).}

\item{rag}{Logical. Whether to enable RAG context retrieval (default \code{TRUE}).}

\item{embed_model}{Character. Ollama embedding model for RAG (default
\code{"nomic-embed-text"}).}

\item{port}{Integer. Port for the Shiny app (default \code{NULL} lets Shiny choose).}

\item{launch.browser}{Logical. Whether to open a browser (default \code{TRUE}).}
}
\value{
None. This function is called to launch the shiny app.
}
\description{
Starts a Shiny app that provides an interactive chat interface to the PRA
risk analysis agent. The agent can select and execute PRA tools (Monte Carlo
simulation, EVM, Bayesian inference, etc.) in response to natural language
questions. Uses shinychat for a polished streaming chat experience with
inline tool result display.
}
\details{
Requires Ollama to be running locally with the specified model downloaded.
}
\examples{
\dontrun{
# Ensure Ollama is running, then:
pra_app()

# With a specific model:
pra_app(model = "qwen2.5")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{parse_command_args}
\alias{parse_command_args}
\title{Parse key=value argument string with bracket-aware splitting}
\usage{
parse_command_args(arg_string, known_args)
}
\arguments{
\item{arg_string}{Character string of arguments (without the command name).}

\item{known_args}{Character vector of recognized argument names.}
}
\value{
A named list of parsed argument values (all character strings).
}
\description{
Splits a string like \verb{n=5000 tasks=[\{"type": "normal", "mean": 10\}]} into
a named list \code{list(n = "5000", tasks = '[{"type": "normal", "mean": 10}]')}.
Tracks bracket/brace/quote nesting so that spaces inside \verb{[]}, \code{{}}, or \code{""}
are not treated as argument separators.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/evm.R
\name{eac}
\alias{eac}
\title{Estimate at Completion (EAC).}
\usage{
eac(bac, method = "typical", cpi = NULL, ac = NULL, ev = NULL, spi = NULL)
}
\arguments{
\item{bac}{Budget at Completion (BAC) (total planned budget).}

\item{method}{The EAC calculation method. One of:
\itemize{
\item "typical" (default): EAC = BAC / CPI. Assumes future work performed at current cost efficiency.
\item "atypical": EAC = AC + (BAC - EV). Assumes future work performed at planned rate.
\item "combined": EAC = AC + (BAC - EV) / (CPI * SPI). Considers both cost and schedule performance.
}}

\item{cpi}{Cost Performance Index (CPI). Required for "typical" and "combined" methods.}

\item{ac}{Actual Cost. Required for "atypical" and "combined" methods.}

\item{ev}{Earned Value. Required for "atypical" and "combined" methods.}

\item{spi}{Schedule Performance Index. Required for "combined" method.}
}
\value{
The function returns the Estimate at Completion (EAC).
}
\description{
Calculates the Estimate at Completion (EAC) using various methods based on
project performance assumptions.
}
\examples{
# Method 1: Typical - assumes current CPI continues
bac <- 100000
cpi <- 0.83
eac <- eac(bac, cpi = cpi)
cat("EAC (typical):", round(eac, 2), "\n")

# Method 2: Atypical - assumes future work at planned rate
ac <- 63000
ev <- 35000
eac <- eac(bac, method = "atypical", ac = ac, ev = ev)
cat("EAC (atypical):", round(eac, 2), "\n")

# Method 3: Combined - considers both CPI and SPI
spi <- 0.875
eac <- eac(bac, method = "combined", cpi = cpi, ac = ac, ev = ev, spi = spi)
cat("EAC (combined):", round(eac, 2), "\n")

}
\references{
Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
}
\seealso{
\code{\link{pv}}, \code{\link{ev}}, \code{\link{ac}}, \code{\link{sv}},
\code{\link{cv}}, \code{\link{spi}}, \code{\link{cpi}}, \code{\link{etc}},
\code{\link{vac}}, \code{\link{tcpi}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/app.R
\name{get_ollama_models}
\alias{get_ollama_models}
\title{Fetch Available Ollama Models}
\usage{
get_ollama_models()
}
\value{
A character vector of model names.
}
\description{
Queries the local Ollama API for available models. Falls back to a default
list if the API is unreachable.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chat.R
\name{pra_system_prompt}
\alias{pra_system_prompt}
\title{Generate the PRA System Prompt}
\usage{
pra_system_prompt()
}
\value{
A character string containing the system prompt.
}
\description{
Creates the system prompt that defines the agent's persona, methodology
guidance, and tool selection heuristics. Kept concise to work well with
smaller local models.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cormat.R
\name{cor_matrix}
\alias{cor_matrix}
\title{Generate Correlation Matrix from Random Samples.}
\usage{
cor_matrix(num_samples = 100, num_vars = 5, dists)
}
\arguments{
\item{num_samples}{The number of samples to generate.}

\item{num_vars}{The number of distributions to sample.}

\item{dists}{A list describing each distribution. Each element should be a function
that generates random samples. The names of the list elements will be used to
identify the distributions.}
}
\value{
The function returns the correlation matrix for the distributions.
}
\description{
This function generates random samples from specified probability distributions
and computes the correlation matrix for the generated samples.
}
\examples{
# List of probability distributions
dists <- list(
  normal = function(n) rnorm(n, mean = 0, sd = 1),
  uniform = function(n) runif(n, min = 0, max = 1),
  exponential = function(n) rexp(n, rate = 1),
  poisson = function(n) rpois(n, lambda = 1),
  binomial = function(n) rbinom(n, size = 10, prob = 0.5)
)

# Generate correlation matrix
cor_matrix <- cor_matrix(num_samples = 100, num_vars = 5, dists = dists)

# Print correlation matrix
print(cor_matrix)

}
\references{
Govan, Paul, and Ivan Damnjanovic. "The resource-based view on project risk management."
Journal of construction engineering and management 142.9 (2016): 04016034.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mcs.R
\name{mcs}
\alias{mcs}
\title{Monte Carlo Simulation.}
\usage{
mcs(num_sims, task_dists, cor_mat = NULL)
}
\arguments{
\item{num_sims}{The number of simulations to run.}

\item{task_dists}{A list of lists describing each task distribution with its parameters.
Each task distribution should be specified as a list with a "type" field (indicating
the distribution type: "normal", "triangular", or "uniform") and the corresponding
parameters: for "normal" (mean, sd), for "triangular" (a, b, c), and for "uniform"
(min, max). For example:
list(
list(type = "normal", mean = 10, sd = 2),
list(type = "triangular", a = 5, b = 10, c = 15),
list(type = "uniform", min = 8, max = 12)
)}

\item{cor_mat}{The correlation matrix for the tasks (Optional). If not provided,
tasks are assumed to be independent.}
}
\value{
The function returns a list of the total mean, variance, standard deviation,
and percentiles for the project.
}
\description{
This function performs a Monte Carlo simulation to estimate the total duration of a project
based on individual task distributions and an optional correlation matrix.
}
\examples{
# Set the number of simulations and task distributions for a toy project.
num_sims <- 10000
task_dists <- list(
  list(type = "normal", mean = 10, sd = 2), # Task A: Normal distribution
  list(type = "triangular", a = 5, b = 10, c = 15), # Task B: Triangular distribution
  list(type = "uniform", min = 8, max = 12) # Task C: Uniform distribution
)

# Set the correlation matrix for the correlations between tasks.
cor_mat <- matrix(c(
  1, 0.5, 0.3,
  0.5, 1, 0.4,
  0.3, 0.4, 1
), nrow = 3, byrow = TRUE)

# Run the Monte Carlo sumulation and print the results.
results <- mcs(num_sims, task_dists, cor_mat)
cat("Mean Total Duration:", results$total_mean, "\n")
cat("Variance of Total Variance:", results$total_variance, "\n")
cat("Standard Deviation of Total Duration:", results$total_sd, "\n")
cat("5th Percentile:", results$percentiles[1], "\n")
cat("Median (50th Percentile):", results$percentiles[2], "\n")
cat("95th Percentile:", results$percentiles[3], "\n")
hist(results$total_distribution,
  breaks = 50, main = "Distribution of Total Project Duration",
  xlab = "Total Duration", col = "skyblue", border = "white"
)
legend("topright", legend = c("Total Duration Distribution"), fill = c("skyblue"))

}
\references{
Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{route_input}
\alias{route_input}
\title{Route User Input to the Appropriate Handler}
\usage{
route_input(input)
}
\arguments{
\item{input}{Character string of user input.}
}
\value{
A list with:
\describe{
\item{mode}{Character: \code{"command"}, \code{"tool"}, or \code{"rag"}}
\item{reason}{Character: brief explanation of the routing decision}
}
For \code{"command"} mode, also includes \code{command} (the command name).
}
\description{
Classifies user input into one of three modes and returns a structured
routing decision. This function encapsulates the three-mode routing
architecture used by both the Shiny app and programmatic interfaces.
}
\details{
Routing logic:
\enumerate{
\item Input starting with \code{/} is routed to the \strong{command} handler
for deterministic execution (no LLM involved).
\item Input containing numerical data patterns (distributions, arrays,
dollar amounts, percentages with surrounding context) is routed
to the \strong{tool} handler where the LLM selects and calls tools.
\item All other input (conceptual questions, explanations) is routed
to the \strong{rag} handler where the LLM answers from the knowledge
base context.
}

Note: modes \code{"tool"} and \code{"rag"} both go through the LLM, but the
distinction affects how the query is framed (tool-calling emphasis vs.
RAG-context emphasis). The LLM ultimately decides whether to call a
tool, but the routing hint improves reliability with smaller models.
}
\examples{
\dontrun{
route_input("/mcs tasks=[...]")
# list(mode = "command", reason = "...", command = "mcs")

route_input("Simulate 3 tasks: Normal(10,2)...")
# list(mode = "tool", reason = "...")

route_input("What is earned value?")
# list(mode = "rag", reason = "...")
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dsm.R
\name{plot.dsm}
\alias{plot.dsm}
\title{Plot a DSM heatmap.}
\usage{
\method{plot}{dsm}(x, main = NULL, col = NULL, ...)
}
\arguments{
\item{x}{A \code{dsm} object returned by \code{\link[=parent_dsm]{parent_dsm()}} or \code{\link[=grandparent_dsm]{grandparent_dsm()}}.}

\item{main}{Optional plot title. If \code{NULL}, a default title is generated.}

\item{col}{Color palette vector. If \code{NULL}, uses \code{\link[grDevices:palettes]{grDevices::heat.colors()}}.}

\item{...}{Additional arguments passed to \code{\link[graphics:image]{graphics::image()}}.}
}
\value{
Invisibly returns \code{x}.
}
\description{
Displays the Design Structure Matrix as a heatmap where color intensity
represents the number of shared resources (parent) or risks (grandparent)
between task pairs.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/app.R
\name{pra_shiny_app}
\alias{pra_shiny_app}
\title{Create the PRA Shiny App Object}
\usage{
pra_shiny_app(model = "llama3.2", rag = TRUE, embed_model = "nomic-embed-text")
}
\arguments{
\item{model}{Ollama model name.}

\item{rag}{Whether to enable RAG.}

\item{embed_model}{Ollama embedding model.}
}
\value{
A Shiny app object.
}
\description{
Create the PRA Shiny App Object
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tools.R
\name{validate_task_dists}
\alias{validate_task_dists}
\title{Validate task distributions parsed from JSON}
\usage{
validate_task_dists(task_dists, tool_name = "mcs_tool")
}
\description{
Validate task distributions parsed from JSON
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/evm.R
\name{tcpi}
\alias{tcpi}
\title{To-Complete Performance Index (TCPI).}
\usage{
tcpi(bac, ev, ac, target = "bac", eac = NULL)
}
\arguments{
\item{bac}{Budget at Completion (BAC) (total planned budget).}

\item{ev}{Earned Value.}

\item{ac}{Actual Cost.}

\item{target}{The target to calculate TCPI against. Either "bac" (default) to meet
original budget, or "eac" to meet revised estimate. If "eac", the eac parameter
must be provided.}

\item{eac}{Estimate at Completion. Required when target = "eac".}
}
\value{
The function returns the To-Complete Performance Index (TCPI).
}
\description{
Calculates the To-Complete Performance Index (TCPI), which indicates the cost
performance required on remaining work to meet a target (BAC or EAC).
TCPI > 1 means efficiency must improve; TCPI < 1 means efficiency can decrease.
}
\examples{
bac <- 100000
ev <- 35000
ac <- 63000

# TCPI to complete within original budget
tcpi_bac <- tcpi(bac, ev, ac)
cat("TCPI (to meet BAC):", round(tcpi_bac, 2), "\n")

# TCPI to complete within revised estimate
eac <- 120482
tcpi_eac <- tcpi(bac, ev, ac, target = "eac", eac = eac)
cat("TCPI (to meet EAC):", round(tcpi_eac, 2), "\n")

}
\references{
Damnjanovic, Ivan, and Kenneth Reinschmidt. Data analytics for engineering and
construction project risk management. No. 172534. Cham, Switzerland: Springer, 2020.
}
\seealso{
\code{\link{eac}}, \code{\link{etc}}, \code{\link{vac}}, \code{\link{cpi}}
}
